@@ -52,13 +52,13 @@
 #include "token.h"
 #include "graph.h"
 
-void start_subtick(double *tick1, double gmin, double dticks);
+double start_subtick(double dsubticks, double dticks, GLEAxis* ax);
 void nice_ticks(double *dticks, double *gmin,double *gmax, double *t1,double *tn,int minset, int maxset);
 void numtrim(char **d,char *s, double dticks);
 void nice_log_ticks(double *start, double *last, double gmin, double gmax) throw (ParserError);
 
-double fnloglen(double v,GLEAxis *ax);
-double fnlogx(double v,GLEAxis *ax);
+double fnloglen(double v, GLEAxis *ax);
+double fnlogx(double v, GLEAxis *ax);
 
 void draw_axis_titles(GLEAxis *ax, double h, double ox, double oy, GLEMeasureBox* measure);
 void draw_axis_titles_v35(GLEAxis *ax, double h, double ox, double oy, double dticks, double llen);
@@ -68,13 +68,6 @@ extern double graph_x2;
 extern double graph_y1;
 extern double graph_y2;
 
-#define fnx(vv) m_fnx(vv)
-#define fnlx(vv) fnlogx(vv,ax)
-#define true (!false)
-#define false 0
-
-static double  gglen,ggmin,ggmax,ggnegate;
-
 void add_tex_labels(string* title)
 {
 	if (g_get_tex_labels() && str_i_str(*title, "\\tex{") == -1) {
@@ -95,13 +88,31 @@ double axis_range_dist_perc(double v1, double v2, GLERange* range, bool log) {
 	return fabs(v1 - v2);
 }
 
-double m_fnx(double v);
+double fnloglen(double v, GLEAxis *ax) {
+	return (v - log10(ax->getMin())) / (log10(ax->getMax()) - log10(ax->getMin())) * ax->length;
+}
 
-double m_fnx(double v) {
-	if (ggnegate) 	{
-		v = ggmax - (v-ggmin);
+double fnAxisX(double v, GLEAxis *ax) {
+	if (ax->negate) {
+		v = ax->getMax() - (v - ax->getMin());
+	}
+	if (ax->log) {
+		return fnloglen(log10(v), ax);
+	} else {
+		return (v - ax->getMin()) / (ax->getMax() - ax->getMin()) * ax->length;
+	}
+}
+
+bool axis_value_equal(double v1, double v2, GLEAxis* ax) {
+	if (ax->log) {
+		if (v2 == 0) {
+			return fabs(v1) < 0.001;
+		} else {
+			return fabs(v1 - v2) / v2 < 0.001;
+		}
+	} else {
+		return fabs(v1 - v2) < ax->dsubticks/100.0;
 	}
-	return (((v-ggmin)/(ggmax-ggmin)) * gglen );
 }
 
 bool axis_is_pos(double pos, int* cnt, double del, vector<double>& vec) {
@@ -142,11 +153,11 @@ void axis_draw_tick(GLEAxis *ax, double fi, int* tick1_cnt, int* tick2_cnt, doub
 	double from = has_tick2 ? -fabs(t)*inv : 0.0;
 	double to = has_tick1 ? fabs(t)*inv : 0.0;
 	if (axis_horizontal(ax->type)) {
-		g_move(ox+fnx(fi), oy+from);
-		g_line(ox+fnx(fi), oy+to);
+		g_move(ox + fnAxisX(fi, ax), oy + from);
+		g_line(ox + fnAxisX(fi, ax), oy + to);
 	} else {
-		g_move(ox+from, oy+fnx(fi));
-		g_line(ox+to, oy+fnx(fi));
+		g_move(ox + from, oy + fnAxisX(fi, ax));
+		g_line(ox + to, oy + fnAxisX(fi, ax));
 	}
 }
 
@@ -160,49 +171,107 @@ void axis_draw_tick_log(GLEAxis *ax, double fi, int* tick1_cnt, int* tick2_cnt,
 	double from = has_tick2 ? -fabs(t)*inv : 0.0;
 	double to = has_tick1 ? fabs(t)*inv : 0.0;
 	if (axis_horizontal(ax->type)) {
-		g_move(ox+fnlx(fi), oy+from);
-		g_line(ox+fnlx(fi), oy+to);
+		g_move(ox + fnAxisX(fi, ax), oy + from);
+		g_line(ox + fnAxisX(fi, ax), oy + to);
 	} else {
-		g_move(ox+from, oy+fnlx(fi));
-		g_line(ox+to, oy+fnlx(fi));
+		g_move(ox + from, oy + fnAxisX(fi, ax));
+		g_line(ox + to, oy + fnAxisX(fi, ax));
 	}
 }
 
-void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
-	double fi,x,y,gmin,gmax,dticks,tick1,tickn;
-	int i,xax,n,savecap;
-	double tlen,stlen,dsubticks,tt,t;
+bool inAxisRange(double value, GLEAxis* ax) {
+	if (value >= ax->getMin() && value <= ax->getMax()) {
+		return true;
+	}
+	if (axis_value_equal(value, ax->getMin(), ax)) {
+		return true;
+	}
+	if (axis_value_equal(value, ax->getMax(), ax)) {
+		return true;
+	}
+	return false;
+}
+
+std::vector<double> getLogSubPlaces(double pos, double gmin, double gmax, int lgset) {
+   std::vector<double> result;
+   if (lgset == GLE_AXIS_LOG_1 || lgset == GLE_AXIS_LOG_25 || lgset == GLE_AXIS_LOG_25B) {
+      for (int i = 2; i <= 9; i++) {
+         if (lgset == GLE_AXIS_LOG_1 || i == 2 || i == 5) {
+            double pos_sub = pos * i;
+            if (pos_sub >= gmin && pos_sub <= gmax) {
+               result.push_back(pos_sub);
+            }
+         }
+      }
+   }
+   return result;
+}
+
+std::string g_format_label(double value, double dticks, GLENumberFormat* format) {
+	std::string result;
+	double absValue = fabs(value);
+	if (absValue < 0.00001 * dticks) {
+		// otherwise place for "zero" may be -8.2556e-25 or some other small number
+		value = 0;
+	}
+	if (format != NULL) {
+		format->format(value, &result);
+	} else {
+		char cbuff[100];
+		char* num_trim_str = NULL;
+		if (value == 0 || (absValue > 1e-5 && absValue < 1e6)) {
+			sprintf(cbuff, "%f", value);
+		} else {
+			sprintf(cbuff, "%e", value);
+		}
+		numtrim(&num_trim_str, cbuff, dticks);
+		if (num_trim_str != NULL) {
+			result = num_trim_str;
+			myfree(num_trim_str);
+		}
+	}
+	return result;
+}
+
+void draw_axis(GLEAxis *ax, GLERectangle* box, DrawAxisPart drawPart) {
+	double x,y,gmin,gmax,dticks,tick1,tickn;
+	int savecap;
 	double h,dist;
-	double bl,br,bu,bd;
 	double ox,oy;
 
 	// Fix for font problems by Jan Struyf
 	g_resetfont();
 
-	xax = axis_horizontal(ax->type);
+	bool xax = axis_horizontal(ax->type);
 	if (ax->off) return;
 	g_get_xy(&ox,&oy);
 
+   bool drawsubticks = true;
 	bool drawticks = true;
 	bool drawrest = true;
 
 	if (ax->hasGrid()) {
-		if (drawgrid) drawrest = false;
-		else drawticks = false;
+      if (drawPart == DRAW_AXIS_GRID_SUBTICKS) {
+         drawticks = false;
+         drawrest = false;
+      } else if (drawPart == DRAW_AXIS_GRID_TICKS) {
+         drawsubticks = false;
+         drawrest = false;
+      } else {
+         CUtilsAssert(drawPart == DRAW_AXIS_ALL);
+         drawsubticks = false;
+         drawticks = false;
+      }
 	} else {
-		if (drawgrid) return;
+		if (drawPart != DRAW_AXIS_ALL) return;
 	}
-
+	
 /*----------------------------- Generate the places for labels to go */
-	ggmin = ax->getMin();
-	ggmax = ax->getMax();
-	ggnegate = ax->negate;
 	gmin = ax->getMin();
 	gmax = ax->getMax();
-	gglen = ax->length;
 	int lgset = ax->lgset;
 
-	vector<bool> subplaces;
+   vector<double> subplaces;	
 	int lgset_def = GLE_AXIS_LOG_OFF;
 	if (ax->log) {
 		h = ax->label_hei;
@@ -210,7 +279,7 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 		nice_log_ticks(&tick1, &tickn, gmin, gmax);
 		/* find suitable value for dticks */
 		if (lgset == GLE_AXIS_LOG_DEFAULT || lgset == GLE_AXIS_LOG_OFF) {
-			dticks = (tickn - tick1) / gglen * 3 * h;
+			dticks = (tickn - tick1) / ax->length * 3 * h;
 			if (dticks >= 4) {
 				dticks = ((int)dticks / 5) * 5.0;
 			} else if (dticks >= 2) {
@@ -269,19 +338,18 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 					}
 				}
 				/* also label subticks? */
-				if (lgset == GLE_AXIS_LOG_1 || lgset == GLE_AXIS_LOG_25 || lgset == GLE_AXIS_LOG_25B) {
-					for (int i = 2; i <= 9; i++) {
-						if (lgset == GLE_AXIS_LOG_1 || i == 2 || i == 5) {
-							double pos_sub = pos * i;
-							if (pos_sub >= gmin && pos_sub <= gmax) {
-								bool_vector_set_expand(&subplaces, ax->getNbPlaces(), true);
-								ax->addPlace(pos_sub);
-							}
-						}
-					}
-				}
+            std::vector<double> crSubPlaces(getLogSubPlaces(pos, gmin, gmax, lgset));
+            for (std::vector<double>::iterator i(crSubPlaces.begin()); i != crSubPlaces.end(); ++i) {
+               ax->addPlace(*i);
+            }
 			}
 		}
+		for (double pw10 = tick1; pw10 <= tickn; pw10 += dticks) {
+         std::vector<double> crSubPlaces(getLogSubPlaces(pow(10.0, pw10), gmin, gmax, lgset));
+         for (std::vector<double>::iterator i(crSubPlaces.begin()); i != crSubPlaces.end(); ++i) {
+            subplaces.push_back(*i);
+         }
+      }
 	} else {
 		dticks = 0;
 		GLERangeSet* range = ax->getRange();
@@ -309,7 +377,7 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 			/* haven't been set, so generate positions. */
 			double prev_tick = GLE_INF;
 			double end_tick = (tickn+dticks/100.0);
-			for (fi = tick1; fi <= end_tick; fi += dticks) {
+			for (double fi = tick1; fi <= end_tick; fi += dticks) {
 				if (fi - prev_tick == 0.0) {
 					/* dticks too small for precision of double */
 					ax->subticks_off = true;
@@ -324,17 +392,16 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 /*------------------------------  Now draw the ticks	(subticks first) */
 
 	// ticks length
-	tlen = ax->base*g_get_fconst(GLEC_TICKSSCALE);
+	double tlen = ax->base*g_get_fconst(GLEC_TICKSSCALE);
 	if (ax->ticks_length!=0) tlen = ax->ticks_length;
 
 	// subticks length and distance
-	stlen = tlen/2;
+	double stlen = tlen/2;
 	if (ax->subticks_length!=0) stlen = ax->subticks_length;
-	dsubticks = 0;
 
 	// draw subticks
 	if (ax->log) {
-		if (!ax->subticks_off && drawticks) {
+		if (!ax->subticks_off && drawsubticks) {
 			g_gsave();
 			g_set_color(ax->subticks_color);
 			g_set_line_width(ax->subticks_lwidth);
@@ -371,19 +438,14 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 			g_grestore();
 		}
 	} else {
+		double dsubticks = 0;
 		if (ax->nsubticks != 0) dsubticks = dticks/(ax->nsubticks+1);
 		if (ax->dsubticks != 0) dsubticks = ax->dsubticks;
 		if (dsubticks == 0) dsubticks = dticks / 2.0;
 		ax->dsubticks = dsubticks;
-		double end_tick = gmax;
-		if (ax->has_ftick) {
-			tick1 += dsubticks;
-			end_tick = tickn;
-		} else {
-			start_subtick(&tick1, gmin, dsubticks);
-		}
-		t = stlen;
-		if (!ax->subticks_off && drawticks) {
+		if (!ax->subticks_off && drawsubticks) {
+			double firstSubtick = start_subtick(dsubticks, dticks, ax);
+			double subtickLimit = ax->getMax() + dsubticks/100.0;
 			int place_cnt = 0;
 			int tick1_cnt = 0;
 			int tick2_cnt = 0;
@@ -391,9 +453,9 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 			g_set_color(ax->subticks_color);
 			g_set_line_width(ax->subticks_lwidth);
 			g_set_line_style(ax->subticks_lstyle);
-			for (fi = tick1; fi <= end_tick; fi += dsubticks) {
-				if (!ax->isPlace(fi, &place_cnt, dsubticks)) {
-					axis_draw_tick(ax, fi, &tick1_cnt, &tick2_cnt, ox, oy, t);
+			for (double fi = firstSubtick; fi <= subtickLimit; fi += dsubticks) {
+				if (inAxisRange(fi, ax) && !ax->isPlace(fi, &place_cnt, dsubticks)) {
+					axis_draw_tick(ax, fi, &tick1_cnt, &tick2_cnt, ox, oy, stlen);
 				}
 			}
 			g_grestore();
@@ -401,8 +463,7 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 	}
 
 /*------------------------------  Now the main ticks */
-	t = tlen;
-	tt = stlen;
+
 	if (!ax->ticks_off && drawticks) {
 		g_gsave();
 		g_set_color(ax->ticks_color);
@@ -410,18 +471,19 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 		g_set_line_style(ax->ticks_lstyle);
 		int tick1_cnt = 0;
 		int tick2_cnt = 0;
+      int subplace_cnt = 0;
 		if (ax->log) {
 			/* Draw log ticks */
-			for (i = 0; i < ax->getNbPlaces(); i++) {
-				fi = ax->places[i];
-				if (!bool_vector_is(&subplaces, i)) {
+			for (int i = 0; i < ax->getNbPlaces(); i++) {
+				double fi = ax->places[i];
+				if (!axis_is_pos_perc(fi, &subplace_cnt, 1e-6, subplaces)) {
 					/* only draw major ticks */
 					axis_draw_tick_log(ax, fi, &tick1_cnt, &tick2_cnt, ox, oy, tlen);
 				}
 			}
 		} else {
-			for (i = 0; i < ax->getNbPlaces(); i++) {
-				fi = ax->places[i];
+			for (int i = 0; i < ax->getNbPlaces(); i++) {
+				double fi = ax->places[i];
 				axis_draw_tick(ax, fi, &tick1_cnt, &tick2_cnt, ox, oy, tlen);
 			}
 		}
@@ -430,6 +492,7 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 
 /*-----------------------------  Ok lets draw the side now */
 /*-----------------------------  Note: after the ticks - this gives better result if ticks are different color */
+
 	if (!ax->side_off && drawrest) {
 		// FIXME: is cap not saved by gsave??
 		g_get_line_cap(&savecap);
@@ -446,6 +509,7 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 	}
 
 /*------------------------------  Now draw the labels */
+
 	double real_llen = 0;
 	if (!ax->ticks_off) {
 		if (tlen < 0) real_llen = -tlen;
@@ -459,10 +523,11 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 		char cbuff[100];
 		GLENumberFormat* format = ax->format == "" ? NULL : new GLENumberFormat(ax->format);
 		if (ax->log) {
-			for (i = 0; i < ax->getNbPlaces(); i++) {
-				fi = ax->places[i];
-				n = (int) floor(.0001 + fi/pow(10.0,floor(log10(fi))));
-				if (!bool_vector_is(&subplaces, i)) {
+			int subplace_cnt = 0;
+			for (int i = 0; i < ax->getNbPlaces(); i++) {
+				double fi = ax->places[i];
+				int n = (int) floor(.0001 + fi/pow(10.0,floor(log10(fi))));
+				if (!axis_is_pos_perc(fi, &subplace_cnt, 1e-6, subplaces)) {
 					n = (int) floor(log10(fi)+0.5);
 					if (format != NULL) {
 						format->format(pow(10.0, n), ax->getNamePtr(i));
@@ -494,25 +559,8 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 				// Get axis labels from a data set
 				ax->getLabelsFromDataSet(ax->getNamesDataSet());
 			} else {
-				for (i = 0; i < ax->getNbPlaces(); i++) {
-					fi = ax->places[i]; x = fabs(fi);
-					if (fabs(x) < 0.00001*dticks) {
-						// otherwise place for "zero" may be -8.2556e-25 or some other small number
-						fi = 0;
-					}
-					if (format != NULL) {
-						format->format(fi, ax->getNamePtr(i));
-					} else {
-						x = fabs(fi);
-						char* num_trim_str = NULL;
-						if (fi == 0 || (x > 1e-5 && x < 1e6)) sprintf(cbuff, "%f", fi);
-						else sprintf(cbuff, "%e", fi);
-						numtrim(&num_trim_str, cbuff, dticks);
-						if (num_trim_str != NULL) {
-							ax->setName(i, num_trim_str);
-							myfree(num_trim_str);
-						}
-					}
+				for (int i = 0; i < ax->getNbPlaces(); i++) {
+					ax->setName(i, g_format_label(ax->places[i], dticks, format));
 				}
 			}
 		}
@@ -527,10 +575,11 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 	GLEMeasureBox measure;
 	measure.measureStart();
 	// If there are no labels, use g_set_bounds to initialize "measure"
+	double bl,br,bu,bd;
 	init_measure_by_axis(ax, ox, oy, real_llen);
 	if (!ax->label_off && drawrest) {
 		int nb_names = ax->getNbNamedPlaces();
-		for (i = 0; i < nb_names; i++) {
+		for (int i = 0; i < nb_names; i++) {
 			string name = ax->names[i];
 			add_tex_labels(&name);
 			g_measure(name, &bl, &br, &bu, &bd);
@@ -540,13 +589,13 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 		}
 		int place_cnt = 0;
 		double angle = ax->getLabelAngle();
-		for (i = 0; i < nb_names; i++) {
-			fi = ax->places[i];
+		for (int i = 0; i < nb_names; i++) {
+			double fi = ax->places[i];
 			string name = ax->names[i];
 			add_tex_labels(&name);
 			if (!ax->isNoPlaceLogOrReg(fi, &place_cnt, dticks) && name != "") {
-				fi = fnx(fi);
-				if (ax->log) fi = fnlx(ax->places[i]);
+				fi = fnAxisX(fi, ax);
+				if (ax->log) fi = fnAxisX(ax->places[i], ax);
 				g_measure(name, &bl, &br, &bu, &bd);
 				switch (ax->type) {
 					case GLE_AXIS_X:
@@ -580,6 +629,10 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 								g_move(ox-llen-maxd, oy+fi+ax->shift);
 								g_rotate(90);
 								g_jtext(JUST_CENTRE);
+							} else if (angle == -90.0) {
+								g_move(ox-llen-maxh+maxd, oy+fi+ax->shift);
+								g_rotate(-90);
+								g_jtext(JUST_CENTRE);
 							} else {
 								g_move(ox-llen, oy+fi+ax->shift);
 								g_rotate(angle);
@@ -604,8 +657,27 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 						}
 						break;
 					case GLE_AXIS_Y2:
-						g_move(ox+llen, oy+fi+ax->shift);
-						g_jtext(JUST_LC);
+						if (angle == 0.0) {
+							g_move(ox+llen, oy+fi+ax->shift);
+							g_jtext(JUST_LC);
+						} else {
+							g_gsave();
+							if (angle == 90.0) {
+								g_move(ox+llen+maxh-maxd, oy+fi+ax->shift);
+								g_rotate(90);
+								g_jtext(JUST_CENTRE);
+							} else if (angle == -90.0) {
+								g_move(ox+llen, oy+fi+ax->shift);
+								g_rotate(-90);
+								g_jtext(JUST_CENTRE);
+							} else {
+								g_move(ox+llen, oy+fi+ax->shift);
+								g_rotate(angle);
+								if (angle > 0.0) g_jtext(JUST_TL);
+								else g_jtext(JUST_BL);
+							}
+							g_grestore();
+						}
 						break;
 				}
 			}
@@ -615,6 +687,7 @@ void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid) {
 	// g_box_stroke(measure.getX1(), measure.getY1(), measure.getX2(), measure.getY2(), false);
 
 /*---------------------------------- Now the axis title. */
+
 	if (ax->title_off || ax->title == "" || !drawrest) {
 		g_update_bounds_box(box);
 		return;
@@ -729,8 +802,8 @@ void draw_axis_titles_v35(GLEAxis *ax, double h, double ox, double oy, double dt
 			string name = ax->names[i];
 			add_tex_labels(&name);
 			if (!ax->isNoPlaceLogOrReg(fi, &place_cnt, dticks) && name != "") {
-				fi = fnx(fi);
-				if (ax->log) fi = fnlx(ax->places[i]);
+				fi = fnAxisX(fi, ax);
+				if (ax->log) fi = fnAxisX(ax->places[i], ax);
 				g_measure(name, &bl, &br, &bu, &bd);
 				switch (ax->type) {
 					case GLE_AXIS_X:
@@ -950,27 +1023,37 @@ void roundrange(GLERange* range, bool extend, bool tozero, double dticks) {
 	}
 }
 
-double chop(double f) {
-	return (double) (int) f;
-}
-
-void start_subtick(double *tick1, double gmin, double dticks) {
-	if (gmin == chop(gmin/dticks) * dticks)
-		*tick1 = gmin;
-	else
-		*tick1 = chop(gmin/dticks) * dticks + dticks;
+double start_subtick(double dsubticks, double dticks, GLEAxis* ax) {
+	double firstTick;
+	if (ax->getNbPlaces() > 0) {
+		firstTick = ax->getPlace(0);
+	} else {
+		GLERange range;
+		range.copy(ax->getRange());
+		roundrange(&range, false, false, dticks);
+		firstTick = range.getMin();
+	}
+	if (firstTick <= ax->getMin()) {
+		return firstTick;
+	} else {
+		double n = ceil((firstTick - ax->getMin()) / dsubticks) + 1;
+		return firstTick - n * dsubticks;
+	}
 }
 
 void numtrime(char *o,char *s) {
-	char *e,*f;
-	strcpy(o,s);
-	e = strchr(s,'e');
-	if (e==NULL) return;
+	char *e, *f;
+	strcpy(o, s);
+	e = strchr(s, 'e');
+	if (e == NULL) return;
 	e--;
-	for (; *e=='0'; e--) ;
-	f = strchr(s,'e');
-	strcpy(e+1,f);
-	strcpy(o,s);
+	for (; *e=='0'; e--);
+	if (*e == '.') {
+		e--;
+	}
+	f = strchr(s, 'e');
+	strcpy(e + 1, f);
+	strcpy(o, s);
 }
 
 void numtrim(char **d,char *s,double dticks) {
@@ -999,24 +1082,6 @@ void numtrim(char **d,char *s,double dticks) {
 	if (nonzero!=NULL) *(nonzero+1) = 0;
 }
 
-double fnloglen(double v,GLEAxis *ax) {
-	return ((v-log10(ax->getMin()))/(log10(ax->getMax())-log10(ax->getMin()))) * ax->length;
-}
-
-double fnlogx(double v,GLEAxis *ax) {
-	if (ax->negate) {
-		v = ax->getMax() - (v-ax->getMin());
-	}
-	return fnloglen(log10(v),ax);
-}
-
-void print_axis(GLEAxis *ax) {
-	cout << "Axis ticks off: " << ax->ticks_off << "\tAxis ticks length: "
-	     << ax->ticks_length << "\tAxis ticks color: " << ax->ticks_color << endl;
-	cout << "Axis subticks off: " << ax->subticks_off << "\tAxis subticks length: "
-	     << ax->subticks_length << "\tAxis subticks color: "  << ax->subticks_color << endl;
-}
-
 bool axis_is_max(int axis) {
 	return axis == GLE_AXIS_X2 || axis == GLE_AXIS_Y2;
 }
@@ -1100,14 +1165,12 @@ GLEAxisQuantileScale::~GLEAxisQuantileScale() {
 }
 
 GLEAxis::GLEAxis() : format() {
-	init(0);
 }
 
 GLEAxis::~GLEAxis() {
 }
 
 void GLEAxis::init(int i) {
-	int c;
 	format = ""; title = "";
 	clearNoTicks();
 	names.clear(); places.clear(); noplaces.clear();
@@ -1130,9 +1193,7 @@ void GLEAxis::init(int i) {
 	label_angle = 0.0;
 	has_subticks_onoff = false;
 	has_label_onoff = false;
-	g_get_color(&c);
-	color = c; side_color = c; ticks_color = c;
-	label_color = c; subticks_color = c;
+	setColor(g_get_color());
 	side_lwidth = -1; ticks_lwidth = -1; subticks_lwidth = -1; label_lwidth = -1;
 	lgset = GLE_AXIS_LOG_DEFAULT;
 	has_ftick = false; ftick = 0.0;
@@ -1156,7 +1217,7 @@ string* GLEAxis::getNamePtr(int i) {
 	return &names[i];
 }
 
-void GLEAxis::setName(int i, const char* name) {
+void GLEAxis::setName(int i, const std::string& name) {
 	while ((int)names.size() <= i) names.push_back(string());
 	names[i] = name;
 }
@@ -1225,35 +1286,69 @@ int GLEAxis::getNbNamedPlaces() {
 	return nb;
 }
 
+double GLEAxis::getLocalAveragePlacesDistance(int i) {
+	int count = 0;
+	double distance = 0.0;
+	if (i > 0) {
+		distance += fabs(places[i] - places[i - 1]);
+		count ++;
+	}
+	if (i < getNbPlaces() - 1) {
+		distance += fabs(places[i] - places[i + 1]);
+		count ++;
+	}
+	if (count == 0) {
+		return GLE_INF;
+	} else {
+		return distance / count;
+	}
+}
+
 void GLEAxis::getLabelsFromDataSet(int ds) {
-	int crpos = 0;
-	if (dp[ds] == NULL) return;
-	int npnts = dp[ds]->np;
-	double* xt = dp[ds]->xv;
-	if (npnts <= 0 || xt == NULL) return;
+	GLEDataSet* dataSet = dp[ds];
+	if (dataSet == NULL || dataSet->np == 0) {
+		return;
+	}
+	GLEDataPairs data;
+	GLEDataPairs::validate(dataSet, 2);
+	data.copyDimension(dataSet, 0);
+	GLEArrayImpl* yv = static_cast<GLEArrayImpl*>(dataSet->getData()->getObject(1));
+	double* xt = data.getX();
 	double min_val = xt[0];
-	double max_val = xt[npnts-1];
-	double half_spc = (max_val - min_val)/npnts/2.0;
+	double max_val = xt[dataSet->np - 1];
+	double half_spc = (max_val - min_val) / dataSet->np / 2.0;
 	min_val -= half_spc;
 	max_val += half_spc;
-	vector<string>* yv_str = dp[ds]->yv_str;
-	if (yv_str == NULL) return;
+	unsigned int crpos = 0;
 	for (int i = 0; i < getNbPlaces(); i++) {
 		double fi = places[i];
+		*getNamePtr(i) = "";
 		if (fi >= min_val && fi <= max_val) {
 			// find last position with x-value smaller than fi
-			while (crpos < npnts && xt[crpos] < fi) crpos++;
-				if (crpos < npnts && crpos >= 0) {
+			while (crpos < dataSet->np && xt[crpos] < fi) {
+				crpos++;
+			}
+			if (crpos < dataSet->np && crpos >= 0) {
 				if (crpos > 0) crpos--;
-				int sel = crpos;
+				unsigned int sel = crpos;
 				double dist = fabs(xt[crpos] - fi);
-				if (crpos+1 < npnts) {
+				if (crpos+1 < dataSet->np) {
 					if (fabs(xt[crpos+1] - fi) < dist) sel = crpos+1;
 				}
-				if (crpos-1 >= 0) {
+				if (crpos >= 1) {
 					if (fabs(xt[crpos-1] - fi) < dist) sel = crpos-1;
 				}
-				if (sel >= 0 && sel < (int)yv_str->size()) *getNamePtr(i) = (*yv_str)[sel];
+				if (sel >= 0 && sel < dataSet->np && !data.getM(sel)) {
+					bool showLabel = true;
+					dist = fabs(xt[sel] - fi);
+					if (!log && dist > getLocalAveragePlacesDistance(i) / 2.0) {
+						showLabel = false;
+					}
+					if (showLabel) {
+						GLERC<GLEString> str(yv->getString(sel));
+						*getNamePtr(i) = str->toUTF8();
+					}
+				}
 			}
 		}
 	}
@@ -1361,3 +1456,11 @@ void GLEAxis::makeUpRange(GLEAxis* copy, GLEAxis* orth, bool extend, bool tozero
 	}
 	getRange()->copyIfNotSet(drange);
 }
+
+
+void GLEAxis::setColor(const GLERC<GLEColor>& color) {
+	side_color = color;
+	ticks_color = color;
+	label_color = color;
+	subticks_color = color;
+}
