@@ -36,6 +36,9 @@
  *                                                                      *
  ************************************************************************/
 
+#ifndef GLE_HAS_AXIS_H
+#define GLE_HAS_AXIS_H
+ 
 #define GLE_AXIS_NONE -1
 #define GLE_AXIS_MAX 7
 #define GLE_AXIS_X   1
@@ -91,7 +94,6 @@ public:
 	double base;
 	double length;
 	double shift;
-	int color;
 	int label_font;
 	double label_hei;
 	double label_scale;
@@ -114,11 +116,14 @@ public:
 	double side_lwidth,label_lwidth;
 	char side_lstyle[9];
 	int title_font;
-	int ticks_color,side_color,subticks_color,label_color;
 	double title_dist, title_adist;
 	double title_hei, title_scale;
 	int title_rot,title_off;
-	int title_color;	/* 0=normal, 1=rotate 180 */
+	GLERC<GLEColor> title_color;	/* 0=normal, 1=rotate 180 */
+	GLERC<GLEColor> ticks_color;
+	GLERC<GLEColor> side_color;
+	GLERC<GLEColor> subticks_color;
+	GLERC<GLEColor> label_color;
 	int names_ds;
 	double label_angle;
 	bool grid, gridtop;
@@ -141,7 +146,7 @@ public:
 	~GLEAxis();
 	void init(int i);
 	string* getNamePtr(int i);
-	void setName(int i, const char* name);
+	void setName(int i, const std::string& name);
 	void setPlace(int i, double place);
 	void clearNoTicks();
 	void addNoTick(double pos);
@@ -156,6 +161,7 @@ public:
 	void performRoundRange(GLERange* range, bool extend, bool tozero);
 	void roundDataRange(bool extend, bool tozero);
 	void makeUpRange(GLEAxis* copy, GLEAxis* orth, bool extend, bool tozero);
+	void setColor(const GLERC<GLEColor>& color);
 	inline bool isPlace(double pos, int* cnt, double delta) { return axis_is_pos(pos, cnt, delta, places); }
 	inline bool isPlaceRel(double pos, int* cnt) { return axis_is_pos_perc(pos, cnt, 1e-6, places); }
 	inline bool isNoPlace(double pos, int* cnt, double delta) { return axis_is_pos(pos, cnt, delta, noplaces); }
@@ -201,8 +207,18 @@ public:
 	inline bool shouldPerformQuantileScale() { return !m_QuantileScale.isNull(); }
 	inline GLEAxisQuantileScale* getQuantileScale() { return m_QuantileScale.get(); }
 	inline void setQuantileScale(GLEAxisQuantileScale* scale)  { m_QuantileScale = scale; }
+private:
+	double getLocalAveragePlacesDistance(int i);
 };
 
 void init_measure_by_axis(GLEAxis* ax, double ox, double oy, double llen);
-void draw_axis(GLEAxis *ax, GLERectangle* box, bool drawgrid); /* Draws the axis */
-void print_axis(GLEAxis *ax);
+
+enum DrawAxisPart {
+   DRAW_AXIS_GRID_SUBTICKS,
+   DRAW_AXIS_GRID_TICKS,
+   DRAW_AXIS_ALL
+};
+
+void draw_axis(GLEAxis *ax, GLERectangle* box, DrawAxisPart drawPart); /* Draws the axis */
+
+#endif // GLE_HAS_AXIS_H
