@@ -43,6 +43,8 @@
 
 #include "../basicconf.h"
 #include "../gprint.h"
+#include "../file_io.h"
+#include "../cutils.h"
 #include "img2ps.h"
 
 GLEBitmap::GLEBitmap() {
@@ -257,38 +259,43 @@ void GLEBitmap::printInfo(ostream& os) {
 void GLEBitmap::close() {
 }
 
+string GLEBitmap::getFName()
+{
+	return "";
+}
+
 GLEFileBitmap::GLEFileBitmap() {
-	m_In = NULL;
 	m_Palette = NULL;
 }
 
+GLEFileBitmap::~GLEFileBitmap() {
+	close();
+}
+
 int GLEFileBitmap::open(const string& fname) {
-	setFName(fname);
-	m_In = fopen(fname.c_str(), "rb");
-	if (m_In == NULL) {
-		return 0;
-	} else {
-		return 1;
-	}
+	m_file.open(fname.c_str(), "rb");
+	return m_file.isOpen() ? 1 : 0;
 }
 
 int GLEFileBitmap::read16BE() {
-	int lh = fgetc(m_In);
-	int ll = fgetc(m_In);
+	int lh = fgetc(m_file.getFile());
+	int ll = fgetc(m_file.getFile());
 	return (lh << 8) | ll;
 }
 
 int GLEFileBitmap::read16LE() {
-	int lh = fgetc(m_In);
-	int ll = fgetc(m_In);
+	int lh = fgetc(m_file.getFile());
+	int ll = fgetc(m_file.getFile());
 	return (ll << 8) | lh;
 }
 
 void GLEFileBitmap::close() {
-	if (m_In != NULL) {
-		fclose(m_In);
-		m_In = NULL;
-	}
+	m_file.close();
+}
+
+string GLEFileBitmap::getFName()
+{
+	return m_file.getName();
 }
 
 GLEByteStream::GLEByteStream() {
@@ -559,46 +566,39 @@ int GLEAlphaRemovalByteStream::endScanLine() {
 	return GLEPipedByteStream::endScanLine();
 }
 
-BicubicIpolData::BicubicIpolData() {
+IpolData::IpolData() {
 }
 
-BicubicIpolData::~BicubicIpolData() {
+IpolData::~IpolData() {
 }
 
-BicubicIpolDoubleMatrix::BicubicIpolDoubleMatrix(double* data, int wd, int hi) {
-	m_X0 = m_Y0 = 0;
-	m_DW = m_Width = wd;
-	m_DH = m_Height = hi;
+IpolDoubleMatrix::IpolDoubleMatrix(double* data, int wd, int hi) {
+	m_Width = wd;
+	m_Height = hi;
 	m_Data = data;
 }
 
-BicubicIpolDoubleMatrix::~BicubicIpolDoubleMatrix() {
+IpolDoubleMatrix::~IpolDoubleMatrix() {
 }
 
-void BicubicIpolDoubleMatrix::setWindow(int x0, int y0, int x1, int y1) {
-	m_X0 = x0; m_Y0 = y0;
-	m_Width = x1 - x0 + 1;
-	m_Height = y1 - y0 + 1;
-}
-
-double BicubicIpolDoubleMatrix::getValue(int x, int y) {
-	x += m_X0;
-	y += m_Y0;
+double IpolDoubleMatrix::getValue(int x, int y) {
 	if (x < 0) x = 0;
-	if (x >= m_DW) x = m_DW-1;
+	if (x >= m_Width) x = m_Width - 1;
 	if (y < 0) y = 0;
-	if (y >= m_DH) y = m_DH-1;
-	return m_Data[y*m_DW + x];
+	if (y >= m_Height) y = m_Height - 1;
+	return m_Data[y * m_Width + x];
 }
 
-BicubicIpol::BicubicIpol(BicubicIpolData* data, int sw, int sh) {
+Ipol::Ipol()
+{
+}
+
+Ipol::~Ipol()
+{
+}
+
+BicubicIpol::BicubicIpol(IpolData* data) {
 	m_Data = data;
-	m_SWidth = sw;
-	m_SHeight = sh;
-	m_Width = data->getWidth();
-	m_Height = data->getHeight();
-	m_SX = (double)m_Width / m_SWidth;
-	m_SY = (double)m_Height / m_SHeight;
 }
 
 double BicubicIpol::R(double x) {
@@ -614,9 +614,9 @@ double BicubicIpol::R(double x) {
 	return sum/6.0;
 }
 
-double BicubicIpol::ipol(int xp, int yp) {
-	double x = xp * m_SX;
-	double y = yp * m_SY;
+double BicubicIpol::ipol(double xp, double yp) {
+	double x = xp * m_Data->getWidth();
+	double y = yp * m_Data->getHeight();
 	int i = (int)floor(x);
 	int j = (int)floor(y);
 	double dx = x - i;
@@ -633,6 +633,15 @@ double BicubicIpol::ipol(int xp, int yp) {
 	return value;
 }
 
+NearestIpol::NearestIpol(IpolData* data):
+	m_Data(data)
+{
+}
+
+double NearestIpol::ipol(double xp, double yp) {
+	return m_Data->getValue(gle_round_int(xp * m_Data->getWidth()), gle_round_int(yp * m_Data->getHeight()));
+}
+
 void GLEBitmapSetPalette(GLEBYTE* pal, int offs, double red, double green, double blue) {
 	int i_red = (int)floor(red*255+0.5);
 	int i_green = (int)floor(green*255+0.5);
