@@ -88,7 +88,6 @@ class GLEByteStream;
 
 class GLEBitmap {
 protected:
-	string m_FName;
 	string m_Error;
 	int  m_Height;
 	int  m_Width;
@@ -124,8 +123,6 @@ public:
 	inline void setEncoding(char encoding) { m_Encoding = encoding; }
 	inline int getBitsPerComponent() { return m_BitsPerComponent; }
 	inline void setBitsPerComponent(int bits) { m_BitsPerComponent = bits; }
-	inline const string& getFName() { return m_FName; }
-	inline void setFName(const string& fname) { m_FName = fname; }
 	inline int isASCII85() { return m_ASCII85; }
 	inline void setASCII85(int val) { m_ASCII85 = val; }
 	inline double getCompress() { return m_Compress; }
@@ -149,19 +146,22 @@ public:
 	virtual int decode(GLEByteStream* output);
 	virtual int coded(GLEByteStream* output);
 	virtual void close();
+	virtual string getFName();
 	void printInfo(ostream& os);
 };
 
 class GLEFileBitmap : public GLEBitmap {
 protected:
-	FILE* m_In;
+	GLEFileIO m_file;
 public:
 	GLEFileBitmap();
+	virtual ~GLEFileBitmap();
 	virtual int open(const string& fname);
 	virtual void close();
+	virtual string getFName();
 	int read16BE();
 	int read16LE();
-	inline int read8() { return fgetc(m_In); }
+	inline int read8() { return fgetc(m_file.getFile()); }
 };
 
 class GLEJPEG : public GLEFileBitmap {
@@ -196,6 +196,7 @@ class GLETIFF : public GLEBitmap {
 protected:
 	TIFF* m_Tiff;
 	uint16 m_TIFFCompress;
+	string m_fname;
 public:
 	GLETIFF();
 	virtual ~GLETIFF();
@@ -204,6 +205,7 @@ public:
 	virtual int prepare(int mode);
 	virtual int decode(GLEByteStream* output);
 	virtual void close();
+	virtual string getFName();
 	int isCCITTCompression();
 	uint16 getTIFFCompression() { return m_TIFFCompress; }
 };
@@ -375,45 +377,55 @@ private:
 	void cleanUp();
 };
 
-class BicubicIpolData {
+enum IpolType
+{
+	IPOL_TYPE_BICUBIC,
+	IPOL_TYPE_NEAREST
+};
+
+class IpolData {
 protected:
 	int m_Width;
 	int m_Height;
 public:
-	BicubicIpolData();
-	virtual ~BicubicIpolData();
+	IpolData();
+	virtual ~IpolData();
 	inline int getWidth() { return m_Width; }
 	inline int getHeight() { return m_Height; }
 	virtual double getValue(int x, int y) = 0;
 };
 
-class BicubicIpolDoubleMatrix : public BicubicIpolData {
+class IpolDoubleMatrix: public IpolData {
 protected:
 	double* m_Data;
-	int m_X0, m_Y0, m_DW, m_DH;
 public:
-	BicubicIpolDoubleMatrix(double* data, int wd, int hi);
-	void setWindow(int x0, int y0, int x1, int y1);
-	virtual ~BicubicIpolDoubleMatrix();
+	IpolDoubleMatrix(double* data, int wd, int hi);
+	virtual ~IpolDoubleMatrix();
 	virtual double getValue(int x, int y);
 };
 
-class BicubicIpol {
-protected:
-	BicubicIpolData* m_Data;
-	int m_SWidth;
-	int m_SHeight;
-	int m_Width;
-	int m_Height;
-	double m_SX, m_SY;
+class Ipol {
 public:
-	BicubicIpol(BicubicIpolData* data, int sw, int sh);
-	double ipol(int xp, int yp);
+	Ipol();
+	virtual ~Ipol();
+	virtual double ipol(double xp, double yp) = 0;
+};
+
+class BicubicIpol: public Ipol {
+protected:
+	IpolData* m_Data;
+public:
+	BicubicIpol(IpolData* data);
+	virtual double ipol(double xp, double yp);
 	double R(double x);
-	inline int getScaleW() { return m_SWidth; }
-	inline int getScaleH() { return m_SHeight; }
-	inline int getWidth() { return m_Width; }
-	inline int getHeight() { return m_Height; }
+};
+
+class NearestIpol: public Ipol {
+protected:
+	IpolData* m_Data;
+public:
+	NearestIpol(IpolData* data);
+	virtual double ipol(double xp, double yp);
 };
 
 void GLEBitmapSetPalette(GLEBYTE* pal, int offs, double red, double green, double blue);
