@@ -375,6 +375,16 @@ bool CmdLineArgSet::hasOnlyValue(int id) {
 	return true;
 }
 
+vector<string> CmdLineArgSet::getValues() {
+	vector<string> result;
+	for (vector<int>::size_type i = 0; i < m_Values.size(); i++) {
+		if (m_HasValue[i] == CMDLINE_YES) {
+			result.push_back(m_Values[i]);
+		}
+	}
+	return result;
+}
+
 CmdLineOption::CmdLineOption(const char* name) {
 	addAlias(name);
 	initialize();
@@ -583,6 +593,11 @@ void CmdLineOptionList::showHelp(int helpid) {
 	if (!expert) cerr << endl << "Show expert options: " << getOptionPrefix() << "help expert" << endl;
 }
 
+bool CmdLineOptionList::hasOption(const string& name) {
+	CmdLineOption* option = getOption(name);
+	return option != 0 && option->hasOption();
+}
+
 bool CmdLineOptionList::hasOption(int id) {
 	if (id >= (int)m_Options.size()) return false;
 	if (m_Options[id] == NULL) return false;
