@@ -72,7 +72,7 @@ char *un_quote(char *ct);
 
 #define skipspace doskip(tk[ct],&ct)
 
-void begin_config(const char* block, int *pln, int *pcode, int *cp) throw(ParserError) {
+void begin_config(const std::string& block, int *pln, int *pcode, int *cp) throw(ParserError) {
 	string block_name(block);
 	ConfigSection* section = g_Config.getSection(block_name);
 	if (section == NULL) {
@@ -259,15 +259,24 @@ bool try_load_config(const string& fname) {
 	}
 }
 
-bool try_save_config(const string& fname, GLEInterface* iface) {
+bool try_save_config(const string& fname, GLEInterface* iface, bool isUser) {
 	ConfigCollection* collection = iface->getConfig()->getRCFile();
 	if (collection->allDefaults()) {
 		return true;
 	}
+	if (fname == "") {
+		return false;
+	}
+	if (IsAbsPath(fname)) {
+		std::string dirname;
+		GetDirName(fname, dirname);
+		EnsureMkDir(dirname);
+	}
 	ofstream fout(fname.c_str());
 	if (!fout.is_open()) {
 		return false;
 	}
+	CmdLineOption* versionOption = collection->getSection(GLE_CONFIG_GLE)->getOption(GLE_CONFIG_GLE_VERSION);
 	ostringstream out;
 	out << "Save configuration to: '" << fname << "'";
 	GLEOutputStream* output = iface->getOutput();
@@ -278,7 +287,7 @@ bool try_save_config(const string& fname, GLEInterface* iface) {
 			fout << "begin config " << sec->getName() << endl;
 			for (int j = 0; j < sec->getNbOptions(); j++) {
 				CmdLineOption* option = sec->getOption(j);
-				if (!option->allDefaults()) {
+				if (!option->allDefaults() && (!isUser || option != versionOption)) {
 					fout << "\t" << option->getName() << " = ";
 					for (int k = 0; k < option->getMaxNbArgs(); k++) {
 						if (k != 0) fout << " ";
@@ -341,15 +350,10 @@ void init_installed_versions(CmdLineObj& cmdline, ConfigCollection* collection)
 	}
 }
 
-void wait_for_enter() {
-	char Line[5];
-	fgets(Line, sizeof(Line), stdin);
-}
-
 void do_wait_for_enter() {
 	if (g_CmdLine.hasOption(GLE_OPT_PAUSE)) {
 		cout << "Press enter to continue ..." << endl;
-		wait_for_enter();
+		GLEReadConsoleInteger();
 	}
 }
 
@@ -555,7 +559,7 @@ bool check_correct_version(const string name, bool has_top, bool has_config, con
 			out << "       '" << name << "'" << endl;
 			out << "Is from GLE version '";
 			if (val == "") { out << "unknown"; } else { out << val; }
-			out << "' (and not '" << GLEVN << "' as espected)." << endl;
+			out << "' (and not '" << GLEVN << "' as expected)." << endl;
 			complain_about_gletop(has_top, out);
 			g_message(out.str().c_str());
 			return false;
@@ -660,7 +664,7 @@ void do_find_deps_sub(GLEInterface* iface, const string& loc) {
 	string myloc = loc;
 #ifdef __WIN32__
 	if (myloc.length() == 0) {
-		myloc = "C:\\Program Files";
+		GLEGetEnv("ProgramFiles", myloc);
 	}
 #endif
 	find_deps(myloc, iface);
@@ -669,10 +673,10 @@ void do_find_deps_sub(GLEInterface* iface, const string& loc) {
 void do_save_config() {
 	GLEInterface* iface = GLEGetInterfacePointer();
 	string conf_name = GLE_TOP_DIR + DIR_SEP + "glerc";
-	bool is_ok = try_save_config(conf_name, iface);
+	bool is_ok = try_save_config(conf_name, iface, false);
 	if (!is_ok) {
 		string user_conf = iface->getUserConfigLocation();
-		is_ok = try_save_config(user_conf, iface);
+		is_ok = try_save_config(user_conf, iface, true);
 	}
 	if (!is_ok) {
 		ostringstream err;
