@@ -49,11 +49,14 @@ typedef enum {
 	GLE_OPT_FULL_PAGE,
 	GLE_OPT_LANDSCAPE,
 	GLE_OPT_OUTPUT,
+	GLE_OPT_NOSAVE,
+	GLE_OPT_COPY,
 	GLE_OPT_PREVIEW,
 	GLE_OPT_GSPREVIEW,
 	GLE_OPT_VERSION,
 	GLE_OPT_COMPAT,
 	GLE_OPT_CALC,
+	GLE_OPT_CATCSV,
 	GLE_OPT_TEX,
 	GLE_OPT_NO_PDFTEX,
 	GLE_OPT_CREATE_INC,
@@ -62,6 +65,7 @@ typedef enum {
 	GLE_OPT_MKINITTEX,
 	GLE_OPT_FINDDEPS,
 	GLE_OPT_NO_COLOR,
+	GLE_OPT_INVERSE,
 	GLE_OPT_TRANSPARENT,
 	GLE_OPT_BBTWEAK,
 	GLE_OPT_NO_CTRL_D,
