@@ -80,6 +80,7 @@
 #define GLE_ARRTIP_SHARP 1
 
 class GLEBitmap;
+class GLEPcodeList;
 
 class GLEArrowProps {
 public:
@@ -92,7 +93,8 @@ public:
 struct gmodel {
 	double image[3][3];
 	double fontn,fontsz;	/* up to here for font caching */
-	colortyp color,fill;
+	GLERC<GLEColor> color;
+	GLERC<GLEColor> fill;
 	double lwidth,lstyled,curx,cury;
 	double endx,endy;
 	double miterlimit;
@@ -146,125 +148,6 @@ public:
 	void measureEndIgnore();
 };
 
-class GLECurve {
-public:
-	GLECurve();
-	virtual ~GLECurve();
-	virtual void getC(double t, GLEPoint& p) = 0;
-	virtual void getCp(double t, GLEPoint& p) = 0;
-	virtual void getCpp(double t, GLEPoint& p) = 0;
-	virtual double getT0();
-	virtual double getT1();
-	virtual double getDist(double t1, double t2);
-	virtual double distToParamValue(double t1, double dist, double t2);
-	virtual double distToParamValue(double t1, double dist);
-	double getDistp(double t);
-protected:
-	double computeDistRecursive(double t1, GLEPoint& p1, double t2, GLEPoint& p2);
-};
-
-class GLECurveT0T1 : public GLECurve {
-protected:
-	double m_T0, m_T1;
-public:
-	GLECurveT0T1();
-	~GLECurveT0T1();
-	virtual double getT0();
-	virtual double getT1();
-};
-
-class GLECircleArc : public GLECurveT0T1 {
-protected:
-	GLEPoint m_Orig;
-	double m_R;
-public:
-	GLECircleArc(const GLEPoint& orig, double r, double t0, double t1);
-	virtual ~GLECircleArc();
-	virtual void getC(double t, GLEPoint& p);
-	virtual void getCp(double t, GLEPoint& p);
-	virtual void getCpp(double t, GLEPoint& p);
-	virtual double getDist(double t1, double t2);
-	virtual double distToParamValue(double t1, double dist, double t2);
-	virtual double distToParamValue(double t1, double dist);
-};
-
-class GLEEllipseArc : public GLECurveT0T1 {
-protected:
-	GLEPoint m_Orig;
-	double m_Rx, m_Ry;
-public:
-	GLEEllipseArc(const GLEPoint& orig, double rx, double ry, double t0, double t1);
-	virtual ~GLEEllipseArc();
-	virtual void getC(double t, GLEPoint& p);
-	virtual void getCp(double t, GLEPoint& p);
-	virtual void getCpp(double t, GLEPoint& p);
-};
-
-class GLEBezier : public GLECurve {
-protected:
-	GLEPoint m_P0, m_P1, m_P2, m_P3;
-	double m_Ax, m_Bx, m_Cx, m_Ay, m_By, m_Cy;
-public:
-	GLEBezier();
-	GLEBezier(double x0, double y0, double x1, double y1, double x2, double y2, double x3, double y3);
-	GLEBezier(const GLEBezier& other);
-	~GLEBezier();
-	inline void setP0(const GLEPoint& p) { m_P0.set(p); }
-	inline void setP1(const GLEPoint& p) { m_P1.set(p); }
-	inline void setP2(const GLEPoint& p) { m_P2.set(p); }
-	inline void setP3(const GLEPoint& p) { m_P3.set(p); }
-	inline void setP0(double x, double y) { m_P0.setXY(x, y); }
-	inline void setP1(double x, double y) { m_P1.setXY(x, y); }
-	inline void setP2(double x, double y) { m_P2.setXY(x, y); }
-	inline void setP3(double x, double y) { m_P3.setXY(x, y); }
-	inline const GLEPoint& getP0() const { return m_P0; }
-	inline const GLEPoint& getP1() const { return m_P1; }
-	inline const GLEPoint& getP2() const { return m_P2; }
-	inline const GLEPoint& getP3() const { return m_P3; }
-	inline GLEPoint& P0() { return m_P0; }
-	inline GLEPoint& P1() { return m_P1; }
-	inline GLEPoint& P2() { return m_P2; }
-	inline GLEPoint& P3() { return m_P3; }
-	virtual void getC(double t, GLEPoint& p);
-	virtual void getCp(double t, GLEPoint& p);
-	virtual void getCpp(double t, GLEPoint& p);
-	void throughPoint(GLEPoint& p, GLEPoint& dir1, GLEPoint& dir2);
-	void updateEquation();
-	void draw();
-	void cutAtParamValue(double t);
-	void cutFromParamValue(double t);
-};
-
-class GLECurvedArrowHead {
-protected:
-	GLECurve* m_Curve;
-	GLEBezier m_Side1, m_Side2;
-	double m_T0, m_TM, m_T1;
-	double m_ArrAlpha, m_ArrSize, m_LWidth;
-	bool m_Enable, m_Sharp;
-	int m_ArrStyle;
-public:
-	GLECurvedArrowHead(GLECurve* curve);
-	virtual ~GLECurvedArrowHead();
-	void getA(double t, double pm, GLEPoint& p);
-	void getAp(double t, double pm1, double pm2, GLEPoint& p);
-	void setArrowAngleSize(int style, double size, double angle);
-	void setArrowAngleSizeSharp(int style, double size, double angle);
-	void drawDirection(bool dir);
-	void setStartEnd(bool dir);
-	double getParamValueEnd();
-	double getArrowCurveDist();
-	void computeAndDraw();
-	void computeArrowHead();
-	void draw();
-	inline int getStyle() { return m_ArrStyle; }
-	inline bool isSharp() { return m_Sharp; }
-	inline void setSharp(bool sharp) { m_Sharp = sharp; }
-	inline void setLineWidth(double lwidth) { m_LWidth = lwidth; }
-	inline bool isEnabled() { return m_Enable; }
-	inline void setEnabled(bool enable) { m_Enable = enable; }
-};
-
 void g_init_arrow_head(GLECurvedArrowHead* head, bool startend);
 
 class GLEPolynomial {
@@ -285,6 +168,36 @@ public:
 	inline void set(int i, double v) { m_Coefs[i] = v; }
 };
 
+class GLECore {
+public:
+	GLECore();
+	~GLECore();
+
+	void reset();
+	bool isComputingLength() const;
+	void setComputingLength(bool computingLength);
+	double getTotalLength() const;
+	void setTotalLength(double length);
+	void addToLength(double value);
+	bool isShowNoteAboutFallback() const;
+	void setShowNoteAboutFallback(bool value);
+
+private:
+	bool m_isComputingLength;
+	double m_totalLength;
+	bool m_showNoteAboutFallback;
+};
+
+class GLEWithoutUpdates {
+public:
+	GLEWithoutUpdates();
+	~GLEWithoutUpdates();
+
+private:
+	GLECore* m_core;
+	bool m_isComputingLength;
+};
+
 unsigned int coreleft();
 unsigned int farcoreleft();
 
@@ -318,6 +231,7 @@ bool g_has_console_output();
 void g_set_console_output(bool set);
 void g_get_usersize(double *x,double *y);
 void g_get_userbox_undev(GLERectangle* rect);
+GLECore* g_get_core();
 
 //
 // -- no need to ifdef these
@@ -325,6 +239,7 @@ void g_get_userbox_undev(GLERectangle* rect);
 //
 void g_bitmap(string& fname, double wx, double wy, int type) throw(ParserError);
 void g_bitmap_info(string& fname, int xvar, int yvar, int type) throw(ParserError);
+int g_bitmap_string_to_type(const std::string& stype);
 int g_bitmap_string_to_type(const char* stype);
 string g_bitmap_supported_types();
 
@@ -340,14 +255,10 @@ void g_arrowpoints(double cx,double cy,double dx,double dy, double *ax1,
                    double *ay1,double *ax2,double *ay2, double *nnx, double *nny);
 void g_psarrow(double x1, double y1, double x2, double y2, int flag);
 
+double g_arc_normalized_angle2(double a1, double a2);
 void g_arc(double r, double t1, double t2, double cx, double cy, int arrow);
 void g_narc(double r, double t1, double t2, double cx, double cy, int arrow);
 
-void g_get_colortyp(colortyp *color);
-void g_get_fill_colortyp(colortyp *color);
-void g_colortyp_to_rgb01(colortyp* c1, rgb01 *c2);
-int g_is_black(colortyp *color);
-
 bool g_is_bbtweak();
 double g_get_angle_deg();
 void g_get_scale(double* sx, double* sy);
@@ -415,7 +326,6 @@ void g_undev(GLERectangle* rect, gmodel* g);
 void g_set_arrow_style(const char* shape) throw (ParserError);
 void g_set_arrow_tip(const char* tip) throw (ParserError);
 
-void g_set_pattern_color(int color);
 void g_bitmap(GLEBitmap*, double wx, double wy, int type) throw(ParserError);
 
 void g_set_tex_scale(const char* ss);
@@ -427,13 +337,15 @@ void g_restore_defaults();
 int g_arrow_style();
 int g_arrow_tip();
 
+GLEPoint g_get_xy();
 void g_get_xy(GLEPoint* pt);
 void g_move(const GLEPoint& pt);
 void g_clear_matrix();
-void g_set_color(GLEColor* color);
-void g_set_fill(GLEColor* color);
+void g_set_color(const GLERC<GLEColor>& color);
+void g_set_fill(int fill);
+void g_set_fill(const GLERC<GLEColor>& fill);
 bool g_is_filled(void);
-void g_set_fill_pattern(int l);
+void g_set_fill_pattern(const GLERC<GLEColor>& pattern);
 void g_set_partial_state(gmodel* s);
 
 unsigned char float_to_color_comp(double value);
@@ -443,8 +355,6 @@ int g_hash_string_to_color(const string& str, colortyp* c);
 int g_verbosity();
 
 int font_get_encoding(int ff);
-int g_get_grey(double g);
-
 void d_devcmd(char *s);
 void g_arcto(dbl x1,dbl y1,dbl x2,dbl y2,dbl rrr);
 void g_beginclip(void);
@@ -461,11 +371,11 @@ void g_clear(void);
 void g_clip(void);
 void g_close(void);
 void g_closepath(void);
-void g_curve(int *pcode);
+void g_curve(GLEPcodeList* pclist, int *pcode);
 void g_dbezier(dbl x1,dbl y1,dbl x2,dbl y2,dbl x3,dbl y3);
 void g_dev(double x, double y,double *xd,double *yd);
 void g_devcmd(const char *s);
-void g_dfont(const char *s);
+void g_dfont(const std::string& s);
 void g_dline(double x, double y);
 void g_dmove(double x, double y);
 void g_dojust(dbl *x1, dbl *y1, dbl *x2, dbl *y2, int jj);
@@ -475,10 +385,12 @@ void g_fill(void);
 void g_fill_ary(int nwk,double *wkx,double *wky);
 void g_flush(void);
 void g_get_bounds(dbl *x1,dbl *y1,dbl *x2,dbl *y2);
-void g_get_color(int *l);
+GLERC<GLEColor> g_get_color();
+GLERC<GLEColor> g_get_fill_clear();
+GLERC<GLEColor> g_get_color_hex(int hexValue);
 void g_get_devsize(dbl *x,dbl *y);
 void g_get_end(dbl *x,dbl *y);
-void g_get_fill(int *f);
+GLERC<GLEColor> g_get_fill();
 void g_get_just(int *j);
 void g_get_line_cap(int *jj);
 void g_get_line_join(int *jj);
@@ -486,8 +398,7 @@ void g_get_line_style(char *s);
 void g_get_line_styled(double *w);
 void g_get_line_width(double *w);
 void g_get_path(int *pathonoff);
-void g_get_rgbf(double *rr, double *gg, double *bb, double *ff);
-void g_get_type(char *t);
+std::string g_get_type();
 void g_get_usersize(dbl *x,dbl *y);
 void g_get_pagesize(dbl *x,dbl *y,int *type);
 void g_get_pagesize(dbl *x,dbl *y);
@@ -518,10 +429,9 @@ void g_rundev(double x, double y,double *xd,double *yd);
 void g_scale(double sx,double sy);
 void g_update_bounds(double x,double y);
 void g_set_color(int l);
-void g_set_background(int b);
+void g_set_background(const GLERC<GLEColor>& color);
 void g_set_end(dbl x,dbl y);
 void g_set_hei(double h);
-void g_set_fill(int f);
 void g_set_fill_method(const char* meth);
 void g_set_font(int j);
 void g_set_font_width(double v);
@@ -534,13 +444,15 @@ void g_set_line_styled(double x);
 void g_set_line_width(double w);
 void g_set_matrix(double newmat[3][3]);
 void g_set_path(int onoff);
-void g_set_rgbf(double rr, double gg, double bb, double ff);
-void g_set_rgbf_fill(double rr, double gg, double bb, double ff);
 void g_set_xy(double x, double y);
 void g_stroke(void);
 void g_text(const string& s);
 void g_translate(double ztx,double zty);
 void g_undev(double ux,double uy, double *x,double *y);
 void my_char(int ff, int cc);
+int g_device_to_bitmap_type(int device);
+bool g_bitmap_supports_type(int type);
+int g_font_fallback(int font);
+std::vector<std::string> g_create_device_string();
 
 #endif
