@@ -38,6 +38,7 @@
 
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <math.h>
 #include <errno.h>
 #include <string.h>
@@ -229,6 +230,10 @@ int str_i_cmp(const char* s1, const char* s2) {
 	}
 }
 
+bool str_i_less::operator()(const std::string& s1, const std::string& s2) const {
+	return str_i_cmp(s1.c_str(), s2.c_str()) < 0;
+}
+
 int str_i_str(const string& haystack, int from, const char* needle) {
 	int endIndex = haystack.length();
 	int patternLength = strlen(needle);
@@ -327,6 +332,15 @@ bool str_starts_with(const string& str, const char* find) {
 	return find[idx] == 0;
 }
 
+bool str_i_starts_with(const string& str, const char* find) {
+	int idx = 0;
+	int len = str.length();
+	while (idx < len && toupper(find[idx]) == toupper(str[idx])) {
+		idx++;
+	}
+	return find[idx] == 0;
+}
+
 bool str_i_ends_with(const string& str, const char* find) {
 	int f_len = strlen(find);
 	int s_len = str.length();
@@ -411,6 +425,29 @@ void str_to_uppercase(string& output) {
 	}
 }
 
+std::vector<std::string> strs_to_uppercase(const std::vector<std::string>& input) {
+	std::vector<std::string> result;
+	result.reserve(input.size());
+	for (std::vector<std::string>::size_type i(0); i != input.size(); ++i) {
+		std::string value(input[i]);
+		str_to_uppercase(value);
+		result.push_back(value);
+	}
+	return result;
+}
+
+std::string str_join(const std::vector<std::string>& input, const char* joinStr) {
+	std::ostringstream strm;
+	for (std::vector<std::string>::size_type i(0); i != input.size(); ++i) {
+		if (i != 0) {
+			strm << joinStr;
+		}
+		strm << input[i];
+	}
+	return strm.str();
+}
+
+
 void str_uppercase_initial_capital(string& str) {
 	if (str.length() >= 1) str[0] = toupper(str[0]);
 }
@@ -627,7 +664,15 @@ int f_i_sign(int *a, int* b) {
 }
 
 int gle_round_int(double val) {
-	return (int)floor(val+0.5);
+	return (int)floor(val + 0.5);
+}
+
+int gle_make_zero_based(int val) {
+	if (val >= 1) {
+		return val - 1;
+	} else {
+		return val;
+	}
 }
 
 void str_get_system_error(ostream& error) {
@@ -711,3 +756,39 @@ bool bool_vector_is(vector<bool>* v, unsigned int i) {
 	if (i >= v->size()) return false;
 	else return (*v)[i];
 }
+
+void split_into_lines(const std::vector<unsigned char>* input, std::vector<std::string>* output) {
+   bool done = false;
+   unsigned int pos = 0;
+   while (!done) {
+      std::ostringstream crLine;
+      bool lineDone = false;
+      while (!lineDone) {
+         if (pos >= input->size()) {
+            done = true;
+            lineDone = true;
+         } else {
+            char ch = (char)input->at(pos++);
+            if (ch == '\n' || ch == '\r') {
+               if (pos < input->size() && input->at(pos) != ch && (input->at(pos) == '\n' || input->at(pos) == '\r')) {
+                  pos++;
+               }
+               lineDone = true;
+            } else {
+               crLine << ch;
+            }
+         }
+      }
+      output->push_back(crLine.str());
+   }
+}
+
+void CUtilsAssertImpl(const char* expr, const char* file, int line, const char* function) {
+	cerr << "Internal error: '" << expr << "' in file '" << file << "' function: '" << function << "' line: " << line << endl;
+	exit(1);
+}
+
+double gle_limit_range(double value, double minValue, double maxValue)
+{
+	return std::min<double>(std::max<double>(value, minValue), maxValue);
+}
