@@ -70,6 +70,11 @@ bool str_i_equals(const char* s1, const char* s2);
 bool str_ni_equals(const char* s1, const char* s2, int max);
 int str_i_cmp(const char* s1, const char* s2);
 
+struct str_i_less
+{
+  bool operator()(const std::string& s1, const std::string& s2) const;
+};
+
 char* str_i_str(const char* haystack, const char* needle);
 int str_i_str(const string& haystack, const char* needle);
 int str_i_str(const string& haystack, int from, const char* needle);
@@ -88,6 +93,10 @@ void str_to_uppercase(const string& input, string& output);
 
 void str_to_uppercase(string& output);
 
+std::vector<std::string> strs_to_uppercase(const std::vector<std::string>& input);
+
+std::string str_join(const std::vector<std::string>& input, const char* joinStr = ", ");
+
 void str_uppercase_initial_capital(string& str);
 
 bool str_only_space(const string& str);
@@ -106,6 +115,8 @@ int str_starts_with_trim(const string& str, const char* test);
 
 bool str_starts_with(const string& str, const char* find);
 
+bool str_i_starts_with(const string& str, const char* find);
+
 bool str_i_ends_with(const string& str, const char* find);
 
 void str_remove_quote(string& str);
@@ -145,6 +156,8 @@ void str_get_system_error(ostream& error);
 
 int gle_round_int(double val);
 
+int gle_make_zero_based(int val);
+
 #if defined(__UNIX__)
 	string str_format(const char* arg_list, ...);
 #else
@@ -153,10 +166,22 @@ int gle_round_int(double val);
 
 void str_format(string* str, const char *format, va_list ap);
 
-#endif
-
 int gle_pass_hex(const char* str, int from, int digits, int* err);
 
 void bool_vector_set_expand(vector<bool>* v, unsigned int i, bool value);
 
 bool bool_vector_is(vector<bool>* v, unsigned int i);
+
+void split_into_lines(const std::vector<unsigned char>* input, std::vector<std::string>* output);
+
+void CUtilsAssertImpl(const char* expr, const char* file, int line, const char* function);
+
+#define CUtilsAssertMessage(msg) \
+   CUtilsAssertImpl(msg, __FILE__, __LINE__, __FUNCTION__)
+
+#define CUtilsAssert(exp) \
+   if (!(exp)) CUtilsAssertImpl(#exp, __FILE__, __LINE__, __FUNCTION__)
+
+#endif
+
+double gle_limit_range(double value, double minValue, double maxValue);
