@@ -42,13 +42,14 @@
 #include "texinterface.h"
 #include "cutils.h"
 #include "gprint.h"
+#include "bitmap/img2ps.h"
 
 #ifdef HAVE_CAIRO
 
+#include <cairo-ps.h>
 #include <cairo-pdf.h>
 #include <cairo-svg.h>
 
-
 /************************************************************************************
  * Fonts in Cairo:
  *
@@ -58,11 +59,21 @@ extern struct gmodel g;
 
 char *font_getname(int i);
 
-GLECairoDevice::GLECairoDevice(bool showerror) {
-	m_ShowError = showerror;
+GLECairoDevice::GLECairoDevice(bool showerror):
+	m_ShowError(showerror),
+	m_width(0.0),
+	m_height(0.0),
+	m_surface(0),
+	m_cr(0),
+	m_FillMethod(GLE_FILL_METHOD_DEFAULT),
+	m_currentFill(g_get_fill_clear())
+{
 }
 
 GLECairoDevice::~GLECairoDevice() {
+	for (unsigned int i = 0; i < m_surfacesToDelete.size(); ++i) {
+		cairo_surface_destroy(m_surfacesToDelete[i]);
+	}
 }
 
 void GLECairoDevice::arc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy) {
@@ -71,32 +82,32 @@ void GLECairoDevice::arc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy) {
 	g_get_xy(&x,&y);
 	//polar_xy(r,t1,&dx,&dy);
 	if (!g.inpath) {
-		if (!g.xinline) cairo_new_path(cr);
+		if (!g.xinline) cairo_new_path(m_cr);
 	}
-	cairo_arc(cr, cx, cy, r, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
+	cairo_arc(m_cr, cx, cy, r, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
 	g.xinline = true;
 	if (!g.inpath) g_move(x,y);
 }
 
 void GLECairoDevice::arcto(dbl x1,dbl y1,dbl x2,dbl y2,dbl rrr) {
 	if (g.xinline==false) move(g.curx,g.cury);
-	cairo_curve_to(cr, x1, y1, x2, y2, x2, y2);
+	cairo_curve_to(m_cr, x1, y1, x2, y2, x2, y2);
 	g.xinline = true;
 }
 
 void GLECairoDevice::beginclip(void)  {
-	cairo_save(cr);
+	cairo_save(m_cr);
 }
 
 void GLECairoDevice::bezier(dbl x1,dbl y1,dbl x2,dbl y2,dbl x3,dbl y3) {
 	double x=g.curx,y=g.cury;
 	if (g.inpath) {
 		if (g.xinline==false) move(g.curx,g.cury);
-		cairo_curve_to(cr, x1, y1, x2, y2, x3, y3);
+		cairo_curve_to(m_cr, x1, y1, x2, y2, x3, y3);
 	} else {
 		g_flush();
-		if (!g.xinline) cairo_move_to(cr, x, y);
-		cairo_curve_to(cr, x1, y1, x2, y2, x3, y3);
+		if (!g.xinline) cairo_move_to(m_cr, x, y);
+		cairo_curve_to(m_cr, x1, y1, x2, y2, x3, y3);
 	}
 	g.xinline = true;
 }
@@ -106,30 +117,30 @@ void GLECairoDevice::box_fill(dbl x1, dbl y1, dbl x2, dbl y2) {
 		xdbox(x1,y1,x2,y2);
 	} else {
 		g_flush();
-		cairo_new_path(cr);
+		cairo_new_path(m_cr);
 		GLERectangle rect(x1, y1, x2, y2);
 		xdbox(x1,y1,x2,y2);
 		ddfill(&rect);
-		cairo_new_path(cr);
+		cairo_new_path(m_cr);
 	}
 }
 
 void GLECairoDevice::box_stroke(dbl x1, dbl y1, dbl x2, dbl y2, bool reverse) {
 	if (g.inpath) {
 		if (reverse) {
-			cairo_move_to(cr, x1, y1);
-			cairo_line_to(cr, x1, y2);
-			cairo_line_to(cr, x2, y2);
-			cairo_line_to(cr, x2, y1);
-			cairo_close_path(cr);
+			cairo_move_to(m_cr, x1, y1);
+			cairo_line_to(m_cr, x1, y2);
+			cairo_line_to(m_cr, x2, y2);
+			cairo_line_to(m_cr, x2, y1);
+			cairo_close_path(m_cr);
 		} else {
 			xdbox(x1,y1,x2,y2);
 		}
 	} else {
 		g_flush();
-		cairo_new_path(cr);
+		cairo_new_path(m_cr);
 		xdbox(x1,y1,x2,y2);
-		cairo_stroke(cr);
+		cairo_stroke(m_cr);
 		//ps_nvec = 0;
 	}
 }
@@ -138,7 +149,7 @@ void GLECairoDevice::dochar(int font, int cc) {
 	if (font_get_encoding(font) > 2) {
 		my_char(font, cc);
 	} else {
-		my_char(17, cc);
+		g_throw_parser_error("PostScript fonts not supported with '-cairo'");
 	}
 }
 
@@ -148,14 +159,14 @@ void GLECairoDevice::resetfont() {
 void GLECairoDevice::circle_fill(double zr) {
 	double x=g.curx,y=g.cury;
 	if (g.inpath) {
-		cairo_arc(cr, x, y, zr, 0, 2*GLE_PI);
+		cairo_arc(m_cr, x, y, zr, 0, 2*GLE_PI);
 	} else {
 		g_flush();
-		cairo_new_path(cr);
-		cairo_arc(cr, x, y, zr, 0, 2*GLE_PI);
+		cairo_new_path(m_cr);
+		cairo_arc(m_cr, x, y, zr, 0, 2*GLE_PI);
 		GLERectangle rect(x-zr, y-zr, x+zr, y+zr);
 		ddfill(&rect);
-		cairo_new_path(cr);
+		cairo_new_path(m_cr);
 	}
 }
 
@@ -163,13 +174,13 @@ void GLECairoDevice::circle_stroke(double zr) {
 	double x,y;
 	g_get_xy(&x,&y);
 	if (g.inpath) {
-		cairo_arc(cr, x, y, zr, 0, 2*GLE_PI);
+		cairo_arc(m_cr, x, y, zr, 0, 2*GLE_PI);
 	} else {
 		g_flush();
-		cairo_new_path(cr);
-		cairo_arc(cr, x, y, zr, 0, 2*GLE_PI);
-		cairo_close_path(cr);
-		cairo_stroke(cr);
+		cairo_new_path(m_cr);
+		cairo_arc(m_cr, x, y, zr, 0, 2*GLE_PI);
+		cairo_close_path(m_cr);
+		cairo_stroke(m_cr);
 	}
 }
 
@@ -177,17 +188,23 @@ void GLECairoDevice::clear(void) {
 }
 
 void GLECairoDevice::clip(void) {
-	cairo_clip(cr);
+	cairo_clip(m_cr);
 }
 
 void GLECairoDevice::closedev(void) {
-    cairo_destroy(cr);
-    cairo_surface_destroy(surface);
-    printf("%s]\n", m_OutputName.getName().c_str());
+    cairo_destroy(m_cr);
+    cairo_surface_destroy(m_surface);
+	if (g_verbosity() > 0) {
+		string mainname;
+		string extension(g_device_to_ext(getDeviceType()));
+		GetMainNameExt(m_OutputName.getName(), extension.c_str(), mainname);
+		cerr << "[" << mainname << "][" << extension << "]";
+		g_set_console_output(false);
+	}
 }
 
 void GLECairoDevice::closepath(void) {
-	cairo_close_path(cr);
+	cairo_close_path(m_cr);
 }
 
 void GLECairoDevice::dfont(char *c) {
@@ -196,22 +213,22 @@ void GLECairoDevice::dfont(char *c) {
 void GLECairoDevice::ellipse_fill(double rx, double ry) {
 	double x=g.curx,y=g.cury;
 	if (g.inpath) {
-		cairo_save (cr);
-		cairo_translate (cr, x, y);
-		cairo_scale (cr, rx, ry);
-		cairo_arc (cr, 0, 0, 1, 0, 2*GLE_PI);
-		cairo_restore (cr);
+		cairo_save (m_cr);
+		cairo_translate (m_cr, x, y);
+		cairo_scale (m_cr, rx, ry);
+		cairo_arc (m_cr, 0, 0, 1, 0, 2*GLE_PI);
+		cairo_restore (m_cr);
 	} else {
 		g_flush();
-		cairo_new_path(cr);
-		cairo_save (cr);
-		cairo_translate (cr, x, y);
-		cairo_scale (cr, rx, ry);
-		cairo_arc (cr, 0, 0, 1, 0, 2*GLE_PI);
-		cairo_restore (cr);
+		cairo_new_path(m_cr);
+		cairo_save (m_cr);
+		cairo_translate (m_cr, x, y);
+		cairo_scale (m_cr, rx, ry);
+		cairo_arc (m_cr, 0, 0, 1, 0, 2*GLE_PI);
+		cairo_restore (m_cr);
 		GLERectangle rect(x-rx, y-ry, x+rx, y+ry);
 		ddfill(&rect);
-		cairo_new_path(cr);
+		cairo_new_path(m_cr);
 	}
 }
 
@@ -219,13 +236,13 @@ void GLECairoDevice::ellipse_stroke(double rx, double ry) {
 	double x,y;
 	g_get_xy(&x,&y);
 	if (!g.inpath) {
-		if (!g.xinline) cairo_new_path(cr);
+		if (!g.xinline) cairo_new_path(m_cr);
 	}
-	cairo_save (cr);
-	cairo_translate (cr, x, y);
-	cairo_scale (cr, rx, ry);
-	cairo_arc (cr, 0, 0, 1, 0, 2*GLE_PI);
-	cairo_restore (cr);
+	cairo_save (m_cr);
+	cairo_translate (m_cr, x, y);
+	cairo_scale (m_cr, rx, ry);
+	cairo_arc (m_cr, 0, 0, 1, 0, 2*GLE_PI);
+	cairo_restore (m_cr);
 	g.xinline = true;
 	if (!g.inpath) g_move(x,y);
 }
@@ -234,13 +251,13 @@ void GLECairoDevice::elliptical_arc(double rx,double ry,double t1,double t2,doub
 	double x,y;
 	g_get_xy(&x,&y);
 	if (!g.inpath) {
-		if (!g.xinline) cairo_new_path(cr);
+		if (!g.xinline) cairo_new_path(m_cr);
 	}
-	cairo_save (cr);
-	cairo_translate (cr, cx, cy);
-	cairo_scale (cr, rx, ry);
-	cairo_arc (cr, 0, 0, 1, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
-	cairo_restore (cr);
+	cairo_save (m_cr);
+	cairo_translate (m_cr, cx, cy);
+	cairo_scale (m_cr, rx, ry);
+	cairo_arc (m_cr, 0, 0, 1, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
+	cairo_restore (m_cr);
 	g.xinline = true;
 	if (!g.inpath) g_move(x,y);
 }
@@ -249,24 +266,24 @@ void GLECairoDevice::elliptical_narc(double rx,double ry,double t1,double t2,dou
 	double x,y;
 	g_get_xy(&x,&y);
 	if (!g.inpath) {
-		if (!g.xinline) cairo_new_path(cr);
+		if (!g.xinline) cairo_new_path(m_cr);
 	}
-	cairo_save (cr);
-	cairo_translate (cr, cx, cy);
-	cairo_scale (cr, rx, ry);
-	cairo_arc_negative (cr, 0, 0, 1, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
-	cairo_restore (cr);
+	cairo_save (m_cr);
+	cairo_translate (m_cr, cx, cy);
+	cairo_scale (m_cr, rx, ry);
+	cairo_arc_negative (m_cr, 0, 0, 1, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
+	cairo_restore (m_cr);
 	g.xinline = true;
 	if (!g.inpath) g_move(x,y);
 }
 
 void GLECairoDevice::endclip(void)  {
 	g_flush();
-	cairo_restore(cr);
-	gmodel* state = (gmodel*) myallocz(SIZEOFSTATE);
+	cairo_restore(m_cr);
+	gmodel* state = new gmodel();
 	g_get_state(state);
 	g_set_state(state);
-	myfree(state);
+	delete state;
 }
 
 void GLECairoDevice::fill(void) {
@@ -276,44 +293,49 @@ void GLECairoDevice::fill(void) {
 }
 
 void GLECairoDevice::ddfill(GLERectangle* bounds) {
-	if (g_cur_fill.b[B_F] == 255) return; /* clear fill, do nothing */
-	if (g_cur_fill.b[B_F] == 2) {
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	if (cur_fill.b[B_F] == 255) {
+		return; /* clear fill, do nothing */
+	} else if (cur_fill.b[B_F] == 2) {
 		shade(bounds);
 		return;
 	}
 	set_fill();			/*because color and fill are the same*/
 	//cairo_fill(cr);
-	cairo_fill_preserve(cr);
+	cairo_fill_preserve(m_cr);
 	set_color();
 }
 
 void GLECairoDevice::shadePattern(void) {
-	int step1 = g_cur_fill.b[B_B];
-	int step2 = g_cur_fill.b[B_G];
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	int step1 = cur_fill.b[B_B];
+	int step2 = cur_fill.b[B_G];
 	int xstep = max(step1, step2);
 	int ystep = max(step1, step2);
-	cairo_save(cr);
+	cairo_save(m_cr);
 	cairo_matrix_t matrix;
-	cairo_get_matrix(cr, &matrix);
-	cairo_surface_t *isurface = cairo_surface_create_similar(surface, CAIRO_CONTENT_COLOR_ALPHA, xstep, ystep);
+	cairo_get_matrix(m_cr, &matrix);
+	cairo_surface_t *isurface = cairo_surface_create_similar(m_surface, CAIRO_CONTENT_COLOR_ALPHA, xstep, ystep);
 	cairo_t *icr = cairo_create(isurface);
-	if (m_Background != (int)GLE_FILL_CLEAR) {
-		if (m_Background == (int)GLE_COLOR_WHITE) {
+	GLERC<GLEColor> background(get_fill_background(m_currentFill.get()));
+	if (!background->isTransparent()) {
+		if (background->getHexValueGLE() == (unsigned int)GLE_COLOR_WHITE) {
 			cairo_set_source_rgb(icr, 1.0, 1.0, 1.0);
 		} else {
-			colortyp col;
-			col.l = m_Background;
-			cairo_set_source_rgb(icr, col.b[B_R]/255.0, col.b[B_G]/255.0, col.b[B_B]/255.0);
+			cairo_set_source_rgb(icr, background->getRed(), background->getGreen(), background->getBlue());
 		}
 		cairo_rectangle(icr, -1, -1, (xstep+1), (ystep+1));
 		cairo_fill(icr);
 	}
-	if (g_cur_fill_color.l == GLE_COLOR_BLACK) {
+	GLERC<GLEColor> foreground(get_fill_foreground(m_currentFill.get()));
+	if (foreground->getHexValueGLE() == GLE_COLOR_BLACK) {
 		cairo_set_source_rgb(icr, 0, 0, 0);
 	} else {
-		cairo_set_source_rgb(icr, g_cur_fill_color.b[B_R]/255.0, g_cur_fill_color.b[B_G]/255.0, g_cur_fill_color.b[B_B]/255.0);
+		cairo_set_source_rgb(icr, foreground->getRed(), foreground->getGreen(), foreground->getBlue());
 	}
-	cairo_set_line_width(icr, (int)g_cur_fill.b[B_R]);
+	cairo_set_line_width(icr, cur_fill.b[B_R]);
 	if (step1 > 0) {
 		cairo_move_to(icr, 0, 0);
 		cairo_line_to(icr, xstep, ystep);
@@ -340,34 +362,35 @@ void GLECairoDevice::shadePattern(void) {
 			cairo_stroke(icr);
 		}
 	}
-	cairo_set_source_rgb(icr, g_cur_color.b[B_R]/255.0, g_cur_color.b[B_G]/255.0, g_cur_color.b[B_B]/255.0);
 	cairo_pattern_t *pattern = cairo_pattern_create_for_surface(isurface);
 	cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);
 	cairo_matrix_init_scale(&matrix, 160.0, 160.0);
 	cairo_pattern_set_matrix(pattern, &matrix);
-	cairo_set_source(cr, pattern);
-	cairo_fill(cr);
-	cairo_restore(cr);
+	cairo_set_source(m_cr, pattern);
+	cairo_fill(m_cr);
+	cairo_restore(m_cr);
 	cairo_pattern_destroy(pattern);
 	cairo_destroy(icr);
 	cairo_surface_destroy(isurface);
 }
 
 void GLECairoDevice::shadeGLE() {
-	double step1 = g_cur_fill.b[B_B]/160.0;
-	double step2 = g_cur_fill.b[B_G]/160.0;
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	double step1 = cur_fill.b[B_B]/160.0;
+	double step2 = cur_fill.b[B_G]/160.0;
 	if (step1 > 0) {
 		for (double x = -40.0; x < 40.0; x += step1) {
-			cairo_move_to(cr, x, 0.0);
-			cairo_line_to(cr, 40.0+x, 40);
-			cairo_stroke(cr);
+			cairo_move_to(m_cr, x, 0.0);
+			cairo_line_to(m_cr, 40.0+x, 40);
+			cairo_stroke(m_cr);
 		}
 	}
 	if (step2 > 0) {
 		for (double x = 0.0; x < 80; x += step2) {
-			cairo_move_to(cr, x, 0.0);
-			cairo_line_to(cr, x-40.0, 40);
-			cairo_stroke(cr);
+			cairo_move_to(m_cr, x, 0.0);
+			cairo_line_to(m_cr, x-40.0, 40);
+			cairo_stroke(m_cr);
 		}
 	}
 }
@@ -376,30 +399,32 @@ void GLECairoDevice::shadeBoundedIfThenElse1(GLERectangle* bounds, double p, dou
 	// if x1+p*s > y1 then
 	if (bounds->getXMax()+p*step1 > bounds->getYMax()) {
 		// aline y1-p*s y1
-		cairo_line_to(cr, bounds->getYMax()-p*step1, bounds->getYMax());
+		cairo_line_to(m_cr, bounds->getYMax()-p*step1, bounds->getYMax());
 	} else {
 		// aline x1 x1+p*s
-		cairo_line_to(cr, bounds->getXMax(), bounds->getXMax()+p*step1);
+		cairo_line_to(m_cr, bounds->getXMax(), bounds->getXMax()+p*step1);
 	}
-	cairo_stroke(cr);
+	cairo_stroke(m_cr);
 }
 
 void GLECairoDevice::shadeBoundedIfThenElse2(GLERectangle* bounds, double p, double step2) {
 	// if p*s-y1 > x0 then
 	if (p*step2-bounds->getYMax() > bounds->getXMin()) {
 		// aline p*s-y1 y1
-		cairo_line_to(cr, p*step2-bounds->getYMax(), bounds->getYMax());
+		cairo_line_to(m_cr, p*step2-bounds->getYMax(), bounds->getYMax());
 	} else {
 		// aline x0 p*s-x0
-		cairo_line_to(cr, bounds->getXMin(), p*step2-bounds->getXMin());
+		cairo_line_to(m_cr, bounds->getXMin(), p*step2-bounds->getXMin());
 	}
-	cairo_stroke(cr);
+	cairo_stroke(m_cr);
 }
 
 void GLECairoDevice::shadeBounded(GLERectangle* bounds) {
-	double step1 = g_cur_fill.b[B_B]/160.0;
-	double step2 = g_cur_fill.b[B_G]/160.0;
-	cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	double step1 = cur_fill.b[B_B]/160.0;
+	double step2 = cur_fill.b[B_G]/160.0;
+	cairo_set_line_cap(m_cr, CAIRO_LINE_CAP_SQUARE);
 	if (step1 > 0) {
 		int p0 = (int)ceil((bounds->getYMax()-bounds->getXMin())/step1-1e-6);
 		if (bounds->getXMin() + p0*step1 > bounds->getYMax()) p0--;
@@ -409,12 +434,12 @@ void GLECairoDevice::shadeBounded(GLERectangle* bounds) {
 		if (bounds->getXMax() + p2*step1 < bounds->getYMin()) p2++;
 		for (int p = p0; p > p1; p--) {
 			// amove x0 x0+p*s
-			cairo_move_to(cr, bounds->getXMin(), bounds->getXMin()+p*step1);
+			cairo_move_to(m_cr, bounds->getXMin(), bounds->getXMin()+p*step1);
 			shadeBoundedIfThenElse1(bounds, p, step1);
 		}
 		for (int p = p1; p >= p2; p--) {
 			// amove y0-p*s y0
-			cairo_move_to(cr, bounds->getYMin()-p*step1, bounds->getYMin());
+			cairo_move_to(m_cr, bounds->getYMin()-p*step1, bounds->getYMin());
 			shadeBoundedIfThenElse1(bounds, p, step1);
 		}
 	}
@@ -427,12 +452,12 @@ void GLECairoDevice::shadeBounded(GLERectangle* bounds) {
 		if (p2*step2 - bounds->getXMax() < bounds->getYMin()) p2++;
 		for (int p = p0; p > p1; p--) {
 			// amove x0 x0+p*s
-			cairo_move_to(cr, bounds->getXMax(), p*step2-bounds->getXMax());
+			cairo_move_to(m_cr, bounds->getXMax(), p*step2-bounds->getXMax());
 			shadeBoundedIfThenElse2(bounds, p, step2);
 		}
 		for (int p = p1; p >= p2; p--) {
 			// amove y0-p*s y0
-			cairo_move_to(cr, p*step2-bounds->getYMin(), bounds->getYMin());
+			cairo_move_to(m_cr, p*step2-bounds->getYMin(), bounds->getYMin());
 			shadeBoundedIfThenElse2(bounds, p, step2);
 		}
 	}
@@ -441,32 +466,26 @@ void GLECairoDevice::shadeBounded(GLERectangle* bounds) {
 void GLECairoDevice::shade(GLERectangle* bounds) {
 	if (m_FillMethod == GLE_FILL_METHOD_GLE ||
 	   (m_FillMethod == GLE_FILL_METHOD_DEFAULT && bounds != NULL)) {
-		cairo_save(cr);
-		if (m_Background != (int)GLE_FILL_CLEAR) {
-			if (m_Background == (int)GLE_COLOR_WHITE) {
-				cairo_set_source_rgb(cr, 1.0, 1.0, 1.0);
-			} else {
-				colortyp col;
-				col.l = m_Background;
-				set_color(col);
-			}
-			cairo_fill_preserve(cr);
+		cairo_save(m_cr);
+		GLERC<GLEColor> background(get_fill_background(m_currentFill.get()));
+		if (!background->isTransparent()) {
+			cairo_set_source_rgb(m_cr, background->getRed(), background->getGreen(), background->getBlue());
+			cairo_fill_preserve(m_cr);
 		}
 		// Implemented by using path as clip and then painting strokes over the clip
-		cairo_clip(cr);
-		cairo_new_path(cr);
-		if (g_cur_fill_color.l == GLE_COLOR_BLACK) {
-			cairo_set_source_rgb(cr, 0.0, 0.0, 0.0);
-		} else {
-			set_color(g_cur_fill_color);
-		}
-		cairo_set_line_width(cr, (double)(g_cur_fill.b[B_R]/160.0));
+		cairo_clip(m_cr);
+		cairo_new_path(m_cr);
+		GLERC<GLEColor> foreground(get_fill_foreground(m_currentFill.get()));
+		cairo_set_source_rgb(m_cr, foreground->getRed(), foreground->getGreen(), foreground->getBlue());
+		colortyp cur_fill;
+		cur_fill.l = m_currentFill->getHexValueGLE();
+		cairo_set_line_width(m_cr, (double)(cur_fill.b[B_R]/160.0));
 		if (m_FillMethod == GLE_FILL_METHOD_DEFAULT && bounds != NULL) {
 			shadeBounded(bounds);
 		} else {
 			shadeGLE();
 		}
-		cairo_restore(cr);
+		cairo_restore(m_cr);
 	} else {
 		shadePattern();
 	}
@@ -479,20 +498,23 @@ void GLECairoDevice::fill_ary(int nwk,double *wkx,double *wky) {
 void GLECairoDevice::flush(void) {
 	if (g.inpath) return;
 	if (g.xinline) {
-		cairo_stroke(cr);
+		cairo_stroke(m_cr);
 		//ps_nvec = 0;
 	}
 }
 
-void GLECairoDevice::get_type(char *t) {
-	strcpy(t,"CAIRO FILLPATH");
+std::string GLECairoDevice::get_type() {
+	std::vector<std::string> temp(g_create_device_string());
+	temp.push_back("FILLPATH");
+	temp.push_back("CAIRO");
+	return str_join(temp);
 }
 
 void GLECairoDevice::line(double zx,double zy) {
 	if (g.xinline==false) {
 		move(g.curx,g.cury);
 	}
-	cairo_line_to(cr, zx, zy);
+	cairo_line_to(m_cr, zx, zy);
 }
 
 void GLECairoDevice::line_ary(int nwk,double *wkx,double *wky) {
@@ -505,11 +527,11 @@ void GLECairoDevice::message(char *s) {
 
 void GLECairoDevice::move(double zx,double zy) {
 	if (g.inpath) {
-		cairo_move_to(cr, zx, zy);
+		cairo_move_to(m_cr, zx, zy);
 	} else {
 		//ps_nvec++;
-		cairo_new_path(cr);
-		cairo_move_to(cr, zx, zy);
+		cairo_new_path(m_cr);
+		cairo_move_to(m_cr, zx, zy);
 	}
 }
 
@@ -519,15 +541,15 @@ void GLECairoDevice::narc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy) {
 	g_get_xy(&x,&y);
 	//polar_xy(r,t1,&dx,&dy);
 	if (!g.inpath) {
-		if (!g.xinline) cairo_new_path(cr);
+		if (!g.xinline) cairo_new_path(m_cr);
 	}
-	cairo_arc_negative(cr, cx, cy, r, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
+	cairo_arc_negative(m_cr, cx, cy, r, t1*GLE_PI/180.0, t2*GLE_PI/180.0);
 	g.xinline = true;
 	if (!g.inpath) g_move(x,y);
 }
 
 void GLECairoDevice::newpath(void) {
-	cairo_new_path(cr);
+	cairo_new_path(m_cr);
 }
 
 void GLECairoDevice::opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) {
@@ -541,50 +563,47 @@ void GLECairoDevice::reverse(void)    /* reverse the order of stuff in the curre
 	cout << "reverse not yet implemented" << endl;
 }
 
-void GLECairoDevice::set_color(colortyp& color) {
-	cairo_set_source_rgb(cr, color.b[B_R]/255.0, color.b[B_G]/255.0, color.b[B_B]/255.0);
+void GLECairoDevice::set_color_impl(const GLERC<GLEColor>& color) {
+	if (color->hasAlpha()) {
+		cairo_set_source_rgba(m_cr, color->getRed(), color->getGreen(), color->getBlue(), color->getAlpha());
+	} else {
+		cairo_set_source_rgb(m_cr, color->getRed(), color->getGreen(), color->getBlue());
+	}
 }
 
 void GLECairoDevice::set_color() {
-	set_color(g_cur_color);
+	set_color_impl(m_currentColor);
 }
-void GLECairoDevice::set_color(int f) {
+
+void GLECairoDevice::set_color(const GLERC<GLEColor>& color) {
 	g_flush();
-	g_cur_color.l = f;
+	m_currentColor = color;
 	set_color();
 }
 
-void GLECairoDevice::set_fill(int f) {
-	g_cur_fill.l = f;
-}
-
-void GLECairoDevice::set_background(int b) {
-	m_Background = b;
+void GLECairoDevice::set_fill(const GLERC<GLEColor>& fill) {
+	m_currentFill = fill;
 }
 
 void GLECairoDevice::set_fill_method(int m) {
 	m_FillMethod = m;
 }
 
-void GLECairoDevice::set_pattern_color(int c) {
-	g_cur_fill_color.l = c;
-}
-
 void GLECairoDevice::set_line_cap(int i) {
 	/*  lcap, 0= butt, 1=round, 2=projecting square */
 	if (!g.inpath) g_flush();
-	cairo_set_line_cap(cr, (cairo_line_cap_t)i);
+	cairo_set_line_cap(m_cr, (cairo_line_cap_t)i);
 }
 
 void GLECairoDevice::set_line_join(int i) {
 	/* 0=miter, 1=round, 2=bevel */
 	if (!g.inpath) g_flush();
-	cairo_set_line_join(cr, (cairo_line_join_t)i);
+	cairo_set_line_join(m_cr, (cairo_line_join_t)i);
 }
 
 void GLECairoDevice::set_line_miterlimit(double d) {
 	if (!g.inpath) g_flush();
-	cairo_set_miter_limit(cr, d);
+	cairo_set_miter_limit(m_cr, d);
 }
 
 void GLECairoDevice::set_line_style(const char *s) {
@@ -600,7 +619,7 @@ void GLECairoDevice::set_line_style(const char *s) {
 	for (int i = 0; i < nb_dashes; i++) {
 		dashes[i] = (s[i]-'0')*g.lstyled;
 	}
-	cairo_set_dash(cr, dashes, nb_dashes, 0);
+	cairo_set_dash(m_cr, dashes, nb_dashes, 0);
 	delete[] dashes;
 }
 
@@ -613,7 +632,7 @@ void GLECairoDevice::set_line_width(double w) {
 	if (w == 0) w = 0.02;
 	if (w < .0002) w = 0;
 	if (!g.inpath) g_flush();
-	cairo_set_line_width(cr, w);
+	cairo_set_line_width(m_cr, w);
 }
 
 void GLECairoDevice::set_matrix(double newmat[3][3]) {
@@ -623,8 +642,12 @@ void GLECairoDevice::set_matrix(double newmat[3][3]) {
 	matrix.yx = - newmat[1][0];
 	matrix.yy = - newmat[1][1];
 	matrix.x0 = newmat[0][2];
-	matrix.y0 = m_height * 72 / CM_PER_INCH - newmat[1][2];
-	cairo_set_matrix(cr, &matrix);
+	double offset = 0.0;
+	if (!g_is_fullpage()) {
+		offset = 2.0 * CM_PER_INCH / 72;
+	}
+	matrix.y0 = (m_height + offset) * 72 / CM_PER_INCH - newmat[1][2];
+	cairo_set_matrix(m_cr, &matrix);
 }
 
 void GLECairoDevice::set_path(int onoff) {
@@ -636,21 +659,21 @@ void GLECairoDevice::source(const char *s) {
 }
 
 void GLECairoDevice::stroke(void) {
-	cairo_stroke_preserve(cr);
+	cairo_stroke_preserve(m_cr);
 }
 
 void GLECairoDevice::set_fill(void) {
-	set_color(g_cur_fill);
+	set_color_impl(m_currentFill);
 }
 
 void GLECairoDevice::xdbox(double x1, double y1, double x2, double y2) {
 	//cout << "xdbox" << endl;
 	//cairo_rectangle_t(cr, x1, y1, x2 - x1, y2 - y1);
-	cairo_move_to(cr, x1, y1);
-	cairo_line_to(cr, x2, y1);
-	cairo_line_to(cr, x2, y2);
-	cairo_line_to(cr, x1, y2);
-	cairo_close_path(cr);
+	cairo_move_to(m_cr, x1, y1);
+	cairo_line_to(m_cr, x2, y1);
+	cairo_line_to(m_cr, x2, y2);
+	cairo_line_to(m_cr, x1, y2);
+	cairo_close_path(m_cr);
 }
 
 void GLECairoDevice::devcmd(const char *s) {
@@ -665,6 +688,166 @@ int GLECairoDevice::getDeviceType() {
 	return GLE_DEVICE_NONE;
 }
 
+void delete_gle_recorded_byte_stream(void* todelete) {
+	GLERecordedByteStream* stream = (GLERecordedByteStream*)todelete;
+	delete stream;
+}
+
+class GLECairoImageByteStream: public GLEByteStream {
+public:
+	GLECairoImageByteStream(cairo_surface_t *image) {
+		m_buffer = cairo_image_surface_get_data(image);
+		m_stride = cairo_image_surface_get_stride(image);
+		m_pos = 0;
+		m_scanLine = 0;
+	}
+
+	virtual int sendByte(GLEBYTE byte) {
+		m_buffer[m_pos++] = byte;
+		return GLE_IMAGE_ERROR_NONE;
+	}
+
+	virtual int endScanLine() {
+		m_scanLine++;
+		m_pos = m_scanLine * m_stride;
+		return GLE_IMAGE_ERROR_NONE;
+	}
+
+protected:
+	unsigned char* m_buffer;
+	int m_stride;
+	int m_pos;
+	int m_scanLine;
+};
+
+cairo_surface_t* GLECairoDevice::bitmapCreateSurface(GLEBitmap* bitmap) {
+	/* Set options */
+	bitmap->setCompress(0.0);
+	bitmap->setASCII85(1);
+	/* Convert bitmap to postscript */
+	bitmap->prepare(GLE_BITMAP_PREPARE_SCANLINE);
+	cairo_format_t imageFormat = CAIRO_FORMAT_RGB24;
+	if (bitmap->isAlpha()) {
+		imageFormat = CAIRO_FORMAT_ARGB32;
+	}
+	if (bitmap->isGrayScale()) {
+		imageFormat = CAIRO_FORMAT_A8;
+		if (bitmap->getBitsPerComponent() == 1) {
+			imageFormat = CAIRO_FORMAT_A1;
+		}
+	}
+	cairo_surface_t* image = cairo_image_surface_create(imageFormat, bitmap->getWidth(), bitmap->getHeight());
+	GLECairoImageByteStream stream(image);
+	GLEByteStream* toDecode = &stream;
+	GLERGBATo32BitByteStream rgbTo32Bit(toDecode, bitmap->isAlpha());
+	if (bitmap->isIndexed() || bitmap->getColorComponents() >= 3) {
+		toDecode = &rgbTo32Bit;
+	}
+	int extra = bitmap->getExtraComponents();
+	int color_alpha = bitmap->getColorComponents();
+	if (bitmap->isAlpha()) {
+		// Do not count alpha as extra component
+		extra--;
+		color_alpha++;
+	}
+	GLEComponentRemovalByteStream crem(toDecode, color_alpha, extra);
+	if (extra != 0) {
+		toDecode = &crem;
+	}
+	GLEPNegateByteStream negated(&stream);
+	if (bitmap->isGrayScale()) {
+		toDecode = &negated;
+	}
+	GLEIndexedToRGBByteStream indexedToRGB(toDecode, bitmap->getPalette());
+	if (bitmap->isIndexed()) {
+		toDecode = &indexedToRGB;
+	}
+	GLEBitsTo32BitByteStream combine(toDecode);
+	if (bitmap->isGrayScale() && bitmap->getBitsPerComponent() == 1) {
+		toDecode = &combine;
+	}
+	bitmap->decode(toDecode);
+	cairo_surface_mark_dirty(image);
+	if (bitmap->getEncoding() == GLE_BITMAP_JPEG) {
+		GLERecordedByteStream* stream = new GLERecordedByteStream();
+		bitmap->coded(stream);
+		cairo_status_t status = cairo_surface_set_mime_data(image,
+															CAIRO_MIME_TYPE_JPEG,
+															stream->getBytes(),
+															stream->getNbBytes(),
+															delete_gle_recorded_byte_stream,
+															(void*)stream);
+		CUtilsAssert(status == CAIRO_STATUS_SUCCESS);
+	}
+	return image;
+}
+
+void GLECairoDevice::bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int type) {
+	/* Store current box */
+	GLERectangle save_box;
+	g_get_bounds(&save_box);
+	/* Generate header in postrscript output */
+	g_gsave();
+	/* Get current position	*/
+	g_scale(scale->getX() / bitmap->getWidth(), scale->getY() / bitmap->getHeight());
+	g_translate(pos->getX(), pos->getY());
+	/* Flip image so that it is not upside down */
+	cairo_matrix_t mirror, current, result;
+	cairo_matrix_init(&mirror, 1, 0, 0, -1, 0, bitmap->getHeight());
+	cairo_get_matrix(m_cr, &current);
+	cairo_matrix_multiply(&result, &mirror, &current);
+	cairo_set_matrix(m_cr, &result);
+	/* Create image */
+	bool inCache = false;
+	cairo_surface_t* image = 0;
+	const std::string& name(bitmap->getFName());
+	if (name.empty()) {
+		image = bitmapCreateSurface(bitmap);
+	} else {
+		inCache = true;
+		image = (cairo_surface_t*)m_bitmapCache.try_get(name);
+		if (image == 0) {
+			image = bitmapCreateSurface(bitmap);
+			m_bitmapCache.add_item(name, image);
+			m_surfacesToDelete.push_back(image);
+		}
+	}
+	cairo_set_source_surface(m_cr, image, 0, 0);
+	cairo_paint(m_cr);
+	if (!inCache) {
+		cairo_surface_destroy(image);
+	}
+	/* Footer */
+	g_grestore();
+	g_set_bounds(&save_box);
+	bitmap->close();
+}
+
+void GLECairoDevice::getRecordedBytes(string* output) {
+	if (!m_recorded.empty()) {
+		*output = std::string(&m_recorded[0], m_recorded.size());
+	} else {
+		output->clear();
+	}
+}
+
+void GLECairoDevice::recordData(const unsigned char *data, unsigned int length) {
+	m_recorded.reserve(m_recorded.size() + length);
+	for (unsigned int i = 0; i < length; ++i) {
+		m_recorded.push_back((char)data[i]);
+	}
+}
+
+void GLECairoDevice::clearRecordedData() {
+	m_recorded.clear();
+}
+
+cairo_status_t gle_cairo_device_write(void* closure, const unsigned char *data, unsigned int length) {
+	GLECairoDevice* cairoDevice = (GLECairoDevice*)closure;
+	cairoDevice->recordData(data, length);
+	return CAIRO_STATUS_SUCCESS;
+}
+
 GLECairoDeviceSVG::GLECairoDeviceSVG(bool showerror) : GLECairoDevice(showerror) {
 }
 
@@ -675,11 +858,15 @@ void GLECairoDeviceSVG::opendev(double width, double height, GLEFileLocation* ou
 	m_width = width;
 	m_height = height;
 	m_OutputName.copy(outputfile);
-	m_OutputName.addExtension("svg");
-	surface = cairo_svg_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
-	cr = cairo_create(surface);
-	g_scale(72.0/CM_PER_INCH, 72.0/CM_PER_INCH);
-	g_translate(1.0*CM_PER_INCH/72, -1.0*CM_PER_INCH/72);
+	m_OutputName.addExtension(g_device_to_ext(getDeviceType()));
+	m_surface = cairo_svg_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
+	cairo_surface_set_fallback_resolution(m_surface, m_resolution, m_resolution);
+	m_cr = cairo_create(m_surface);
+	computeBoundingBox(width, height);
+	g_scale(PS_POINTS_PER_INCH/CM_PER_INCH, PS_POINTS_PER_INCH/CM_PER_INCH);
+	if (!g_is_fullpage()) {
+		g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	}
 }
 
 int GLECairoDeviceSVG::getDeviceType() {
@@ -693,20 +880,93 @@ GLECairoDevicePDF::~GLECairoDevicePDF() {
 }
 
 void GLECairoDevicePDF::opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) {
+	clearRecordedData();
 	m_width = width;
 	m_height = height;
 	m_OutputName.copy(outputfile);
-	m_OutputName.addExtension("pdf");
-	surface = cairo_pdf_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
-	cr = cairo_create(surface);
-	g_scale(72.0/CM_PER_INCH, 72.0/CM_PER_INCH);
-	g_translate(1.0*CM_PER_INCH/72, -1.0*CM_PER_INCH/72);
+	m_OutputName.addExtension(g_device_to_ext(getDeviceType()));
+	if (isRecordingEnabled()) {
+		m_surface = cairo_pdf_surface_create_for_stream(gle_cairo_device_write, (void*)this, 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
+	} else {
+		m_surface = cairo_pdf_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
+	}
+	cairo_surface_set_fallback_resolution(m_surface, m_resolution, m_resolution);
+	m_cr = cairo_create(m_surface);
+	computeBoundingBox(width, height);
+	g_scale(PS_POINTS_PER_INCH/CM_PER_INCH, PS_POINTS_PER_INCH/CM_PER_INCH);
+	if (!g_is_fullpage()) {
+		g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	}
 }
 
 int GLECairoDevicePDF::getDeviceType() {
 	return GLE_DEVICE_CAIRO_PDF;
 }
 
+GLECairoDeviceEPS::GLECairoDeviceEPS(bool showerror) : GLECairoDevice(showerror) {
+}
+
+GLECairoDeviceEPS::~GLECairoDeviceEPS() {
+}
+
+void GLECairoDeviceEPS::opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) {
+	clearRecordedData();
+	m_width = width;
+	m_height = height;
+	m_OutputName.copy(outputfile);
+	m_OutputName.addExtension(g_device_to_ext(getDeviceType()));
+	if (isRecordingEnabled()) {
+		m_surface = cairo_ps_surface_create_for_stream(gle_cairo_device_write, (void*)this, 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
+	} else {
+		m_surface = cairo_ps_surface_create(m_OutputName.getFullPath().c_str(), 72*width/CM_PER_INCH+2, 72*height/CM_PER_INCH+2);
+	}
+	cairo_surface_set_fallback_resolution(m_surface, m_resolution, m_resolution);
+	cairo_ps_surface_set_eps(m_surface, true);
+	int int_bb_x = 0, int_bb_y = 0;
+	computeBoundingBox(width, height, &int_bb_x, &int_bb_y);
+	std::ostringstream bbLoRes;
+	std::ostringstream bbHiRes;
+	bbLoRes << "%%BoundingBox: 0 0 " << int_bb_x << " " << int_bb_y;
+	bbHiRes << "%%HiResBoundingBox: 0 0 " << m_boundingBox.getX() << " " << m_boundingBox.getY();
+	cairo_ps_surface_dsc_comment(m_surface, bbLoRes.str().c_str());
+	cairo_ps_surface_dsc_comment(m_surface, bbHiRes.str().c_str());
+	m_cr = cairo_create(m_surface);
+	g_scale(PS_POINTS_PER_INCH/CM_PER_INCH, PS_POINTS_PER_INCH/CM_PER_INCH);
+	if (!g_is_fullpage()) {
+		g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	}
+}
+
+void GLECairoDeviceEPS::getRecordedBytes(string* output) {
+	// work-around for bug in Cairo 1.10.x
+	int int_bb_x = 0, int_bb_y = 0;
+	computeBoundingBox(m_width, m_height, &int_bb_x, &int_bb_y);
+	std::ostringstream bbLoRes;
+	std::ostringstream bbHiRes;
+	bbLoRes << "%%BoundingBox: 0 0 " << int_bb_x << " " << int_bb_y;
+	bbHiRes << "%%HiResBoundingBox: 0 0 " << m_boundingBox.getX() << " " << m_boundingBox.getY();
+	// do conversion
+	std::stringstream inp;
+	std::ostringstream outp;
+	inp.write(&m_recorded[0], m_recorded.size());
+	while (inp.good()) {
+		string line;
+		getline(inp, line);
+		if (str_starts_with(line, "%%BoundingBox:")) {
+			outp << bbLoRes.str() << std::endl;
+		} else if (str_starts_with(line, "%%HiResBoundingBox:")) {
+			outp << bbHiRes.str() << std::endl;
+		} else {
+			outp << line << std::endl;
+		}
+	}
+	*output = outp.str();
+}
+
+int GLECairoDeviceEPS::getDeviceType() {
+	return GLE_DEVICE_CAIRO_EPS;
+}
+
 #ifdef __WIN32__
 
 #include <cairo-win32.h>
@@ -749,8 +1009,12 @@ void GLECairoDeviceEMF::set_matrix(double newmat[3][3]) {
     matrix.yx = - newmat[1][0];
     matrix.yy = - newmat[1][1];
     matrix.x0 = newmat[0][2];
-    matrix.y0 = m_height * m_DPI/CM_PER_INCH - newmat[1][2];
-    cairo_set_matrix(cr, &matrix);
+	double offset = 0.0;
+	if (!g_is_fullpage()) {
+		offset = 2.0 * CM_PER_INCH / 72;
+	}
+    matrix.y0 = (m_height + offset) * m_DPI/CM_PER_INCH - newmat[1][2];
+    cairo_set_matrix(m_cr, &matrix);
 }
 
 void GLECairoDeviceEMF::opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) {
@@ -758,8 +1022,8 @@ void GLECairoDeviceEMF::opendev(double width, double height, GLEFileLocation* ou
 	m_height = height;
 	m_OutputName.copy(outputfile);
 	m_OutputName.addExtension("emf");
-	double myWidth = width + 2.0/72.0*CM_PER_INCH;
-	double myHeight = height + 2.0/72.0*CM_PER_INCH;
+	double myWidth = width + 2.0/PS_POINTS_PER_INCH*CM_PER_INCH;
+	double myHeight = height + 2.0/PS_POINTS_PER_INCH*CM_PER_INCH;
 	HDC hdcRef = GetDC(NULL);
 	int iWidthMM = GetDeviceCaps(hdcRef, HORZSIZE);    // iWidthMM is the display width in millimeters.
 	int iHeightMM = GetDeviceCaps(hdcRef, VERTSIZE);   // iHeightMM is the display height in millimeters.
@@ -797,10 +1061,14 @@ void GLECairoDeviceEMF::opendev(double width, double height, GLEFileLocation* ou
 	SetWindowOrgEx(m_WinInfo->hdc, 0, 0, NULL);
 	SetWindowExtEx(m_WinInfo->hdc, windowWidth, windowHeight, NULL);
 	SetViewportExtEx(m_WinInfo->hdc, viewportWidth, viewportHeight, NULL);
-	surface = cairo_win32_printing_surface_create(m_WinInfo->hdc);
-	cr = cairo_create(surface);
+	m_surface = cairo_win32_printing_surface_create(m_WinInfo->hdc);
+	cairo_surface_set_fallback_resolution(m_surface, m_resolution, m_resolution);
+	m_cr = cairo_create(m_surface);
+	computeBoundingBox(width, height);
 	g_scale(m_DPI/CM_PER_INCH, m_DPI/CM_PER_INCH);
-	g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	if (!g_is_fullpage()) {
+		g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
+	}
 }
 
 BOOL PutEnhMetafileOnClipboard(HWND hWnd, HENHMETAFILE hEMF) {
