@@ -45,9 +45,13 @@
 class GLEBitmap;
 class GLEPoint;
 
+int setMaxPSVector(int newMax);
+
 class GLEDevice {
 protected:
-	bool m_Recording;
+	bool m_recording;
+	GLEPoint m_boundingBox;
+	double m_resolution;
 public:
 	GLEDevice();
 	virtual ~GLEDevice();
@@ -75,7 +79,7 @@ public:
 	virtual void fill_ary(int nwk,double *wkx,double *wky) = 0;
 	virtual void flush(void) = 0;
 	// virtual void get_line_cap(int *i) = 0;
-	virtual void get_type(char *t) = 0;
+	virtual std::string get_type() = 0;
 	virtual void line(double zx,double zy) = 0;
 	virtual void line_ary(int nwk,double *wkx,double *wky) = 0;
 	virtual void message(char *s) = 0;
@@ -85,9 +89,8 @@ public:
 	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) = 0;
 	virtual void pscomment(char* ss) = 0;
 	virtual void reverse(void)    /* reverse the order of stuff in the current path */ = 0;
-	virtual void set_color(int f) = 0;
-	virtual void set_fill(int f) = 0;
-	virtual void set_pattern_color(int c) = 0;
+	virtual void set_color(const GLERC<GLEColor>& color) = 0;
+	virtual void set_fill(const GLERC<GLEColor>& fill) = 0;
 	virtual void set_line_cap(int i) = 0;
 	virtual void set_line_join(int i) = 0;
 	virtual void set_line_miterlimit(double d) = 0;
@@ -100,19 +103,20 @@ public:
 	virtual void stroke(void) = 0;
 	virtual void set_color(void) = 0;
 	virtual void set_fill(void) = 0;
-	virtual void set_background(int b);
 	virtual void set_fill_method(int m);
 	virtual void xdbox(double x1, double y1, double x2, double y2) = 0;
 	virtual void devcmd(const char *s) = 0;
 	virtual FILE* get_file_pointer(void) = 0;
 	virtual int getDeviceType() = 0;
 	virtual void bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int type);
-	virtual const char* getExtension();
 	virtual void getRecordedBytes(string* output);
-	void writeRecordedOutputFile(const string& fname, string* buffer = 0) throw (ParserError);
+	void computeBoundingBox(double width, double height, int* int_bb_x, int* int_bb_y);
+	void computeBoundingBox(double width, double height);
+	inline GLEPoint* getBoundingBox() { return &m_boundingBox; }
+	void setResolution(double resolution) { m_resolution = resolution; }
 public:
-	inline void setRecordingEnabled(bool rec) { m_Recording = rec; }
-	inline bool isRecordingEnabled() { return m_Recording; }
+	inline void setRecordingEnabled(bool rec) { m_recording = rec; }
+	inline bool isRecordingEnabled() { return m_recording; }
 };
 
 class PSGLEDevice : public GLEDevice {
@@ -129,9 +133,8 @@ protected:
 	FILE *psfile;
 	int i,l,j;
 	int m_FillMethod;
-	int m_Background;
-	colortyp g_cur_fill, g_cur_color, g_cur_fill_color;
-	GLEPoint m_BBox;
+	GLERC<GLEColor> m_currentColor;
+	GLERC<GLEColor> m_currentFill;
 public:
 	PSGLEDevice(bool eps);
 	virtual ~PSGLEDevice();
@@ -159,7 +162,7 @@ public:
 	virtual void fill_ary(int nwk,double *wkx,double *wky);
 	virtual void flush(void);
 	// virtual void get_line_cap(int *i);
-	virtual void get_type(char *t);
+	virtual std::string get_type();
 	virtual void line(double zx,double zy);
 	virtual void line_ary(int nwk,double *wkx,double *wky);
 	virtual void message(char *s);
@@ -169,9 +172,8 @@ public:
 	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
 	virtual void pscomment(char* ss);
 	virtual void reverse(void)    /* reverse the order of stuff in the current path */;
-	virtual void set_color(int f);
-	virtual void set_fill(int f);
-	virtual void set_pattern_color(int c);
+	virtual void set_color(const GLERC<GLEColor>& color);
+	virtual void set_fill(const GLERC<GLEColor>& fill);
 	virtual void set_line_cap(int i);
 	virtual void set_line_join(int i);
 	virtual void set_line_miterlimit(double d);
@@ -184,17 +186,15 @@ public:
 	virtual void stroke(void);
 	virtual void set_color(void);
 	virtual void set_fill(void);
-	virtual void set_background(int b);
 	virtual void set_fill_method(int m);
 	virtual void xdbox(double x1, double y1, double x2, double y2);
 	virtual void devcmd(const char *s);
 	virtual FILE* get_file_pointer(void);
 	virtual int getDeviceType();
 	virtual void bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int type);
-	virtual const char* getExtension();
 	virtual void getRecordedBytes(string* output);
 protected:
-	void set_color(colortyp& color);
+	void set_color_impl(const GLERC<GLEColor>& color);
 	void ddfill(GLERectangle* bounds = NULL);
 	void shadeGLE();
 	void shadeBoundedIfThenElse1(GLERectangle* bounds, double step1);
@@ -209,91 +209,11 @@ public:
 	inline bool isEps() { return m_IsEps; }
 	inline bool isOutputPageSize() { return m_IsPageSize; }
 	inline void setOutputPageSize(bool out) { m_IsPageSize = out; }
-	inline GLEPoint* getBoundingBox() { return &m_BBox; }
 	inline ostream& out() { return *m_Out; }
 	void startRecording();
 	void initialPS();
 };
 
-class SVGGLEDevice : public GLEDevice {
-protected:
-	GLEFileLocation m_OutputName;
-	int ps_nvec;
-	vector<string> comments;
-	double linewidth;
-	string linecap;
-	string linejoin;
-	string miterlimit;
-	double Width;
-	double Height;
-	FILE *SVGFile;
-	int i,l,j;
-	colortyp g_cur_fill, g_cur_color;
-public:
-	SVGGLEDevice();
-	virtual ~SVGGLEDevice();
-	virtual void arc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy);
-	virtual void arcto(dbl x1,dbl y1,dbl x2,dbl y2,dbl rrr);
-	virtual void beginclip(void) ;
-	virtual void bezier(dbl x1,dbl y1,dbl x2,dbl y2,dbl x3,dbl y3);
-	virtual void box_fill(dbl x1, dbl y1, dbl x2, dbl y2);
-	virtual void box_stroke(dbl x1, dbl y1, dbl x2, dbl y2, bool reverse);
-	virtual void dochar(int font, int cc);
-	virtual void resetfont();
-	virtual void circle_fill(double zr);
-	virtual void circle_stroke(double zr);
-	virtual void clear(void);
-	virtual void clip(void);
-	virtual void closedev(void);
-	virtual void closepath(void);
-	virtual void dfont(char *c);
-	virtual void ellipse_fill(double rx, double ry);
-	virtual void ellipse_stroke(double rx, double ry);
-	virtual void elliptical_arc(double rx,double ry,double t1,double t2,double cx,double cy);
-	virtual void elliptical_narc(double rx,double ry,double t1,double t2,double cx,double cy);
-	virtual void endclip(void) ;
-	virtual void fill(void);
-	virtual void fill_ary(int nwk,double *wkx,double *wky);
-	virtual void flush(void);
-	// virtual void get_line_cap(int *i);
-	virtual void get_type(char *t);
-	virtual void line(double zx,double zy);
-	virtual void line_ary(int nwk,double *wkx,double *wky);
-	virtual void message(char *s);
-	virtual void move(double zx,double zy);
-	virtual void narc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy);
-	virtual void newpath(void);
-	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
-	virtual void pscomment(char* ss);
-	virtual void reverse(void)    /* reverse the order of stuff in the current path */;
-	virtual void set_color(int f);
-	virtual void set_fill(int f);
-	virtual void set_pattern_color(int c);
-	virtual void set_line_cap(int i);
-	virtual void set_line_join(int i);
-	virtual void set_line_miterlimit(double d);
-	virtual void set_line_style(const char *s);
-	virtual void set_line_styled(double dd);
-	virtual void set_line_width(double w);
-	virtual void set_matrix(double newmat[3][3]);
-	virtual void set_path(int onoff);
-	virtual void source(const char *s);
-	virtual void stroke(void);
-	virtual void set_color(void);
-	virtual void set_fill(void);
-	virtual void xdbox(double x1, double y1, double x2, double y2);
-	virtual void devcmd(const char *s);
-	virtual FILE* get_file_pointer(void);
-	virtual int getDeviceType();
-protected:
-	void ddfill(void);
-	void shade(void);
-	void test_SVGFile();
-	void reapsfont(void);
-	string GetColor();
-	inline double AY(double y) { return Height-y; }
-};
-
 #ifdef HAVE_CAIRO
 
 #include <cairo.h>
@@ -304,11 +224,14 @@ protected:
 	bool m_ShowError;
 	double m_width;
 	double m_height;
-	cairo_surface_t *surface;
-	cairo_t *cr;
+	cairo_surface_t *m_surface;
+	cairo_t *m_cr;
 	int m_FillMethod;
-	int m_Background;
-	colortyp g_cur_fill, g_cur_color, g_cur_fill_color;
+	GLERC<GLEColor> m_currentColor;
+	GLERC<GLEColor> m_currentFill;
+	std::vector<char> m_recorded;
+	StringVoidPtrHash m_bitmapCache;
+	std::vector<cairo_surface_t*> m_surfacesToDelete;
 public:
 	GLECairoDevice(bool showerror);
 	virtual ~GLECairoDevice();
@@ -335,7 +258,7 @@ public:
 	virtual void fill(void);
 	virtual void fill_ary(int nwk,double *wkx,double *wky);
 	virtual void flush(void);
-	virtual void get_type(char *t);
+	virtual std::string get_type();
 	virtual void line(double zx,double zy);
 	virtual void line_ary(int nwk,double *wkx,double *wky);
 	virtual void message(char *s);
@@ -345,11 +268,9 @@ public:
 	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
 	virtual void pscomment(char* ss);
 	virtual void reverse(void)    /* reverse the order of stuff in the current path */;
-	virtual void set_color(int f);
-	virtual void set_fill(int f);
-	virtual void set_background(int b);
+	virtual void set_color(const GLERC<GLEColor>& color);
+	virtual void set_fill(const GLERC<GLEColor>& fill);
 	virtual void set_fill_method(int m);
-	virtual void set_pattern_color(int c);
 	virtual void set_line_cap(int i);
 	virtual void set_line_join(int i);
 	virtual void set_line_miterlimit(double d);
@@ -366,8 +287,11 @@ public:
 	virtual void devcmd(const char *s);
 	virtual FILE* get_file_pointer(void);
 	virtual int getDeviceType();
+	virtual void bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int type);
+	virtual void getRecordedBytes(string* output);
+	void recordData(const unsigned char *data, unsigned int length);
 protected:
-	void set_color(colortyp& color);
+	void set_color_impl(const GLERC<GLEColor>& color);
 	void ddfill(GLERectangle* bounds = NULL);
 	void shadePattern();
 	void shadeGLE();
@@ -375,6 +299,9 @@ protected:
 	void shadeBoundedIfThenElse2(GLERectangle* bounds, double p, double step2);
 	void shadeBounded(GLERectangle* bounds);
 	void shade(GLERectangle* bounds);
+	void clearRecordedData();
+private:
+	cairo_surface_t* bitmapCreateSurface(GLEBitmap* bitmap);
 };
 
 class GLECairoDevicePDF : public GLECairoDevice {
@@ -385,6 +312,15 @@ public:
 	virtual int getDeviceType();
 };
 
+class GLECairoDeviceEPS : public GLECairoDevice {
+public:
+	GLECairoDeviceEPS(bool showerror);
+	virtual ~GLECairoDeviceEPS();
+	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
+	virtual int getDeviceType();
+	virtual void getRecordedBytes(string* output);
+};
+
 class GLECairoDeviceSVG : public GLECairoDevice {
 public:
 	GLECairoDeviceSVG(bool showerror);
@@ -446,7 +382,7 @@ public:
 	virtual void fill(void);
 	virtual void fill_ary(int nwk,double *wkx,double *wky);
 	virtual void flush(void);
-	virtual void get_type(char *t);
+	virtual std::string get_type();
 	virtual void line(double zx,double zy);
 	virtual void line_ary(int nwk,double *wkx,double *wky);
 	virtual void message(char *s);
@@ -456,9 +392,8 @@ public:
 	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
 	virtual void pscomment(char* ss);
 	virtual void reverse(void)    /* reverse the order of stuff in the current path */;
-	virtual void set_color(int f);
-	virtual void set_fill(int f);
-	virtual void set_pattern_color(int c);
+	virtual void set_color(const GLERC<GLEColor>& color);
+	virtual void set_fill(const GLERC<GLEColor>& fill);
 	virtual void set_line_cap(int i);
 	virtual void set_line_join(int i);
 	virtual void set_line_miterlimit(double d);
@@ -548,7 +483,7 @@ public:
 	virtual void fill_ary(int nwk,double *wkx,double *wky);
 	virtual void flush(void);
 	// virtual void get_line_cap(int *i);
-	virtual void get_type(char *t);
+	virtual std::string get_type();
 	virtual void line(double zx,double zy);
 	virtual void line_ary(int nwk,double *wkx,double *wky);
 	virtual void message(char *s);
@@ -558,9 +493,8 @@ public:
 	virtual void opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError);
 	virtual void pscomment(char* ss);
 	virtual void reverse(void)    /* reverse the order of stuff in the current path */;
-	virtual void set_color(int f);
-	virtual void set_fill(int f);
-	virtual void set_pattern_color(int c);
+	virtual void set_color(const GLERC<GLEColor>& color);
+	virtual void set_fill(const GLERC<GLEColor>& fill);
 	virtual void set_line_cap(int i);
 	virtual void set_line_join(int i);
 	virtual void set_line_miterlimit(double d);
