@@ -41,6 +41,7 @@
 /*--------------------------------------------------------------*/
 
 #include "all.h"
+#include "file_io.h"
 #include "bitmap/img2ps.h"
 #include "core.h"
 #include "d_interface.h"
@@ -49,7 +50,7 @@
 #include "op_def.h"
 #include "cmdline.h"
 #include "config.h"
-#include "file_io.h"
+#include "mem_limits.h"
 #ifdef __WIN32__
 #include <time.h>
 #endif
@@ -100,15 +101,20 @@ static char ellipse_fcn[] = "\
 //
 // -- global variables can we get rid of these?
 //
-extern int MAX_VECTOR; /* Cant send POSTSCRIPT too complex a path */
 extern bool control_d;
 extern struct gmodel g;
-extern bool BLACKANDWHITE;
 extern bool GS_PREVIEW;
 extern int gle_debug;
-
 extern ConfigCollection g_Config;
 
+int MAX_VECTOR = MAXIMUM_PS_VECTOR; /* Cant send POSTSCRIPT too complex a path */
+
+int setMaxPSVector(int newMax) {
+	int oldValue = MAX_VECTOR;
+	MAX_VECTOR = newMax;
+	return oldValue;
+}
+
 //
 // -- function prototypes
 //
@@ -123,12 +129,11 @@ PSGLEDevice::PSGLEDevice(bool eps) : GLEDevice() {
 	m_IsPageSize = false;
 	first_ellipse = 1;
 	ps_nvec = 0;
-	g_cur_fill_color.l = GLE_COLOR_BLACK;
 	m_Out = NULL;
 	m_OutputFile = NULL;
 	m_OutputBuffer = NULL;
 	m_FillMethod = GLE_FILL_METHOD_DEFAULT;
-	m_Background = GLE_COLOR_WHITE;
+	m_currentFill = g_get_fill_clear();
 }
 
 PSGLEDevice::~PSGLEDevice() {
@@ -150,9 +155,11 @@ void PSGLEDevice::source(const char *s) {
 	dbg out() << "%% SOURCE, " << s;
 }
 
-void PSGLEDevice::get_type(char *t) {
-	strcpy(t,"HARDCOPY, PS, FILLPATH");
-	if (isEps()) strcat(t,", EPS,");
+std::string PSGLEDevice::get_type() {
+	std::vector<std::string> temp(g_create_device_string());
+	temp.push_back("FILLPATH");
+	temp.push_back("POSTSCRIPT");
+	return str_join(temp);
 }
 
 void PSGLEDevice::set_path(int onoff) {
@@ -174,7 +181,7 @@ FILE* PSGLEDevice::get_file_pointer(void) {
 void PSGLEDevice::opendev(double width, double height, GLEFileLocation* outputfile, const string& inputfile) throw(ParserError) {
 	first_ellipse = 1;
 	m_OutputName.copy(outputfile);
-	m_OutputName.addExtension(getExtension());
+	m_OutputName.addExtension(g_device_to_ext(getDeviceType()));
 #ifdef ENABLE_GS_PREVIEW
 	setRecordingEnabled(true);
 #endif
@@ -204,24 +211,10 @@ void PSGLEDevice::opendev(double width, double height, GLEFileLocation* outputfi
 		out() << "%% " << comments[i] << endl;
 	}
 	comments.clear();
-	int int_bb_x = 0;
-	int int_bb_y = 0;
-	if (g_is_fullpage()) {
-		m_BBox.setX(72*width/CM_PER_INCH);
-		m_BBox.setY(72*height/CM_PER_INCH);
-		// Just round the bounding box to make sure papersize detection works in GhostView
-		int_bb_x = (int)floor(m_BBox.getX()+0.5);
-		int_bb_y = (int)floor(m_BBox.getY()+0.5);
-	} else {
-		// Make bounding box a little larger (bounding box tweak)
-		m_BBox.setX(72*width/CM_PER_INCH+2);
-		m_BBox.setY(72*height/CM_PER_INCH+2);
-		// Make sure approximate bounding box is large enough for ImageMagick and GhostView
-		int_bb_x = (int)ceil(m_BBox.getX()+1e-6);
-		int_bb_y = (int)ceil(m_BBox.getY()+1e-6);
-	}
+	int int_bb_x = 0, int_bb_y = 0;
+	computeBoundingBox(width, height, &int_bb_x, &int_bb_y);
 	out() << "%%BoundingBox: 0 0 " << int_bb_x << " " << int_bb_y << endl;
-	out() << "%%HiResBoundingBox: 0 0 " << m_BBox.getX() << " " << m_BBox.getY() << endl;
+	out() << "%%HiResBoundingBox: 0 0 " << m_boundingBox.getX() << " " << m_boundingBox.getY() << endl;
 	out() << "%%EndComments" << endl;
 	out() << "%%EndProlog" << endl;
 	if (isOutputPageSize()) {
@@ -241,7 +234,7 @@ void PSGLEDevice::initialPS() {
 	out() << "matrix currentmatrix /originmat exch def" << endl;
 	out() << "/umatrix {originmat matrix concatmatrix setmatrix} def" << endl;
 	// Measure distance in cm
-	g_scale(72.0/CM_PER_INCH,72.0/CM_PER_INCH);
+	g_scale(PS_POINTS_PER_INCH/CM_PER_INCH, PS_POINTS_PER_INCH/CM_PER_INCH);
 	if (!g_is_fullpage()) {
 		// Bounding box tweak
 		g_translate(1.0*CM_PER_INCH/72, 1.0*CM_PER_INCH/72);
@@ -362,8 +355,11 @@ void PSGLEDevice::fill() {
 }
 
 void PSGLEDevice::ddfill(GLERectangle* bounds) {
-	if (g_cur_fill.b[B_F] == 255) return; /* clear fill, do nothing */
-	if (g_cur_fill.b[B_F] == 2) {
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	if (cur_fill.b[B_F] == 255) {
+		return; /* clear fill, do nothing */
+	} else if (cur_fill.b[B_F] == 2) {
 		shade(bounds);
 		return;
 	}
@@ -373,8 +369,10 @@ void PSGLEDevice::ddfill(GLERectangle* bounds) {
 }
 
 void PSGLEDevice::shadeGLE() {
-	double step1 = g_cur_fill.b[B_B]/160.0;
-	double step2 = g_cur_fill.b[B_G]/160.0;
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	double step1 = cur_fill.b[B_B]/160.0;
+	double step2 = cur_fill.b[B_G]/160.0;
 	if (step1 > 0) {
 		out() << -40.0 << " " << step1 << " " << 40.0 << " { /x exch def" << endl;
 		out() << "x 0 moveto 40 x add 40 lineto stroke" << endl;
@@ -406,8 +404,10 @@ void PSGLEDevice::shadeBoundedIfThenElse2(GLERectangle* bounds, double step2) {
 }
 
 void PSGLEDevice::shadeBounded(GLERectangle* bounds) {
-	double step1 = g_cur_fill.b[B_B]/160.0;
-	double step2 = g_cur_fill.b[B_G]/160.0;
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	double step1 = cur_fill.b[B_B]/160.0;
+	double step2 = cur_fill.b[B_G]/160.0;
 	out() << "2 setlinecap" << endl;
 	if (step1 > 0) {
 		int p0 = (int)ceil((bounds->getYMax()-bounds->getXMin())/step1-1e-6);
@@ -452,8 +452,10 @@ void PSGLEDevice::shadeBounded(GLERectangle* bounds) {
 }
 
 void PSGLEDevice::shadePostScript() {
-	int step1 = g_cur_fill.b[B_B];
-	int step2 = g_cur_fill.b[B_G];
+	colortyp cur_fill;
+	cur_fill.l = m_currentFill->getHexValueGLE();
+	int step1 = cur_fill.b[B_B];
+	int step2 = cur_fill.b[B_G];
 	out() << "<< /PatternType 1" << endl;
 	out() << "/PaintType 1" << endl;
 	out() << "/TilingType 1" << endl;
@@ -466,22 +468,14 @@ void PSGLEDevice::shadePostScript() {
 	out() << "{ pop" << endl;
 	out() << "0 setlinecap" << endl;
 	out() << "0 setlinejoin" << endl;
-	if (m_Background != (int)GLE_FILL_CLEAR) {
-		if (m_Background == (int)GLE_COLOR_WHITE) {
-			out() << "1 setgray" << endl;
-		} else {
-			colortyp col;
-			col.l = m_Background;
-			set_color(col);
-		}
+	GLERC<GLEColor> background(get_fill_background(m_currentFill.get()));
+	if (!background->isTransparent()) {
+		set_color_impl(background);
 		out() << "-1 -1 " << (xstep+1) << " " << (ystep+1) << " rectfill" << endl;
 	}
-	if (g_cur_fill_color.l == GLE_COLOR_BLACK) {
-		out() << "0 setgray" << endl;
-	} else {
-		set_color(g_cur_fill_color);
-	}
-	out() << (int)g_cur_fill.b[B_R] << " setlinewidth" << endl;
+	GLERC<GLEColor> foreground(get_fill_foreground(m_currentFill.get()));
+	set_color_impl(foreground);
+	out() << (int)cur_fill.b[B_R] << " setlinewidth" << endl;
 	if (step1 > 0) {
 		out() << "0 0 moveto" << endl;
 		out() << xstep << " " << ystep << " l" << endl;
@@ -520,15 +514,10 @@ void PSGLEDevice::shadePostScript() {
 void PSGLEDevice::shade(GLERectangle* bounds) {
 	if (m_FillMethod == GLE_FILL_METHOD_GLE ||
 	   (m_FillMethod == GLE_FILL_METHOD_DEFAULT && bounds != NULL)) {
-		if (m_Background != (int)GLE_FILL_CLEAR) {
+		GLERC<GLEColor> background(get_fill_background(m_currentFill.get()));
+		if (!background->isTransparent()) {
 			out() << "gsave" << endl;
-			if (m_Background == (int)GLE_COLOR_WHITE) {
-				out() << "1 setgray" << endl;
-			} else {
-				colortyp col;
-				col.l = m_Background;
-				set_color(col);
-			}
+			set_color_impl(background);
 			out() << "fill" << endl;
 			out() << "grestore" << endl;
 		}
@@ -536,12 +525,11 @@ void PSGLEDevice::shade(GLERectangle* bounds) {
 		out() << "gsave" << endl;
 		out() << "clip" << endl;
 		out() << "newpath" << endl;
-		if (g_cur_fill_color.l == GLE_COLOR_BLACK) {
-			out() << "0 setgray" << endl;
-		} else {
-			set_color(g_cur_fill_color);
-		}
-		out() << (double)(g_cur_fill.b[B_R]/160.0) << " setlinewidth" << endl;
+		GLERC<GLEColor> foreground(get_fill_foreground(m_currentFill.get()));
+		set_color_impl(foreground);
+		colortyp cur_fill;
+		cur_fill.l = m_currentFill->getHexValueGLE();
+		out() << (double)(cur_fill.b[B_R]/160.0) << " setlinewidth" << endl;
 		if (m_FillMethod == GLE_FILL_METHOD_DEFAULT && bounds != NULL) {
 			shadeBounded(bounds);
 		} else {
@@ -616,7 +604,7 @@ void PSGLEDevice::line(double zx,double zy) {
 		move(g.curx,g.cury);
 	}
 	ps_nvec++;
-	if (ps_nvec>MAX_VECTOR) {
+	if (MAX_VECTOR != -1 && ps_nvec > MAX_VECTOR) {
 		//gprint("Warning, complex path, if filling fails then try /nomaxpath \n");
 		ps_nvec = 0; g_flush(); move(g.curx,g.cury);
 	}
@@ -809,43 +797,44 @@ void PSGLEDevice::bezier(dbl x1,dbl y1,dbl x2,dbl y2,dbl x3,dbl y3) {
 	g.xinline = true;
 }
 
-void PSGLEDevice::set_color(colortyp& color) {
-	if (BLACKANDWHITE) {
-		double gray = (color.b[B_R]*3.0/255.0+color.b[B_G]*2.0/255.0+color.b[B_B]/255.0) / 6;
-		out() << gray << " setgray" << endl;
+void PSGLEDevice::set_color_impl(const GLERC<GLEColor>& color) {
+	unsigned int hexValue = color->getHexValueGLE();
+	if (hexValue == (unsigned int)GLE_COLOR_WHITE) {
+		out() << "1 setgray" << endl;
+	} else if (hexValue == (unsigned int)GLE_COLOR_BLACK) {
+		out() << "0 setgray" << endl;
 	} else {
-		out() << color.b[B_R]/255.0 << " " << color.b[B_G]/255.0 << " " << color.b[B_B]/255.0 << " setrgbcolor" << endl;
+		if (color->getRed() == color->getGreen() && color->getRed() == color->getBlue()) {
+			out() << color->getRed() << " setgray" << endl;
+		} else {
+			out() << color->getRed() << " " << color->getGreen() << " " << color->getBlue() << " setrgbcolor" << endl;
+		}
+	}
+	if (color->hasAlpha()) {
+		g_throw_parser_error("semi-transparency only supported with command line option '-cairo'");
 	}
 }
 
 void PSGLEDevice::set_color() {
-	set_color(g_cur_color);
+	set_color_impl(m_currentColor);
 }
 
 void PSGLEDevice::set_fill() {
-	set_color(g_cur_fill);
-}
-
-void PSGLEDevice::set_background(int b) {
-	m_Background = b;
+	set_color_impl(m_currentFill);
 }
 
 void PSGLEDevice::set_fill_method(int m) {
 	m_FillMethod = m;
 }
 
-void PSGLEDevice::set_color(int f) {
+void PSGLEDevice::set_color(const GLERC<GLEColor>& color) {
 	g_flush();
-	g_cur_color.l = f;
+	m_currentColor = color;
 	set_color();
 }
 
-void PSGLEDevice::set_fill(int f) {
-	g_cur_fill.l = f;
-}
-
-void PSGLEDevice::set_pattern_color(int c) {
-	g_cur_fill_color.l = c;
+void PSGLEDevice::set_fill(const GLERC<GLEColor>& fill) {
+	m_currentFill = fill;
 }
 
 void PSGLEDevice::beginclip() {
@@ -855,10 +844,10 @@ void PSGLEDevice::beginclip() {
 void PSGLEDevice::endclip() {
 	g_flush();
 	out() << "grestore" << endl;
-	gmodel* state = (gmodel*) myallocz(SIZEOFSTATE);
+	gmodel* state = new gmodel();
 	g_get_state(state);
 	g_set_state(state);
-	myfree(state);
+	delete state;
 }
 
 struct psfont_struct {char *sname; char *lname;};
@@ -1002,7 +991,7 @@ void PSGLEDevice::read_psfont(void) {
 	static int init_done;
 	FILE *fptr;
 	char *s;
-	char inbuff[90];
+	char inbuff[200];
 	if (init_done) return;
 	init_done = true;
 
@@ -1013,15 +1002,20 @@ void PSGLEDevice::read_psfont(void) {
 	fptr = fopen(fname.c_str(), "r");
 	if (fptr == 0) return; /* if not exists then don't bother */
 
-	for (fgets(inbuff,200,fptr);!feof(fptr);fgets(inbuff,200,fptr)) {
-		s = strchr(inbuff,'!');
-		if (s!=NULL) *s=0;
-		s = strtok(inbuff," \t,\n");
-		if (s!=NULL) if (*s!='\n') {
-			psf[i].sname = sdup(s);
-			s = strtok(0," \t,\n");
-			psf[i].lname = sdup(s);
-			i++;
+	if (fgets(inbuff, 200, fptr) != 0) {
+		while (!feof(fptr)) {
+			s = strchr(inbuff,'!');
+			if (s!=NULL) *s=0;
+			s = strtok(inbuff," \t,\n");
+			if (s!=NULL) if (*s!='\n') {
+				psf[i].sname = sdup(s);
+				s = strtok(0," \t,\n");
+				psf[i].lname = sdup(s);
+				i++;
+			}
+			if (fgets(inbuff, 200, fptr) == 0) {
+				break;
+			}
 		}
 	}
 	psf[i].sname = NULL;
@@ -1108,10 +1102,6 @@ void PSGLEDevice::bitmap(GLEBitmap* bitmap, GLEPoint* pos, GLEPoint* scale, int
 	g_set_bounds(&save_box);
 }
 
-const char* PSGLEDevice::getExtension() {
-	return isEps() ? "eps" : "ps";
-}
-
 void PSGLEDevice::startRecording() {
 	if (m_OutputFile != NULL) {
 		delete m_OutputFile;
