@@ -93,6 +93,11 @@
 	#endif
 #endif
 
+/* PATH_MAX is not defined on gnu-hurd */
+#ifndef PATH_MAX
+       #define PATH_MAX 8192
+#endif
+
 using namespace std;
 
 #include "tokens/stokenizer.h"
@@ -762,8 +767,12 @@ int GLESystem(const string& cmd, bool wait, bool redirout, istream* ins, ostream
 	int* const pin = fd;
 	int* const pout = fd+2;
 	if (wait) {
-		pipe(pin);
-		pipe(pout);
+		if (pipe(pin) != 0) {
+			return GLE_SYSTEM_ERROR;
+		}
+		if (pipe(pout) != 0) {
+			return GLE_SYSTEM_ERROR;
+		}
 		fcntl(pin[GLESYS_PIPE_WR], F_SETFL, O_NONBLOCK);
 		fcntl(pout[GLESYS_PIPE_RD], F_SETFL, O_NONBLOCK);
 	}
@@ -1368,24 +1377,24 @@ bool GetExeName(const char* appname, char **argv, string& exe_name) {
 	}
 	/* try to read location from the /proc/self/maps file */
 	ifstream maps("/proc/self/maps");
-	if (!maps.is_open()) {
-		return false;
-	}
-	string f1 = DIR_SEP + appname;
-	string f2 = f1 + ".exe";
-	while (!maps.eof()) {
-		string line;
-		ReadFileLine(maps, line);
-		char_separator separator(" ", "");
-		tokenizer<char_separator> tokens(line, separator);
-		while (tokens.has_more()) {
-			exe_name = tokens.next_token();
-			if (str_i_ends_with(exe_name, f1.c_str()) || str_i_ends_with(exe_name, f2.c_str())) {
-				return true;
+	if (maps.is_open()) {
+		string f1 = DIR_SEP + appname;
+		string f2 = f1 + ".exe";
+		while (!maps.eof()) {
+			string line;
+			ReadFileLine(maps, line);
+			char_separator separator(" ", "");
+			tokenizer<char_separator> tokens(line, separator);
+			while (tokens.has_more()) {
+				exe_name = tokens.next_token();
+				if (str_i_ends_with(exe_name, f1.c_str())
+				    || str_i_ends_with(exe_name, f2.c_str())) {
+					return true;
+				}
 			}
 		}
+		maps.close();
 	}
-	maps.close();
 #endif
 	string arg0 = argv[0];
 	if (IsAbsPath(arg0)) {
@@ -1517,8 +1526,7 @@ string GLEExpandEnvironmentVariables(const string& str) {
 
 bool GLEReadFile(const string& name, vector<string>* lines) {
 	ifstream inFile(name.c_str());
-	if (!inFile.is_open())
-	{
+	if (!inFile.is_open()) {
 		return false;
 	}
 	while (inFile.good()) {
@@ -1530,3 +1538,131 @@ bool GLEReadFile(const string& name, vector<string>* lines) {
 	return true;
 }
 
+bool GLEReadFileBinary(const string& name, std::vector<char>* contents) {
+	ifstream inFile(name.c_str(), ios::in | ios::binary | ios::ate);
+	if (!inFile.is_open()) {
+		return false;
+	}
+	int length = inFile.tellg();
+	inFile.seekg(0, ios::beg);
+	contents->resize(length, 0);
+	inFile.read(&(*contents)[0], length);
+	inFile.close();
+	return true;
+}
+
+GLEFileIO::GLEFileIO():
+	m_file(0)
+{
+}
+
+GLEFileIO::~GLEFileIO()
+{
+}
+
+void GLEFileIO::open(const char* fname, const char* flags)
+{
+	m_fname = fname;
+	m_file = fopen(fname, flags);
+}
+
+bool GLEFileIO::isOpen() const
+{
+	return m_file != 0;
+}
+
+void GLEFileIO::close()
+{
+	if (m_file != 0) {
+		fclose(m_file);
+		m_file = 0;
+	}
+}
+
+void GLEFileIO::fread(void *ptr, size_t size, size_t nmemb)
+{
+	size_t nbRead = ::fread(ptr, size, nmemb, m_file);
+	if (nbRead != nmemb) {
+		std::cerr << "error reading from file '" << m_fname << "'";
+	}
+}
+
+void GLEFileIO::fwrite(const void *ptr, size_t size, size_t nmemb)
+{
+	size_t nbWritten = ::fwrite(ptr, size, nmemb, m_file);
+	if (nbWritten != nmemb) {
+		std::cerr << "error writing to file '" << m_fname << "'";
+	}
+}
+
+FILE* GLEFileIO::getFile()
+{
+	return m_file;
+}
+
+std::string GLEFileIO::getName() const
+{
+	return m_fname;
+}
+
+void GLEFileIO::setName(const std::string& name)
+{
+	m_fname = name;
+}
+
+int GLEFileIO::fgetc()
+{
+	return ::fgetc(m_file);
+}
+
+void GLEFileIO::fputc(int value)
+{
+	::fputc(value, m_file);
+}
+
+int GLEFileIO::feof()
+{
+	return ::feof(m_file);
+}
+
+long GLEFileIO::ftell()
+{
+	return ::ftell(m_file);
+}
+
+int GLEFileIO::fseek(long offset, int whence)
+{
+	return ::fseek(m_file, offset, whence);
+}
+
+void GLEFileIO::fgetcstr(char* s) {
+	int i = fgetc();
+	if (i != 0) {
+		fread(s, 1, i);
+		s[i] = 0;
+	}
+}
+
+void GLEFileIO::fsendstr(const char *s) {
+	if (s == NULL) {
+		::fputc(0, m_file);
+	} else {
+		::fputc(strlen(s), m_file);
+		fwrite(s, 1, strlen(s));
+	}
+}
+
+int GLEReadConsoleInteger()
+{
+	char* ptr = 0;
+	std::string line;
+	getline(std::cin, line);
+	int result = strtol(line.c_str(), &ptr, 10);
+	if (ptr == 0 || *ptr != 0) {
+		return 0;
+	} else {
+		return result;
+	}
+}
+
+
