@@ -48,7 +48,7 @@
 /* and give it the next unused index number			                 */
 
 /* ret = Type of returned value,  1=number, 2=string                 */
-/* np = number of paramters expected                                 */
+/* np = number of parameters expected                                 */
 /* Parameters p0...p4  ==  1=number, 2=string, 0=none (e.g. height()).*/
 
 // current max is 97 ==> FN_GETENV
@@ -104,6 +104,7 @@ struct keyw { const char *word; int index; int ret,np,p[5]; } keywfn[] = {
 	{ "HEIGHT",      9,             1,1, { 0,0,0,0,0 } },
 	{ "INT",         10,            1,1, { 1,0,0,0,0 } },
 	{ "ISNAME",      FN_ISNAME,     1,1, { 2,0,0,0,0 } },
+    { "JUSTIFY",     FN_JUSTIFY,    1,1, { 2,0,0,0,0 } },
 	{ "LEFT$",       11,            2,2, { 2,1,0,0,0 } },
 	{ "LEN",         12,            1,1, { 2,0,0,0,0 } },
 	{ "LOG",         13,            1,1, { 1,0,0,0,0 } },
@@ -112,6 +113,7 @@ struct keyw { const char *word; int index; int ret,np,p[5]; } keywfn[] = {
 	{ "MIN",         FN_MIN,        1,2, { 1,1,0,0,0 } },
 	{ "NARGS",       FN_NARGS,      1,0, { 0,0,0,0,0 } },
 	{ "NDATA",       FN_NDATA,      1,1, { 2,0,0,0,0 } },
+	{ "NDATASETS",   FN_NDATASETS,  1,0, { 0,0,0,0,0 } },
 	{ "NOT",         15,            1,1, { 1,0,0,0,0 } },
 	{ "NUM$",        16,            2,1, { 1,0,0,0,0 } },
 	{ "NUM1$",       17,            2,1, { 1,0,0,0,0 } },
@@ -125,6 +127,8 @@ struct keyw { const char *word; int index; int ret,np,p[5]; } keywfn[] = {
 	{ "PTY",         78,            1,1, { 2,0,0,0,0 } },
 	{ "RGB",         47,            1,3, { 1,1,1,0,0 } },
 	{ "RGB255",      80,            1,3, { 1,1,1,0,0 } },
+	{ "RGBA",        FN_RGBA,       1,4, { 1,1,1,1,0 } },
+	{ "RGBA255",     FN_RGBA255,    1,4, { 1,1,1,1,0 } },
 	{ "RIGHT$",      21,            2,2, { 2,1,0,0,0 } },
 	{ "RND",         22,            1,1, { 1,0,0,0,0 } },
 	{ "SDIV",        FN_SDIV,       1,2, { 1,1,0,0,0 } },
@@ -149,6 +153,7 @@ struct keyw { const char *word; int index; int ret,np,p[5]; } keywfn[] = {
 	{ "XBAR",        FN_XBAR,       1,2, { 1,1,0,0,0 } },
 	{ "XEND",        35,            1,0, { 0,0,0,0,0 } },
 	{ "XG",          36,            1,1, { 1,0,0,0,0 } },
+	{ "XG3D",        FN_XG3D,       1,3, { 1,1,1,0,0 } },
 	{ "XGRAPH",      36,            1,1, { 1,0,0,0,0 } },
 	{ "XMAX",        37,            1,1, { 0,0,0,0,0 } },
 	{ "XMIN",        38,            1,1, { 0,0,0,0,0 } },
@@ -156,6 +161,7 @@ struct keyw { const char *word; int index; int ret,np,p[5]; } keywfn[] = {
 	{ "XY2ANGLE",    FN_XY2ANGLE,   1,2, { 1,1,0,0,0 } },
 	{ "YEND",        40,            1,0, { 0,0,0,0,0 } },
 	{ "YG",          41,            1,1, { 1,0,0,0,0 } },
+	{ "YG3D",        FN_YG3D,       1,3, { 1,1,1,0,0 } },
 	{ "YGRAPH",      41,            1,1, { 1,0,0,0,0 } },
 	{ "YMAX",        42,            1,1, { 0,0,0,0,0 } },
 	{ "YMIN",        43,            1,1, { 0,0,0,0,0 } },
