@@ -41,6 +41,36 @@
 #include "var.h"
 #include "core.h"
 
+const char* gle_object_type_to_string(GLEObjectType type) {
+	switch (type) {
+	case GLEObjectTypeBool:
+		return "bool";
+	case GLEObjectTypeInt:
+		return "integer";
+	case GLEObjectTypeDouble:
+		return "double";
+	case GLEObjectTypeString:
+		return "string";
+	case GLEObjectTypeArray:
+		return "array";
+	case GLEObjectTypeColor:
+		return "color";
+	case GLEObjectTypeDynamicSub:
+		return "subroutine";
+	case GLEObjectTypeObjectRep:
+		return "object";
+	case GLEObjectTypePoint:
+		return "point";
+	case GLEObjectTypeClassDefinition:
+		return "definition";
+	case GLEObjectTypeClassInstance:
+		return "instance";
+	case GLEObjectTypeUnknown:
+		return "unknown";
+	}
+	return "unknown";
+}
+
 bool gle_memory_cell_equals(GLEMemoryCell* a, GLEMemoryCell* b) {
 	if (a->Type != b->Type) return false;
 	switch (a->Type) {
@@ -95,17 +125,39 @@ bool gle_memory_cell_to_double(GLEMemoryCell* a, double* result) {
 	}
 }
 
+int gle_memory_cell_type(GLEMemoryCell* a) {
+	switch (a->Type) {
+		case GLE_MC_UNKNOWN: return GLEObjectTypeUnknown;
+		case GLE_MC_BOOL:    return GLEObjectTypeBool;
+		case GLE_MC_INT:     return GLEObjectTypeInt;
+		case GLE_MC_DOUBLE:  return GLEObjectTypeDouble;
+		case GLE_MC_OBJECT:  return a->Entry.ObjectVal->getType();
+	}
+	return GLEObjectTypeUnknown;
+}
+
+void gle_memory_cell_check(GLEMemoryCell* a, int expected) {
+	int cellType = gle_memory_cell_type(a);
+	if (cellType != expected) {
+		std::ostringstream msg;
+		msg << "found type '" << gle_object_type_to_string((GLEObjectType)cellType) << "' (value = '";
+		gle_memory_cell_print(a, msg);
+		msg << "') but expected '" << gle_object_type_to_string((GLEObjectType)expected) << "'";
+		g_throw_parser_error(msg.str());
+	}
+}
+
 GLEDataObject::GLEDataObject() {
 }
 
 GLEDataObject::~GLEDataObject() {
 }
 
-int GLEDataObject::getType() {
+int GLEDataObject::getType() const {
 	return GLEObjectTypeUnknown;
 }
 
-bool GLEDataObject::equals(GLEDataObject* obj) {
+bool GLEDataObject::equals(GLEDataObject* obj) const {
 	return false;
 }
 
@@ -118,11 +170,11 @@ GLEPointDataObject::GLEPointDataObject(double x, double y) : m_point(x, y) {
 GLEPointDataObject::~GLEPointDataObject() {
 }
 
-int GLEPointDataObject::getType() {
+int GLEPointDataObject::getType() const {
 	return GLEObjectTypePoint;
 }
 
-bool GLEPointDataObject::equals(GLEDataObject* obj) {
+bool GLEPointDataObject::equals(GLEDataObject* obj) const {
 	if (obj->getType() != GLEObjectTypePoint) return false;
 	GLEPointDataObject* other = (GLEPointDataObject*)obj;
 	return m_point.getX() == other->m_point.getX()
@@ -164,7 +216,9 @@ GLEString::~GLEString() {
 }
 
 void GLEString::fromUTF8(const char* str) {
-	fromUTF8(str, strlen(str));
+	if (str != 0) {
+		fromUTF8(str, strlen(str));
+	}
 }
 
 void GLEString::fromUTF8(const string& str) {
@@ -420,7 +474,36 @@ bool GLEString::equalsI(const char* str) {
 	return true;
 }
 
-bool GLEString::equals(GLEDataObject* obj) {
+bool GLEString::equalsI(GLEString* other) {
+	if (m_Length != other->m_Length) return false;
+	for (unsigned int i = 0; i < m_Length; ++i) {
+		if (getI(i) != other->getI(i)) {
+			return false;
+		}
+	}
+	return true;
+}
+
+bool GLEString::containsI(unsigned int ch) {
+	for (unsigned int i = 0; i < m_Length; ++i) {
+		if (getI(i) == ch) {
+			return true;
+		}
+	}
+	return false;
+}
+
+int GLEString::strICmp(GLEString* other) const {
+	unsigned int s1 = 0;
+	unsigned int s2 = 0;
+	while (true) {
+		int c1 = s1 < m_Length ? getI(s1++) : 0;
+		int c2 = s2 < other->m_Length ? other->getI(s2++) : 0;
+		if (c1 == 0 || c1 != c2) return c1 - c2;
+	}
+}
+
+bool GLEString::equals(GLEDataObject* obj) const {
 	if (obj->getType() != GLEObjectTypeString) return false;
 	GLEString* other = (GLEString*)obj;
 	if (m_Length != other->length()) return false;
@@ -430,6 +513,29 @@ bool GLEString::equals(GLEDataObject* obj) {
 	return true;
 }
 
+GLEString* GLEString::concat(GLEString* other) const {
+	GLEString* res = new GLEString();
+	int size = m_Length + other->m_Length;
+	res->resize(size);
+	res->m_Length = size;
+	unsigned int idx = 0;
+	for (unsigned int i = 0; i < m_Length; ++i) {
+		res->set(idx++, get(i));
+	}
+	for (unsigned int i = 0; i < other->m_Length; ++i) {
+		res->set(idx++, other->get(i));
+	}
+	return res;
+}
+
+GLEString* GLEString::substringWithLength(unsigned int from, unsigned int size) const {
+	if (size == 0) {
+		return new GLEString();
+	} else {
+		return substring(from, from + size - 1);
+	}
+}
+
 GLEString* GLEString::substring(unsigned int from, unsigned int to) const {
 	if (m_Length == 0) {
 		return new GLEString();
@@ -451,6 +557,30 @@ GLEString* GLEString::substring(unsigned int from, unsigned int to) const {
 	}
 }
 
+int GLEString::find(GLEString* needle, unsigned int from) {
+	int endPattern = (int)m_Length - needle->length() + 1;
+	if (endPattern < 0) {
+		return -1;
+	}
+	if (needle->length() <= 0) {
+		return 0;
+	}
+	unsigned int patternChar0 = needle->getI(0);
+	for (int ctrSrc = from; ctrSrc <= endPattern; ctrSrc++) {
+		if (getI(ctrSrc) != patternChar0) {
+			continue;
+		}
+		int ctrPat;
+		for (ctrPat = 1; ctrPat < int(needle->length()) && getI(ctrSrc + ctrPat) == needle->getI(ctrPat); ctrPat++) {
+			; // just loop
+		}
+		if (ctrPat == int(needle->length())) {
+			return ctrSrc;
+		}
+	}
+	return -1;
+}
+
 GLEArrayImpl* GLEString::split(char bych) const {
 	GLEArrayImpl* res = new GLEArrayImpl();
 	unsigned int pos = 0;
@@ -525,7 +655,18 @@ void GLEString::resize(unsigned int size) {
 	}
 }
 
-int GLEString::getType() {
+unsigned int GLEString::toStringIndex(int value)
+{
+	if (value < 0) {
+		return (unsigned int)std::max<int>(0, (int)m_Length + value);
+	} else if (value > 0) {
+		return value - 1;
+	} else {
+		return 0;
+	}
+}
+
+int GLEString::getType() const {
 	return GLEObjectTypeString;
 }
 
@@ -552,7 +693,7 @@ GLEArrayImpl::~GLEArrayImpl() {
 
 void GLEArrayImpl::clear() {
 	if (m_Data != NULL) {
-		for (unsigned int i = 0; i < m_Length; i++) {
+		for (unsigned int i = 0; i < m_Alloc; i++) {
 			GLE_MC_DEL_INTERN(&m_Data[i]);
 		}
 		free(m_Data);
@@ -562,15 +703,12 @@ void GLEArrayImpl::clear() {
 	m_Alloc = 0;
 }
 
-int GLEArrayImpl::getType(unsigned int i) {
-	switch (m_Data[i].Type) {
-		case GLE_MC_UNKNOWN: return GLEObjectTypeUnknown;
-		case GLE_MC_BOOL:    return GLEObjectTypeBool;
-		case GLE_MC_INT:     return GLEObjectTypeInt;
-		case GLE_MC_DOUBLE:  return GLEObjectTypeDouble;
-		case GLE_MC_OBJECT:  return m_Data[i].Entry.ObjectVal->getType();
-	}
-	return GLEObjectTypeUnknown;
+void GLEArrayImpl::checkType(unsigned int i, int expected) {
+	gle_memory_cell_check(get(i), expected);
+}
+
+int GLEArrayImpl::getType(unsigned int i) const {
+	return gle_memory_cell_type(get(i));
 }
 
 void GLEArrayImpl::init(unsigned int i) {
@@ -659,10 +797,7 @@ GLERC<GLEString> GLEArrayImpl::getString(unsigned int i) {
 
 void GLEArrayImpl::ensure(unsigned int size) {
 	if (m_Alloc < size) extend(size);
-	while (m_Length < size) {
-		GLE_MC_INIT(m_Data[m_Length]);
-		m_Length++;
-	}
+	m_Length = std::max<int>(m_Length, size);
 }
 
 void GLEArrayImpl::resize(unsigned int size) {
@@ -678,6 +813,9 @@ void GLEArrayImpl::resize(unsigned int size) {
 void GLEArrayImpl::resizeMemory(unsigned int size) {
 	if (m_Alloc < size) {
 		m_Data = (GLEMemoryCell*)realloc(m_Data, size*sizeof(GLEMemoryCell));
+		for (unsigned int i = m_Alloc; i < size; ++i) {
+			GLE_MC_INIT(m_Data[i]);
+		}
 		m_Alloc = size;
 	}
 }
@@ -702,10 +840,19 @@ void GLEArrayImpl::enumStrings(ostream& out) {
 	}
 }
 
-int GLEArrayImpl::getType() {
+int GLEArrayImpl::getType() const {
 	return GLEObjectTypeArray;
 }
 
+GLERC<GLEArrayImpl> doublesToArray(double* args, int nb) {
+	GLERC<GLEArrayImpl> result(new GLEArrayImpl());
+	result->ensure(nb);
+	for (int i = 0; i < nb; ++i) {
+		result->setDouble(i, args[i]);
+	}
+	return result;
+}
+
 GLEStringHash::GLEStringHash() : GLEArrayImpl() {
 }
 
@@ -756,7 +903,7 @@ GLEDynamicSub::~GLEDynamicSub() {
 	if (m_State != NULL) delete m_State;
 }
 
-int GLEDynamicSub::getType() {
+int GLEDynamicSub::getType() const {
 	return GLEObjectTypeDynamicSub;
 }
 
@@ -789,7 +936,7 @@ void GLEObjectRepresention::enableChildObjects() {
 	if (m_SubObjs.isNull()) m_SubObjs = new GLEStringHash();
 }
 
-int GLEObjectRepresention::getType() {
+int GLEObjectRepresention::getType() const {
 	return GLEObjectTypeObjectRep;
 }
 
@@ -846,11 +993,11 @@ void GLEClassDefinition::addField(const char* fieldName) {
 	m_FieldNames->addObject(new GLEString(fieldName));
 }
 
-int GLEClassDefinition::getType() {
+int GLEClassDefinition::getType() const {
 	return GLEObjectTypeClassDefinition;
 }
 
-bool GLEClassDefinition::equals(GLEDataObject* obj) {
+bool GLEClassDefinition::equals(GLEDataObject* obj) const {
 	return false;
 }
 
@@ -862,11 +1009,11 @@ GLEClassInstance::GLEClassInstance(GLEClassDefinition* definition):
 {
 }
 
-int GLEClassInstance::getType() {
+int GLEClassInstance::getType() const {
 	return GLEObjectTypeClassInstance;
 }
 
-bool GLEClassInstance::equals(GLEDataObject* obj) {
+bool GLEClassInstance::equals(GLEDataObject* obj) const {
 	return false;
 }
 
