@@ -195,7 +195,7 @@ bool GLEInterface::initializeGLE(const char* appname, int argc, char **argv) {
 		init_config(&g_Config);
 		init_option_args(g_CmdLine);
 		return do_load_config(appname, argv, g_CmdLine, g_Config);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
@@ -242,7 +242,7 @@ GLERC<GLEScript> GLEInterface::newGLEFile(const char* glecode, const char* tmpfi
 		}
 		file->trim(1);
 		script->getSource()->initFromMain();
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
@@ -256,7 +256,7 @@ GLERC<GLEScript> GLEInterface::loadGLEFile(const char* glefile) {
 		if (m_FileInfoMap != NULL) m_FileInfoMap->clear();
 		g_set_compatibility(GLE_COMPAT_MOST_RECENT);
 		return load_gle_code_sub(glefile, g_CmdLine);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
@@ -316,7 +316,7 @@ void GLEInterface::renderGLE(GLEScript* script, const char* outfile, int device,
 		if (isMakeDrawObjects()) script->clear();
 		load_one_file_sub(script, g_CmdLine, &exit_code);
 		m_Output->setExitCode(get_nb_errors());
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
@@ -401,7 +401,7 @@ void GLEInterface::commitChangesGLE(GLEScript* script) {
 		g_restore_device(old_device);
 		setMakeDrawObjects(false);
 		setCommitMode(false);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
@@ -632,16 +632,22 @@ string GLEInterface::getToolLocation(const char* name) {
 }
 
 string GLEInterface::getUserConfigLocation() {
+	string location;
 	#if defined(__UNIX__) || defined (__OS2__)
-		const char* home = getenv("HOME");
-		if (home != NULL && home[0] != 0) {
-			string home_str(home);
-			AddDirSep(home_str);
-			home_str += ".glerc";
-			return home_str;
+		GLEGetEnv("HOME", location);
+	#endif
+	#ifdef __WIN32__	
+		GLEGetEnv("APPDATA", location);
+		if (location != "") {
+			AddDirSep(location);
+			location += "gle-graphics.org";
 		}
 	#endif
-	return string("");
+	if (location != "") {
+		AddDirSep(location);
+		location += ".glerc";
+	}
+	return location;
 }
 
 string GLEInterface::getManualLocation() {
@@ -715,15 +721,16 @@ void GLEInterface::evalString(const char* str, GLEScript* script) {
 			sub_clear(false);
 			clear_run();
 			f_init();
-			var_def("PI", GLE_PI);
+			gle_set_constants();
 		}
 		GLEPolish polish;
 		polish.initTokenizer();
 		string value;
-		polish.eval_string(str, &value, true);
+		GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+		polish.evalString(stk.get(), str, &value, true);
 		g_message_first_newline(false);
 		g_message(value);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		output_error(err);
 	}
 }
@@ -1041,7 +1048,7 @@ GLEScript* GLEObjectDOConstructor::getScript() {
 }
 
 void output_error(ParserError& err);
-void eval(int *pcode,int *cp,double *oval,const char **ostr,int *otyp) throw(ParserError);
+void eval(GLEArrayImpl* stk, GLEPcodeList* pclist, int *pcode, int *cp, double *oval, GLEString **ostr, int *otyp) throw(ParserError);
 
 GLEObjectDO* GLEObjectDOConstructor::constructObject() {
 	GLEObjectDO* obj = new GLEObjectDO(this);
@@ -1676,8 +1683,7 @@ void GLEObjectDO::render() {
 		GLEMeasureBox measure;
 		measure.measureStart();
 		g_move(0.0, 0.0);
-		double oval = 0.0;
-		int otyp = 0, cp = 0;
+		int cp = 0;
 		GLEPcodeList pc_list;
 		GLEPcode pcode(&pc_list);
 		pcode.addInt(PCODE_EXPR);    /* Expression follows */
@@ -1694,18 +1700,14 @@ void GLEObjectDO::render() {
 				} else {
 					GLEString* args = (GLEString*)arr->getObject(i);
 					args->toUTF8(argi);
-					try {
-						polish->polish(argi.c_str(), pcode, &rtype);
-					} catch (ParserError err) {
-						err.setParserString(argi);
-						throw err;
-					}
+					polish->polish(argi.c_str(), pcode, &rtype);
 				}
 			}
 		}
 		pcode.addFunction(sub->getIndex() + LOCAL_START_INDEX);
 		pcode.setInt(savelen, pcode.size() - savelen - 1);
-		eval((int*)&pcode[0], &cp, &oval, NULL, &otyp);
+		GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+		evalGeneric(stk.get(), &pc_list, (int*)&pcode[0], &cp);
 		// g_flush() required to make sure that all line segments are output
 		g_flush();
 		measure.measureEnd();
@@ -1717,7 +1719,7 @@ void GLEObjectDO::render() {
 		// Return the resulting PostScript code
 		dev->getRecordedBytes(getPostScriptPtr());
 		saved_state.restore();
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		newobj->getRectangle()->setXMin(-1.0);
 		output_error(err);
 	}
@@ -1739,7 +1741,7 @@ void GLEObjectDO::computeReferencePoint(GLEPoint* pt) {
 		} else {
 			pt->setXY(0, 0);
 		}
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		output_error(err);
 	}
 }
@@ -1885,7 +1887,7 @@ GLEColor* GLEColor::clone() {
 	return result;
 }
 
-int GLEColor::getType() {
+int GLEColor::getType() const {
 	return GLEObjectTypeColor;
 }
 
@@ -1897,7 +1899,7 @@ bool GLEColor::equalsApprox(GLEColor* other) {
 		   m_Transparent == other->m_Transparent;
 }
 
-bool GLEColor::equals(GLEDataObject* obj) {
+bool GLEColor::equals(GLEDataObject* obj) const {
 	if (obj->getType() != GLEObjectTypeColor) return false;
 	GLEColor* other = (GLEColor*)obj;
 	return m_Red == other->m_Red &&
@@ -1907,7 +1909,7 @@ bool GLEColor::equals(GLEDataObject* obj) {
 	       m_Transparent == other->m_Transparent;
 }
 
-void GLEColor::toString(ostream& out) {
+void GLEColor::print(ostream& out) const {
 	if (isTransparent()) {
 		out << "clear";
 	} else {
@@ -2126,7 +2128,7 @@ void GLEProperty::getPropertyAsString(string* result, GLEMemoryCell* value) {
 			break;
 		case GLEPropertyTypeColor:
 			color = (GLEColor*)value->Entry.ObjectVal;
-			color->toString(str);
+			color->print(str);
 			break;
 		case GLEPropertyTypeFont:
 			font = (GLEFont*)value->Entry.ObjectVal;
