@@ -88,6 +88,8 @@ using namespace std;
 #define GDO_FLAG_DELETED  1
 #define GDO_FLAG_MODIFIED 2
 
+typedef void (*gle_write_func)(void* closure, char* data, int length);
+
 class GLEInterface;
 class GLEScript;
 class GLEFont;
@@ -271,6 +273,8 @@ inline ostream& operator<<(ostream& os, const GLEPoint& pt) {
 	return pt.write(os);
 }
 
+#include "gle-shapemath.h"
+
 class DLLCLASS GLEPoint3D {
 public:
 	double m_C[3];
@@ -335,6 +339,18 @@ public:
 	inline double getB() const { return m_B; }
 };
 
+class DLLCLASS GLELineSegment {
+private:
+	GLEPoint m_p1;
+	GLEPoint m_p2;
+
+public:
+	GLELineSegment(const GLEPoint& p1, const GLEPoint& p2);
+	GLELineSegment(double x1, double y1, double x2, double y2);
+	const GLEPoint& getP1() const { return m_p1; }
+	const GLEPoint& getP2() const { return m_p2; }
+};
+
 #include "gle-datatype.h"
 
 enum GLEFontStyle {
@@ -348,6 +364,7 @@ protected:
 	GLERC<GLEFont> m_Bold;
 	GLERC<GLEFont> m_Italic;
 	GLERC<GLEFont> m_BoldItalic;
+	GLEFont* m_Parent;
 	int m_Index;
 	int m_Number;
 public:
@@ -361,45 +378,106 @@ public:
 	inline void setName(const string& name) { m_Name = name; }
 	inline void setFullName(const string& name) { m_FullName = name; }
 	inline bool hasStyle(GLEFontStyle style) { return getStyle(style) != NULL; }
+	DLLFCT GLEFontStyle checkStyle(GLEFont* child);
 	DLLFCT GLEFont* getStyle(GLEFontStyle style);
 	void setStyle(GLEFontStyle style, GLEFont* font);
 	inline int getIndex() { return m_Index; }
 	inline void setIndex(int value) { m_Index = value; }
 	inline int getNumber() { return m_Number; }
 	inline void setNumber(int value) { m_Number = value; }
+	inline GLEFont* getBaseFont() { return m_Parent != 0 ? m_Parent : this; }
+	inline void setParent(GLEFont* parent) { m_Parent = parent; }
+};
+
+enum GLEFillType {
+	GLE_FILL_TYPE_PATTERN
+};
+
+class GLEFillBase : public GLERefCountObject {
+public:
+	GLEFillBase();
+	virtual ~GLEFillBase();
+	virtual GLEFillType getFillType() = 0;
+	virtual GLEFillBase* clone() = 0;
+};
+
+class GLEPatternFill : public GLEFillBase {
+public:
+	GLEPatternFill(int fillDescr);
+	virtual ~GLEPatternFill();
+	virtual GLEFillType getFillType();
+	virtual GLEFillBase* clone();
+
+	inline void setFillDescription(int fillDescription) { m_fillDescription = fillDescription; }
+	inline int getFillDescription() const { return m_fillDescription; }
+	inline void setBackground(GLEColor* color) { m_background = color; }
+	inline GLEColor* getBackground() const { return m_background.get(); }
+
+private:
+	int m_fillDescription;
+	GLERC<GLEColor> m_background;
 };
 
 class DLLCLASS GLEColor : public GLEDataObject {
 protected:
 	bool m_Transparent;
-	double m_Red, m_Green, m_Blue;
+	double m_Red, m_Green, m_Blue, m_Alpha;
 	string* m_Name;
+	GLERC<GLEFillBase> m_Fill;
 public:
 	GLEColor();
 	GLEColor(double r, double g, double b);
+	GLEColor(double r, double g, double b, double a);
+	explicit GLEColor(double gray);
 	~GLEColor();
-	virtual int getType();
-	virtual bool equals(GLEDataObject* obj);
+	virtual int getType() const;
+	virtual bool equals(GLEDataObject* obj) const;
+	virtual void print(ostream& out) const;
+	bool equalsApprox(GLEColor* other);
 	DLLFCT void setRGB(double r, double g, double b);
+	DLLFCT void setRGBA(double r, double g, double b, double a);
 	DLLFCT void setRGB255(int r, int g, int b);
+    DLLFCT void setRGB255(double r, double g, double b);
+    void setRGBA255(double r, double g, double b, double a);
+	void setGray(double gray);
 	DLLFCT const char* getName();
 	void setName(const string& name);
+	void setName(const string* name);
 	void setHexValue(unsigned int v);
+	void setDoubleEncoding(double v);
+	double getDoubleEncoding();
+	void setHexValueGLE(unsigned int hexValue);
 	unsigned int getHexValueGLE();
-	inline double getRed() { return m_Red; }
-	inline double getGreen() { return m_Green; }
-	inline double getBlue() { return m_Blue; }
-	inline unsigned char getRedI() { return float_to_color_comp(m_Red); }
-	inline unsigned char getGreenI() { return float_to_color_comp(m_Green); }
-	inline unsigned char getBlueI() { return float_to_color_comp(m_Blue); }
+	double getGray();
+	GLEColor* clone();
+	inline double getRed() const { return m_Red; }
+	inline double getGreen() const { return m_Green; }
+	inline double getBlue() const { return m_Blue; }
+	inline double getAlpha() const { return m_Alpha; }
+	inline bool hasAlpha() const { return float_to_color_comp(m_Alpha) != 255; }
+	inline unsigned char getRedI() const { return float_to_color_comp(m_Red); }
+	inline unsigned char getGreenI() const { return float_to_color_comp(m_Green); }
+	inline unsigned char getBlueI() const { return float_to_color_comp(m_Blue); }
+	inline unsigned char getAlphaI() const { return float_to_color_comp(m_Alpha); }
 	inline void setRed(double v) { m_Red = v; }
 	inline void setGreen(double v) { m_Green = v; }
 	inline void setBlue(double v) { m_Blue = v; }
-	inline bool isTransparent() { return m_Transparent; }
+	inline bool isTransparent() const { return m_Transparent; }
 	inline void setTransparent(bool transp) { m_Transparent = transp; }
-	DLLFCT void toString(ostream& out);
+	inline bool isFill() const { return !m_Fill.isNull(); }
+	inline GLEFillBase* getFill() { return m_Fill.get(); }
+	inline void setFill(GLEFillBase* fill) { m_Fill = fill; }
+	inline std::string* getNameS() { return m_Name; }
 };
 
+GLERC<GLEColor> color_or_fill_from_int(int hexValue);
+void update_color_foreground(GLEColor* updateMe, GLEColor* color);
+void update_color_foreground_and_pattern(GLEColor* updateMe, GLEColor* color);
+void update_color_fill_pattern(GLEColor* updateMe, GLEPatternFill* fill);
+void update_color_fill_background(GLEColor* updateMe, GLEColor* color);
+GLERC<GLEColor> get_fill_background(GLEColor* fill);
+GLERC<GLEColor> get_fill_foreground(GLEColor* fill);
+
 // Each drawable object has a reference to a property store
 // To, e.g., get the line width of a given object, call:
 //
@@ -428,8 +506,8 @@ enum GLEPropertyType {
 enum GLEPropertyID {
 	GLEDOPropertyColor, GLEDOPropertyFillColor, GLEDOPropertyJustify,
 	GLEDOPropertyLineWidth, GLEDOPropertyLineStyle, GLEDOPropertyLineCap,
-	GLEDOPropertyFont, GLEDOPropertyFontStyle, GLEDOPropertyFontSize,
-	GLEDOPropertyArrowSize, GLEDOPropertyArrowAngle, GLEDOPropertyArrowStyle, GLEDOPropertyArrowTip,
+	GLEDOPropertyFont, GLEDOPropertyFontSize, GLEDOPropertyArrowSize,
+	GLEDOPropertyArrowAngle, GLEDOPropertyArrowStyle, GLEDOPropertyArrowTip,
 	GLEDOPropertyUserArg
 };
 
@@ -672,6 +750,7 @@ protected:
 	bool m_MakeDrawObjs;
 	bool m_CommitMode;
 	GLERCVector<GLEFont> m_Fonts;
+	GLERCVector<GLEFont> m_AllFonts;
 	StringIntHash* m_FontHash;
 	IntIntHash* m_FontIndexHash;
 
@@ -686,7 +765,7 @@ public:
 	~GLEInterface();
 
 	// Initialize GLE - call once before calling any of the other functions
-	DLLFCT bool initializeGLE(const char* appname, char **argv);
+	DLLFCT bool initializeGLE(const char* appname, int argc, char **argv);
 
 	// Returns the version of GLE
 	DLLFCT string getGLEVersion();
@@ -751,7 +830,7 @@ public:
 	// Return the font matching a given name
 	GLEFont* getFont(const char* name);
 
-	int getFontIndex(int font);
+	GLEFont* getFontIndex(int font);
 
 	// Return the property store model for a text string
 	inline GLEPropertyStoreModel* getTextPropertyStoreModel() { return m_TextModel.get(); }
@@ -780,6 +859,7 @@ public:
 	DLLFCT void clearAllCmdLine();
 	DLLFCT void setCmdLineOption(const char* name);
 	DLLFCT void setCmdLineOptionString(const char* name, const char* value, int arg = 0);
+	DLLFCT bool hasCmdLineOptionString(const char* name);
 
 	DLLFCT string getGhostScriptLocation();
 	DLLFCT string getToolLocation(const char* name);
@@ -804,6 +884,17 @@ public:
 
 	// Get information about 3D surface plots
 	DLLFCT surface_struct* getSurface();
+
+	DLLFCT void convertPDFToImage(char* pdfData,
+                                  int pdfLength,
+                                  double resolution,
+                                  int device,
+                                  int options,
+                                  gle_write_func writeFunc,
+                                  void* closure);
+                                  
+   DLLFCT bool readFileOrGZIPTxt(const char* name, std::string* result);
+
 public:
 	inline void setCommitMode(bool commit) { m_CommitMode = commit; }
 	inline bool isCommitMode() { return m_CommitMode; }
@@ -813,6 +904,7 @@ public:
 public:
 	// For GLE internal use only
 	void addFont(GLEFont* font);
+	void addSubFont(GLEFont* font);
 };
 
 class DLLCLASS GLEErrorMessage {
@@ -919,10 +1011,19 @@ public:
 	void getPropertyAsString(GLEPropertyID id, string* result) { m_Properties->getPropertyAsString(id, result); }
 };
 
-class GLELineDO : public GLEDrawObject {
+class GLEHasArrowBase {
+public:
+	GLEHasArrowBase();
+	inline GLEHasArrow getArrow() { return m_Arrow; }
+	inline void setArrow(GLEHasArrow arrow) { m_Arrow = arrow; }
+
+protected:
+	GLEHasArrow m_Arrow;
+};
+
+class GLELineDO : public GLEDrawObject, public GLEHasArrowBase {
 protected:
 	GLEPoint m_P1, m_P2;
-	GLEHasArrow m_Arrow;
 public:
 	GLELineDO();
 	GLELineDO(double x1, double y1, double x2, double y2);
@@ -933,8 +1034,6 @@ public:
 	inline GLEPoint& getP2() { return m_P2; }
 	inline void setP1(const GLEPoint& p1) { m_P1 = p1; }
 	inline void setP2(const GLEPoint& p2) { m_P2 = p2; }
-	inline GLEHasArrow getArrow() { return m_Arrow; }
-	inline void setArrow(GLEHasArrow arrow) { m_Arrow = arrow; }
 	virtual void initProperties(GLEInterface* iface);
 	virtual bool needsAMove(GLEPoint& pt);
 	virtual void createGLECode(string& code);
@@ -973,7 +1072,7 @@ public:
 	virtual void initProperties(GLEInterface* iface);
 };
 
-class GLEArcDO : public GLEEllipseDO {
+class GLEArcDO : public GLEEllipseDO, public GLEHasArrowBase {
 protected:
 	double m_Angle1, m_Angle2;
 public:
@@ -995,7 +1094,6 @@ public:
 	DLLFCT GLEPoint& getPoint1(GLEPoint& pt);
 	DLLFCT GLEPoint& getPoint2(GLEPoint& pt);
 	DLLFCT GLEPoint& getPointMid(GLEPoint& pt);
-	double getNormalizedAngle2();
 	DLLFCT void normalize();
 };
 
@@ -1119,7 +1217,8 @@ protected:
 	GLEGlobalSource m_File;
 	GLEPoint m_Size, m_BoundingBox, m_BoundingBoxOrigin;
 	GLERCVector<GLEDrawObject> m_NewObjs;
-	string m_PostScript;
+	string m_PostScriptCode;
+	string m_PDFCode;
 	int m_CurrObject;
 public:
 	GLEScript();
@@ -1141,7 +1240,6 @@ public:
 	inline const GLEPoint& getBoundingBox() { return m_BoundingBox; }
 	inline const GLEPoint& getBoundingBoxOrigin() { return m_BoundingBoxOrigin; }
 
-
 	inline void setSize(double wd, double hi) { m_Size.setXY(wd, hi); }
 	inline void setBoundingBox(double wd, double hi) { m_BoundingBox.setXY(wd, hi); }
 	inline void setBoundingBoxOrigin(double x, double y) { m_BoundingBoxOrigin.setXY(x, y); }
