@@ -134,7 +134,7 @@ void GLESourceFile::performUpdates() {
 	}
 	m_Code.clear();
 	/* copy back one by one */
-	int ins_pos = 0;
+	unsigned int ins_pos = 0;
 	#ifdef DEBUG_UPDATE
 		cout << "Performing updates: " << m_ToInsertIdx.size() << endl;
 	#endif
