@@ -49,20 +49,21 @@
 #include "config.h"
 #include "drawit.h"
 #include "gle-interface/gle-interface.h"
+#include "gle-poppler.h"
 
 #ifdef __WIN32__
 	#include <io.h>
 #endif
 
+#include <set>
+
 /*------------ GLOBAL VARIABLES --------------*/
 //
 // -- these globals are needed. they are called in other source files using extern...
 //
-int MAX_VECTOR=MAXIMUM_PS_VECTOR;
 int gle_debug;
 bool control_d = true;
 extern int trace_on;
-bool BLACKANDWHITE = false;
 bool IS_INSTALL = false;
 bool GS_PREVIEW = false;
 
@@ -102,6 +103,12 @@ void gle_cat_csv(vector<string>* files) {
 		string fname(files->at(i));
 		GLECSVData csvData;
 		csvData.read(fname);
+		GLECSVError* error = csvData.getError();
+		if (error->errorCode != GLECSVErrorNone) {
+			cout << "error: " << error->errorString << endl;
+		} else {
+			csvData.print(cout);
+		}
 	}
 }
 
@@ -111,6 +118,7 @@ int GLEMain(int argc, char **argv) {
 int main(int argc, char **argv) {
 #endif
 	try {
+      gle_glib_init(argc, argv);
 		g_init();
 
 		/* Init and process command line arguments */
@@ -174,7 +182,7 @@ int main(int argc, char **argv) {
 
 		/* Process optional arguments */
 		process_option_args(g_CmdLine, g_Options);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		ostringstream err_str;
 		err_str << "Error: ";
 		err.write(err_str);
@@ -284,30 +292,35 @@ void do_show_info() {
 	string version, bdate;
 	g_get_version(&version);
 	g_get_build_date(&bdate);
-	cout << "GLE version:   " << version << endl;
+	cout << "GLE version:             " << version << endl;
 	if (!bdate.empty()) {
-		cout << "Build date:    " << bdate << endl;
+		cout << "Build date:              " << bdate << endl;
 	}
-	cout << "GLE_TOP:       " << GLE_TOP_DIR << endl;
-	cout << "GLE_BIN:       " << GLE_BIN_DIR << endl;
+	cout << "GLE_TOP:                 " << GLE_TOP_DIR << endl;
+	cout << "GLE_BIN:                 " << GLE_BIN_DIR << endl;
 	/* Location of GhostScript */
 	string gs_dir;
 	ConfigSection* tools = g_Config.getSection(GLE_CONFIG_TOOLS);
 	const string& gs_cmd = ((CmdLineArgString*)tools->getOptionValue(GLE_TOOL_GHOSTSCRIPT_CMD))->getValue();
 	if (gs_cmd != "") {
-		cout << "GhostScript:   " << gs_cmd << endl;
+		cout << "GhostScript:             " << gs_cmd << endl;
 	}
 	CmdLineArgString* gslib = (CmdLineArgString*)tools->getOptionValue(GLE_TOOL_GHOSTSCRIPT_LIB);
 	if (!gslib->isDefault()) {
-		cout << "GS library:    " << gslib->getValue() << endl;
+		cout << "GS library:              " << gslib->getValue() << endl;
 	}
 	/* Bitmap import */
-	cout << "Bitmap import: " << g_bitmap_supported_types() << endl;
+	cout << "Bitmap import:           " << g_bitmap_supported_types() << endl;
 	/* Cairo support */
 #ifdef HAVE_CAIRO
-	cout << "Cairo support: Yes" << endl;
+	cout << "Cairo rendering support: Yes" << endl;
 #else
-	cout << "Cairo support: No" << endl;
+	cout << "Cairo rendering support: No" << endl;
+#endif
+#ifdef HAVE_POPPLER
+	cout << "Poppler PDF support:     Yes" << endl;
+#else
+	cout << "Poppler PDF support:     No" << endl;
 #endif
 	do_wait_for_enter_exit(0);
 }
@@ -341,10 +354,8 @@ void process_option_args(CmdLineObj& cmdline, GLEOptions& options) {
 	options.ASK_DEBUG = cmdline.hasOption(GLE_OPT_DEBUG);
 	control_d = !cmdline.hasOption(GLE_OPT_NO_CTRL_D);
 	if (cmdline.hasOption(GLE_OPT_NO_MAXPATH)) {
-		/* Huh - what does this do? */
-		MAX_VECTOR = 10*MAXIMUM_PS_VECTOR;
+		setMaxPSVector(-1);
 	}
-	BLACKANDWHITE = cmdline.hasOption(GLE_OPT_NO_COLOR);
 	if (cmdline.hasOption(GLE_OPT_BBTWEAK)) g_psbbtweak();
 	GS_PREVIEW = cmdline.hasOption(GLE_OPT_GSPREVIEW);
 	// .ps output implies full page
@@ -434,6 +445,9 @@ void init_option_args(CmdLineObj& cmdline) {
 	option = new CmdLineOption("nocolor", "bw");
 	option->setHelp("Forces grayscale output");
 	cmdline.addOption(option, GLE_OPT_NO_COLOR);
+	option = new CmdLineOption("inverse");
+	option->setHelp("Render black as white for using on dark backgrounds");
+	cmdline.addOption(option, GLE_OPT_INVERSE);
 	option = new CmdLineOption("transparent", "tr");
 	option->setHelp("Creates transparent output (with -d png)");
 	cmdline.addOption(option, GLE_OPT_TRANSPARENT);
@@ -625,7 +639,7 @@ bool has_pdflatex(CmdLineObj* cmdline) {
 	}
 }
 
-bool has_eps_based_device(CmdLineArgSet* device, CmdLineObj& cmdline) {
+bool has_eps_or_pdf_based_device(CmdLineArgSet* device, CmdLineObj& cmdline) {
 	if (cmdline.hasOption(GLE_OPT_TEX)) return true;
 	if (device->hasValue(GLE_DEVICE_EPS)) return true;
 	if (device->hasValue(GLE_DEVICE_PDF)) return true;
@@ -634,16 +648,6 @@ bool has_eps_based_device(CmdLineArgSet* device, CmdLineObj& cmdline) {
 	return false;
 }
 
-bool requires_tex_eps(CmdLineArgSet* device, CmdLineObj* cmdline) {
-	if (!cmdline->hasOption(GLE_OPT_CREATE_INC)) {
-		if (device->hasValue(GLE_DEVICE_EPS)) return true;
-		if (device->hasValue(GLE_DEVICE_PDF) && !has_pdflatex(cmdline)) return true;
-	}
-	if (device->hasValue(GLE_DEVICE_JPEG)) return true;
-	if (device->hasValue(GLE_DEVICE_PNG)) return true;
-	return false;
-}
-
 bool requires_tex(CmdLineArgSet* device, CmdLineObj* cmdline) {
 	if (!cmdline->hasOption(GLE_OPT_CREATE_INC)) {
 		if (device->hasValue(GLE_DEVICE_EPS)) return true;
@@ -736,6 +740,22 @@ void delete_temp_file(const string& file, const char* ext) {
 	}
 }
 
+void writeRecordedOutputFile(const string& fname, int deviceCode, string* buffer) throw (ParserError) {
+	string outf = fname;
+	outf.append(g_device_to_ext(deviceCode));
+	ofstream out(outf.c_str(), ios::out | ios::binary);
+	if (!out.is_open()) {
+		g_throw_parser_error("failed to create file '", outf.c_str(), "'");
+	}
+	out.write(buffer->data(), buffer->size());
+	out.close();
+}
+
+void writeRecordedOutputFile(const string& fname, int deviceCode, GLEScript* script) throw (ParserError) {
+	string* buffer = script->getRecordedBytesBuffer(deviceCode);
+	writeRecordedOutputFile(fname, deviceCode, buffer);
+}
+
 class GLELoadOneFileManager {
 protected:
 	GLEScript* m_Script;
@@ -744,24 +764,38 @@ protected:
 	GLEDevice* m_Device;
 	GLEFileLocation m_IncName;
 	bool m_HasTeXFile;
-	bool m_HasIncEPS;
-	bool m_HasIncPDF;
-	bool m_HasEPSFile;
 	bool m_HasTempDotDir;
 	bool m_HasTempFile;
+	std::set<int> m_hasGenerated;
+	std::set<int> m_hasFile;
+	std::set<int> m_hasIncFile;
 public:
 	GLELoadOneFileManager(GLEScript* script, CmdLineObj* cmdline, GLEFileLocation* outname);
 	~GLELoadOneFileManager();
 	void update_bounding_box();
-	bool process_one_file_eps() throw(ParserError);
+	void delete_previous_output(int deviceCode);
+	void create_cairo_eps() throw(ParserError);
+	bool process_one_file_tex() throw(ParserError);
 	void do_output_type(const char* type);
 	void cat_stdout(const char* ext);
 	void cat_stdout_and_del(const char* ext);
 	void create_latex_eps_ps_pdf() throw(ParserError);
+	void convert_eps_to_pdf_no_latex() throw(ParserError);
 	istream* get_eps_stream();
-	void write_eps() throw(ParserError);
+	bool hasGenerated(int deviceCode);
+	bool hasFile(int deviceCode);
+	bool hasIncFile(int deviceCode);
+	void setHasGenerated(int deviceCode, bool value);
+	void setHasFile(int deviceCode, bool value);
+	void setHasIncFile(int deviceCode, bool value);
+	void write_recorded_data(int deviceCode) throw(ParserError);
 	void clean_tex_temp_files();
-	void delete_original_eps();
+	void clean_inc_file(int deviceCode);
+	void delete_original_eps_pdf();
+	void delete_original_eps_pdf_impl(int device);
+	bool requires_tex_eps(CmdLineArgSet* device, CmdLineObj* cmdline);
+	bool requires_tex_pdf(CmdLineArgSet* device, CmdLineObj* cmdline);
+	bool has_cairo_pdf_based_device(CmdLineArgSet* device, CmdLineObj* cmdline);
 };
 
 GLELoadOneFileManager::GLELoadOneFileManager(GLEScript* script, CmdLineObj* cmdline, GLEFileLocation* outname) {
@@ -770,9 +804,6 @@ GLELoadOneFileManager::GLELoadOneFileManager(GLEScript* script, CmdLineObj* cmdl
 	m_OutName = outname;
 	m_Device = NULL;
 	m_HasTeXFile = false;
-	m_HasIncEPS = false;
-	m_HasIncPDF = false;
-	m_HasEPSFile = false;
 	m_HasTempDotDir = false;
 	m_HasTempFile = false;
 }
@@ -780,6 +811,8 @@ GLELoadOneFileManager::GLELoadOneFileManager(GLEScript* script, CmdLineObj* cmdl
 GLELoadOneFileManager::~GLELoadOneFileManager() {
 }
 
+
+
 void GLELoadOneFileManager::update_bounding_box() {
 	double size_x, size_y;
 	g_get_usersize(&size_x, &size_y);
@@ -789,9 +822,47 @@ void GLELoadOneFileManager::update_bounding_box() {
 	m_Script->setBoundingBoxOrigin(0.0, 0.0);
 }
 
-bool GLELoadOneFileManager::process_one_file_eps() throw(ParserError) {
+void GLELoadOneFileManager::delete_previous_output(int deviceCode) {
 	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
-	m_Device = g_select_device(GLE_DEVICE_EPS);
+	if (device->hasValue(deviceCode) && !m_OutName->isStdout()) {
+		/* Delete output so that QGLE does not display previous rendering if there are errors */
+		DeleteFileWithExt(m_OutName->getFullPath(), g_device_to_ext(deviceCode));
+	}
+}
+
+void GLELoadOneFileManager::create_cairo_eps() throw(ParserError) {
+	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
+	if (!hasGenerated(GLE_DEVICE_EPS) && device->hasValue(GLE_DEVICE_EPS)) {
+		setHasGenerated(GLE_DEVICE_EPS, true);
+		m_Device = g_select_device(GLE_DEVICE_CAIRO_EPS);
+		m_Device->setRecordingEnabled(true);
+		if (g_verbosity() > 0) {
+			cerr << endl;
+		}
+		DrawIt(m_Script, m_OutName, m_CmdLine);
+		m_Device->getRecordedBytes(m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS));
+	}
+}
+
+bool GLELoadOneFileManager::process_one_file_tex() throw(ParserError) {
+	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
+	delete_previous_output(GLE_DEVICE_EPS);
+	delete_previous_output(GLE_DEVICE_PDF);
+	if (m_CmdLine->hasOption(GLE_OPT_CAIRO)) {
+		if (has_cairo_pdf_based_device(device, m_CmdLine)) {
+			setHasGenerated(GLE_DEVICE_PDF, true);
+			m_Device = g_select_device(GLE_DEVICE_CAIRO_PDF);
+		} else {
+			setHasGenerated(GLE_DEVICE_EPS, true);
+			m_Device = g_select_device(GLE_DEVICE_CAIRO_EPS);
+		}
+	} else {
+		setHasGenerated(GLE_DEVICE_EPS, true);
+		m_Device = g_select_device(GLE_DEVICE_EPS);
+	}
+	if (m_CmdLine->hasOption(GLE_OPT_DPI)) {
+		m_Device->setResolution(((CmdLineArgInt*)m_CmdLine->getOption(GLE_OPT_DPI)->getArg(0))->getValue());
+	}
 	m_Device->setRecordingEnabled(true);
 	/* In some cases two passes are required to measure size of TeX objects */
 	int done = 1;
@@ -828,10 +899,14 @@ bool GLELoadOneFileManager::process_one_file_eps() throw(ParserError) {
 		iter++;
 	} while (done == 1);
 	/* get recorded PostScript code */
-	m_Device->getRecordedBytes(m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS));
+	if (hasGenerated(GLE_DEVICE_PDF)) {
+		m_Device->getRecordedBytes(m_Script->getRecordedBytesBuffer(GLE_DEVICE_PDF));
+	} else {
+		m_Device->getRecordedBytes(m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS));
+	}
 	update_bounding_box();
-	/* do TeX stuff */
 	interface->checkObjectDimensions();
+	create_cairo_eps();
 	if (m_CmdLine->hasOption(GLE_OPT_CREATE_INC)) {
 		interface->createInc(m_CmdLine->getStringValue(GLE_OPT_TEXINCPREF));
 	}
@@ -874,6 +949,48 @@ void GLELoadOneFileManager::cat_stdout_and_del(const char* ext) {
 	delete_temp_file(m_OutName->getFullPath(), ext);
 }
 
+void GLELoadOneFileManager::convert_eps_to_pdf_no_latex() throw(ParserError) {
+	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
+	if (device->hasValue(GLE_DEVICE_PDF) && !hasGenerated(GLE_DEVICE_PDF)) {
+		setHasFile(GLE_DEVICE_PDF, true);
+		int dpi = m_CmdLine->getIntValue(GLE_OPT_DPI);
+		create_pdf_file_ghostscript(m_OutName, dpi, m_Script);
+		do_output_type(".pdf");
+	}
+}
+
+bool GLELoadOneFileManager::requires_tex_eps(CmdLineArgSet* device, CmdLineObj* cmdline) {
+	if (!cmdline->hasOption(GLE_OPT_CREATE_INC)) {
+		if (device->hasValue(GLE_DEVICE_EPS)) return true;
+		if (device->hasValue(GLE_DEVICE_PDF) && !has_pdflatex(cmdline)) return true;
+	}
+	if (!hasGenerated(GLE_DEVICE_PDF)) {
+		if (device->hasValue(GLE_DEVICE_JPEG)) return true;
+		if (device->hasValue(GLE_DEVICE_PNG)) return true;
+	}
+	return false;
+}
+
+bool GLELoadOneFileManager::has_cairo_pdf_based_device(CmdLineArgSet* device, CmdLineObj* cmdline) {
+	if (m_CmdLine->hasOption(GLE_OPT_CAIRO)) {
+		if (device->hasValue(GLE_DEVICE_PDF)) return true;
+		if (device->hasValue(GLE_DEVICE_JPEG)) return true;
+		if (device->hasValue(GLE_DEVICE_PNG)) return true;
+	}
+	return false;
+}
+
+bool GLELoadOneFileManager::requires_tex_pdf(CmdLineArgSet* device, CmdLineObj* cmdline) {
+	if (!cmdline->hasOption(GLE_OPT_CREATE_INC)) {
+		if (device->hasValue(GLE_DEVICE_PDF)) return true;
+	}
+	if (hasGenerated(GLE_DEVICE_PDF)) {
+		if (device->hasValue(GLE_DEVICE_JPEG)) return true;
+		if (device->hasValue(GLE_DEVICE_PNG)) return true;
+	}
+	return false;
+}
+
 void GLELoadOneFileManager::create_latex_eps_ps_pdf() throw(ParserError) {
 	/* m_OutName has no path and no extension */
 	m_IncName.fromAbsolutePath(m_OutName->getFullPath() + "_inc");
@@ -886,20 +1003,24 @@ void GLELoadOneFileManager::create_latex_eps_ps_pdf() throw(ParserError) {
 	/* Create "_inc.eps" */
 	if (device->hasOnlyValue(GLE_DEVICE_PDF) && (has_pdftex || create_inc)) {
 		/* "_inc.eps" not required in this case */
-	} else {
-		m_HasIncEPS = true;
-		m_Device->writeRecordedOutputFile(m_IncName.getFullPath());
+	} else if (hasGenerated(GLE_DEVICE_EPS)) {
+		setHasIncFile(GLE_DEVICE_EPS, true);
+		writeRecordedOutputFile(m_IncName.getFullPath(), GLE_DEVICE_EPS, m_Script);
 	}
 	/* Create "_inc.pdf" */
-	if (device->hasValue(GLE_DEVICE_PDF) && (has_pdftex || create_inc)) {
-		m_HasIncPDF = true;
-		create_pdf_file_ghostscript(&m_IncName, dpi, m_Script);
-		do_output_type(".pdf");
+	if ((device->hasValue(GLE_DEVICE_PDF) || hasGenerated(GLE_DEVICE_PDF)) && (has_pdftex || create_inc)) {
+		setHasIncFile(GLE_DEVICE_PDF, true);
+		if (hasGenerated(GLE_DEVICE_PDF)) {
+			writeRecordedOutputFile(m_IncName.getFullPath(), GLE_DEVICE_PDF, m_Script);
+		} else {
+			create_pdf_file_ghostscript(&m_IncName, dpi, m_Script);
+			do_output_type(".pdf");
+		}
 	}
 	/* Exit if no EPS/PS/PDF required */
 	if (!requires_tex_eps(device, m_CmdLine) &&
-	    !device->hasValue(GLE_DEVICE_PS) &&
-	    !device->hasValue(GLE_DEVICE_PDF)) {
+		!requires_tex_pdf(device, m_CmdLine) &&
+	    !device->hasValue(GLE_DEVICE_PS)) {
 			return;
 	}
 	/* Create EPS/PS/PDF based on .TeX */
@@ -909,22 +1030,23 @@ void GLELoadOneFileManager::create_latex_eps_ps_pdf() throw(ParserError) {
 	GLEChDir(out_name_dir);
 	if (requires_tex_eps(device, m_CmdLine)) {
 		create_eps_file_latex_dvips(out_name_np, m_Script);
-		m_HasEPSFile = true;
+		writeRecordedOutputFile(m_OutName->getFullPath(), GLE_DEVICE_EPS, m_Script);
+		setHasFile(GLE_DEVICE_EPS, true);
+	}
+	if ((device->hasValue(GLE_DEVICE_PDF) && !create_inc) || requires_tex_pdf(device, m_CmdLine)) {
+		setHasFile(GLE_DEVICE_PDF, true);
+		if (has_pdftex) {
+			create_pdf_file_pdflatex(out_name_np, m_Script);
+		} else {
+			create_pdf_file_ghostscript(m_OutName, dpi, m_Script);
+			do_output_type(".pdf");
+		}
 	}
 	if (device->hasValue(GLE_DEVICE_PS)) {
 		create_ps_file_latex_dvips(out_name_np);
 		if (m_OutName->isStdout()) cat_stdout_and_del(".ps");
 		do_output_type(".ps");
 	}
-	if (device->hasValue(GLE_DEVICE_PDF) && !create_inc) {
-		if (has_pdftex) {
-			create_pdf_file_pdflatex(out_name_np);
-		} else {
-			create_pdf_file_ghostscript(m_OutName, dpi, m_Script);
-			do_output_type(".pdf");
-		}
-		if (m_OutName->isStdout()) cat_stdout_and_del(".pdf");
-	}
 	GLEChDir(m_Script->getLocation()->getDirectory());
 }
 
@@ -932,37 +1054,77 @@ istream* GLELoadOneFileManager::get_eps_stream() {
 	return NULL;
 }
 
-void GLELoadOneFileManager::write_eps() throw (ParserError) {
+bool GLELoadOneFileManager::hasGenerated(int deviceCode) {
+	return m_hasGenerated.find(deviceCode) != m_hasGenerated.end();
+}
+
+bool GLELoadOneFileManager::hasFile(int deviceCode) {
+	return m_hasFile.find(deviceCode) != m_hasFile.end();
+}
+
+bool GLELoadOneFileManager::hasIncFile(int deviceCode) {
+	return m_hasIncFile.find(deviceCode) != m_hasIncFile.end();
+}
+
+void GLELoadOneFileManager::setHasGenerated(int deviceCode, bool value) {
+	if (value) {
+		m_hasGenerated.insert(deviceCode);
+	} else {
+		m_hasGenerated.erase(deviceCode);
+	}
+}
+
+void GLELoadOneFileManager::setHasFile(int deviceCode, bool value) {
+	if (value) {
+		m_hasFile.insert(deviceCode);
+	} else {
+		m_hasFile.erase(deviceCode);
+	}
+}
+
+void GLELoadOneFileManager::setHasIncFile(int deviceCode, bool value) {
+	if (value) {
+		m_hasIncFile.insert(deviceCode);
+	} else {
+		m_hasIncFile.erase(deviceCode);
+	}
+}
+
+void GLELoadOneFileManager::write_recorded_data(int deviceCode) throw (ParserError) {
+	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
+	if (!device->hasValue(deviceCode)) {
+		return;
+	}
 	if (m_CmdLine->hasOption(GLE_OPT_CREATE_INC) || m_CmdLine->hasOption(GLE_OPT_NOSAVE)) {
 		return;
 	}
 	if (m_OutName->isStdout()) {
-		/* Write .eps data to stdout */
-		string* buffer = m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS);
-		cout.write(buffer->data(), buffer->size());
+		/* Write to stdout */
+		if (hasFile(deviceCode)) {
+			cat_stdout(g_device_to_ext(deviceCode));
+		} else {
+			string* buffer = m_Script->getRecordedBytesBuffer(deviceCode);
+			cout.write(buffer->data(), buffer->size());
+		}
 	} else {
-		/* Write .eps data to output file */
-		m_Device->writeRecordedOutputFile(m_OutName->getFullPath(), m_Script->getRecordedBytesBuffer(GLE_DEVICE_EPS));
+		if (!hasFile(deviceCode)) {
+			writeRecordedOutputFile(m_OutName->getFullPath(), deviceCode, m_Script);
+		}
+	}
+}
+
+void GLELoadOneFileManager::clean_inc_file(int deviceCode) {
+	/* Remove _inc.* if not required anymore */
+	bool create_inc = m_CmdLine->hasOption(GLE_OPT_CREATE_INC);
+	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
+	if (hasIncFile(deviceCode) && (!create_inc || !device->hasValue(deviceCode))) {
+		delete_temp_file(m_IncName.getFullPath(), g_device_to_ext(deviceCode));
 	}
 }
 
 void GLELoadOneFileManager::clean_tex_temp_files() {
-	bool create_inc = m_CmdLine->hasOption(GLE_OPT_CREATE_INC);
-	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
-	/* Remove _inc.{eps,pdf} if not required anymore */
-	if (create_inc) {
-		if (m_HasIncEPS && !device->hasValue(GLE_DEVICE_EPS)) {
-			delete_temp_file(m_IncName.getFullPath(), ".eps");
-		}
-	} else {
-		/* No create_inc, so for sure no "_inc.{eps,pdf}" required */
-		if (m_HasIncEPS) {
-			delete_temp_file(m_IncName.getFullPath(), ".eps");
-		}
-		if (m_HasIncPDF) {
-			delete_temp_file(m_IncName.getFullPath(), ".pdf");
-		}
-	}
+	clean_inc_file(GLE_DEVICE_EPS);
+	clean_inc_file(GLE_DEVICE_PDF);
 	/* Remove .tex file */
 	if (m_HasTeXFile) {
 		delete_temp_file(m_OutName->getFullPath(), ".tex");
@@ -974,20 +1136,25 @@ void GLELoadOneFileManager::clean_tex_temp_files() {
 	}
 }
 
-void GLELoadOneFileManager::delete_original_eps() {
+void GLELoadOneFileManager::delete_original_eps_pdf_impl(int deviceCode) {
 	CmdLineArgSet* device = (CmdLineArgSet*)m_CmdLine->getOption(GLE_OPT_DEVICE)->getArg(0);
-	bool should_del = m_HasEPSFile;
-	if (device->hasValue(GLE_DEVICE_EPS)) {
-		/* don't delete .eps if we wanted this as output */
+	bool should_del = hasFile(deviceCode);
+	if (device->hasValue(deviceCode)) {
+		/* don't delete .eps/.pdf if we wanted this as output */
 		/* and we did not wanted this printed to stout */
 		if (!m_OutName->isStdout() && !m_CmdLine->hasOption(GLE_OPT_CREATE_INC)) {
 			should_del = false;
 		}
 	}
 	if (should_del) {
-		/* delete original .eps file */
-		delete_temp_file(m_OutName->getFullPath(), ".eps");
+		/* delete original .eps/.pdf file */
+		delete_temp_file(m_OutName->getFullPath(), g_device_to_ext(deviceCode));
 	}
+}
+
+void GLELoadOneFileManager::delete_original_eps_pdf() {
+	delete_original_eps_pdf_impl(GLE_DEVICE_EPS);
+	delete_original_eps_pdf_impl(GLE_DEVICE_PDF);
 	/* has temporary output file */
 	if (m_HasTempFile) {
 		#ifdef __WIN32__
@@ -998,6 +1165,12 @@ void GLELoadOneFileManager::delete_original_eps() {
 	}
 }
 
+void complain_latex_not_supported(int device) {
+	if (TeXInterface::getInstance()->hasObjects()) {
+		g_throw_parser_error(">> LaTeX expressions not supported in '", g_device_to_ext(device), "' output");
+	}
+}
+
 void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code) throw(ParserError) {
 	GLEFileLocation out_name; /* out_name has no extension */
 	GLEGetInterfacePointer()->getConfig()->setAllowConfigBlocks(false);
@@ -1008,20 +1181,13 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 	GLEGetColorList()->reset();
 	if (cmdline.hasOption(GLE_OPT_DEBUG)){
 		printf("Debug options 16=do_pcode, 8=pass 4=polish, 2=eval ");
-		printf("Debug "); scanf("%d",&gle_debug);
-		printf("Trace "); scanf("%d",&trace_on);
+		printf("Debug "); gle_debug = GLEReadConsoleInteger();
+		printf("Trace "); trace_on = GLEReadConsoleInteger();
 	}
 	GLELoadOneFileManager manager(script, &cmdline, &out_name);
 	CmdLineArgSet* device = (CmdLineArgSet*)cmdline.getOption(GLE_OPT_DEVICE)->getArg(0);
-	if (device->hasValue(GLE_DEVICE_PDF) && cmdline.hasOption(GLE_OPT_CAIRO)) {
-		g_select_device(GLE_DEVICE_CAIRO_PDF);
-		DrawIt(script, &out_name, &cmdline);
-	} else if (has_eps_based_device(device, cmdline)) {
-		if (device->hasValue(GLE_DEVICE_EPS) && !out_name.isStdout()) {
-			/* Delete output .eps so that QGLE does not display previous rendering if there are errors */
-			DeleteFileWithExt(out_name.getFullPath(), ".eps");
-		}
-		bool has_tex = manager.process_one_file_eps();
+	if (has_eps_or_pdf_based_device(device, cmdline)) {
+		bool has_tex = manager.process_one_file_tex();
 		/* Return if there were errors */
 		if (get_nb_errors() > 0) {
 			if (g_verbosity() > 0) cerr << endl;
@@ -1034,25 +1200,26 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 		if (has_tex) {
 			manager.create_latex_eps_ps_pdf();
 		} else {
-			if (device->hasValue(GLE_DEVICE_PDF)) {
-				create_pdf_file_ghostscript(&out_name, dpi, script);
-				manager.do_output_type(".pdf");
-			}
+			manager.convert_eps_to_pdf_no_latex();
 		}
 		/* Create bitmap outputs */
-		bool is_bw = cmdline.hasOption(GLE_OPT_NO_COLOR);
-		bool is_transp = cmdline.hasOption(GLE_OPT_TRANSPARENT);
+		int output_options = 0;
+		if (cmdline.hasOption(GLE_OPT_NO_COLOR)) {
+			output_options |= GLE_OUTPUT_OPTION_GRAYSCALE;
+		}
+		if (cmdline.hasOption(GLE_OPT_TRANSPARENT)) {
+			output_options |= GLE_OUTPUT_OPTION_TRANSPARENT;
+		}
 		for (int i = 0; i < device->getNbValues(); i++) {
 			if (is_bitmap_device(i) && device->hasValue(i)) {
-				create_bitmap_file(&out_name, i, dpi, is_bw, is_transp, script);
+				create_bitmap_file(&out_name, i, dpi, output_options, script);
 				manager.do_output_type(g_device_to_ext(i));
 			}
 		}
 		/* Output .eps to stdout? */
-		if (device->hasValue(GLE_DEVICE_EPS)) {
-			manager.write_eps();
-		}
-		manager.delete_original_eps();
+		manager.write_recorded_data(GLE_DEVICE_EPS);
+		manager.write_recorded_data(GLE_DEVICE_PDF);
+		manager.delete_original_eps_pdf();
 		if (has_tex) manager.clean_tex_temp_files();
 		if (g_verbosity() > 0) cerr << endl;
 	}
@@ -1063,15 +1230,23 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 			g_message(">> To include LaTeX expressions, use \"gle -tex -d ps file.gle\"");
 		}
 		if (psdev->isRecordingEnabled()) {
-			psdev->writeRecordedOutputFile(out_name.getFullPath());
+			std::string recorded;
+			psdev->getRecordedBytes(&recorded);
+			writeRecordedOutputFile(out_name.getFullPath(), GLE_DEVICE_PS, &recorded);
+		}
+		if (out_name.isStdout()) {
+			manager.cat_stdout_and_del(".ps");
 		}
-		if (out_name.isStdout()) manager.cat_stdout_and_del(".ps");
 		cerr << endl;
 	}
 	if (device->hasValue(GLE_DEVICE_SVG)) {
 		g_select_device(GLE_DEVICE_CAIRO_SVG);
 		DrawIt(script, &out_name, &cmdline);
-		if (out_name.isStdout()) manager.cat_stdout_and_del(".svg");
+		complain_latex_not_supported(GLE_DEVICE_SVG);
+		if (out_name.isStdout()) {
+			manager.cat_stdout_and_del(".svg");
+		}
+		cerr << endl;
 	}
 	if (device->hasValue(GLE_DEVICE_X11)) {
 		g_select_device(GLE_DEVICE_X11);
@@ -1090,6 +1265,8 @@ void load_one_file_sub(GLEScript* script, CmdLineObj& cmdline, size_t* exit_code
 			dev->setDPI(2000);
 		}
 		DrawIt(script, &out_name, &cmdline);
+		complain_latex_not_supported(GLE_DEVICE_EMF);
+		cerr << endl;
 	}
 #endif
 }
@@ -1104,7 +1281,7 @@ void load_one_file(const char* name, CmdLineObj& cmdline, size_t* exit_code) {
 	try {
 		GLERC<GLEScript> script = load_gle_code_sub(name, cmdline);
 		load_one_file_sub(script.get(), cmdline, exit_code);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
@@ -1117,7 +1294,7 @@ void load_one_file_stdin(CmdLineObj& cmdline, size_t* exit_code) {
 	try {
 		GLERC<GLEScript> script = load_gle_code_sub_stdin(cmdline);
 		load_one_file_sub(script.get(), cmdline, exit_code);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		string err_str;
 		err.toString(err_str);
 		str_uppercase_initial_capital(err_str);
