@@ -111,6 +111,8 @@ public:
 	inline double* getData() { return m_Data; }
 };
 
+bool GLEReadFileOrGZIPTxt(const std::string& name, std::string* result);
+
 typedef unsigned char GLEBYTE;
 
 enum GLECSVDataStatus {
@@ -119,21 +121,58 @@ enum GLECSVDataStatus {
 	GLECSVDataStatusEOF
 };
 
+enum GLECSVErrorCode {
+	GLECSVErrorNone,
+	GLECSVErrorFileNotFound,
+	GLECSVErrorUnterminatedString,
+	GLECSVErrorInconsistentNrColumns
+};
+
+class GLECSVError {
+public:
+	GLECSVError();
+	~GLECSVError();
+	GLECSVErrorCode errorCode;
+	unsigned int errorLine;
+	unsigned int errorColumn;
+	string errorString;
+};
+
 class GLECSVData {
 protected:
 	vector<GLEBYTE> m_buffer;
-	vector<int> m_index;
-	vector<int> m_rowStart;
-	vector<int> m_rowSize;
+	vector<unsigned int> m_cellPos;
+	vector<unsigned int> m_cellSize;
+	vector<unsigned int> m_firstCell;
 	GLEBYTE* m_data;
 	bool* m_delims;
 	unsigned int m_size;
 	unsigned int m_pos;
 	unsigned int m_writePos;
+	unsigned int m_lines;
+	unsigned int m_firstColumn;
+	unsigned int m_nextLine;
+	unsigned int m_ignoreHeader;
+	GLECSVError m_error;
+	string m_fileName;
+	string m_comment;
+	bool m_lastDelimWasSpace;
 public:
 	GLECSVData();
 	~GLECSVData();
 	bool read(const std::string& file);
+	void readBuffer(const char* buffer);
+	void print(ostream& os);
+	void setDelims(const char* delims);
+	void setCommentIndicator(const char* comment);
+	void setIgnoreHeader(unsigned int ignore);
+	GLECSVError* getError();
+	unsigned int getNbLines();
+	unsigned int getNbColumns(unsigned int line);
+	const char* getCell(unsigned int row, unsigned int column, unsigned int* size);
+	string getCellString(unsigned int row, unsigned int column);
+	void setCellTrim(unsigned int row, unsigned int column, const char* data);
+	unsigned int validateIdenticalNumberOfColumns();
 private:
 	bool readBlock(const std::string& file);
 	void parseBlock();
@@ -142,11 +181,14 @@ private:
 	bool isSpace(GLEBYTE ch);
 	bool isEol(GLEBYTE ch);
 	bool isComment(GLEBYTE ch);
-	void skipTillEol();
+	GLECSVDataStatus skipTillEol();
+	GLECSVDataStatus ignoreHeader();
 	GLECSVDataStatus readCellString(GLEBYTE quote);
 	GLECSVDataStatus readCell();
 	void createCell(unsigned int cellSize, unsigned int cellPos);
-	GLECSVDataStatus removeTrailingEOLs();
+	GLECSVDataStatus readNewline(GLEBYTE prevCh);
+	bool isSizeCheckOKEndOfLine(unsigned int cellSize);
+	bool isSizeCheckOKAtDelim(GLEBYTE delim, unsigned int cellSize);
 	void goBack();
 	unsigned int lastCharPos();
 	GLEBYTE readChar();
@@ -154,6 +196,9 @@ private:
 	void initWritePos();
 	void writeChar(GLEBYTE ch);
 	GLECSVDataStatus skipSpacesAndFirstDelim(GLEBYTE ch);
+	void createErrorString(const string& str);
+	unsigned int getUTF8Column(unsigned int cellPos);
+	unsigned int getFirstCell(unsigned int line);
 };
 
 #endif
