@@ -41,6 +41,9 @@
 
 #include "tokens/Tokenizer.h"
 
+class GLEColor;
+class GLEString;
+
 #define SC (char *)
 #define UC (unsigned char *)
 
@@ -69,9 +72,10 @@ int df_circle_stroke(dbl r);
 int df_narc(dbl r,dbl t1,dbl t2,dbl cx,dbl cy);
 int mem_total(void);
 int mem_worst(void);
-int pass_color(const char *s) throw(ParserError);
-int pass_color_var(const char *s) throw(ParserError);
-int pass_font(const char *s);
+GLERC<GLEColor> pass_color_var(const std::string& token) throw(ParserError);
+int pass_font(const std::string& token);
+int pass_justify(const std::string& token);
+int pass_marker(char *s) throw(ParserError);
 int testfree(int i);
 int text_printf(int *in,int ilen);
 int w_message(char *s);
@@ -112,6 +116,7 @@ void polar_xy(double rx, double ry, double angle, double *dx, double *dy);
 void scheck(int v);
 void set_glue(int *in,int ilen,double actual,double width,double stretch,double shrink,double *setlen);
 void setdstr(char **s, const char *in);
+void setdstr(char **s, const std::string& in);
 void setsstr(char **s, const char *in);
 void showpcode(int *p);
 void spop(int v);
@@ -119,7 +124,7 @@ void spush(int v);
 void sub_clear(bool undef);
 void sub_get_startend(int idx, int *ss, int *ee);
 void sub_set_return(double d);
-void sub_set_return_str(const char* s);
+void sub_set_return_str(GLEString* s);
 void subscript();
 void text_block(const string& s,double width,int justify);
 void text_block(const string& s,double width,int justify, int innerjust);
@@ -144,5 +149,8 @@ void validate_open_input_stream(ifstream& input, const string& fname) throw(Pars
 FILE* validate_fopen(const string& fname, const char *mode, bool isread) throw(ParserError);
 void validate_file_name(const string& fname, bool isread) throw(ParserError);
 string fontdir(const char *s);
-int check_has_font(const char* name);
+int check_has_font(const std::string& name);
+int get_marker_string(const string& marker, IThrowsError* error);
+void gle_set_constants();
+
 #endif
