@@ -560,6 +560,9 @@ void do_bar(int& ct, GLEGraphBlockInstance* graphBlock) {
 		3dbar .5 .5 top black,black,black side red,green,blue  notop */
 	int ng = 0,fi;
 	char *ss;
+	if (g_nbar + 1 >= MAX_NB_BAR) {
+		g_throw_parser_error("too many bar commands in graph block");
+	}
 	g_nbar++;
 	br[g_nbar] = new bar_struct();
 	br[g_nbar]->ngrp = 0;
@@ -650,6 +653,9 @@ void do_bar(int& ct, GLEGraphBlockInstance* graphBlock) {
 /* fill d1,d2 color green xmin 1 xmax 2 ymin 1 ymax 2   */
 /* fill d1 color green xmin 1 xmax 2 ymin 1 ymax 2      */
 void do_fill(int& ct, GLEGraphBlockInstance* graphBlock) {
+	if (nfd + 1 >= MAX_NB_FILL) {
+		g_throw_parser_error("too many fill commands in graph block");
+	}
 	fd[++nfd] = new fill_data();
 	GLEGraphDataSetOrder* order = graphBlock->getData()->getOrder();
 	GLEClassDefinition* classDef = graphBlock->getGraphBlockBase()->getClassDefinitions()->getFill();
@@ -823,15 +829,29 @@ void do_colormap(int& ct) {
 	ct++;
 	while (ct <= ntk) {
 		kw("COLOR") g_colormap->setColor(true);
-		kw("INVERT") g_colormap->setInvert(true);
-		kw("ZMIN") g_colormap->setZMin(next_exp);
-		kw("ZMAX") g_colormap->setZMax(next_exp);
-		kw("PALETTE") {
+		else kw("INVERT") g_colormap->setInvert(true);
+		else kw("ZMIN") g_colormap->setZMin(next_exp);
+		else kw("ZMAX") g_colormap->setZMax(next_exp);
+		else kw("INTERPOLATE") {
+			string tmp;
+			next_str_cpp(tmp);
+			IpolType ipolType(IPOL_TYPE_BICUBIC);
+			if (str_i_equals(tmp, "BICUBIC")) {
+				ipolType = IPOL_TYPE_BICUBIC;
+			} else if (str_i_equals(tmp, "NEAREST")) {
+				ipolType = IPOL_TYPE_NEAREST;
+			} else {
+				g_throw_parser_error("unknown interpolation type '", tmp.c_str(), "'");
+			}
+			g_colormap->setIpolType(ipolType);
+		}
+		else kw("PALETTE") {
 			string tmp;
 			next_str_cpp(tmp);
 			str_to_uppercase(tmp);
 			g_colormap->setPalette(tmp);
 		}
+		else g_throw_parser_error("expecting colormap sub command, not '", tk[ct], "'");
 		ct++;
 	}
 	g_colormap->readData();
@@ -1284,7 +1304,7 @@ void do_set_bar_color(const char* tk, bar_struct* bar, int type) {
 				break;
 			case BAR_SET_FILL:
 				ensure_fill_created(bar, fi);
-				update_color_foreground(bar->fill[fi].get(), color.get());
+				update_color_foreground_and_pattern(bar->fill[fi].get(), color.get());
 				update_key_fill(bar, fi);
 				break;
 			case BAR_SET_TOP:
@@ -1522,6 +1542,14 @@ void prepare_graph_key_and_clip(double ox, double oy, KeyInfo* keyinfo) {
 	}
 }
 
+void key_update_bounds(double ox, double oy, KeyInfo* keyinfo) {
+	if (!keyinfo->hasHei()) keyinfo->setHei(g_fontsz);
+	measure_key(keyinfo);
+	if (keyinfo->getNbEntries() > 0 && !keyinfo->isDisabled()) {
+		g_update_bounds(keyinfo->getRect());
+	}
+}
+
 void draw_graph(KeyInfo* keyinfo, GLEGraphBlockInstance* graphBlock) throw (ParserError) {
 	GLERectangle box;
 	double ox,oy;
@@ -1589,6 +1617,7 @@ void draw_graph(KeyInfo* keyinfo, GLEGraphBlockInstance* graphBlock) throw (Pars
 		measure.measureStart();
 		graphBlock->getAxis()->setBox(&dummy);
 		graphBlock->getAxis()->drawLayer(GLE_GRAPH_LAYER_UNDEFINED);
+		key_update_bounds(ox, oy, keyinfo);
 		measure.measureEnd();
 		g_restore_device(old_device);
 		if (g_auto_s_h) {
@@ -1640,9 +1669,8 @@ void draw_graph(KeyInfo* keyinfo, GLEGraphBlockInstance* graphBlock) throw (Pars
 	}
 
 	if (g_colormap != NULL) {
-		g_colormap->setXRange(xx[1].getMin(), xx[1].getMax());
-		g_colormap->setYRange(xx[2].getMin(), xx[2].getMax());
-		g_colormap->draw(graph_x1, graph_y1, xlength, ylength);
+		GLEToGraphView view(&xx[GLE_AXIS_X], &xx[GLE_AXIS_Y]);
+		g_colormap->draw(&view, graph_x1, graph_y1, xlength, ylength);
 		delete g_colormap;
 		g_colormap = NULL;
 	}
@@ -1656,12 +1684,13 @@ void draw_graph(KeyInfo* keyinfo, GLEGraphBlockInstance* graphBlock) throw (Pars
 		g_endclip();
 		g_grestore();
 	}
-	draw_key_after_measure(keyinfo);
 
-	g_move(ox,oy);
 	g_grestore();
 	g_init_bounds();
 	g_set_bounds(&box);
+
+	draw_key_after_measure(keyinfo);
+	g_move(ox,oy);
 	return;
 }
 
@@ -1743,21 +1772,21 @@ bool is_dataset_identifier(const char* ds) {
 		return true;
 	}
 	char* ptr = NULL;
-	strtol(ds+1, &ptr, 10);
-	return ptr != NULL && *ptr == 0;
+	int result = strtol(ds+1, &ptr, 10);
+	return ptr != NULL && *ptr == 0 && result >= 0;
 }
 
-int get_dataset_identifier(const char* ds, bool def) throw(ParserError) {
-	int len = strlen(ds);
+int get_dataset_identifier(const std::string& ds, bool def) throw(ParserError) {
+	int len = ds.size();
 	if (len <= 1 || toupper(ds[0]) != 'D') {
-		g_throw_parser_error("illegal data set identifier '", ds, "'");
+		g_throw_parser_error("illegal data set identifier '", ds.c_str(), "'");
 	}
 	if (str_i_equals(ds, "dn")) {
 		return 0;
 	}
 	if (len > 3 && ds[1] == '[' && ds[len - 1] == ']') {
 		double id;
-		string str(ds + 2, len - 3);
+		string str(ds.c_str() + 2, len - 3);
 		polish_eval((char*)str.c_str(), &id);
 		int result = (int)floor(id + 0.5);
 		if (result < 0 || result >= MAX_NB_DATA) {
@@ -1765,18 +1794,21 @@ int get_dataset_identifier(const char* ds, bool def) throw(ParserError) {
 			err << "data set identifier out of range: '" << result << "'";
 			g_throw_parser_error(err.str());
 		}
+		if (def && dp[result] == NULL) {
+			g_throw_parser_error("data set '", ds.c_str(), "' not defined");
+		}
 		return result;
 	} else {
 		char* ptr = NULL;
-		int result = strtol(ds+1, &ptr, 10);
+		int result = strtol(ds.c_str() + 1, &ptr, 10);
 		if (*ptr != 0) {
-			g_throw_parser_error("illegal data set identifier '", ds, "'");
+			g_throw_parser_error("illegal data set identifier '", ds.c_str(), "'");
 		}
 		if (result < 0 || result >= MAX_NB_DATA) {
-			g_throw_parser_error("data set identifier out of range '", ds, "'");
+			g_throw_parser_error("data set identifier out of range '", ds.c_str(), "'");
 		}
 		if (def && dp[result] == NULL) {
-			g_throw_parser_error("data set '", ds, "' not defined");
+			g_throw_parser_error("data set '", ds.c_str(), "' not defined");
 		}
 		return result;
 	}
@@ -1986,6 +2018,9 @@ void get_data_value(GLECSVData* csvData, int dn, GLEArrayImpl* array, int arrayI
 }
 
 void createDataSet(int dn) {
+	if (dn < 0 || dn >= MAX_NB_DATA) {
+		g_throw_parser_error("too many data sets");
+	}
 	if (dn > ndata) ndata = dn;
 	if (dp[dn] == NULL) {
 		dp[dn] = new GLEDataSet(dn);
