@@ -38,6 +38,8 @@
 
 #include "gle-block.h"
 #include "axis.h"
+#include "file_io.h"
+#include "bitmap/img2ps.h"
 
 class GLEGraphBlockBase;
 class GLEGraphDrawCommand;
@@ -392,10 +394,10 @@ public:
 	void noLogZero(bool xlog, bool ylog);
 	vector<double>* getDimension(unsigned int i);
 	double getMinXInterval();
-	inline unsigned int size() { return m_X.size(); }
-	inline double getX(int i) { return m_X[i]; }
-	inline double getY(int i) { return m_Y[i]; }
-	inline int getM(int i) { return m_M[i]; }
+	inline unsigned int size() const { return m_X.size(); }
+	inline double getX(int i) const { return m_X[i]; }
+	inline double getY(int i) const { return m_Y[i]; }
+	inline int getM(int i) const { return m_M[i]; }
 	inline double* getX() { return &m_X[0]; }
 	inline double* getY() { return &m_Y[0]; }
 	inline int* getM() { return &m_M[0]; }
@@ -465,6 +467,7 @@ public:
 	GLEAxis* getAxis(int i);
 	void clip(double *x, double *y);
 	bool contains(double x, double y);
+	bool contains(const GLEPoint& p);
 	void checkRanges() throw(ParserError);
 	void copyRangeIfRequired(int dimension);
 	vector<int> getMissingValues();
@@ -511,7 +514,8 @@ public:
 #define GRAPHDEF
 #endif
 
-#define MAX_NB_FILL 20
+#define MAX_NB_FILL 100
+#define MAX_NB_BAR 100
 
 GRAPHDEF double graph_x1,graph_y1,graph_x2,graph_y2;  /* in cm */
 GRAPHDEF double graph_xmin,graph_ymin,graph_xmax,graph_ymax; /* graph units */
@@ -533,13 +537,13 @@ GRAPHDEF int gntmp;
 
 GRAPHDEF GLEDataSet *dp[MAX_NB_DATA];
 
-GRAPHDEF bar_struct *br[20];
+GRAPHDEF bar_struct *br[MAX_NB_BAR];
 void vinit_axis(int i);
 void vinit_title_axis();
 void draw_bar(double x, double yf, double yt, double wd, bar_struct* barset, int di, GLEDataSet* toDataSet) throw(ParserError);
 void get_dataset_ranges();
 void set_bar_axis_places();
-int get_dataset_identifier(const char* ds, bool def = false) throw(ParserError);
+int get_dataset_identifier(const std::string& ds, bool def = false) throw(ParserError);
 int get_dataset_identifier(const string& ds, GLEParser* parser, bool def) throw(ParserError);
 
 double graph_bar_pos(double xpos, int bar, int set) throw(ParserError);
@@ -558,26 +562,59 @@ string dimension2String(unsigned int dimension);
 
 class GLEZData;
 
+class GLEToView {
+public:
+	GLEToView();
+	virtual ~GLEToView();
+	virtual GLEPoint fnXY(const GLEPoint& xy) = 0;
+	virtual GLEPoint fnXYInv(const GLEPoint& xy) = 0;
+};
+
+class GLEToRectangularView: public GLEToView {
+public:
+	GLEToRectangularView();
+	virtual ~GLEToRectangularView();
+	virtual GLEPoint fnXY(const GLEPoint& xy);
+	virtual GLEPoint fnXYInv(const GLEPoint& xy);
+	void setXRange(double from, double to) { m_xRange.setMinMax(from, to); }
+	void setYRange(double from, double to) { m_yRange.setMinMax(from, to); }
+	void setOrigin(const GLEPoint& origin) { m_origin.set(origin); }
+	void setSize(const GLEPoint& size) { m_size.set(size); }
+private:
+	GLERange m_xRange;
+	GLERange m_yRange;
+	GLEPoint m_origin;
+	GLEPoint m_size;
+};
+
+class GLEToGraphView: public GLEToView {
+public:
+	GLEToGraphView(GLEAxis* xAxis, GLEAxis* yAxis);
+	virtual ~GLEToGraphView();
+	virtual GLEPoint fnXY(const GLEPoint& xy);
+	virtual GLEPoint fnXYInv(const GLEPoint& xy);
+private:
+	GLEAxis* m_xAxis;
+	GLEAxis* m_yAxis;
+};
+
 class GLEColorMap {
 public:
 	string m_function;
 	string m_palette;
 	int m_wd, m_hi;
 	bool m_color;
-	double m_xmin, m_xmax;
-	double m_ymin, m_ymax;
 	double m_zmin, m_zmax;
 	bool m_has_zmin;
 	bool m_has_zmax;
 	bool m_invert;
 	bool m_haspal;
+	IpolType m_ipolType;
 	GLEZData* m_Data;
 public:
 	GLEColorMap();
 	~GLEColorMap();
-	void draw(double x0, double y0, double wd, double hi);
-	void setXRange(double min, double max);
-	void setYRange(double min, double max);
+	void draw(GLEToView* toView, double x0, double y0, double wd, double hi);
 	void setZMin(double val);
 	void setZMax(double val);
 	void setPalette(const string& pal);
@@ -586,16 +623,12 @@ public:
 	inline bool hasZMax() { return m_has_zmax; }
 	inline double getZMin() { return m_zmin; }
 	inline double getZMax() { return m_zmax; }
-	inline void setFunction(const char* f) { m_function = f; }
+	inline void setFunction(const std::string& f) { m_function = f; }
 	inline void setWidth(int wd) { m_wd = wd; }
 	inline void setHeight(int hi) { m_hi = hi; }
 	inline const string& getFunction() { return m_function; }
 	inline int getWidth() { return m_wd; }
 	inline int getHeight() { return m_hi; }
-	inline double getXMin() { return m_xmin; }
-	inline double getXMax() { return m_xmax; }
-	inline double getYMin() { return m_ymin; }
-	inline double getYMax() { return m_ymax; }
 	inline bool isColor() { return m_color; }
 	inline void setColor(bool color) { m_color = color; }
 	inline void setInvert(bool inv) { m_invert = inv; }
@@ -603,6 +636,8 @@ public:
 	inline bool hasPalette() { return m_haspal; }
 	inline const string& getPaletteFunction() { return m_palette; }
 	inline GLEZData* getData() { return m_Data; }
+	inline void setIpolType(IpolType type) { m_ipolType = type; }
+	inline IpolType getIpolType() const { return m_ipolType; }
 };
 
 
