@@ -39,16 +39,19 @@
 #include "all.h"
 #include "mem_limits.h"
 #include "token.h"
+#include "cutils.h"
 #include "core.h"
 #include "glearray.h"
 #include "polish.h"
 #include "pass.h"
+#include "gle-block.h"
 #include "key.h"
 #include "justify.h"
 #include "color.h"
 #include "gprint.h"
-#include "cutils.h"
 #include "op_def.h"
+#include "var.h"
+#include "sub.h"
 
 /* for key command and gx(), gy() */
 extern double graph_x1,graph_y1,graph_x2,graph_y2;  /* in cm */
@@ -77,125 +80,203 @@ double get_next_exp(TOKENS tk,int ntk,int *curtok);
 #define next_fill ((ct+=1),pass_color_var(tk[ct]))
 #define next_str(s)  (ct+=1,strcpy(s,tk[ct]))
 #define next_vstr(s)  (ct+=1,mystrcpy(&s,tk[ct]))
-#define next_vquote(s) (ct+=1,mystrcpy(&s,un_quote(tk[ct])))
 #define next_quote(s) (ct+=1,strcpy(&s,un_quote(tk[ct])))
 #define next_vquote_cpp(s) (ct+=1,skipspace,pass_file_name(tk[ct],s))
 
 #define KEY_FILL_HEI_FY 0.66
 #define KEY_FILL_HEI_FX 0.7
 
-key_struct *kd[100];
-int g_nkd, g_keycol;
-
-void begin_key(int *pln, int *pcode, int *cp) throw(ParserError) {
-    int nkd=0;
-    double khei=0,zzhei;
-    int st;
-    int ct;
-    int col = 0;
-    bool has_pattern = false;
-    KeyInfo info;
-    g_get_hei(&zzhei);
-    (*pln)++;
-    begin_init();
-    for (;;) {
-		st = begin_token(&pcode,cp,pln,srclin,tk,&ntk,outbuff);
-		if (!st) {
-			/* exit loop */
-			break;
-		}
-		/* line count variable*/
-		ct = 1;
-		while (ct<=ntk) {
-			skipspace;
-			kw("OFFSET") {
-				info.setOffsetX(next_exp);
-				info.setOffsetY(next_exp);
-			}
-			else kw("MARGINS") {
-				double mx = next_exp;
-				double my = next_exp;
-				info.setMarginXY(mx, my);
-			}
-			else kw("ABSOLUTE") {
-				if (ct <= ntk-1) {
-					info.setOffsetX(next_exp);
-					info.setOffsetY(next_exp);
-				}
-				info.setAbsolute(true);
-			}
-			else if (!has_pattern && str_i_equals(tk[ct],"BACKGROUND")) info.setBackgroundColor(next_color);
-			else kw("ROW") info.setBase(next_exp);
-			else kw("BASE") info.setBase(next_exp);
-			else kw("LPOS") info.setLinePos(next_exp);
-			else kw("LLEN") info.setLineLen(next_exp);
-			else kw("NOBOX") info.setNoBox(true);
-			else kw("NOLINE") info.setNoLines(true);
-			else kw("COMPACT") info.setCompact(true);
-			else kw("OFF") info.setDisabled(true);
-			else kw("HEI") khei = next_exp;
-			else kw("POSITION") next_str(info.getJustify());
-			else kw("POS") next_str(info.getJustify());
-			else kw("BOXCOLOR") info.setBoxColor(next_color);
-			else kw("SEPARATOR") {
-				if (nkd == 0) {
-					g_throw_parser_error("key: 'separator' should come after a valid key entry");
-				} else {
-					ct++;
-					kw("LSTYLE") {
-						kd[nkd]->sepstyle = (int)floor(next_exp + 0.5);
-					} else {
-						ct--;
-					}
-					col++;
-				}
-			}
-			else kw("JUSTIFY") {
-				next_str(info.getJustify());
-				info.setPosOrJust(false);
-			}
-			else kw("JUST") {
-				next_str(info.getJustify());
-				info.setPosOrJust(false);
-			}
-			else kw("DIST") info.setDist(next_exp);
-			else kw("COLDIST") info.setColDist(next_exp);
-			else {
-				if (ct==1) {
-					nkd++;
-					kd[nkd] = new key_struct(col);
-				}
-				if (nkd==0) return;
-				kw("TEXT") {
-					next_vquote_cpp(kd[nkd]->descrip);
-					if (g_get_tex_labels()) {
-						kd[nkd]->descrip.insert(0, "\\tex{");
-						kd[nkd]->descrip.append("}");
-					}
-				}
-				else kw("MARKER") {
-					kd[nkd]->marker = next_marker;
-				}
-				else kw("MSIZE") kd[nkd]->msize = next_exp;
-				else kw("MSCALE") kd[nkd]->msize = (next_exp) * zzhei;
-				else kw("COLOR") kd[nkd]->color = next_color;
-				else kw("FILL") kd[nkd]->fill = next_fill;
-				else kw("PATTERN") {
-					kd[nkd]->pattern = next_fill;
-					has_pattern = true;
-				}
-				else kw("BACKGROUND") kd[nkd]->background = next_fill;
-				else kw("LSTYLE") next_str(kd[nkd]->lstyle);
-				else kw("LINE") strcpy(kd[nkd]->lstyle,"1");
-				else kw("LWIDTH") kd[nkd]->lwidth = next_exp;
-				else g_throw_parser_error("unrecognised KEY sub command: '",tk[ct],"'");
-			}
-			ct++;
-		}
+class ParseGeneralKeyInfo {
+public:
+	ParseGeneralKeyInfo() {
+		GLESubDefinitionHelper passKeyInfo("key");
+		offset = passKeyInfo.addPointArgument("offset", new GLEPointDataObject(0.0, 0.0), false);
+		margins = passKeyInfo.addPointArgument("margins", new GLEPointDataObject(0.0, 0.0), false);
+		absolute = passKeyInfo.addPointArgument("absolute", 0, false);
+		// background = passKeyInfo.addPointArgument("background", GLE_FILL_CLEAR, false);
+		base = passKeyInfo.addDoubleArgumentNoDefault("base", false);
+		passKeyInfo.addArgumentAlias(base, "row");
+		linepos = passKeyInfo.addDoubleArgumentNoDefault("lpos", false);
+		linelen = passKeyInfo.addDoubleArgumentNoDefault("llen", false);
+/*
+		else kw("NOBOX") info.setNoBox(true);
+		else kw("NOLINE") info.setNoLines(true);
+		else kw("COMPACT") info.setCompact(true);
+		else kw("OFF") info.setDisabled(true);
+		else kw("HEI") khei = next_exp;
+		else kw("POSITION") next_str(info.getJustify());
+		else kw("POS") next_str(info.getJustify());
+		else kw("BOXCOLOR") info.setBoxColor(next_color);
+		else kw("SEPARATOR") {
+*/
 	}
-	info.setHei(khei);
-	info.setNbEntries(nkd);
-	draw_key(&info);
+
+public:
+	unsigned int offset;
+	unsigned int margins;
+	unsigned int absolute;
+	unsigned int background;
+	unsigned int base;
+	unsigned int linepos;
+	unsigned int linelen;
+};
+
+class GLEKeyBlockInstance : public GLEBlockInstance {
+public:
+	GLEKeyBlockInstance(GLEKeyBlockBase* parent);
+	virtual ~GLEKeyBlockInstance();
+	virtual void executeLine(GLESourceLine& sline);
+	virtual void endExecuteBlock();
+
+private:
+	KeyInfo m_info;
+	bool m_hasPattern;
+};
+
+GLEKeyBlockInstance::GLEKeyBlockInstance(GLEKeyBlockBase* parent):
+	GLEBlockInstance(parent),
+	m_hasPattern(false)
+{
+}
+
+GLEKeyBlockInstance::~GLEKeyBlockInstance() {
+}
+
+void GLEKeyBlockInstance::executeLine(GLESourceLine& sline)	{
+	double zzhei;
+	g_get_hei(&zzhei);
+	begin_init();
+	int st = begin_token(sline, srclin, tk, &ntk, outbuff, true);
+	if (!st) {
+		/* exit loop */
+		return;
+	}
+	/* line count variable*/
+	int ct = 1;
+	KeyEntry* entry = m_info.lastEntry();
+	while (ct <= ntk) {
+		skipspace;
+		kw("OFFSET") {
+			m_info.setOffsetX(next_exp);
+			m_info.setOffsetY(next_exp);
+		}
+		else kw("MARGINS") {
+			double mx = next_exp;
+			double my = next_exp;
+			m_info.setMarginXY(mx, my);
+		}
+		else kw("ABSOLUTE") {
+			if (ct <= ntk-1) {
+				m_info.setOffsetX(next_exp);
+				m_info.setOffsetY(next_exp);
+			}
+			m_info.setAbsolute(true);
+		}
+		else if (!m_hasPattern && str_i_equals(tk[ct],"BACKGROUND")) m_info.setBackgroundColor(next_color);
+		else kw("ROW") m_info.setBase(next_exp);
+		else kw("BASE") m_info.setBase(next_exp);
+		else kw("LPOS") m_info.setLinePos(next_exp);
+		else kw("LLEN") m_info.setLineLen(next_exp);
+		else kw("NOBOX") m_info.setNoBox(true);
+		else kw("NOLINE") m_info.setNoLines(true);
+		else kw("COMPACT") m_info.setCompact(true);
+		else kw("OFF") m_info.setDisabled(true);
+		else kw("HEI") m_info.setHei(next_exp);
+		else kw("POSITION") next_str(m_info.getJustify());
+		else kw("POS") next_str(m_info.getJustify());
+		else kw("BOXCOLOR") m_info.setBoxColor(next_color);
+		else kw("SEPARATOR") {
+			if (entry == 0) {
+				g_throw_parser_error("key: 'separator' should come after a valid key entry");
+			} else {
+				ct++;
+				kw("LSTYLE") {
+					entry->sepstyle = (int)floor(next_exp + 0.5);
+				} else {
+					ct--;
+				}
+				m_info.addColumn();
+			}
+		}
+		else kw("JUSTIFY") {
+			next_str(m_info.getJustify());
+			m_info.setPosOrJust(false);
+		}
+		else kw("JUST") {
+			next_str(m_info.getJustify());
+			m_info.setPosOrJust(false);
+		}
+		else kw("DIST") m_info.setDist(next_exp);
+		else kw("COLDIST") m_info.setColDist(next_exp);
+		else {
+			if (ct == 1) {
+				entry = m_info.createEntry();
+			}
+			if (entry == 0) {
+				return;
+			}
+			kw("TEXT") {
+				next_vquote_cpp(entry->descrip);
+				if (g_get_tex_labels()) {
+					entry->descrip.insert(0, "\\tex{");
+					entry->descrip.append("}");
+				}
+			}
+			else kw("MARKER") {
+				entry->marker = next_marker;
+			}
+			else kw("MSIZE") entry->msize = next_exp;
+			else kw("MSCALE") entry->msize = (next_exp) * zzhei;
+			else kw("COLOR") entry->color = next_color;
+			else kw("TEXTCOLOR") entry->textcolor = next_color;
+			else kw("FILL") {
+				GLERC<GLEColor> fillColor(next_fill);
+				update_color_foreground_and_pattern(entry->fill.get(), fillColor.get());
+			} else kw("PATTERN") {
+				GLERC<GLEColor> fillPattern(next_fill);
+				if (fillPattern->isFill() && fillPattern->getFill()->getFillType() == GLE_FILL_TYPE_PATTERN) {
+					update_color_fill_pattern(entry->fill.get(), static_cast<GLEPatternFill*>(fillPattern->getFill()));
+				} else {
+					g_throw_parser_error("expected fill pattern");
+				}
+			} else kw("BACKGROUND") {
+				GLERC<GLEColor> backColor(next_fill);
+				update_color_fill_background(entry->fill.get(), backColor.get());
+			}
+			else kw("LSTYLE") next_str(entry->lstyle);
+			else kw("LINE") strcpy(entry->lstyle, "1");
+			else kw("LWIDTH") entry->lwidth = next_exp;
+			else g_throw_parser_error("unrecognised KEY sub command: '", tk[ct], "'");
+		}
+		ct++;
+	}
+}
+
+void GLEKeyBlockInstance::endExecuteBlock() {
+	draw_key(&m_info);
+}
+
+GLEKeyBlockBase::GLEKeyBlockBase():
+	GLEBlockWithSimpleKeywords("key", false)
+{
+	const char* commands[] = {
+		"OFFSET", "MARGINS", "ABSOLUTE", "BACKGROUND", "ROW",
+		"BASE", "LPOS", "LLEN", "NOBOX", "NOLINE", "COMPACT",
+		"OFF", "HEI", "POSITION", "POS", "BOXCOLOR", "SEPARATOR",
+		"LSTYLE", "JUSTIFY", "JUST", "DIST", "COLDIST", "TEXT",
+		"MARKER", "MSIZE", "MSCALE", "COLOR", "TEXTCOLOR", "FILL",
+		"PATTERN", "LINE", "LWIDTH", ""};
+	for (int i = 0; commands[i][0] != 0; ++i) {
+		addKeyWord(commands[i]);
+	}
+}
+
+GLEKeyBlockBase::~GLEKeyBlockBase()
+{
+}
+
+GLEBlockInstance* GLEKeyBlockBase::beginExecuteBlockImpl(GLESourceLine& /* sline */, int* /* pcode */, int* /* cp */) {
+	return new GLEKeyBlockInstance(this);
 }
 
 KeyRCInfo::KeyRCInfo() {
@@ -245,12 +326,18 @@ KeyInfo::KeyInfo() {
 	m_NoLines = false;
 	m_Disabled = false;
 	m_BoxColor = 0;
-	m_NbEntries = 0;
 	m_ExtraY = 0.0;
-	m_BackgroundColor = GLE_FILL_CLEAR;
+	m_BackgroundColor = g_get_fill_clear();
+	m_col = 0;
 	strcpy(m_Justify, "");
 }
 
+KeyInfo::~KeyInfo() {
+	for (size_t i = 0; i < m_entries.size(); ++i) {
+		delete m_entries[i];
+	}
+}
+
 void KeyInfo::setOffsetX(double x) {
 	m_OffsX = x;
 	m_HasOffset = true;
@@ -261,7 +348,7 @@ void KeyInfo::setOffsetY(double y) {
 	m_HasOffset = true;
 }
 
-void KeyInfo::setBoxColor(int col) {
+void KeyInfo::setBoxColor(const GLERC<GLEColor>& col) {
 	m_BoxColor = col;
 	m_HasBoxColor = true;
 }
@@ -292,14 +379,28 @@ void KeyInfo::expandToRow(int row) {
 	}
 }
 
+KeyEntry* KeyInfo::createEntry() {
+	KeyEntry* entry = new KeyEntry(m_col);
+	m_entries.push_back(entry);
+	return entry;
+}
+
+KeyEntry* KeyInfo::lastEntry() {
+	if (m_entries.empty()) {
+		return 0;
+	} else {
+		return m_entries.back();
+	}
+}
+
 void draw_key(KeyInfo* info) {
 	if (info->getNbEntries() == 0) {
 		return;
 	}
 	GLEPoint savept;
 	g_get_xy(&savept);
-	if (info->getBackgroundColor() == (int)GLE_FILL_CLEAR) {
-		info->setBackgroundColor(GLE_COLOR_WHITE);
+	if (info->getBackgroundColor()->isTransparent()) {
+		info->setBackgroundColor(g_get_color_hex(GLE_COLOR_WHITE));
 	}
 	measure_key(info);
 	draw_key_after_measure(info);
@@ -308,8 +409,8 @@ void draw_key(KeyInfo* info) {
 
 void measure_key_v_recent(KeyInfo* info, GLEPoint* orig) {
 	/* Add separator dist */
-	for (int i = 1; i <= info->getNbEntries(); i++) {
-		info->getCol(kd[i]->column)->size += kd[i]->sepdist;
+	for (int i = 0; i < info->getNbEntries(); i++) {
+		info->getCol(info->getEntry(i)->column)->size += info->getEntry(i)->sepdist;
 	}
 	/* Compute sum of column widths and max number of rows */
 	int maxrow = 0;
@@ -436,31 +537,34 @@ void do_draw_key_v35(double ox, double oy, KeyInfo* info){
 	double cr = info->getBase();
 	KeyRCInfo* col_info = info->getCol(0);
 	g_set_hei(info->getHei());
-	for (int i = info->getNbEntries(); i >= 1; i--) {
-		g_move(ox+0.6*cr, oy+0.6*cr+cr*(info->getNbEntries()-i));
-		if (kd[i]->color != 0) g_set_color(kd[i]->color);
+	for (int i = info->getNbEntries() - 1; i >= 0; i--) {
+		KeyEntry* entry = info->getEntry(i);
+		g_move(ox+0.6*cr, oy+0.6*cr+cr*(info->getNbEntries()-i-1));
+		if (!entry->color.isNull()) {
+			g_set_color(entry->color);
+		}
 		if (col_info->hasMarker()) {
 			g_rmove(cr/2, info->getHei()*.35);
-			double z = kd[i]->msize;
+			double z = entry->msize;
 			if (z==0) z = info->getHei();
-			if (kd[i]->marker!=0) g_marker(kd[i]->marker, z);
+			if (entry->marker!=0) g_marker(entry->marker, z);
 			g_rmove(cr, -info->getHei()*.35);
 		}
 		if (col_info->hasLine()) {
 			double savelw;
-			g_set_line_style(kd[i]->lstyle);
+			g_set_line_style(entry->lstyle);
 			g_get_line_width(&savelw);
-			g_set_line_width(kd[i]->lwidth);
+			g_set_line_width(entry->lwidth);
 			g_rmove(0.0, cr*.3);
-			if (kd[i]->lstyle[0] == 0) g_rmove(1.5*cr, 0.0);
+			if (entry->lstyle[0] == 0) g_rmove(1.5*cr, 0.0);
 			else g_rline(1.5*cr, 0.0);
 			g_rmove(cr/2, -cr*.3);
 			g_set_line_style("1");
 			g_set_line_width(savelw);
 		}
 		if (col_info->hasFill()) {
-			if (kd[i]->fill!=0) {
-				g_set_fill(kd[i]->fill);
+			if (entry->hasFill()) {
+				g_set_fill(entry->fill);
 				double cx, cy;
 				g_get_xy(&cx,&cy);
 				g_box_fill(cx,cy,cx+cr*.7, cy+cr*.66);
@@ -468,22 +572,23 @@ void do_draw_key_v35(double ox, double oy, KeyInfo* info){
 			}
 			g_rmove(1.3*cr, 0.0);
 		}
-		if (kd[i]->color != 0) g_set_color(info->getDefaultColor());
+		if (!entry->color.isNull()) {
+			g_set_color(info->getDefaultColor());
+		}
 		g_set_just(JUST_LEFT);
-		if (kd[i]->descrip != "") g_text((char*)kd[i]->descrip.c_str());
+		if (entry->descrip != "") g_text((char*)entry->descrip.c_str());
 	}
 }
 
 void measure_key(KeyInfo* info) {
 	GLEPoint orig;
-	int old_color, old_fill;
 	double save_hei;
 	GLERectangle save_bounds;
 	/* Initialize */
 	info->initPosition();
 	g_get_xy(&orig);
-	g_get_color(&old_color);
-	g_get_fill(&old_fill);
+	GLERC<GLEColor> old_color(g_get_color());
+	GLERC<GLEColor> old_fill(g_get_fill());
 	g_get_hei(&save_hei);
 	g_get_bounds(&save_bounds);
 	if (!info->hasHei()) {
@@ -514,8 +619,10 @@ void measure_key(KeyInfo* info) {
 		info->setLineLen(1.5*rowhi);
 	}
 	/* Use fill somewhere? */
-	for (int i = 1; i <= info->getNbEntries(); i++) {
-		if (kd[i]->fill != 0) info->setHasFill(true);
+	for (int i = 0; i < info->getNbEntries(); i++) {
+		if (info->getEntry(i)->hasFill()) {
+			info->setHasFill(true);
+		}
 	}
 	/* Empty key? */
 	if (info->getNbEntries() == 0) {
@@ -526,30 +633,33 @@ void measure_key(KeyInfo* info) {
 	/* Measure all labels and count rows in each column */
 	g_set_hei(khei);
 	// cout << "key height: " << khei << endl;
-	double linePos = 1e30;
-	for (int i = 1; i <= info->getNbEntries(); i++) {
-		int col = kd[i]->column;
+	double linePos = GLE_INF;
+	for (int i = 0; i < info->getNbEntries(); i++) {
+		KeyEntry* entry = info->getEntry(i);
+		int col = entry->column;
 		KeyRCInfo* colinfo = info->expandToCol(col);
 		int row = colinfo->elems;
 		info->expandToRow(row);
-		if (!str_only_space(kd[i]->descrip)) {
+		if (!str_only_space(entry->descrip)) {
 			double bl, br, bu, bd;
-			g_measure(kd[i]->descrip,&bl,&br,&bu,&bd);
+			g_measure(entry->descrip,&bl,&br,&bu,&bd);
 			if (colinfo->size < br) colinfo->size = br;
 			if (-bd > info->getRow(row)->descent) info->getRow(row)->descent = -bd;
 			// size should not include descent, only ascent of the text
 			if (bu > info->getRow(row)->size) info->getRow(row)->size = bu;
-			// cout << "key = " << kd[i]->descrip << " bl = " << bl << " br = " << br << " bu = " << bu << " bd = " << bd << endl;
+			// cout << "key = " << entry->descrip << " bl = " << bl << " br = " << br << " bu = " << bu << " bd = " << bd << endl;
 			if (bu/2 < linePos) linePos = bu/2;
 		}
 		/* Set booleans: kl = lstyle, km = marker, kf = fill */
-		if (kd[i]->lstyle[0] == 0 && kd[i]->lwidth > 0) {
-			strcpy(kd[i]->lstyle, "1");
+		if (entry->lstyle[0] == 0 && entry->lwidth > 0) {
+			strcpy(entry->lstyle, "1");
+		}
+		if (entry->lstyle[0] != 0) colinfo->setHasLine(true);
+		if (entry->lwidth > 0) colinfo->setHasLine(true);
+		if (entry->marker != 0) colinfo->setHasMarker(true);
+		if (entry->hasFill()) {
+			colinfo->setHasFill(true);
 		}
-		if (kd[i]->lstyle[0] != 0) colinfo->setHasLine(true);
-		if (kd[i]->lwidth > 0) colinfo->setHasLine(true);
-		if (kd[i]->marker != 0) colinfo->setHasMarker(true);
-		if (kd[i]->fill != 0) colinfo->setHasFill(true);
 		/* Adjust row height in case of fill */
 		if (info->hasFill()) {
 			if (rowhi*KEY_FILL_HEI_FY > info->getRow(row)->size) {
@@ -557,13 +667,13 @@ void measure_key(KeyInfo* info) {
 				info->getRow(row)->size = rowhi*KEY_FILL_HEI_FY;
 			}
 		}
-		if (kd[i]->marker != 0) {
-			double z = kd[i]->msize;
+		if (entry->marker != 0) {
+			double z = entry->msize;
 			if (z == 0) z = khei;
 			GLEMeasureBox marksize;
 			marksize.measureStart();
 			g_move(0.0, 0.0);
-			g_marker(kd[i]->marker,z);
+			g_marker(entry->marker,z);
 			marksize.measureEnd();
 			if (info->isCompact() && !info->isNoLines()) {
 				double yval = marksize.getYMin();
@@ -607,14 +717,13 @@ void draw_key_after_measure(KeyInfo* info) {
 	if (info->getNbEntries() == 0 || info->isDisabled()) {
 		return;
 	}
-	int old_color, old_fill;
 	double save_hei;
 	g_get_hei(&save_hei);
-	g_get_color(&old_color);
-	g_get_fill(&old_fill);
+	GLERC<GLEColor> old_color(g_get_color());
+	GLERC<GLEColor> old_fill(g_get_fill());
 	double ox = info->getRect()->getXMin();
 	double oy = info->getRect()->getYMin();
-	if (!info->getNoBox() && info->getBackgroundColor() != (int)GLE_FILL_CLEAR) {
+	if (!info->getNoBox() && !info->getBackgroundColor()->isTransparent()) {
 		g_set_fill(info->getBackgroundColor());
 		g_box_fill(info->getRect());
 	}
@@ -623,13 +732,13 @@ void draw_key_after_measure(KeyInfo* info) {
 	} else {
 		do_draw_key(ox+info->getComputedMargins()->getX(), oy+info->getComputedMargins()->getY()+info->getExtraY(), false, info);
 		int prev_col = 0;
-		for (int i = 1; i <= info->getNbEntries(); i++) {
-			if (prev_col != kd[i]->column) {
-				prev_col = kd[i]->column;
-				if (i > 1 && kd[i-1]->sepstyle != -1) {
+		for (int i = 0; i < info->getNbEntries(); i++) {
+			if (prev_col != info->getEntry(i)->column) {
+				prev_col = info->getEntry(i)->column;
+				if (i >= 1 && info->getEntry(i - 1)->sepstyle != -1) {
 					char msep[9];
 					// should implement g_set_line_style that takes int
-					sprintf(msep, "%d", kd[i-1]->sepstyle);
+					sprintf(msep, "%d", info->getEntry(i - 1)->sepstyle);
 					g_set_line_style(msep);
 					double xsep = ox+info->getComputedMargins()->getX()+info->getCol(prev_col)->offs-info->getColDist()/2;
 					g_move(xsep, oy);
@@ -654,65 +763,62 @@ void do_draw_key(double ox, double oy, bool notxt, KeyInfo* info) {
 	double khei = info->getHei();
 	double rowhi = info->getBase();
 	g_set_hei(khei);
-	for (int i = 1; i <= info->getNbEntries(); i++) {
-		if (prev_col != kd[i]->column) {
+	for (int i = 0; i < info->getNbEntries(); i++) {
+		KeyEntry* entry = info->getEntry(i);
+		if (prev_col != entry->column) {
 			row = 0;
-			prev_col = kd[i]->column;
+			prev_col = entry->column;
 		}
 		KeyRCInfo* col_info = info->getCol(prev_col);
 		cx = ox+col_info->offs;
 		cy = oy+info->getRow(row)->offs;
 		g_move(cx, cy);
 		g_update_bounds(cx, cy);
-		if (kd[i]->color!=0) g_set_color(kd[i]->color);
+		if (!entry->color.isNull()) {
+			g_set_color(entry->color);
+		}
 		if (col_info->hasMarker()) {
 			g_rmove(col_info->mleft, info->getLinePos());
-			if (col_info->hasLine() && info->isCompact() && !info->isNoLines() && kd[i]->lstyle[0] != 0) {
-				g_set_line_style(kd[i]->lstyle);
-				g_get_line_width(&savelw);
-				g_set_line_width(kd[i]->lwidth);
+			g_get_line_width(&savelw);
+			g_set_line_width(entry->lwidth);
+			if (col_info->hasLine() && info->isCompact() && !info->isNoLines() && entry->lstyle[0] != 0) {
+				g_set_line_style(entry->lstyle);
 				g_rmove(-info->getLineLen()/2, 0.0);
 				g_rline(info->getLineLen(), 0.0);
 				g_rmove(-info->getLineLen()/2, 0.0);
 				g_set_line_style("1");
-				g_set_line_width(savelw);
 			}
-			if (kd[i]->marker != 0) {
-				double z = kd[i]->msize;
+			if (entry->marker != 0) {
+				double z = entry->msize;
 				if (z == 0) z = khei;
-				g_marker(kd[i]->marker, z);
+				g_marker(entry->marker, z);
 			}
+			g_set_line_width(savelw);         
 			g_rmove(col_info->mright+info->getDist(),-info->getLinePos());
 		}
 		if (col_info->hasLine() && !info->isCompact() && !info->isNoLines()) {
-			g_set_line_style(kd[i]->lstyle);
+			g_set_line_style(entry->lstyle);
 			g_get_line_width(&savelw);
-			g_set_line_width(kd[i]->lwidth);
+			g_set_line_width(entry->lwidth);
 			g_rmove(0.0,info->getLinePos());
-			if (kd[i]->lstyle[0]==0) g_rmove(info->getLineLen(),0.0);
+			if (entry->lstyle[0]==0) g_rmove(info->getLineLen(),0.0);
 			else g_rline(info->getLineLen(),0.0);
 			g_rmove(info->getDist(),-info->getLinePos());
 			g_set_line_style("1");
 			g_set_line_width(savelw);
 		}
-		if (kd[i]->color!=0) g_set_color(info->getDefaultColor());
+		if (!entry->color.isNull()) {
+			g_set_color(info->getDefaultColor());
+		}
 		if (col_info->hasFill()) {
-			if (kd[i]->fill!=0) {
-				int save_color;
-				if (kd[i]->pattern != -1 && kd[i]->pattern != (int)GLE_FILL_CLEAR) {
-					g_set_fill(kd[i]->pattern);
-					g_set_pattern_color(kd[i]->fill);
-					g_set_background(kd[i]->background);
-				} else {
-					g_set_pattern_color(GLE_COLOR_BLACK);
-					g_set_fill(kd[i]->fill);
-				}
+			if (entry->hasFill()) {
+				g_set_fill(entry->fill);
 				g_get_xy(&cx,&cy);
 				g_box_fill(cx,cy,cx+rowhi*KEY_FILL_HEI_FX, cy+rowhi*KEY_FILL_HEI_FY);
-				g_get_color(&save_color);
+				GLERC<GLEColor> save_color(g_get_color());
 				if (info->hasBoxColor()) {
-					int boxcolor = info->getBoxColor();
-					if (boxcolor != (int)GLE_FILL_CLEAR) {
+					GLERC<GLEColor> boxcolor = info->getBoxColor();
+					if (!boxcolor->isTransparent()) {
 						g_set_color(boxcolor);
 						g_box_stroke(cx,cy,cx+rowhi*KEY_FILL_HEI_FX, cy+rowhi*KEY_FILL_HEI_FY);
 						g_set_color(save_color);
@@ -726,7 +832,13 @@ void do_draw_key(double ox, double oy, bool notxt, KeyInfo* info) {
 		g_get_xy(&cx,&cy);
 		if (!notxt) {
 			g_set_just(JUST_LEFT);
-			if (kd[i]->descrip != "") g_text((char*)kd[i]->descrip.c_str());
+			if (entry->descrip != "") {
+				g_set_color(entry->textcolor);
+				g_text((char*)entry->descrip.c_str());
+				if (!entry->textcolor.isNull()) {
+					g_set_color(info->getDefaultColor());
+				}
+			}
 		} else {
 			g_update_bounds(cx+col_info->size, cy+info->getRow(row)->size);
 		}
@@ -734,17 +846,22 @@ void do_draw_key(double ox, double oy, bool notxt, KeyInfo* info) {
 	}
 }
 
-key_struct::key_struct(int col) {
+KeyEntry::KeyEntry(int col) {
 	column = col;
 	sepstyle = -1;
 	sepdist = 0.0;
-	pattern = -1;
 	lstyle[0] = 0;
 	lwidth = 0.0;
-	color = 0; fill = 0;
-	marker = 0; msize = 0.0;
-	background = GLE_FILL_CLEAR;
+	color = 0;
+	marker = 0;
+	msize = 0.0;
+	fill = new GLEColor();
+	fill->setTransparent(true);
 }
 
-key_struct::~key_struct() {
+KeyEntry::~KeyEntry() {
+}
+
+bool KeyEntry::hasFill() const {
+	return !fill.isNull() && !fill->isTransparent();
 }
