@@ -79,6 +79,15 @@ public:
 	virtual void format(double number, string* output);
 };
 
+class GLENumberFormatterPercent : public GLENumberFormatter {
+protected:
+	int m_NbDecPlaces;
+public:
+	virtual ~GLENumberFormatterPercent();
+	virtual void parseOptions(GLENumberFormat* format);
+	virtual void format(double number, string* output);
+};
+
 class GLENumberFormatterRound : public GLENumberFormatter {
 protected:
 	int m_Sig;
@@ -308,6 +317,22 @@ void GLENumberFormatterFix::format(double number, string* output) {
 	doAll(output);
 }
 
+GLENumberFormatterPercent::~GLENumberFormatterPercent() {
+}
+
+void GLENumberFormatterPercent::parseOptions(GLENumberFormat* format) {
+	m_NbDecPlaces = format->nextInt();
+}
+
+void GLENumberFormatterPercent::format(double number, string* output) {
+	char format[20], result[100];
+	sprintf(format, "%%.%df", m_NbDecPlaces);
+	sprintf(result, format, 100*number);
+	*output = result;
+	*output += "%";
+	doAll(output);
+}
+
 GLENumberFormatterSci::GLENumberFormatterSci() {
 	m_Mode = GLE_NF_SCI_SMALL_E;
 	m_HasExpDigits = false;
@@ -374,7 +399,7 @@ void GLENumberFormatterSci::formatExpPart(int exp, string* output) {
 				if (output->length() != 0) toAdd << "\\cdot ";
 				toAdd << "10^{" << exp_str << "}";
 				if (g_get_tex_labels()) toAdd << "$";
-				*output = toAdd.str();
+				*output += toAdd.str();
 			}
 			break;
 	}
@@ -666,6 +691,9 @@ GLENumberFormat::GLENumberFormat(const string& formats) :
 		if (name == "fix") {
 			incTokens();
 			format = new GLENumberFormatterFix();
+		} else if (name == "percent") {
+			incTokens();
+			format = new GLENumberFormatterPercent();
 		} else if (name == "dec") {
 			incTokens();
 			format = new GLENumberFormatterInt(GLE_NF_INT_DEC);
@@ -788,9 +816,9 @@ void GLENumberFormat::nextString(string& value) {
 	incTokens();
 }
 
-void format_number_to_string(char* out, const char* format, double value) {
+std::string format_number_to_string(const std::string& format, double value) {
 	string result;
 	GLENumberFormat fmt(format);
 	fmt.format(value, &result);
-	strcpy(out, result.c_str());
+	return result;
 }
