@@ -36,29 +36,36 @@
  *                                                                      *
  ************************************************************************/
 
+#define GRAPHDEF extern
+
 #include "all.h"
+#include "cutils.h"
 #include "mem_limits.h"
 #include "token.h"
 #include "core.h"
 #include "glearray.h"
+#include "graph.h"
 #include "polish.h"
 #include "pass.h"
 #include "var.h"
 #include "sub.h"
 #include "op_def.h"
-#include "cutils.h"
 #include "gprint.h"
 #include "keyword.h"
+#include "gle-block.h"
+#include "key.h"
+#include "surface/gsurface.h"
+#include "run.h"
 
 GLEParser* g_parser;
 
 extern GLESubMap g_Subroutines;
-extern int **gpcode;   /* gpcode is a pointer to an array of poiter to int */
 
 void get_cap(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
 void get_join(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
 void g_marker_def(char *s1, char *s2);
 void font_load(void) throw(ParserError);
+bool execute_graph(GLESourceLine& sline, bool isCommandCheck);
 
 extern int this_line;
 extern int gle_debug;
@@ -75,7 +82,6 @@ struct sub_st{
 struct sub_st *psub;
 
 #define skip_space
-#define dbg if ((gle_debug & 8)>0)
 
 char *mark_name[30];
 char *mrk_fname[61];
@@ -101,6 +107,14 @@ void get_key_info(OPKEY lkey, int* count, int* width) {
 	}
 }
 
+void GLEParserInitTokenizer(Tokenizer* tokens) {
+	TokenizerLanguage* lang = tokens->get_language();
+	lang->setLineCommentTokens("!");
+	lang->setSpaceTokens(" \t\r\n");
+	lang->enableCComment();
+	lang->setSingleCharTokens(",;=@()[]{}");
+}
+
 GLESubCallAdditParam::GLESubCallAdditParam() {
 }
 
@@ -137,9 +151,19 @@ GLEParser::GLEParser(GLEScript* script, GLEPolish* polish) : m_lang(), m_tokens(
 	m_insub = false;
 	// difference between m_CrSub and m_insub:
 	// m_CrSub still points to last sub also after "end sub"
+
+	m_blockTypes = new GLEBlocks();
+	m_blockTypes->addBlock(GLE_OPBEGIN_GRAPH, new GLEGraphBlockBase());
+	m_blockTypes->addBlock(GLE_OPBEGIN_KEY, new GLEKeyBlockBase());
+	m_blockTypes->addBlock(GLE_OPBEGIN_SURF, new GLESurfaceBlockBase());
 }
 
 GLEParser::~GLEParser() {
+	delete m_blockTypes;
+}
+
+GLEBlocks* GLEParser::getBlockTypes() {
+	return m_blockTypes;
 }
 
 void GLEParser::get_block_type(int type, string& result) {
@@ -231,10 +255,7 @@ void GLEParser::checkmode() throw(ParserError) {
 
 void GLEParser::initTokenizer() {
 	TokenizerLanguage* lang = getTokens()->get_language();
-	lang->setLineCommentTokens("!");
-	lang->setSpaceTokens(" \t\r\n");
-	lang->enableCComment();
-	lang->setSingleCharTokens(",;=@()[]{}");
+	GLEParserInitTokenizer(getTokens());
 		// ; -> allows more commands on single line
 		// = -> for variable assignments
 		// @ -> for user defined subroutine calls
@@ -248,7 +269,7 @@ void GLEParser::initTokenizer() {
 	multi->setEndToken(';');
 	multi->setEndToken(',');
 	multi->setEndToken(')');
-   multi->setEndToken(']');
+	multi->setEndToken(']');
 	lang->setMulti(multi);
 }
 
@@ -276,14 +297,12 @@ void GLEParser::checkValidName(const string& name, const char* type, int pos) th
 
 double GLEParser::evalTokenToDouble() throw(ParserError) {
 	double x = 0.0;
-	GLEPcodeList pc_list;
-	GLEPcode pcode(&pc_list);
 	Tokenizer* tokens = getTokens();
 	string& expr = tokens->next_multilevel_token();
 	int pos = tokens->token_pos_col();
 	try {
 		m_polish->internalEval(expr.c_str(), &x);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		err.incColumn(pos-1);
 		throw err;
 	}
@@ -291,14 +310,12 @@ double GLEParser::evalTokenToDouble() throw(ParserError) {
 }
 
 void GLEParser::evalTokenToString(string* str) throw(ParserError) {
-	GLEPcodeList pc_list;
-	GLEPcode pcode(&pc_list);
 	Tokenizer* tokens = getTokens();
 	string& expr = tokens->next_multilevel_token();
 	int pos = tokens->token_pos_col();
 	try {
 		m_polish->internalEvalString(expr.c_str(), str);
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		err.incColumn(pos-1);
 		throw err;
 	}
@@ -321,8 +338,8 @@ void GLEParser::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 	// cout << "pos = " << pos << endl;
 	try {
 		// cout << "Polish: '" << expr << "'" << endl;
-		m_polish->polish(expr.c_str(), pcode, rtype);
-	} catch (ParserError err) {
+		m_polish->internalPolish(expr.c_str(), pcode, rtype);
+	} catch (ParserError& err) {
 		err.incColumn(pos-1);
 		throw err;
 	}
@@ -335,18 +352,13 @@ void GLEParser::polish_eol(GLEPcode& pcode, int *rtype) throw(ParserError) {
 }
 
 void GLEParser::polish(const char* str, GLEPcode& pcode, int *rtype) throw(ParserError) {
-	try {
-		m_polish->polish(str, pcode, rtype);
-	} catch (ParserError err) {
-		err.setParserString(str);
-		throw err;
-	}
+	m_polish->polish(str, pcode, rtype);
 }
 
 void GLEParser::polish_pos(const string& arg, int pos, GLEPcode& pcode, int* rtype) throw(ParserError) {
 	try {
-		m_polish->polish(arg.c_str(), pcode, rtype);
-	} catch (ParserError err) {
+		m_polish->internalPolish(arg.c_str(), pcode, rtype);
+	} catch (ParserError& err) {
 		err.incColumn(pos-1);
 		throw err;
 	}
@@ -398,8 +410,8 @@ void GLEParser::get_if(GLEPcode& pcode) throw(ParserError) {
 	try {
 		int rtype = 1;
 		// cout << "Polish: '" << expr << "'" << endl;
-		m_polish->polish(expr.c_str(), pcode, &rtype);
-	} catch (ParserError err) {
+		m_polish->internalPolish(expr.c_str(), pcode, &rtype);
+	} catch (ParserError& err) {
 		err.incColumn(pos-1);
 		throw err;
 	}
@@ -545,6 +557,37 @@ void GLEParser::pass_subroutine_call(GLESubCallInfo* info, int poscol) throw(Par
 	}
 }
 
+void GLEParser::gen_subroutine_call_polish_arg(GLESubCallInfo* info, int i, GLEPcode& pcode) throw(ParserError) {
+	GLESub* sub = info->getSub();
+	try {
+		int vtype = sub->getParamType(i);
+		m_polish->internalPolish(info->getParamVal(i).c_str(), pcode, &vtype);
+	} catch (ParserError& err) {
+		if (info->getParamPos(i) != -2) {
+			/* in place given argument */
+			err.incColumn(info->getParamPos(i)-1);
+		} else {
+			/* default argument */
+			err.setParserString(info->getParamVal(i));
+		}
+		throw err;
+	}
+}
+
+void GLEParser::evaluate_subroutine_arguments(GLESubCallInfo* info, GLEArrayImpl* arguments) {
+	GLESub* sub = info->getSub();
+	int np = sub->getNbParam();
+	arguments->resize(np);
+	GLEPcodeList pcodeList;
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	for (int i = 0; i < np; i++) {
+		GLEPcode pcode(&pcodeList);
+		gen_subroutine_call_polish_arg(info, i, pcode);
+		int cp = 0;
+		arguments->set(i, evalGeneric(stk.get(), &pcodeList, (int*)&pcode[0], &cp));
+	}
+}
+
 void GLEParser::gen_subroutine_call_code(GLESubCallInfo* info, GLEPcode& pcode) throw(ParserError) {
 	/* pass all arguments */
 	GLESub* sub = info->getSub();
@@ -553,24 +596,7 @@ void GLEParser::gen_subroutine_call_code(GLESubCallInfo* info, GLEPcode& pcode)
 	int savelen = pcode.size();  /* Used to set acutal length at end */
 	pcode.addInt(0);             /* Length of expression */
 	for (int i = 0; i < np; i++) {
-		int vtype = sub->getParamType(i);
-		if (info->getParamPos(i) != -2) {
-			/* in place given argument */
-			try {
-				m_polish->polish(info->getParamVal(i).c_str(), pcode, &vtype);
-			} catch (ParserError err) {
-				err.incColumn(info->getParamPos(i)-1);
-				throw err;
-			}
-		} else {
-			/* default argument */
-			try {
-				m_polish->polish(info->getParamVal(i).c_str(), pcode, &vtype);
-			} catch (ParserError err) {
-				err.setParserString(info->getParamVal(i));
-				throw err;
-			}
-		}
+		gen_subroutine_call_polish_arg(info, i, pcode);
 	}
 	pcode.addFunction(sub->getIndex()+LOCAL_START_INDEX);
 	pcode.setInt(savelen, pcode.size() - savelen - 1);
@@ -681,7 +707,7 @@ int GLEParser::get_optional(OPKEY lkey, GLEPcode& pcode) throw(ParserError) {
 	return ret;
 }
 
-ParserError GLEParser::create_option_error(OPKEY lkey, int count, string& token) {
+ParserError GLEParser::create_option_error(OPKEY lkey, int count, const string& token) {
 	stringstream strm;
 	if (count == 1) {
 		strm << "found '" << token << "', but expecting '" << lkey[0].name << "'";
@@ -769,9 +795,12 @@ int GLEParser::get_one_option(op_key* lkey, GLEPcode& pcode, int plen) throw(Par
 }
 
 int GLEParser::get_first(OPKEY lkey) throw(ParserError) {
+	return get_first(m_tokens.next_token(), lkey);
+}
+
+int GLEParser::get_first(const string& token, OPKEY lkey) throw(ParserError) {
 	int count, width;
 	get_key_info(lkey, &count, &width);
-	string& token = m_tokens.next_token();
 	for (int i = 0; i < count; i++) {
 		if (str_i_equals(token.c_str(), lkey[i].name)) {
 			return lkey[i].idx;
@@ -801,69 +830,120 @@ void GLEParser::get_fill(GLEPcode& pcode) throw (ParserError) {
 	get_color(pcode);
 }
 
-void GLEParser::get_color(GLEPcode& pcode) throw (ParserError) {
-	int vtype = 1;
-	string& token = m_tokens.next_token();
-	if (token.length() > 1 && token[0] == '#') {
-		if (token.length() != 7) {
-			throw error(string("illegal color specification '")+token+"'");
+bool pass_color_hash_value(const string& color, int* result, IThrowsError* error) {
+	if (color.length() > 1 && color[0] == '#') {
+		if (color.length() != 7) {
+			throw error->throwError("illegal color specification '", color.c_str(), "'");
 		}
 		colortyp c;
-		int err = g_hash_string_to_color(token, &c);
+		int err = g_hash_string_to_color(color, &c);
 		if (err != 0) {
-			int pos = m_tokens.token_pos_col() + err;
-			throw error(pos, string("illegal color specification '")+token+"'");
+			int pos = error->getErrorPosition() + err;
+			throw error->throwError(pos, string("illegal color specification '") + color + "'");
 		}
-		pcode.addInt(8);
-		pcode.addInt(c.l);
-	} else if (str_i_str(token.c_str(), "RGB") != NULL) {
-		m_tokens.pushback_token();
-		get_exp(pcode);
-	} else if (token == "(") {
-		string expr = string("CVTGRAY(")+m_tokens.next_token()+")";
-		polish(expr.c_str(), pcode, &vtype);
-		m_tokens.ensure_next_token(")");
-	} else if (is_float(token)) {
-		string expr = string("CVTGRAY(")+token+")";
-		polish(expr.c_str(), pcode, &vtype);
-	} else	if (strchr(token.c_str(),'$') != NULL) {
-		string expr = string("CVTCOLOR(")+token+")";
-		polish(expr.c_str(), pcode, &vtype);
+		*result = c.l;
+      return true;
+   } else {
+      return false;
+   }
+}
+
+GLERC<GLEColor> pass_color_list_or_fill(const string& color, IThrowsError* error) {
+	GLERC<GLEColor> result;
+	string uc_color;
+	str_to_uppercase(color, uc_color);
+	GLEColor* gleColor = GLEGetColorList()->get(uc_color);
+	if (gleColor != NULL) {
+		result = gleColor->clone();
 	} else {
-		string uc_token;
-		str_to_uppercase(token, uc_token);
-		GLEColor* color = GLEGetColorList()->get(uc_token);
-		if (color != NULL) {
-			pcode.addInt(8);
-			pcode.addInt((int)color->getHexValueGLE());
-		} else {
-			int result = 0;
-			if (gt_firstval_err(op_fill_typ, uc_token.c_str(), &result)) {
-				pcode.addInt(8);
-				pcode.addInt(result);
+		int fillDescr = 0;
+		if (gt_firstval_err(op_fill_typ, uc_color.c_str(), &fillDescr)) {
+			result = new GLEColor();
+			if (unsigned(fillDescr) == GLE_FILL_CLEAR) {
+				result->setTransparent(true);
 			} else {
-				throw error(string("found '")+token+"', but expecting color or fill specification");
+				result->setFill(new GLEPatternFill(fillDescr));
+			}
+		} else {
+			char *endp;
+			const char* colorCStr = color.c_str();
+			double dvalue = strtod(colorCStr, &endp);
+			if (colorCStr != endp && *endp == 0) {
+				result = new GLEColor(dvalue);
+			} else if (error != 0) {
+				throw error->throwError("found '", color.c_str(), "', but expecting color or fill specification");
 			}
 		}
 	}
+	return result;
 }
 
-void GLEParser::get_marker(GLEPcode& pcode) throw (ParserError) {
-	int vtype = 1;
-	string& token = m_tokens.next_token();
-	if (token == "(" || is_float(token)) {
-		string expr = string("CVTINT(")+token+")";
-		polish(expr.c_str(), pcode, &vtype);
-	} else	if (strchr(token.c_str(),'$') != NULL) {
-		string expr = string("CVTMARKER(")+token+")";
-		polish(expr.c_str(), pcode, &vtype);
+GLERC<GLEColor> memory_cell_to_color(GLEPolish* polish, GLEArrayImpl* stk, GLEMemoryCell* cell, IThrowsError* throwsError, int depth) {
+	if (depth >= 5) {
+		throwsError->throwError("maximum depth exceeded while parsing color expression");
+	}
+	GLERC<GLEColor> color(new GLEColor());
+	switch (gle_memory_cell_type(cell)) {
+		case GLEObjectTypeDouble:
+			color->setGray(cell->Entry.DoubleVal);
+			break;
+		case GLEObjectTypeString:
+			{
+				int result = 0;
+				std::string token(((GLEString*)cell->Entry.ObjectVal)->toUTF8());
+				if (token.empty()) {
+					throwsError->throwError("expecting color name, but found empty string");
+				} else if (pass_color_hash_value(token, &result, throwsError)) {
+					color->setHexValue(result);
+				} else if (((GLEString*)cell->Entry.ObjectVal)->containsI('(')) {
+					color = memory_cell_to_color(polish, stk, polish->evalGeneric(stk, token.c_str()), throwsError, depth + 1);
+				} else {
+					color = pass_color_list_or_fill(token, throwsError);
+				}
+				break;
+			}
+		default:
+			gle_memory_cell_check(cell, GLEObjectTypeColor);
+			color = (GLEColor*)cell->Entry.ObjectVal;
+			break;
+	}
+	return color;
+}
+
+GLERC<GLEColor> pass_color_var(const std::string& token) throw(ParserError) {
+	GLERC<GLEColor> color(new GLEColor());
+	int result = 0;
+	if (token.empty()) {
+		g_throw_parser_error("expecting color name, but found empty string");
+	} else if (pass_color_hash_value(token, &result, g_get_throws_error())) {
+		color->setHexValue(result);
 	} else {
-		pcode.addInt(8);
-		pcode.addInt(pass_marker(token));
+		GLEPolish* polish = get_global_polish();
+		GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+		color = memory_cell_to_color(polish, stk.get(), polish->evalGeneric(stk.get(), token.c_str()), g_get_throws_error(), 0);
+	}
+	return color;
+}
+
+void GLEParser::get_color(GLEPcode& pcode) throw (ParserError) {
+	int result = 0;
+	GLERC<GLEColor> color;
+	string& token = m_tokens.next_token();
+	if (pass_color_hash_value(token, &result, &m_tokens)) {
+		color = new GLEColor();
+		color->setHexValue(result);
+	} else {
+		color = pass_color_list_or_fill(token, 0);
+	}
+	if (color.isNull()) {
+		m_tokens.pushback_token();
+		get_exp(pcode);
+	} else {
+		pcode.addColor(color.get());
 	}
 }
 
-int GLEParser::pass_marker(const string& marker) throw (ParserError) {
+int get_marker_string(const string& marker, IThrowsError* error) {
 	/* if 0, maybe its a user defined marker, ie a subroutine */
 	/* Use -ve to signify subroutine instead of normal marker */
 	int mark_idx = 0;
@@ -881,8 +961,31 @@ int GLEParser::pass_marker(const string& marker) throw (ParserError) {
 			}
 		}
 	}
-	if (mark_idx == 0) throw error("invalid marker name");
-	return mark_idx;
+	if (mark_idx == 0) {
+      throw error->throwError("invalid marker name '", marker.c_str(), "'");
+   }
+   return mark_idx;
+}
+
+void GLEParser::get_marker(GLEPcode& pcode) throw (ParserError) {
+	int vtype = 1;
+	string& token = m_tokens.next_token();
+	if (token == "(" || is_float(token)) {
+		string expr = string("CVTINT(")+token+")";
+		polish(expr.c_str(), pcode, &vtype);
+	} else if (str_starts_with(token, "\"") || str_var_valid_name(token)) {
+		string expr = string("CVTMARKER(")+token+")";
+		polish(expr.c_str(), pcode, &vtype);
+	} else {
+		pcode.addInt(8);
+		pcode.addInt(get_marker_string(token, &m_tokens));
+	}
+}
+
+int pass_marker(char *name) throw(ParserError) {
+	string marker;
+	polish_eval_string(name, &marker);
+   return get_marker_string(marker, g_get_throws_error());   
 }
 
 void GLEParser::define_marker_1(GLEPcode& pcode) throw (ParserError) {
@@ -906,25 +1009,29 @@ void GLEParser::define_marker_2(GLEPcode& pcode) throw (ParserError) {
 	g_marker_def((char*)name.c_str(), (char*)sub.c_str());
 }
 
-void GLEParser::get_font(GLEPcode& pcode) throw (ParserError) {
-	int etype = 1;
+
+/*
+
+    Data types:
+    
+    - fill
+    - color
+    - marker
+    - font
+    - justify
+    - join
+    - cap
+
+*/
+
+int get_font_index(const string& token, IThrowsError* error) {
 	if (get_nb_fonts() == 0) font_load();
-	string& token = m_tokens.next_token();
-	int token_len = token.length();
-	char first_char = token_len > 0 ? token[0] : ' ';
-	if (first_char == '"' || token.find("$") != string::npos) {
-		string parse = "CVTFONT("+token+")";
-		polish(parse.c_str(), pcode, &etype);
-		return;
-	}
-	pcode.addInt(8);
 	int count = get_nb_fonts();
 	// font zero is a dummy!
 	for (int i = 1; i < count; i++) {
 		const char* name = get_font_name(i);
 		if (str_i_equals(name, token.c_str())) {
-			pcode.addInt(i);
-			return;
+			return i;
 		}
 	}
 	stringstream strm;
@@ -949,7 +1056,32 @@ void GLEParser::get_font(GLEPcode& pcode) throw (ParserError) {
 			idx++;
 		}
 	}
-	throw m_tokens.error(strm.str());
+	throw error->throwError(strm.str());
+}
+
+void GLEParser::get_font(GLEPcode& pcode) throw (ParserError) {
+	string& token = m_tokens.next_token();
+	if (str_starts_with(token, "\"") || str_var_valid_name(token)) {
+   	int etype = 1;
+		string parse("CVTFONT(" + token + ")");
+		polish(parse.c_str(), pcode, &etype);
+		return;
+	}
+	pcode.addInt(8);
+    pcode.addInt(get_font_index(token, &m_tokens));
+}
+
+int pass_font(const std::string& token) {
+	if (str_starts_with(token, "\"") || str_var_valid_name(token)) {
+      int result = 0;
+      double xx = 0.0;
+		string parse("CVTFONT(" + token + ")");
+		polish_eval((char*)parse.c_str(), &xx);
+		memcpy(&result, &xx, sizeof(int));
+		return result;
+	} else {
+      return get_font_index(token, g_get_throws_error());
+	}
 }
 
 void GLEParser::get_papersize(GLEPcode& pcode) throw (ParserError) {
@@ -966,7 +1098,27 @@ void GLEParser::get_papersize(GLEPcode& pcode) throw (ParserError) {
 }
 
 void GLEParser::get_justify(GLEPcode& pcode) throw (ParserError) {
-	pcode.addInt(get_first(op_justify));
+	const string& token = m_tokens.next_token();
+	if (str_starts_with(token, "\"") || str_var_valid_name(token)) {
+		int etype = 1;
+		string parse("JUSTIFY(" + token + ")");
+		polish(parse.c_str(), pcode, &etype);
+		return;
+	}
+    pcode.addInt(8);
+	pcode.addInt(get_first(token, op_justify));
+}
+
+int pass_justify(const std::string& token) {
+	if (str_starts_with(token, "\"") || str_var_valid_name(token)) {
+		int result = 0;
+      	double xx = 0.0;
+		string parse("JUSTIFY(" + token + ")");
+		polish_eval((char*)parse.c_str(), &xx);
+		memcpy(&result, &xx, sizeof(int));
+		return result;
+	}
+	return gt_firstval(op_justify, token.c_str());
 }
 
 void GLEParser::get_join(GLEPcode& pcode) throw (ParserError) {
@@ -1022,6 +1174,24 @@ GLESub* GLEParser::is_draw_sub(const string& str) {
 	return sub_find((char*)subname.c_str());
 }
 
+bool GLEParser::pass_block_specific(GLESourceLine& sourceLine, GLEPcode& pcode) {
+	for (int i = m_blocks.size() - 1; i >= 0; i--) {
+		GLESourceBlock* block = &m_blocks[i];
+		GLEBlockBase* blockType = getBlockTypes()->getBlockIfExists(block->getType() - GLE_SRCBLK_MAGIC);
+		if (blockType != 0) {
+			if (blockType->checkLine(sourceLine)) {
+				int pos_start = pcode.size();
+				pcode.addInt(0);
+				pcode.addInt(GLE_KW_BLOCK_COMMAND);
+				pcode.addInt(block->getType() - GLE_SRCBLK_MAGIC);
+				pcode.setInt(pos_start, pcode.size()-pos_start);
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
 void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError) {
 	resetSpecial();
 	static int i,f,vtyp,v,vidx;
@@ -1030,7 +1200,7 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 	int fctkey;
 	int nbcmd = 0;
 	int position;
-	GLESourceBlock* block;
+	GLESourceBlock* block = last_block();
 	string first, temp_str;
 	if (cur_mode != 0) {
 		do_text_mode(SLine, getTokens(), pcode);
@@ -1039,19 +1209,23 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 	setAllowSpace(false);
 	bool single_cmd = false;
 	Tokenizer* tokens = getTokens();
-	if (m_auto_endif) {
-		// Allow for "IF test THEN code" blocks	without "END IF"
-		block = last_block();
-		if (block->getType() == GLE_SRCBLK_MAGIC+GLE_OPBEGIN_IF) {
-			string& token = tokens->try_next_token();
-			if (str_i_equals(token, "ELSE")) m_auto_endif = false;
-			if (token != "") tokens->pushback_token();
-		} else if (block->getType() != GLE_SRCBLK_ELSE) {
-			m_auto_endif = false;
-		}
+	if (block != 0) {
 		if (m_auto_endif) {
-			m_auto_endif = false;
-			do_endif(srclin, pcode);
+			// Allow for "IF test THEN code" blocks	without "END IF"
+			if (block->getType() == GLE_SRCBLK_MAGIC+GLE_OPBEGIN_IF) {
+				string& token = tokens->try_next_token();
+				if (str_i_equals(token, "ELSE")) m_auto_endif = false;
+				if (token != "") tokens->pushback_token();
+			} else if (block->getType() != GLE_SRCBLK_ELSE) {
+				m_auto_endif = false;
+			}
+			if (m_auto_endif) {
+				m_auto_endif = false;
+				do_endif(srclin, pcode);
+			}
+		}
+		if (pass_block_specific(SLine, pcode)) {
+			return;
 		}
 	}
 	int pos_start = pcode.size();
@@ -1190,7 +1364,6 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 					get_optional(op_begin_text, pcode);
 					break;
 				  case 10: /* graph */
-					cur_mode = 10;
 					break;
 				  case 11: /* xaxis */
 					cur_mode = 11;
@@ -1206,9 +1379,6 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 					break;
 				  case 15: /* curve */
 					break;
-				  case 16: /* KEY */
-					cur_mode = 16;
-					break;
 				  case 18: /* table */
 					cur_mode = 18;
 					break;
@@ -1219,9 +1389,6 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 				  case 23: /* tex preamble */
 					cur_mode = 23;
 					break;
-				  case 24: /* surface */
-					cur_mode = 24;
-					break;
 				  case 25: /* letz */
 					cur_mode = 25;
 					break;
@@ -1252,6 +1419,9 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 						pcode.addInt(m_CrSub->getIndex());
 					}
 					break;
+				  case OP_BEGIN_LENGTH:
+					get_var(pcode);
+					break;
 				}
 				/* here should copy source line across for "begin width 3 */
 				if (cur_mode>0)	pcode.addInt(0);
@@ -1305,7 +1475,10 @@ void GLEParser::passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError)
 				pcode.setLast(GLE_KW_COMMENT);
 				temp_str =  m_tokens.next_token();
 				str_remove_quote(temp_str);
-				GLEGetColorList()->defineColor(temp_str, pass_color(m_tokens.next_token().c_str()));
+				{
+					GLERC<GLEColor> color(pass_color_var(m_tokens.next_token().c_str()));
+					GLEGetColorList()->defineColor(temp_str, color.get());
+				}
 				break;
 			  case GLE_KW_COMPATIBILITY:
 				pcode.setLast(GLE_KW_COMMENT);
@@ -2015,7 +2188,7 @@ const char* GLESourceBlockName(int type) {
 		int count, width;
 		get_key_info(op_begin, &count, &width);
 		for (int i = 0; i < count; i++) {
-			if (op_begin[i].idx == type - GLE_SRCBLK_MAGIC) return op_begin[i].name;
+			if (int(op_begin[i].idx) == type - GLE_SRCBLK_MAGIC) return op_begin[i].name;
 		}
 	}
 	switch (type) {
@@ -2028,20 +2201,6 @@ const char* GLESourceBlockName(int type) {
 	}
 }
 
-void gt_xy(int *curtok, TOKENS tk, int *ntok, int *pcode, int *plen) {
-	int etype;
-	etype = 1;
-	if (*ntok < *curtok) {
-		gprint("Expecting x expression on end of line\n");
-	}
-	polish(tok((*curtok)++),(char *) pcode,plen,&etype);
-	etype = 1;
-	if (*ntok < *curtok) {
-		gprint("Expecting y expression on end of line\n");
-	}
-	polish(tok((*curtok)++),(char *) pcode,plen,&etype);
-}
-
 void gt_find_error(const char* found, OPKEY lkey, int nk) {
 	stringstream ss;
 	ss << "found '" << found << "', but expecting one of:" << endl;
@@ -2095,22 +2254,6 @@ int gt_firstval(OPKEY lkey, const char *s) {
 	return 0;
 }
 
-/*
-int gt_firstval(OPKEY lkey, const char *s) {
-	int nk,i,width=0,p;
-	for (i=0; lkey[i].typ!=typ_end; i++) {
-	}
-	nk = i;
-	for (i=0; i<nk; i++) {
-		if (str_i_equals(lkey[i].name,s)) {
-			dbg gprint("Got match {%s} \n",s);
-			return lkey[i].idx;
-		}
-	}
-	gt_find_error(s, lkey, nk);
-	return 0;
-}*/
-
 int gt_index(OPKEY lkey,char *s) {
 	for (int i = 0; lkey[i].typ!=typ_end; i++) {
 		if (str_i_equals(lkey[i].name,s)) {
@@ -2136,78 +2279,6 @@ void mystrcpy(char **d, const char *s) {
 /* pos=   Offset to find the data			*/
 /* idx=   For switches, which can only have one value. 	*/
 
-int pass_justify(const char *s) {
-	return gt_firstval(op_justify,s);
-}
-
-int pass_color(const char *s) throw(ParserError) {
-	double xx = 0.0;
-	char vv[80];
-	if (s[0] == '#') {
-		int len = strlen(s);
-		if (len != 7) {
-			g_throw_parser_error("illegal color specification '", s, "'");
-		}
-		colortyp c;
-		int err = g_hash_string_to_color(string(s), &c);
-		if (err != 0) {
-			g_throw_parser_error("illegal color specification '", s, "'");
-		}
-		return c.l;
-	} else if (str_i_str(s,"RGB")!=NULL) {
-		polish_eval((char*)s,&xx);
-	} else if (*s=='.' || *s=='(' || isdigit(*s)) {
-		strcpy(vv,"cvtgray(");
-		strcat(vv,s); strcat(vv,")");
-		polish_eval(vv,&xx);
-	} else if (strchr(s,'$') != NULL) {
-		strcpy(vv,"cvtcolor(");
-		strcat(vv,s); strcat(vv,")");
-		polish_eval(vv,&xx);
-	} else {
-		if (s[0] == 0) {
-			g_throw_parser_error("expecting color name, but found empty string");
-		} else {
-			string uc_token = s;
-			str_to_uppercase(uc_token);
-			str_remove_quote(uc_token);
-			GLEColor* color = GLEGetColorList()->get(uc_token);
-			if (color != NULL) {
-				return (int)color->getHexValueGLE();
-			} else {
-				int result = 0;
-				if (gt_firstval_err(op_fill_typ, s, &result)) {
-					return result;
-				} else {
-					g_throw_parser_error("found '", uc_token.c_str(), "', but expecting color or fill specification");
-				}
-			}
-		}
-	}
-	int j;
-	memcpy(&j,&xx,sizeof(int));
-	return j;
-}
-
-int pass_color_var(const char *s) throw(ParserError) {
-	if (strchr(s,'$') != NULL) {
-		int idx, typ;
-		char tmpcol[100];
-		string var_str = s;
-		str_to_uppercase(var_str);
-		var_find((char*)var_str.c_str(), &idx, &typ);
-		if (idx >= 0) {
-			var_getstr(idx, tmpcol);
-			return pass_color(tmpcol);
-		} else {
-			g_throw_parser_error("color '", s, "' not defined");
-			return 0;
-		}
-	} else {
-		return pass_color(s);
-	}
-}
-
 struct mark_struct { const char *name; const char *font; int cc; double rx; double ry; double scl; bool center;};
 
 struct mark_struct stdmark_v35[] = {
@@ -2265,7 +2336,7 @@ struct mark_struct stdmark[] = {
 	{ "DDAG",     "TEXSY",   122,0,0,0.79,       true  },   /* dagg */
 	{ "DIAMOND",  "GLEMARK", 10,-0.5,-0.5,0.7,   false },   /* diamond */
 	{ "DIAMONDZ", "TEXSY",   125,0,0,0.81,       true  },   /* diamondz */
-	{ "DOT",      "RM",      46,0,0,3.0,         true  },   /* dot */
+	{ "DOT",      "GLEMARK", 8,-0.5,-0.5,0.3325, false },   /* dot */
 	{ "FCIRCLE",  "GLEMARK", 8,-0.5,-0.5,0.7,    false },   /* fcircle */
 	{ "FDIAMOND", "GLEMARK", 11,-0.5,-0.5,0.7,   false },   /* fdiamond */
 	{ "FLOWER",   "PSZD",    96,0,0.03,0.97,     true  },   /* ZapDingbats */
@@ -2345,32 +2416,6 @@ void mark_clear(void) {
 	}
 }
 
-int pass_marker(char *name) throw(ParserError) {
-	int f = 0;
-	string marker;
-	polish_eval_string(name, &marker);
-	/* if 0, maybe its a user defined marker, ie a subroutine */
-	/* Use -ve to signify subroutine instead of normal marker */
-	for (int i = 0; i < nmark; i++) {
-		if (str_i_equals(mark_name[i], marker.c_str())) {
-			f = -(++i);
-			break;
-		}
-	}
-	if (f == 0)  {
-		for (int i = nmrk-1; i >= 0; i--) {
-			if (str_i_equals(mrk_name[i], marker.c_str())) {
-				f = ++i;
-				break;
-			}
-		}
-	}
-	if (f == 0) {
-		g_throw_parser_error("invalid marker name '", marker.c_str(), "'");
-	}
-	return f;
-}
-
 void pass_file_name(const char* name, string& file) {
 	// Support operations on strings, but make sure it also works with unquoted strings containing "/" chars
 	if (str_contains(name, '"') || str_contains(name, '$') || str_contains(name, '+')) {
