@@ -53,12 +53,17 @@ typedef struct op_key OPKEY[];
 #define GLE_SRCBLK_RETURN 5
 #define GLE_SRCBLK_MAGIC  100
 
-#define GLE_OPBEGIN_IF   6
-#define GLE_OPBEGIN_SUB  7
-#define GLE_OPBEGIN_BOX  2
+#define GLE_OPBEGIN_IF    6
+#define GLE_OPBEGIN_SUB   7
+#define GLE_OPBEGIN_BOX   2
+#define GLE_OPBEGIN_GRAPH 10
+#define GLE_OPBEGIN_KEY   16
+#define GLE_OPBEGIN_SURF  24
 
 class GLESubMap;
 
+void GLEParserInitTokenizer(Tokenizer* tokens);
+
 class GLESourceBlock {
 protected:
 	int m_block_type;
@@ -126,6 +131,8 @@ public:
 	inline GLESub* getSub() { return m_Sub; }
 };
 
+class GLEBlocks;
+
 class GLEParser {
 protected:
 	GLEScript* m_Script;
@@ -133,6 +140,7 @@ protected:
 	StringTokenizer m_tokens;
 	GLEPolish* m_polish;
 	GLESub* m_CrSub;
+	GLEBlocks* m_blockTypes;
 	string m_include;
 	int m_special;
 	bool m_auto_endif;
@@ -141,10 +149,12 @@ protected:
 public:
 	GLEParser(GLEScript* script, GLEPolish* polish);
 	~GLEParser();
+	GLEBlocks* getBlockTypes();
 	void initTokenizer();
 	double evalTokenToDouble() throw(ParserError);
-   void evalTokenToString(string* str) throw(ParserError);
-   void evalTokenToFileName(string* str) throw(ParserError);
+	void evalTokenToString(string* str) throw(ParserError);
+	void evalTokenToFileName(string* str) throw(ParserError);
+	bool pass_block_specific(GLESourceLine& sourceLine, GLEPcode& pcode);
 	void passt(GLESourceLine &SLine, GLEPcode& pcode) throw(ParserError);
 	void polish_eol(GLEPcode& pcode, int *rtype) throw(ParserError);
 	void polish(GLEPcode& pcode, int *rtype) throw(ParserError);
@@ -158,6 +168,8 @@ public:
 	int get_anyexp(GLEPcode& pcode) throw(ParserError);
 	void pass_subroutine_call(GLESubCallInfo* info, int poscol) throw(ParserError);
 	void gen_subroutine_call_code(GLESubCallInfo* info, GLEPcode& pcode) throw(ParserError);
+	void gen_subroutine_call_polish_arg(GLESubCallInfo* info, int i, GLEPcode& pcode) throw(ParserError);
+	void evaluate_subroutine_arguments(GLESubCallInfo* info, GLEArrayImpl* arguments);
 	void get_subroutine_call(GLEPcode& pcode, string* name = NULL, int poscol = 0) throw(ParserError);
 	GLESub* get_subroutine_declaration(GLEPcode& pcode) throw(ParserError);
 	void get_subroutine_default_param(GLESub* sub) throw(ParserError);
@@ -169,6 +181,7 @@ public:
 	void do_endsub(int srclin, GLEPcode& pcode);
 	int get_optional(OPKEY lkey, GLEPcode& pcode) throw(ParserError);
 	int get_first(OPKEY lkey) throw(ParserError);
+   int get_first(const string& token, OPKEY lkey) throw(ParserError);
 	void get_token(const char* token) throw(ParserError);
 	bool try_get_token(const char* token) throw(ParserError);
 	void get_fill(GLEPcode& pcode) throw (ParserError);
@@ -186,7 +199,7 @@ public:
 	void do_text_mode(GLESourceLine &SLine, Tokenizer* tokens, GLEPcode& pcode) throw (ParserError);
 	void checkmode() throw(ParserError);
 	void get_block_type(int type, string& result);
-	ParserError create_option_error(OPKEY lkey, int count, string& token);
+	ParserError create_option_error(OPKEY lkey, int count, const string& token);
 	int get_one_option(op_key* lkey, GLEPcode& pcode, int plen) throw(ParserError);
 	void duplicate_error(GLEPcode& pcode, int pos) throw(ParserError);
 	void checkValidName(const string& name, const char* type, int pos) throw(ParserError);
@@ -221,30 +234,19 @@ public:
 	inline GLEGlobalSource* getSource() { return m_Script->getSource(); }
 };
 
-int gt_first(OPKEY lkey, int *curtok, TOKENS tk, int *ntok, int *pcode, int *plen);
 int gt_firstval(OPKEY lkey, const char *s);
 bool gt_firstval_err(OPKEY lkey, const char *s, int* result);
 int gt_index(OPKEY lkey,char *s);
-int gt_optional(OPKEY lkey, int *curtok, TOKENS tk, int *ntok, int *pcode, int *plen);
-int pass_color(const char *s) throw(ParserError);
-int pass_justify(const char *s);
+int pass_justify(const std::string& s);
 int pass_marker(char *s) throw(ParserError);
-void get_color(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
-void get_fill(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
-void get_font(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
-void get_justify(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
-void get_marker(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen);
-void gt_xy(int *curtok, TOKENS tk, int *ntok,	int *pcode, int *plen);
 void mark_clear(void);
 void pass_checkmode(void);
-//void passt(_itGLESource sc_line_it,TOKENS tk,int *ntok,int *pcode,int *plen);
 void spop(int v);
 void spush(int v);
 void scheck(int v);
 //
 // used in utils
 //
-int zpolish(TOKENS tk,int *ntok,int *curtok,char *pcode,int *plen,int *rtype);
 
 void set_global_parser(GLEParser* parser);
 GLEParser* get_global_parser();
