@@ -55,11 +55,12 @@ extern int gle_debug;
 #include "cutils.h"
 #include "keyword.h"
 #include "run.h"
+#include "fn.h"
 
 /*---------------------------------------------------------------------------*/
 /* bin = 10..29, binstr = 30..49, fn= 60...139, userfn=LOCAL_START_INDEX..nnn */
-#define stack_bin(i,p) 	stack_op(pcode,stk,stkp,&nstk,i-10+(last_typ*20),p+curpri)
-#define stack_fn(i) 	stack_op(pcode,stk,stkp,&nstk,i+60,10+curpri)
+#define stack_bin(i,p) stack_op(pcode, stk, stkp, &nstk, i + BINARY_OPERATOR_OFFSET, p + curpri)
+#define stack_fn(i)    stack_op(pcode, stk, stkp, &nstk, i + FN_BUILTIN_MAGIC, 10 + curpri)
 #define dbg if ((gle_debug & 4)>0)
 
 // #define dbg
@@ -91,6 +92,12 @@ void GLEPolish::initTokenizer() {
 	m_tokens.select_language(0);
 }
 
+void GLEPolish::get_array_index(GLEPcode& pcode) throw(ParserError) {
+	int vtype = 1;
+	internalPolish(pcode, &vtype);
+	m_tokens.ensure_next_token("]");
+}
+
 void GLEPolish::get_params(GLEPcode& pcode, int np, int* plist, const string& name) throw(ParserError) {
 	int nb_param = 0;
 	if (!m_tokens.is_next_token(")")) {
@@ -101,7 +108,7 @@ void GLEPolish::get_params(GLEPcode& pcode, int np, int* plist, const string& na
 				throw error(string("too many parameters in call to '")+name+err_str);
 			}
 			int vtype = *(plist + nb_param);
-			polish(pcode, &vtype);
+			internalPolish(pcode, &vtype);
 			int next_token = m_tokens.is_next_token_in(",)");
 			if (next_token == -1) {
 				throw error(string("expecting ',' or ')' in parameter list of function '")+name+"'");
@@ -118,26 +125,33 @@ void GLEPolish::get_params(GLEPcode& pcode, int np, int* plist, const string& na
 }
 
 void GLEPolish::polish(const char *expr, GLEPcode& pcode, int *rtype) throw(ParserError) {
+	try {
+		internalPolish(expr, pcode, rtype);
+	} catch (ParserError& err) {
+		err.setParserString(expr);
+		throw err;
+	}
+}
+
+void GLEPolish::internalPolish(const char *expr, GLEPcode& pcode, int *rtype) throw(ParserError) {
 	#ifdef DEBUG_POLISH
 		gprint("==== Start of expression {%s} \n",expr);
 	#endif
 	m_tokens.set_string(expr);
-	polish(pcode, rtype);
+	internalPolish(pcode, rtype);
 }
 
-void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
+void GLEPolish::internalPolish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 	GLESub* sub;
 	string uc_token;
 	int idx, ret, np, *plist, term_bracket = false;
 	int curpri = 0;
 	int nstk = 0, stk[50], stkp[50];   /* stack for operators */
 	int unary = 1;                     /* binary or unary operation expected */
-	/* last_typ, 1=number,2=string */
 	bool isa_string = false;
 	bool not_string = false;
 	if (*rtype==1) not_string = true;
 	if (*rtype>0) term_bracket = true;
-	int last_typ = *rtype;
 	pcode.addInt(PCODE_EXPR);   /* Expression follows */
 	int savelen = pcode.size(); /* Used to set acutal length at end */
 	pcode.addInt(0);	    /* Length of expression */
@@ -148,14 +162,14 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 		char first_char = token_len > 0 ? token[0] : ' ';
 		// cout << "Token: '" << token << "'" << endl;
 		// end of stream, or found ',' or ')'
-		if (token_len == 0 || (token_len == 1 && (first_char == ',' || (first_char == ')' && curpri == 0)))) {
+		if (token_len == 0 || (token_len == 1 && (first_char == ',' || (first_char == ')' && curpri == 0) || (first_char == ']' && curpri == 0)))) {
 			if (token_len != 0) {
 				m_tokens.pushback_token();
 			}
-			*rtype = last_typ;
+			*rtype = 0;
 			dbg gprint("Found END OF EXPRESSION \n");
 			if (curpri != 0) {
-				throw error("unexpected end of expression, missing closing ')'");
+				throw error("unexpected end of expression, missing closing ')' or ']'");
 			}
 			/* Pop everything off the stack */
 			for (int i = nstk; i > 0; i--) {
@@ -178,10 +192,6 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 				dbg gprint("Found number {%s}\n",token.c_str());
 				double value = atof(token.c_str());
 				pcode.addDouble(value);
-				if (last_typ == 2) {
-					throw error(token_col, "expecting string, but found number");
-				}
-				last_typ = 1;
 				unary = 2;
 				break;
 			}
@@ -195,11 +205,7 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 				//
 				dbg gprint("Found built in function \n");
 				get_params(pcode, np, plist, uc_token);
-				if (last_typ == (3-ret)) {
-					throw error(token_col, string("illegal function return type: ")+ns[ret]);
-				}
-				last_typ = ret;
-				pcode.addFunction(idx+60);
+				pcode.addFunction(idx + FN_BUILTIN_MAGIC);
 				unary = 2;
 				break;
 			} else if (idx > 0 && idx <= 3) {
@@ -216,7 +222,6 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 //				printf("User cts=%s  idx=%d ret=%d np=%d plist=%d\n",cts,idx,ret,np,plist);
 				dbg gprint("Found user function \n");
 				get_params(pcode, sub->getNbParam(), sub->getParamTypes(), uc_token);
-				if (ret > 0 && ret < 3) last_typ = ret;
 				pcode.addFunction(sub->getIndex()+LOCAL_START_INDEX);
 				unary = 2;
 				break;
@@ -226,11 +231,10 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 			var_find((char*)uc_token.c_str(), &v, &ret);
 			if (v >= 0) {
 				// cout << "found var: '" << uc_token << "' -> " << v << endl;
-				last_typ = ret;
 				if (ret == 2) pcode.addStrVar(v);
 				else pcode.addVar(v);
 				unary = 2;
-				if (m_vars != NULL) {
+				if (m_vars != NULL && m_vars->try_get(uc_token) == -1) {
 					/* Add it to list of vars */
 					m_vars->add_item(uc_token, v);
 				}
@@ -239,18 +243,23 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 			/* Is it a string */
 			if (first_char == '"' || first_char == '\'') {
 				dbg gprint("Found string \n");
-				last_typ = 2;
 				string str_no_quote = token;
 				str_remove_quote(str_no_quote);
 				pcode.addString(str_no_quote);
 				unary = 2;
 				break;
 			}
+			if ((first_char == 'd' || first_char == 'D') && token_len == 1 && m_tokens.is_next_token("[")) {
+				get_array_index(pcode);
+				pcode.addFunction(FN_DI + FN_BUILTIN_MAGIC);
+				unary = 2;
+				break;
+			}
 			if (first_char == '(' && token_len == 1) {
 				curpri = curpri + 100;
 				break;
 			}
-			if (first_char == ')' && token_len == 1) {
+			if ((first_char == ')' || first_char == ')') && token_len == 1) {
 				throw error("constant, function, or unary operator expected");
 			}
 			if (m_tokens.is_next_token("(")) {
@@ -265,19 +274,17 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 					throw error(token_col, "illegal variable name '"+uc_token+"'");
 				}
 				var_findadd((char*)uc_token.c_str(), &v, &ret);
-				last_typ = ret;
 				if (ret == 2) pcode.addStrVar(v);
 				else pcode.addVar(v);
 				not_string = true;
 				unary = 2;
-				if (m_vars != NULL) {
+				if (m_vars != NULL && m_vars->try_get(uc_token) == -1) {
 					/* Add it to list of vars */
 					m_vars->add_item(uc_token, v);
 				}
 				break;
 			}
-			last_typ = 2;
-			dbg printf("Unquoted string (%s) \n",token.c_str());
+			// std::cout << "Unquoted string '" << token << "'" << std::endl;
 			pcode.addString(token);
 			if (!valid_unquoted_string(token)) {
 				throw error(token_col, "invalid unquoted string '"+token+"'");
@@ -299,43 +306,39 @@ void GLEPolish::polish(GLEPcode& pcode, int *rtype) throw(ParserError) {
 			int priority = 0;
 			if (token_len == 1) {
 				switch (first_char) {
-					case '+' : v = 1;  priority = 2; break;
-					case '-' : v = 2;  priority = 2; break;
-					case '*' : v = 3;  priority = 3; break;
-					case '/' : v = 4;  priority = 3; break;
-					case '%' : v = 14; priority = 3; break;
-					case '^' : v = 5;  priority = 4; break;
-					case '=' : v = 6;  priority = 1; break;
-					case '&' : v = 12; priority = 1; break;
-					case '|' : v = 13; priority = 1; break;
-					case '<' : v = 7;  priority = 1; break;
-					case '>' : v = 9;  priority = 1; break;
-					case '.' : v = 15;  priority = 2; break;
+					case '+' : v = BIN_OP_PLUS;  priority = 2; break;
+					case '-' : v = BIN_OP_MINUS;  priority = 2; break;
+					case '*' : v = BIN_OP_MULTIPLY;  priority = 3; break;
+					case '/' : v = BIN_OP_DIVIDE;  priority = 3; break;
+					case '%' : v = BIN_OP_MOD; priority = 3; break;
+					case '^' : v = BIN_OP_POW;  priority = 4; break;
+					case '=' : v = BIN_OP_EQUALS;  priority = 1; break;
+					case '&' : v = BIN_OP_AND; priority = 1; break;
+					case '|' : v = BIN_OP_OR; priority = 1; break;
+					case '<' : v = BIN_OP_LT;  priority = 1; break;
+					case '>' : v = BIN_OP_GT;  priority = 1; break;
+					case '.' : v = BIN_OP_DOT;  priority = 2; break;
 					default  : v = 0;
 				}
 			} else {
 				str_to_uppercase(token, uc_token);
 				if (token == "<=") {
-					v = 8; priority = 1;
+					v = BIN_OP_LE; priority = 1;
 				} else if (token == "<>") {
-					v = 11; priority = 1;
+					v = BIN_OP_NOT_EQUALS; priority = 1;
 				} else if (token == ">=") {
-					v = 10; priority = 1;
+					v = BIN_OP_GE; priority = 1;
 				} else if (token == "**") {
-					v = 5;  priority = 4;
+					v = BIN_OP_POW;  priority = 4;
 				} else if (uc_token == "AND") {
-					v = 12; priority = 1;
+					v = BIN_OP_AND; priority = 1;
 				} else if (uc_token == "OR") {
-					v = 13; priority = 1;
+					v = BIN_OP_OR; priority = 1;
 				} else {
 					v = 0;
 				}
 			}
 			if (v > 0) {
-				if (last_typ < 1 || last_typ > 3) {
-					last_typ = 1;
-				}
-				dbg gprint("stack, i %d, type %d \n",v,last_typ);
 				stack_bin(v, priority);
 				dbg gprint("Found binary operator \n");
 				unary = 1;
@@ -360,70 +363,52 @@ Tokenizer* GLEPolish::getTokens(const string& str) {
 	return &m_tokens;
 }
 
-void GLEPolish::eval(const char *exp, double *x) throw(ParserError) {
-	int rtype = 1, otyp = 0, cp = 0;
-	GLEPcodeList pc_list;
-	GLEPcode pcode(&pc_list);
-	// cout << "eval '" << exp << "'" << endl;
-	try {
-		polish(exp, pcode, &rtype);
-	} catch (ParserError err) {
-		err.setParserString(exp);
-		throw err;
-	}
-	::eval((int*)&pcode[0], &cp, x, NULL, &otyp);
-}
-
 void GLEPolish::internalEval(const char *exp, double *x) throw(ParserError) {
-	int rtype = 1, otyp = 0, cp = 0;
+	// difference with eval: no try / catch
+	int rtype = 1, cp = 0;
 	GLEPcodeList pc_list;
 	GLEPcode pcode(&pc_list);
-	polish(exp, pcode, &rtype);
-	::eval((int*)&pcode[0], &cp, x, NULL, &otyp);
+	internalPolish(exp, pcode, &rtype);
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	*x = evalDouble(stk.get(), &pc_list, (int*)&pcode[0], &cp);
 }
 
 void GLEPolish::internalEvalString(const char* exp, string* str) throw(ParserError) {
-   double oval;
-	const char* ostr;
-	int rtype = 2, otyp = 0, cp = 0;
+	// difference with eval_string: no try / catch
+	int rtype = 2, cp = 0;
+	GLEPcodeList pc_list;
+	GLEPcode pcode(&pc_list);
+	internalPolish(exp, pcode, &rtype);
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	GLERC<GLEString> result(::evalString(stk.get(), &pc_list, (int*)&pcode[0], &cp, true));
+	*str = result->toUTF8();
+}
+
+void GLEPolish::eval(GLEArrayImpl* stk, const char *exp, double *x) throw(ParserError) {
+	int rtype = 1, cp = 0;
 	GLEPcodeList pc_list;
 	GLEPcode pcode(&pc_list);
 	polish(exp, pcode, &rtype);
-	::eval((int*)&pcode[0], &cp, &oval, &ostr, &otyp);
-	if (otyp == 1) {
-		stringstream str_cnv;
-		str_cnv << oval;
-		*str = str_cnv.str();
-	} else {
-		*str = ostr;
-	}
+	*x = evalDouble(stk, &pc_list, (int*)&pcode[0], &cp);
 }
 
-void GLEPolish::eval_string(const char *exp, string *str, bool allownum) throw(ParserError) {
-	const char* ostr;
-	double oval = 0.0;
+void GLEPolish::evalString(GLEArrayImpl* stk, const char *exp, string *str, bool allownum) throw(ParserError) {
 	int rtype = allownum ? 0 : 2;
-	int otyp = 0, cp = 0;
+	int cp = 0;
 	GLEPcodeList pc_list;
 	GLEPcode pcode(&pc_list);
-	try {
-		polish(exp, pcode, &rtype);
-	} catch (ParserError err) {
-		err.setParserString(exp);
-		throw err;
-	}
-	::eval((int*)&pcode[0], &cp, &oval, &ostr, &otyp);
-	if (otyp == 1) {
-		if (allownum) {
-			stringstream str_cnv;
-			str_cnv << oval;
-			*str = str_cnv.str();
-		} else {
-			throw error(string("expression does not evaluate to string '")+exp+"'");
-		}
-	} else {
-		*str = ostr;
-	}
+	polish(exp, pcode, &rtype);
+	GLERC<GLEString> result(::evalString(stk, &pc_list, (int*)&pcode[0], &cp, allownum));
+	*str = result->toUTF8();
+}
+
+GLEMemoryCell* GLEPolish::evalGeneric(GLEArrayImpl* stk, const char *exp) throw(ParserError) {
+	int cp = 0;
+	int rtype = 0;
+	GLEPcodeList pc_list;
+	GLEPcode pcode(&pc_list);
+	polish(exp, pcode, &rtype);
+	return ::evalGeneric(stk, &pc_list, (int*)&pcode[0], &cp);
 }
 
 bool valid_unquoted_string(const string& str) {
@@ -452,58 +437,92 @@ void stack_op(GLEPcode& pcode, int stk[], int stkp[], int *nstk,  int i, int p)
 void polish(char *expr, GLEPcode& pcode, int *rtype) throw(ParserError) {
 	GLEPolish* polish = get_global_polish();
 	if (polish != NULL) {
-		try {
-			polish->polish(expr, pcode, rtype);
-		} catch (ParserError err) {
-			err.setParserString(expr);
-			throw err;
-		}
+		polish->polish(expr, pcode, rtype);
 	}
 }
 
 void eval_pcode(GLEPcode& pcode, double* x) {
-	int otyp = 0, cp = 0;
-	::eval((int*)&pcode[0], &cp, x, NULL, &otyp);
+	int cp = 0;
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	*x = evalDouble(stk.get(), pcode.getPcodeList(), (int*)&pcode[0], &cp);
 }
 
 void eval_pcode_str(GLEPcode& pcode, string& x) {
-	const char* ostr;
-	int otyp = 1, cp = 0;
-	double value;
-	::eval((int*)&pcode[0], &cp, &value, &ostr, &otyp);
-	x = ostr;
-}
-
-void polish(char *expr, char *pcode, int *plen, int *rtype) throw(ParserError) {
-	GLEPolish* polish = get_global_polish();
-	if (polish != NULL) {
-		GLEPcodeList pc_list;
-		GLEPcode my_pcode(&pc_list);
-		try {
-			polish->polish(expr, my_pcode, rtype);
-		} catch (ParserError err) {
-			err.setParserString(expr);
-			throw err;
-		}
-		*plen = my_pcode.size();
-		memcpy(pcode,&my_pcode[0],my_pcode.size()*sizeof(int));
-	}
+	int cp = 0;
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	GLERC<GLEString> result(::evalString(stk.get(), pcode.getPcodeList(), (int*)&pcode[0], &cp, true));
+	x = result->toUTF8();
 }
 
 void polish_eval(char *exp, double *x) throw(ParserError) {
 	GLEPolish* polish = get_global_polish();
-	if (polish != NULL) polish->eval(exp, x);
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	if (polish != NULL) polish->eval(stk.get(), exp, x);
 }
 
 void polish_eval_string(const char *exp, string *str, bool allownum) throw(ParserError) {
 	GLEPolish* polish = get_global_polish();
-	if (polish != NULL) polish->eval_string(exp, str, allownum);
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	if (polish != NULL) polish->evalString(stk.get(), exp, str, allownum);
+}
+
+std::string gle_operator_to_string(int op) {
+	switch (op) {
+		case BIN_OP_PLUS:
+			return "+";
+		case BIN_OP_MINUS:
+			return "-";
+		case BIN_OP_MULTIPLY:
+			return "*";
+		case BIN_OP_DIVIDE:
+			return "/";
+		case BIN_OP_POW:
+			return "^";
+		case BIN_OP_EQUALS:
+			return "=";
+		case BIN_OP_LT:
+			return "<";
+		case BIN_OP_LE:
+			return "<=";
+		case BIN_OP_GT:
+			return ">";
+		case BIN_OP_GE:
+			return ">=";
+		case BIN_OP_NOT_EQUALS:
+			return "<>";
+		case BIN_OP_AND:
+			return "AND";
+		case BIN_OP_OR:
+			return "OR";
+		case BIN_OP_MOD:
+			return "%";
+		case BIN_OP_DOT:
+			return ".";
+		default:
+			break;
+	}
+	{
+		std::ostringstream msg;
+		msg << "OP" << op;
+		return msg.str();
+	}
 }
 
 GLEPcode::GLEPcode(GLEPcodeList* list) {
 	m_PCodeList = list;
 }
 
+void GLEPcode::addColor(GLEColor* color) {
+	addInt(PCODE_EXPR);
+	int savelen = size(); /* Used to set acutal length at end */
+	addInt(0);	          /* Length of expression */
+	addInt(PCODE_OBJECT);
+	int pos = getPcodeList()->size();
+	getPcodeList()->push_back(color);
+	addInt(pos);
+	setInt(savelen, size() - savelen - 1);
+}
+
 void GLEPcode::addDoubleExpression(double val) {
 	addInt(PCODE_EXPR);
 	int savelen = size();
@@ -616,15 +635,10 @@ GLEFunctionParserPcode::~GLEFunctionParserPcode() {
 void GLEFunctionParserPcode::polish(const char* fct, StringIntHash* vars) throw(ParserError) {
 	GLEPolish* polish = get_global_polish();
 	if (polish != NULL) {
-		try {
-			int rtype = 1;
-			polish->setExprVars(vars);
-			polish->polish(fct, m_Pcode, &rtype);
-			polish->setExprVars(NULL);
-		} catch (ParserError err) {
-			err.setParserString(fct);
-			throw err;
-		}
+		int rtype = 1;
+		polish->setExprVars(vars);
+		polish->polish(fct, m_Pcode, &rtype);
+		polish->setExprVars(NULL);
 	}
 }
 
@@ -634,9 +648,9 @@ void GLEFunctionParserPcode::polishPos(const char* fct, int pos, StringIntHash*
 		try {
 			int rtype = 1;
 			polish->setExprVars(vars);
-			polish->polish(fct, m_Pcode, &rtype);
+			polish->internalPolish(fct, m_Pcode, &rtype);
 			polish->setExprVars(NULL);
-		} catch (ParserError err) {
+		} catch (ParserError& err) {
 			err.incColumn(pos-1);
 			throw err;
 		}
@@ -652,3 +666,9 @@ double GLEFunctionParserPcode::evalDouble() {
 	eval_pcode(m_Pcode, &value);
 	return value;
 }
+
+bool GLEFunctionParserPcode::evalBool() {
+	int cp = 0;
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
+	return ::evalBool(stk.get(), m_Pcode.getPcodeList(), (int*)&m_Pcode[0], &cp);
+}
