@@ -68,7 +68,7 @@ extern int *gplen;     /* gpcode is a pointer to an array of int */
 
 string get_b_name(int jj) {
 	for (int i=0; op_begin[i].typ!=0; i++) {
-		if( op_begin[i].idx == jj){
+		if(int(op_begin[i].idx) == jj){
 			return string(op_begin[i].name);
 		}
 	}
@@ -78,27 +78,18 @@ string get_b_name(int jj) {
 void text_def(uchar *ss);
 void run_bigfile(char *ss);
 
-void begin_config(const char* block, int *pln, int *pcode, int *cp);
+void begin_config(const std::string& block, int *pln, int *pcode, int *cp);
 void begin_tex_preamble(int *pln, int *pcode, int *cp);
 void begin_tex(GLERun* run, int *pln, int *pcode, int *cp);
 void begin_surface(int *pln, int *pcode, int *cp) throw(ParserError);
-void begin_letz(int *pln, int *pcode, int *cp) throw(ParserError);
+void begin_letz(int *pln, GLEPcodeList* pclist, int *pcode, int *cp) throw(ParserError);
 void begin_fitz(int *pln, int *pcode, int *cp) throw(ParserError);
 void begin_contour(int *pln, int *pcode, int *cp) throw(ParserError);
 // void begin_fitls(int *pln, int *pcode, int *cp) throw(ParserError);
 
 class GLEBox {
-protected:
-	const char* m_Name; // Should be replaced by string* later on
-	bool m_HasStroke;
-	bool m_HasReverse;
-	double m_Add;
-	bool m_IsRound;
-	double m_Round;
-	GLERC<GLEColor> m_Fill;
 public:
 	GLEBox();
-	GLEBox(const GLEBox& box);
 	void setFill(const GLERC<GLEColor>& fill);
 	void setRound(double round);
 	void draw(GLERun* run, double x1, double y1, double x2, double y2);
@@ -110,29 +101,24 @@ public:
 	inline GLERC<GLEColor> getFill() const { return m_Fill; }
 	inline bool isRound() const { return m_IsRound; }
 	inline double getRound() const { return m_Round; }
-	inline void setNamePtr(const char* name) { m_Name = name; }
-	inline bool hasName() const { return m_Name != NULL; }
-	inline const char* getName() const { return m_Name; }
+	inline void setName(GLEString* name) { m_name = name; }
+	inline bool hasName() const { return !m_name.isNull(); }
+	inline GLEString* getName() const { return m_name.get(); }
 	inline void setAdd(double add) { m_Add = add; }
 	inline double getAdd() const { return m_Add; }
+protected:
+	bool m_HasStroke;
+	bool m_HasReverse;
+	double m_Add;
+	bool m_IsRound;
+	double m_Round;
+	GLERC<GLEString> m_name;
+	GLERC<GLEColor> m_Fill;
 };
 
 class GLEStoredBox : public GLEBox {
-public:
-	GLERectangle m_SaveBounds;
-	GLEPoint m_Orig;
-protected:
-	string m_Name;
-	bool m_HasName;
-	bool m_SecondPass;
-	GLEDevice* m_Device;
-	GLERC<GLEObjectRepresention> m_Object;
 public:
 	GLEStoredBox();
-	GLEStoredBox(const GLEStoredBox& box);
-	void setName(const char* name);
-	inline bool hasName() const { return m_HasName; }
-	inline const string& getName() const { return m_Name; }
 	inline bool isSecondPass() const { return m_SecondPass; }
 	inline void setSecondPass(bool second) { m_SecondPass = second; }
 	inline GLEDevice* getDevice() { return m_Device; }
@@ -143,6 +129,12 @@ public:
 	inline GLERectangle* getSaveBounds() { return &m_SaveBounds; }
 	void setOrigin(double x0, double y0) { m_Orig.setXY(x0, y0); }
 	const GLEPoint& getOrigin() { return m_Orig; }
+private:
+	GLERectangle m_SaveBounds;
+	GLEPoint m_Orig;
+	bool m_SecondPass;
+	GLEDevice* m_Device;
+	GLERC<GLEObjectRepresention> m_Object;
 };
 
 class GLEBoxStack {
@@ -190,9 +182,9 @@ public:
 	void gotoNewLine() throw(ParserError);
 	void resetLang();
 	void setLangChars(int type, const char* str);
-	inline void setCommentChars(const char* str) { setLangChars(0, str); }
-	inline void setSpaceTokens(const char* str) { setLangChars(1, str); }
-	inline void setSingleCharTokens(const char* str) { setLangChars(2, str); }
+	inline void setCommentChars(const std::string& str) { setLangChars(0, str.c_str()); }
+	inline void setSpaceTokens(const std::string& str) { setLangChars(1, str.c_str()); }
+	inline void setSingleCharTokens(const std::string& str) { setLangChars(2, str.c_str()); }
 	inline bool isRead() { return m_ReadWrite; }
 	inline void setRdWr(bool rd) { m_ReadWrite = rd; }
 	inline FILE* getOutput() { return m_Output; }
@@ -232,11 +224,12 @@ extern int gle_debug;
 int can_fillpath = false;
 vector<int> g_drobj;
 
-#define readval(x) eval(pcode,&cp,&x,&ostr,&otyp)
-#define readxy(x,y) {eval(pcode,&cp,&x,&ostr,&otyp);eval(pcode,&cp,&y,&ostr,&otyp);}
-#define readstr(s) {eval(pcode,&cp,&x,&ostr,&otyp); s = ostr;}
+#define readval(x) x=evalDouble(getStack(),getPcodeList(),pcode,&cp)
+#define readxy(x,y) {x=evalDouble(getStack(),getPcodeList(),pcode,&cp); y=evalDouble(getStack(),getPcodeList(),pcode,&cp);}
+#define readstr(s) {ostr=evalStringPtr(getStack(),getPcodeList(),pcode,&cp); s=ostr->toUTF8();}
 #define readlong(i) i = *(pcode+cp++)
-#define readvalp(x,p) {zzcp=0; eval(p,&zzcp,&x,&ostr,&otyp);}
+#define readvalp(x,p) {zzcp=0; x=evalDouble(getStack(),getPcodeList(),p,&zzcp);}
+#define readstrp(p) {zzcp=0; ostr=evalStringPtr(getStack(),getPcodeList(),p,&zzcp);}
 
 #define PCODE_UNKNOWN_COMMAND 1
 
@@ -250,11 +243,10 @@ void byte_code_error(int err) throw(ParserError) {
 }
 
 void clear_run() {
-	char ss[500];
 	npath = 0;
 	done_open = false;
-	g_get_type(ss);
-	if (strstr(ss,"FILLPATH")!=NULL) can_fillpath = true;
+	std::string ss(g_get_type());
+	if (strstr(ss.c_str(), "FILLPATH")!=NULL) can_fillpath = true;
 	else can_fillpath = false;
 	g_drobj.clear();
 }
@@ -311,7 +303,7 @@ bool tryHandleChangedPropertiesPrevSet(GLEGlobalSource* source, vector<GLEProper
 				ss << " " << set_opt << " " << set_value;
 			}
 		}
-	} catch (ParserError err) {
+	} catch (ParserError& err) {
 		return false;
 	}
 	for (vector<GLEProperty*>::size_type i = 0; i < changed.size(); i++) {
@@ -465,15 +457,18 @@ void handleNewDrawObject(GLEDrawObject* obj, bool mkdrobjs, GLEPoint* orig = NUL
 	}
 }
 
-GLERun::GLERun(GLEScript* script, GLEFileLocation* outfile) {
+GLERun::GLERun(GLEScript* script, GLEFileLocation* outfile, GLEPcodeIndexed* pcode) {
 	m_Script = script;
 	m_OutFile = outfile;
 	m_Vars = getVarsInstance();
 	m_CrObj = new GLEObjectRepresention();
+	m_stack = new GLEArrayImpl();
 	m_blockTypes = 0;
+	m_pcode = pcode;
 	for (int i = 0; i < GLE_KW_NB; i++) {
 		m_AllowBeforeSize[i] = false;
 	}
+	GLE_MC_INIT(m_returnValue);
 	allowBeforeSize(GLE_KW_ASSIGNMENT);
 	allowBeforeSize(GLE_KW_BITMAP_INFO);
 	allowBeforeSize(GLE_KW_BLANK);
@@ -509,11 +504,16 @@ GLERun::GLERun(GLEScript* script, GLEFileLocation* outfile) {
 	allowBeforeSize(GLE_KW_SUB);
 	allowBeforeSize(GLE_KW_UNTIL);
 	allowBeforeSize(GLE_KW_WHILE);
+	allowBeforeSize(GLE_KW_END);
 }
 
 GLERun::~GLERun() {
 }
 
+GLEPcodeList* GLERun::getPcodeList() {
+	return m_pcode->getPcodeList();
+}
+
 void GLERun::setDeviceIsOpen(bool open) {
 	done_open = open;
 }
@@ -531,12 +531,14 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 /* pcode =  a pointer to the pcode output buffer */
 /* plne =   a pointer to the length of the pcode output */
 	union {double d; int l; int ll[2];} both;
-	int otyp,cp=*pend,i,zzcp;
-	const char* ostr;
+	int cp=*pend,i,zzcp;
+	GLEString* ostr;
+	GLERC<GLEString> gstr;
 	string temp_str;
 	double x,y,ox,oy,x1,y1,x2,y2,x3,y3,a1,a2,r,z,rx,ry;
 	int t,j,jj,jj2,ptr,ptr_fill,mask_just,mask_nostroke,marrow;
-	static char ss[255],ss2[80],ss1[90];
+	static char ss[255];
+    string ss1, ss2;
 	static bool jump_back = false;
 	GLEPoint orig;
 	GLERC<GLEColor> colorBackup;
@@ -651,9 +653,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			break;
 		  case 51: /* Assignment  var=exp */
 			readlong(jj);
-			readval(x);
-			if (otyp==1) var_set(jj,x);
-			if (otyp==2) var_setstr(jj, (char*)ostr);
+			getVars()->set(jj, evalGeneric(getStack(), getPcodeList(), pcode, &cp));
 			break;
 		  case 5:  /* BEGIN box | path | scale | rotate | EXTERNAL */
 			g_flush();
@@ -672,8 +672,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 						path_fill[npath] = g_get_fill_clear();
 					}
 					if (ptr) {
-						readvalp(z,pcode+cp+ptr);
-						path_fill[npath] = color_from_double_encoding(z);
+						path_fill[npath] = evalColor(getStack(), getPcodeList(), pcode + cp + ptr, 0);
 					}
 					cp++;
 					g_set_path(true);
@@ -690,8 +689,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 						}
 						ptr = *(pcode + ++cp);
 						if (ptr) {
-							readvalp(z,pcode+cp+ptr);
-							box->setFill(color_from_double_encoding(z));
+							box->setFill(evalColor(getStack(), getPcodeList(), pcode + cp + ptr, 0));
 						}
 						if (*(pcode + ++cp)) {
 							box->setStroke(false);
@@ -703,7 +701,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 						}
 						ptr = *(pcode + ++cp);
 						if (ptr) {
-							readvalp(z,pcode+cp+ptr);
+							readstrp(pcode+cp+ptr);
 							box->setName(ostr);
 						}
 						if (box->isFilled() && !g_is_dummy_device()) {
@@ -744,8 +742,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					{
 						GLEStoredBox* box = box_start();
 						box->setStroke(false);
-						readval(z);
-						box->setName(ostr);
+						box->setName(evalStringPtr(getStack(), getPcodeList(), pcode, &cp));
 						ptr = *(pcode + cp);
 						if (ptr) {
 							readvalp(z, pcode+cp+ptr);
@@ -785,14 +782,14 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					g_move(0.0,0.0);
 					break;
 				case 22: /* config */
-					readval(z);
-					begin_config(ostr,srclin,pcode,&cp);
+					ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+					begin_config(ostr->toUTF8(), srclin, pcode, &cp);
 					break;
 				case 23: /* tex preamble */
 					begin_tex_preamble(srclin,pcode,&cp);
 					break;
 				case 25: /* letz */
-					begin_letz(srclin,pcode,&cp);
+					begin_letz(srclin, getPcodeList(), pcode, &cp);
 					break;
 				case 26: /* fitz */
 					begin_fitz(srclin,pcode,&cp);
@@ -815,8 +812,8 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 							done_open = true;
 						}
 						readlong(jj);
-						readval(z);
-						begin_object(ostr, getSubroutines()->get(jj));
+						ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+						begin_object(ostr->toUTF8(), getSubroutines()->get(jj));
 					} else {
 						/* statical object - identical to sub */
 						readlong(jj);
@@ -852,8 +849,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				}
 				ptr = *(pcode + ++cp);
 				if (ptr) {
-					readvalp(z,pcode+cp+ptr);
-					box.setFill(color_from_double_encoding(z));
+					box.setFill(evalColor(getStack(), getPcodeList(), pcode + cp + ptr, 0));
 				}
 				if (*(pcode + ++cp)) {
 					box.setStroke(false);
@@ -868,14 +864,14 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				}
 				ptr = *(pcode + ++cp);
 				if (ptr) {
-					readvalp(z,pcode+cp+ptr);
-					box.setNamePtr(ostr);
+					readstrp(pcode+cp+ptr);
+					box.setName(ostr);
 				}
 				box.draw(this, ox, oy, x, y);
 			}
 			break;
 		  case 52:  /* CALL or @ */
-			readval(r);
+			evalGeneric(getStack(), getPcodeList(), pcode, &cp);
 			break;
 		  case 8:  /* CIRCLE */
 			readval(r);
@@ -891,8 +887,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			colorBackup = g_get_fill();
 			ptr_fill = *(pcode + cp);
 			if (ptr_fill) {
-				readvalp(z,pcode + cp + ptr_fill);
-				g_set_fill(color_from_double_encoding(z));
+				g_set_fill(evalColor(getStack(), getPcodeList(), pcode + cp + ptr_fill, 0));
 			}
 			if (mkdrobjs) {
 				GLEEllipseDO drawobj(ox, oy, r);
@@ -919,8 +914,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			mask_nostroke = *(pcode + cp++);
 			ptr_fill = *(pcode + cp);
 			if (ptr_fill) {
-				readvalp(z,pcode + cp + ptr_fill);
-				g_set_fill(color_from_double_encoding(z));
+				g_set_fill(evalColor(getStack(), getPcodeList(), pcode + cp + ptr_fill, 0));
 			}
 			if (mkdrobjs) {
 				GLEEllipseDO drawobj(ox, oy, rx, ry);
@@ -980,13 +974,13 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			g_closepath();
 			break;
 		  case 10: /* CURVE  x y x y ...  change to BEGIN CURVE ... END CURVE */
-			g_curve(pcode+cp);
+			g_curve(getPcodeList(), pcode + cp);
 			break;
 		  case 11: /* DEFINE  MARKER name  subname */
 			break;
 		  case 12: /* DFONT */
-			readval(x);
-			g_dfont(ostr);
+			ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+			g_dfont(ostr->toUTF8());
 			break;
 		  case 14: /* END */
 			readlong(jj);
@@ -1117,14 +1111,11 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					readlong(t);
 					if (t!=49) gprint("WRITE, PCODE ERROR, %d  cp %d plen %d\n",t,cp,plen);
 					readlong(t);
-					if (t==1) {
-						readval(x);
-						sprintf(ss,"%g ",x);
-						temp_str += ss;
-					} else {
-						readval(x);
-						temp_str += ostr;
+					gstr = evalString(getStack(), getPcodeList(), pcode, &cp, true);
+					if (!temp_str.empty()) {
+						temp_str += " ";
 					}
+					temp_str += gstr->toUTF8();
 				}
 				if (p==64) temp_str += "\n";
 				fprintf(file->getOutput(),"%s",temp_str.c_str());
@@ -1150,12 +1141,12 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				if (chn == -1) break;
 				GLEFile* file = g_Files[chn];
 				file->resetLang();
-				readval(x);
-				file->setCommentChars(ostr);
-				readval(x);
-				file->setSpaceTokens(ostr);
-				readval(x);
-				file->setSingleCharTokens(ostr);
+				ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+				file->setCommentChars(ostr->toUTF8());
+				ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+				file->setSpaceTokens(ostr->toUTF8());
+				ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+				file->setSingleCharTokens(ostr->toUTF8());
 			} break;
 		  case 77: /* papersize */
   			if (done_open) error_before_drawing_cmds("papersize");
@@ -1211,14 +1202,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 		  case 21: /* ICON */
 			break;
 		  case 22: /* IF EXP */
-			readval(x);
-			readlong(jj);   /* jump to line */
-			readlong(jj2);  /* jump pcode offset */
-			if (x == 0) {
-				*pend = jj2;
-				*srclin = jj-1;
-				return;
-			}
+		  	{
+		  		bool ifValue = evalBool(getStack(), getPcodeList(), pcode, &cp);
+				readlong(jj);   /* jump to line */
+				readlong(jj2);  /* jump pcode offset */
+				if (!ifValue) {
+					*pend = jj2;
+					*srclin = jj-1;
+					return;
+				}
+		  	}
 			break;
 		  case 23: /* INCLUDE (done in pass,  already included) */
 			break;
@@ -1226,19 +1219,17 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 		  	break;
 		  case 25: /* JOIN  str1,type,str2 */
 			{
-				readval(z);
-				strcpy(ss1,ostr);
+				GLERC<GLEString> s1(evalStringPtr(getStack(), getPcodeList(), pcode, &cp));
 				readlong(jj);
-				readval(z);
-				strcpy(ss2,ostr);
+				GLERC<GLEString> s2(evalStringPtr(getStack(), getPcodeList(), pcode, &cp));
 				ptr = *(pcode + cp); /* curve angle1 angle2 d1 d2 */
 				if (ptr) {
 					cp += ptr;
 					readxy(x2, y2);
 					readxy(x3, y3);
-					name_join(ss1, ss2, (int)jj, x2, y2, x3, y3);
+					name_join(s1.get(), s2.get(), (int)jj, x2, y2, x3, y3);
 				} else {
-					name_join(ss1, ss2, (int)jj, 0, 0, 0, 0);
+					name_join(s1.get(), s2.get(), (int)jj, 0, 0, 0, 0);
 				}
 			}
 			break;
@@ -1254,8 +1245,8 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			break;
 		  case 27: /* MOVE  name */
 		  	{
-		  		readval(z);
 		  		GLEPoint pt;
+		  		ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
 		  		name_to_point(ostr, &pt);
 		  		g_move(pt);
 		  		break;
@@ -1327,8 +1318,8 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				int bm_type = 0;
 				ptr = *(pcode + cp); /* type */
 				if (ptr) {
-					readvalp(z, pcode + cp + ptr);
-					bm_type = g_bitmap_string_to_type(ostr);
+					readstrp(pcode + cp + ptr);
+					bm_type = g_bitmap_string_to_type(ostr->toUTF8());
 				}
 				g_bitmap(temp_str, x1, y1, bm_type);
 			}
@@ -1340,8 +1331,8 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				int bm_type = 0;
 				ptr = *(pcode + cp); /* type */
 				if (ptr) {
-					readvalp(z, pcode + cp + ptr);
-					bm_type = g_bitmap_string_to_type(ostr);
+					readstrp(pcode + cp + ptr);
+					bm_type = g_bitmap_string_to_type(ostr->toUTF8());
 				}
 				g_bitmap_info(temp_str, jj, jj2, bm_type);
 			}
@@ -1349,12 +1340,13 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 		  case GLE_KW_COLORMAP:
 		  	{
 				GLEColorMap map;
-				readval(x);
-				map.setFunction(ostr);
+				GLEToRectangularView view;
+				ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+				map.setFunction(ostr->toUTF8());
 				readxy(x1, y1);
-				map.setXRange(x1, y1);
+				view.setXRange(x1, y1);
 				readxy(x1, y1);
-				map.setYRange(x1, y1);
+				view.setYRange(x1, y1);
 				readxy(x1, y1);
 				map.setWidth((int)floor(x1+0.5));
 				map.setHeight((int)floor(y1+0.5));
@@ -1365,14 +1357,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 				}
 				ptr = *(pcode + ++cp); /* palette */
 				if (ptr) {
-					readvalp(z, pcode + cp + ptr);
-					temp_str = ostr;
+					readstrp(pcode + cp + ptr);
+					temp_str = ostr->toUTF8();
 					// because palette can be subroutine name!
 					str_to_uppercase(temp_str);
 					map.setPalette(temp_str.c_str());
 				}
 				g_get_xy(&ox,&oy);
-				map.draw(ox, oy, x1, y1);
+				view.setOrigin(GLEPoint(ox, oy));
+				view.setSize(GLEPoint(x1, y1));
+				map.draw(&view, ox, oy, x1, y1);
 			}
 			break;
 		  case 33: /* RBEZIER */
@@ -1387,12 +1381,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 		  case 34: /* REGION */
 			break;
 		  case 50: /* RETURN exp */
-			readval(x);
-			if (otyp == 1) {
-				sub_set_return(x);
-			} else {
-				sub_set_return_str(ostr);
-			}
+			GLE_MC_COPY(&m_returnValue, evalGeneric(getStack(), getPcodeList(), pcode, &cp));
 			readlong(jj);   /* jump to line */
 			*srclin = jj-1;
 			break;
@@ -1429,7 +1418,7 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			break;
 		  case 39: /* SAVE  name */
 			g_get_xy(&x,&y);
-			readval(z);
+			ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
 			name_set(ostr,x,y,x,y);
 			break;
 		  case 40: /* SCALE */
@@ -1457,20 +1446,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					g_set_just(both.l);
 					break;
 				  case 4: /* color */
-					readval(x);
-					g_set_color(color_from_double_encoding(x));
+					g_set_color(evalColor(getStack(), getPcodeList(), pcode, &cp));
 					break;
 				  case OP_SET_BACKGROUND: /* background */
-					readval(x);
-					g_set_background(color_from_double_encoding(x));
+					g_set_background(evalColor(getStack(), getPcodeList(), pcode, &cp));
 					break;
 				  case OP_SET_FILL: /* fill */
-					readval(x);
-					g_set_fill(color_from_double_encoding(x));
+					g_set_fill(evalColor(getStack(), getPcodeList(), pcode, &cp));
 					break;
 				  case OP_SET_FILL_PATTERN: /* fill pattern */
-					readval(x);
-					g_set_fill_pattern(color_from_double_encoding(x));
+					g_set_fill_pattern(evalColor(getStack(), getPcodeList(), pcode, &cp));
 					break;
 				  case 5: /* dashlen */
 					readval(x);
@@ -1499,16 +1484,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					g_set_line_cap(jj);
 					break;
 				  case OP_SET_FILL_METHOD:
-					readval(x);
-				  	g_set_fill_method(ostr);
+					readstr(ss1);
+				  	g_set_fill_method(ss1.c_str());
 				  	break;
 				  case OP_SET_ARROW_STYLE:
-					readval(x);
-				  	g_set_arrow_style(ostr);
+					readstr(ss1);
+				  	g_set_arrow_style(ss1.c_str());
 				  	break;
 				  case OP_SET_ARROW_TIP:
-					readval(x);
-					g_set_arrow_tip(ostr);
+					readstr(ss1);
+					g_set_arrow_tip(ss1.c_str());
 					break;
 				  case OP_SET_ARROW_SIZE:
 				  	readval(x);
@@ -1519,12 +1504,12 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 					g_set_arrow_angle(x);
 					break;
 				  case OP_SET_IMAGE_FORMAT:
-					readval(x);
-					g_set_pdf_image_format(ostr);
+					readstr(ss1);
+					g_set_pdf_image_format(ss1.c_str());
 					break;
 				  case OP_SET_TEX_SCALE:
-					readval(x);
-					g_set_tex_scale(ostr);
+					readstr(ss1);
+					g_set_tex_scale(ss1.c_str());
 					break;
 				  case OP_SET_TEX_LABELS:
 				  	readval(x);
@@ -1573,19 +1558,18 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			readlong(jj);
 			if (jj == 0) {
 				// draw static object
-				readval(z);
-				string path = ostr;
-				readval(z);
-				string name = ostr;
+				string path;
+				string name;
+				readstr(path);
+				readstr(name);
 				draw_object_static(path, name, pcode, &cp, mkdrobjs);
 			} else {
 				// draw dynamic object (as before?)
-				readval(z);
-				temp_str = ostr;
+				readstr(temp_str);
 				ptr = *(pcode + ++cp); /* name */
 				if (ptr) {
-					readvalp(z, pcode + cp + ptr);
-					string obj_name = ostr;
+					readstrp(pcode + cp + ptr);
+					string obj_name = ostr->toUTF8();
 					draw_object(temp_str, obj_name.c_str());
 				} else {
 					draw_object(temp_str, NULL);
@@ -1607,10 +1591,12 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			if (x) *srclin = jj;
 			break;
 		  case 48: /* WHILE */
-			readval(x);
-			readlong(jj);
-			jump_back = false;
-			if (!x) *srclin = jj;
+		    {
+				bool ifValue = evalBool(getStack(), getPcodeList(), pcode, &cp);
+				readlong(jj);
+				jump_back = false;
+				if (!ifValue) *srclin = jj;
+		  	}
 			break;
 		  case 32: /* PRINT */
 		  case 49: /* WRITE */
@@ -1618,16 +1604,15 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 			temp_str = "";
 			while (cp < cmd_plen) {
 				readlong(t);
-				if (t!=49 && t!=32) gprint("WRITE, PCODE ERROR, %d  cp %d plen %d\n",t,cp,plen);
-				readlong(t);
-				if (t == 1) {
-					readval(x);
-					sprintf(ss,"%g ",x);
-					temp_str += ss;
-				} else {
-					readval(x);
-					temp_str += ostr;
+				if (t != 49 && t != 32) {
+					g_throw_parser_error("pcode error in print/write");
 				}
+				readlong(t);
+				gstr = evalString(getStack(), getPcodeList(), pcode, &cp, true);
+				if (!temp_str.empty()) {
+					temp_str += " ";
+				}
+				temp_str += gstr->toUTF8();
 			}
 			if (p == 49) {
 				/* WRITE */
@@ -1650,16 +1635,16 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 		  case 74: /* TeX */
 			{
 				x = 0.0;
-				readval(z);
 				GLERectangle box;
-				TeXInterface::getInstance()->draw(ostr, &box);
+				ostr = evalStringPtr(getStack(), getPcodeList(), pcode, &cp);
+				TeXInterface::getInstance()->draw(ostr->toUTF8(), &box);
 				ptr = *(pcode + cp); /* add */
 				if (ptr) {
 					readvalp(x, pcode + cp + ptr);
 				}
 				ptr = *(pcode + ++cp); /* name */
 				if (ptr) {
-					readvalp(z, pcode + cp + ptr);
+					readstrp(pcode + cp + ptr);
 					box.getDimensions(&x1,&y1,&x2,&y2);
 					x1 -= x; x2 += x; y1 -= x; y2 += x;
 					name_set(ostr,x1,y1,x2,y2);
@@ -1687,26 +1672,15 @@ void GLERun::do_pcode(GLESourceLine &sline, int *srclin, int *pcode, int plen, i
 }
 
 GLEBox::GLEBox():
-	m_Name(NULL),
 	m_HasStroke(true),
 	m_HasReverse(false),
 	m_Add(0.0),
 	m_IsRound(false),
+	m_Round(0.0),
 	m_Fill(g_get_fill_clear())
 {
 }
 
-GLEBox::GLEBox(const GLEBox& box):
-	m_Name(box.getName()),
-	m_HasStroke(box.hasStroke()),
-	m_HasReverse(box.hasReverse()),
-	m_Add(box.getAdd()),
-	m_IsRound(box.isRound()),
-	m_Round(box.getRound()),
-	m_Fill(box.getFill())
-{
-}
-
 void GLEBox::setFill(const GLERC<GLEColor>& fill) {
 	m_Fill = fill;
 }
@@ -1768,24 +1742,15 @@ void GLEBox::draw(GLERun* run, double x1, double y1, double x2, double y2) {
 	}
 	g_set_fill(cur_fill);
 	if (hasName()) {
-		run->name_set((char*)getName(), rect.getXMin(), rect.getYMin(), rect.getXMax(), rect.getYMax());
+		run->name_set(getName(), rect.getXMin(), rect.getYMin(), rect.getXMax(), rect.getYMax());
 	}
 	g_move(ox, oy);
 }
 
-GLEStoredBox::GLEStoredBox() : GLEBox() {
-	m_SecondPass = false; m_HasName = false;
-	m_Device = NULL;
-}
-
-GLEStoredBox::GLEStoredBox(const GLEStoredBox& box) : GLEBox(box), m_SaveBounds(box.m_SaveBounds), m_Object(box.getObjectRep()) {
-	m_SecondPass = box.m_SecondPass; m_Device = box.m_Device;
-	m_HasName = box.hasName(); m_Name = box.getName();
-}
-
-void GLEStoredBox::setName(const char* name) {
-	m_HasName = true;
-	m_Name = name;
+GLEStoredBox::GLEStoredBox():
+	m_SecondPass(false),
+	m_Device(0)
+{
 }
 
 GLEStoredBox* box_start() {
@@ -1826,7 +1791,7 @@ bool GLERun::box_end() throw (ParserError) {
 	if (box->getDevice() != NULL) {
 		g_restore_device(box->getDevice());
 	}
-	box->setNamePtr(box->hasName() ? box->getName().c_str() : NULL);
+	box->setName(box->hasName() ? box->getName() : NULL);
 	box->draw(this, x1, y1, x2, y2);
 	if (box->getSaveBounds()->isValid()) {
 		g_update_bounds(box->getSaveBounds());
@@ -1843,12 +1808,11 @@ bool GLERun::box_end() throw (ParserError) {
 
 void nm_adjust(GLEJustify jj, double *sx, double *sy, double ex, double ey, GLERectangle* r);
 
-void GLERun::name_set(const char *n, double x1, double y1, double x2, double y2) {
-	GLERC<GLEString> name(new GLEString(n));
+void GLERun::name_set(GLEString* name, double x1, double y1, double x2, double y2) {
 	GLERC<GLEObjectRepresention> obj(new GLEObjectRepresention());
 	obj->getRectangle()->setDimensions(x1, y1, x2, y2);
 	g_dev(obj->getRectangle());
-	if (!getCRObjectRep()->setChildObject(name.get(), obj.get())) {
+	if (!getCRObjectRep()->setChildObject(name, obj.get())) {
 		char ostr[80];
 		int idx, type;
 		name->toUTF8(ostr);
@@ -1935,13 +1899,11 @@ GLEObjectRepresention* GLERun::name_to_object(GLEObjectRepresention* obj, GLEArr
 	return obj;
 }
 
-bool GLERun::is_name(const char *name) {
-	char str[80];
+bool GLERun::is_name(GLEString* name) {
 	int idx, type;
-	GLEString sname(name);
-	GLERC<GLEArrayImpl> path(sname.split('.'));
+	GLERC<GLEArrayImpl> path(name->split('.'));
 	GLEString* objname = (GLEString*)path->getObjectUnsafe(0);
-	objname->toUTF8(str);
+	std::string str(objname->toUTF8());
 	getVars()->find(str, &idx, &type);
 	if (idx != -1) {
 		GLEDataObject* obj = getVars()->getObject(idx);
@@ -1956,13 +1918,11 @@ bool GLERun::is_name(const char *name) {
 	return false;
 }
 
-GLEObjectRepresention* GLERun::name_to_object(const char *name, GLEJustify* just) throw(ParserError) {
-	char str[80];
+GLEObjectRepresention* GLERun::name_to_object(GLEString* name, GLEJustify* just) throw(ParserError) {
 	int idx, type;
-	GLEString sname(name);
-	GLERC<GLEArrayImpl> path(sname.split('.'));
+	GLERC<GLEArrayImpl> path(name->split('.'));
 	GLEString* objname = (GLEString*)path->getObjectUnsafe(0);
-	objname->toUTF8(str);
+	std::string str(objname->toUTF8());
 	getVars()->find(str, &idx, &type);
 	if (idx != -1) {
 		GLEDataObject* obj = getVars()->getObject(idx);
@@ -1985,7 +1945,7 @@ GLEObjectRepresention* GLERun::name_to_object(const char *name, GLEJustify* just
 	return NULL;
 }
 
-void GLERun::name_to_point(const char *name, GLEPoint* point) throw(ParserError) {
+void GLERun::name_to_point(GLEString* name, GLEPoint* point) throw(ParserError) {
 	GLEJustify just;
 	GLEObjectRepresention* obj = name_to_object(name, &just);
 	if (obj != NULL) {
@@ -1998,7 +1958,7 @@ void GLERun::name_to_point(const char *name, GLEPoint* point) throw(ParserError)
 	}
 }
 
-void GLERun::name_to_size(const char *name, double *wd, double *hi) throw(ParserError) {
+void GLERun::name_to_size(GLEString* name, double *wd, double *hi) throw(ParserError) {
 	GLEJustify just;
 	GLEObjectRepresention* obj = name_to_object(name, &just);
 	if (obj != NULL) {
@@ -2012,7 +1972,7 @@ void GLERun::name_to_size(const char *name, double *wd, double *hi) throw(Parser
 	}
 }
 
-void GLERun::name_join(const char *n1, const char *n2, int marrow, double a1, double a2, double d1, double d2)  throw(ParserError) {
+void GLERun::name_join(GLEString *n1, GLEString *n2, int marrow, double a1, double a2, double d1, double d2)  throw(ParserError) {
 	GLEJustify j1, j2;
 	GLEObjectRepresention* obj1 = name_to_object(n1, &j1);
 	GLEObjectRepresention* obj2 = name_to_object(n2, &j2);
@@ -2044,8 +2004,6 @@ void GLERun::name_join(const char *n1, const char *n2, int marrow, double a1, do
 }
 
 void GLERun::draw_object_static(const string& path, const string& name, int* pcode, int* cp, bool mkdrobjs) throw (ParserError) {
-	int otyp;
-	double x;
 	int cp_backup = *cp;
 	GLEPoint orig;
 	g_get_xy(&orig);
@@ -2066,8 +2024,9 @@ void GLERun::draw_object_static(const string& path, const string& name, int* pco
 	GLEMeasureBox measure;
 	measure.measureStart();
 	g_move(0.0, 0.0);
+	GLERC<GLEArrayImpl> stk(new GLEArrayImpl());
 	if (mkdrobjs) {
-		GLESub* sub = eval_subroutine_call(pcode, cp, &otyp);
+		GLESub* sub = eval_subroutine_call(stk.get(), getPcodeList(), pcode, cp);
 		sub->setScript(getScript());
 		GLEObjectDOConstructor* cons = sub->getObjectDOConstructor();
 		GLEObjectDO objdo(cons);
@@ -2075,10 +2034,10 @@ void GLERun::draw_object_static(const string& path, const string& name, int* pco
 		GLEString* refpt = new GLEString();
 		refpt->join('.', a_path.get(), 1);
 		objdo.setRefPointString(refpt);
-		eval_do_object_block_call(sub, &objdo);
+		eval_do_object_block_call(stk.get(), sub, &objdo);
 		handleNewDrawObject(&objdo, mkdrobjs, &orig);
 	} else {
-		eval(pcode, cp, &x, NULL, &otyp);
+		evalGeneric(getStack(), getPcodeList(), pcode, cp);
 	}
 	if (hasoffs) measure.measureEndIgnore();
 	else measure.measureEnd();
@@ -2102,7 +2061,7 @@ void GLERun::draw_object_static(const string& path, const string& name, int* pco
 			g_translate(transl.getX(), transl.getY());
 			*cp = cp_backup;
 			g_move(0.0, 0.0);
-			eval(pcode, cp, &x, NULL, &otyp);
+			evalGeneric(getStack(), getPcodeList(), pcode, cp);
 			g_grestore();
 		} else {
 			/* dummy device, just update bounds */
@@ -2222,12 +2181,10 @@ void GLERun::draw_object_dynamic(int idx, GLEObjectRepresention* newobj, GLEArra
 		}
 		g_move(0.0, 0.0);
 		g_set_partial_state(oldstate);
-		int s_start = sub->getStart();
-		int s_end = sub->getEnd();
 		int endp = 0;
 		bool mkdrobjs = false;
-		for (int i = s_start + 1; i < s_end; i++) {
-			GLESourceLine* line = getSource()->getLine(i);
+		for (int i = sub->getStart() + 1; i < sub->getEnd(); i++) {
+			GLESourceLine* line = getSource()->getLine(i - 1);
 			do_pcode(*line, &i, gpcode[i], gplen[i], &endp, mkdrobjs);
 		}
 		if (local_vars != NULL) {
@@ -2290,7 +2247,7 @@ void GLERun::draw_object(const string& path, const char* newname) throw (ParserE
 	g_move(orig);
 }
 
-void GLERun::begin_object(const char* name, GLESub* sub) throw (ParserError) {
+void GLERun::begin_object(const std::string& name, GLESub* sub) throw (ParserError) {
 	GLEStoredBox* box = box_start();
 	box->setStroke(false);
 	box->setObjectRep(getCRObjectRep());
@@ -2300,7 +2257,7 @@ void GLERun::begin_object(const char* name, GLESub* sub) throw (ParserError) {
 	setCRObjectRep(newobj);
 	/* set name */
 	int idx, type;
-	getVars()->findAdd(name, &idx, &type);
+	getVars()->findAdd(name.c_str(), &idx, &type);
 	getVars()->setObject(idx, newobj);
 	/* create corresponding dynamic subroutine */
 	GLEDynamicSub* dynsub = new GLEDynamicSub(sub);
