@@ -45,6 +45,8 @@
 #include "keyword.h"
 #include "run.h"
 #include "cutils.h"
+#include "glearray.h"
+#include "polish.h"
 
 extern int **gpcode;   /* gpcode is a pointer to an array of poiter to int */
 extern int *gplen;     /* gpcode is a pointer to an array of int */
@@ -54,19 +56,29 @@ extern int gle_debug;
 
 GLESubMap g_Subroutines;
 
-int return_type = 1;
-double return_value = 0.0;
-string return_value_str;
-vector<string> return_str_stack;
-
-GLESubArgNames::GLESubArgNames() :
-	m_ArgNames(new GLEStringHash())
-{
+GLESubArgNames::GLESubArgNames() {
 }
 
 void GLESubArgNames::addArgName(const char* argName) {
-	GLERC<GLEString> strArgName(new GLEString(argName));
-	m_ArgNames->setObjectByKey(strArgName, strArgName.get());
+	addArgName(m_ArgNames.size(), argName);
+}
+
+void GLESubArgNames::addArgName(unsigned int argIndex, const char* argName) {
+	GLERC<GLEString> argNameStr(new GLEString(argName));
+	GLEStringHashData::iterator found(m_ArgNameHash.find(argNameStr));
+	if (found == m_ArgNameHash.end()) {
+		m_ArgNameHash.insert(make_pair(argNameStr, argIndex));
+	}
+	m_ArgNames.resize(argIndex + 1);
+	m_ArgNames.setObject(argIndex, argNameStr.get());
+}
+
+void GLESubArgNames::addArgNameAlias(unsigned int argIndex, const char* argName) {
+	GLERC<GLEString> argNameStr(new GLEString(argName));
+	GLEStringHashData::iterator found(m_ArgNameHash.find(argNameStr));
+	if (found == m_ArgNameHash.end()) {
+		m_ArgNameHash.insert(make_pair(argNameStr, argIndex));
+	}
 }
 
 GLESubRoot::GLESubRoot(GLEString* name, GLESubArgNames* argNames) :
@@ -80,6 +92,43 @@ GLESubRoot::GLESubRoot(GLEString* name, GLESubArgNames* argNames) :
 void GLESubRoot::updateArgNames(GLESubArgNames* argNames) {
 }
 
+GLESubDefinitionHelper::GLESubDefinitionHelper(const std::string& name) {
+	m_Defaults = new GLEArrayImpl();
+	m_ArgNames = new GLESubArgNames();
+	m_Name = new GLEString(name);
+}
+
+void GLESubDefinitionHelper::addArgumentAlias(unsigned int argIndex, const std::string& name) {
+	m_ArgNames->addArgNameAlias(argIndex, name.c_str());
+}
+
+unsigned int GLESubDefinitionHelper::addArgument(const std::string& name, unsigned int type, bool mandatory) {
+	unsigned int nbArguments = m_ArgTypes.size();
+	m_ArgTypes.push_back(type);
+	m_isMandatory.push_back(mandatory);
+	m_Defaults->resize(nbArguments + 1);
+	m_ArgNames->addArgName(nbArguments, name.c_str());
+	return nbArguments;
+}
+
+unsigned int GLESubDefinitionHelper::addDoubleArgument(const std::string& name, double defaultValue, bool mandatory) {
+	unsigned int result = addArgument(name, GLEObjectTypeDouble, mandatory);
+	m_Defaults->setDouble(result, defaultValue);
+	return result;
+}
+
+unsigned int GLESubDefinitionHelper::addDoubleArgumentNoDefault(const std::string& name, bool mandatory) {
+	return addArgument(name, GLEObjectTypeDouble, mandatory);
+}
+
+unsigned int GLESubDefinitionHelper::addPointArgument(const std::string& name, GLEPointDataObject* defaultValue, bool mandatory) {
+	unsigned int result = addArgument(name, GLEObjectTypePoint, mandatory);
+	if (defaultValue != 0) {
+		m_Defaults->setObject(result, defaultValue);
+	}
+	return result;
+}
+
 GLESubSignature::GLESubSignature(GLESubRoot* root) :
 	m_Root(root),
 	m_Subroutines(new GLEArrayWithFreeList()),
@@ -315,113 +364,80 @@ GLESub* sub_get(int idx) throw(ParserError) {
 
 extern int this_line;
 
+/*
+int pos = --npm;
+std::cerr << "param: " << i << " from stack pos: " << pos << " = ";
+gle_memory_cell_print(stk->get(pos), std::cerr);
+std::cerr << std::endl;
+*/
+
 /* 	Run a user defined function  */
-void GLERun::sub_call(int idx, double *pval, char **pstr, int *npm, int *otyp) throw(ParserError) {
+void GLERun::sub_call_stack(GLESub* sub, GLEArrayImpl* stk) throw(ParserError) {
 	// Save current return value
-	int save_return_type = return_type;
-	double save_return_value = return_value;
-	if (return_type == 2) {
-		// Efficiency of this will be improved if RefCount objects are introduced
-		// see glearray.cpp (Jan Struyf)
-		return_str_stack.push_back(return_value_str);
-	}
-	GLESub* sub = sub_get(idx);
+	GLEMemoryCell save_return_value;
+	GLE_MC_INIT(save_return_value);
+	GLE_MC_COPY(&save_return_value, &m_returnValue);
 	// Set local variable map
 	GLEVarMap* sub_map = sub->getLocalVars();
 	GLEVarMap* save_var_map = var_swap_local_map(sub_map);
 	var_alloc_local(sub_map);
 	// Copy parameters to local variables
+	int npm = stk->size();
 	for (int i = sub->getNbParam()-1; i >= 0; i--) {
-		int var = i | GLE_VAR_LOCAL_BIT;
-		if (sub->getParamType(i) == 1)  {
-			var_set(var, *(pval+(*npm)--));
-		} else {
-			var_setstr(var, *(pstr+(*npm)--));
-		}
+		getVars()->set(i | GLE_VAR_LOCAL_BIT, stk->get(--npm));
 	}
 	// Run subroutine
-	int s_start = sub->getStart();
-	int s_end = sub->getEnd();
 	int endp = 0;
 	bool mkdrobjs = false;
 	int oldline = this_line;
-	// cout << "calling routine: " << sub->getName() << endl;
-	for (int i = s_start + 1; i < s_end; i++) {
-		// cout << "executing line " << i << " of " << getSource()->getNbLines() << endl;
-		GLESourceLine* line = getSource()->getLine(i);
-		do_pcode(*line,&i,gpcode[i],gplen[i],&endp,mkdrobjs);
-		dbg gprint("AFTER DO_PCODE I = %d \n",i);
+	for (int i = sub->getStart() + 1; i < sub->getEnd(); i++) {
+		GLESourceLine* line = getSource()->getLine(i - 1);
+		do_pcode(*line, &i, gpcode[i], gplen[i], &endp, mkdrobjs);
 	}
 	// FIXME: Find more elegant way to back up current line
 	this_line = oldline;
-	// Return type of subroutine
-	if (return_type == 1) {
-		*(pval + ++(*npm)) = return_value;
-		*otyp = 1;
-	} else {
-		(*npm) = (*npm) + 1;
-		if (pstr[(*npm)] != NULL) myfree(pstr[(*npm)]);
-		pstr[(*npm)] = sdup((char*)return_value_str.c_str());
-		*otyp = 2;
-	}
-	// Restore var map
+	stk->decrementSize(sub->getNbParam() - 1);
+	stk->ensure(npm + 1);
+	stk->set(npm, &m_returnValue);
 	var_set_local_map(save_var_map);
-	// Get saved return values back
-	return_type = save_return_type;
-	if (return_type == 1) {
-		return_value = save_return_value;
-	} else {
-		return_value_str = return_str_stack.back();
-		return_str_stack.pop_back();
-	}
+	GLE_MC_COPY(&m_returnValue, &save_return_value);
 	var_free_local();
 }
 
 /* 	Run a user defined function  */
-void GLERun::sub_call(GLESub* sub) throw(ParserError) {
-	// Save current return value
-	int save_return_type = return_type;
-	double save_return_value = return_value;
-	if (return_type == 2) {
-		// Efficiency of this will be improved if RefCount objects are introduced
-		// see glearray.cpp (Jan Struyf)
-		return_str_stack.push_back(return_value_str);
-	}
+void GLERun::sub_call(GLESub* sub, GLEArrayImpl* arguments) throw(ParserError) {
+	GLEMemoryCell save_return_value;
+	GLE_MC_INIT(save_return_value);
+	GLE_MC_COPY(&save_return_value, &m_returnValue);
 	// Set local variable map
 	GLEVarMap* sub_map = sub->getLocalVars();
 	GLEVarMap* save_var_map = var_swap_local_map(sub_map);
 	var_alloc_local(sub_map);
+	// Copy parameters to local variables
+	if (arguments != 0) {
+		CUtilsAssert(sub->getNbParam() == (int)arguments->size());
+		for (int i = 0; i < sub->getNbParam(); ++i) {
+			getVars()->set(i | GLE_VAR_LOCAL_BIT, arguments->get(i));
+		}
+	}
 	// Run subroutine
-	int s_start = sub->getStart();
-	int s_end = sub->getEnd();
 	int endp = 0;
 	bool mkdrobjs = false;
 	int oldline = this_line;
-	for (int i = s_start + 1; i < s_end; i++) {
-		dbg gprint("=Call do pcode, line %d ",i);
-		GLESourceLine* line = getSource()->getLine(i);
+	for (int i = sub->getStart() + 1; i < sub->getEnd(); i++) {
+		GLESourceLine* line = getSource()->getLine(i - 1);
 		do_pcode(*line, &i, gpcode[i], gplen[i], &endp, mkdrobjs);
-		dbg gprint("AFTER DO_PCODE I = %d \n",i);
 	}
 	// FIXME: Find more elegant way to back up current line
 	this_line = oldline;
-	// Restore var map
 	var_set_local_map(save_var_map);
-	// Get saved return values back
-	return_type = save_return_type;
-	if (return_type == 1) {
-		return_value = save_return_value;
-	} else {
-		return_value_str = return_str_stack.back();
-		return_str_stack.pop_back();
-	}
+	GLE_MC_COPY(&m_returnValue, &save_return_value);
 	var_free_local();
 }
 
 void call_sub_byname(const string& name, double* args, int nb, const char* err_inf) throw(ParserError) {
 	GLESub* sub = sub_find(name);
-	int idx = sub != NULL ? sub->getIndex() : -1;
-	if (idx == -1)  {
+	if (sub == NULL)  {
 		stringstream err;
 		err << "subroutine '" << name << "' not found";
 		if (err_inf != NULL) err << " " << err_inf;
@@ -432,16 +448,8 @@ void call_sub_byname(const string& name, double* args, int nb, const char* err_i
 		if (err_inf != NULL) err << " " << err_inf;
 		g_throw_parser_error(err.str());
 	}
-	for (int i = 0; i < nb; i++) {
-		if (sub->getParamType(i) != 1)  {
-			stringstream err;
-			err << "all parameters of subroutine '" << name << "' should be numeric";
-			if (err_inf != NULL) err << " " << err_inf;
-			g_throw_parser_error(err.str());
-		}
-	}
-	int otype;
-	getGLERunInstance()->sub_call(idx, args, NULL, &nb, &otype);
+	GLERC<GLEArrayImpl> stk(doublesToArray(args, nb));
+	getGLERunInstance()->sub_call(sub, stk.get());
 }
 
 void call_sub_byid(int idx, double* args, int nb, const char* err_inf) throw(ParserError) {
@@ -453,25 +461,6 @@ void call_sub_byid(int idx, double* args, int nb, const char* err_inf) throw(Par
 		if (err_inf != NULL) err << " " << err_inf;
 		g_throw_parser_error(err.str());
 	}
-	for (int i = 0; i < nb; i++) {
-		if (sub->getParamType(i) != 1)  {
-			stringstream err;
-			err << "all parameters of subroutine '" << sub->getName() << "' should be numeric";
-			if (err_inf != NULL) err << " " << err_inf;
-			g_throw_parser_error(err.str());
-		}
-	}
-	int otype;
-	getGLERunInstance()->sub_call(idx, args, NULL, &nb, &otype);
+	GLERC<GLEArrayImpl> stk(doublesToArray(args, nb));
+	getGLERunInstance()->sub_call(sub, stk.get());
 }
-
-void sub_set_return(double d) {
-	return_type = 1;
-	return_value = d;
-}
-
-void sub_set_return_str(const char* s) {
-	return_type = 2;
-	return_value_str = s;
-}
-
