@@ -123,11 +123,11 @@ void set_parskip(double v);
 void set_lineskip(double v);
 int ncat(char *a,char *b,int n);
 void pp_fntchar(int f, int c,int *out, int *lout);
-void cmd_token(uchar **in,char *cmdstr);
+void cmd_token(uchar **in, uchar *cmdstr);
 void set_base_size(void);
 int find_primcmd(char *cmd);
 void text_box(const string& s,double width,int *tbuff, int *rplen);
-uchar* cmdParam(uchar **inp, char** pm, int* pmlen, int npm);
+uchar* cmdParam(uchar **inp, uchar **pm, int* pmlen, int npm);
 void text_tomacro(const string& in, uchar *out);
 void texint(char *s, int *i);
 void texint(const string& s, int *i);
@@ -171,7 +171,7 @@ typedef struct def_table_struct deftable;
 
 bool tex_def(const char *name, const char *defn,int npm);
 int tex_mathdef(const char *name, int defn);
-int *tex_findmathdef(const char *s);
+int* tex_findmathdef(const char *s);
 void tex_chardef(int c, const char *defn);
 char *tex_findchardef(int c);
 deftable *tex_finddef(const char *s) ;
@@ -241,8 +241,9 @@ void set_base_size() {
 	g_get_hei(&base_size);
 }
 
-void set_tex_font(int fnt) {
+GLECoreFont* set_tex_font(int fnt) {
 	p_fnt = g_font_fallback(fnt);
+	return get_core_font_ensure_loaded(p_fnt);
 }
 
 void try_get_next_char(uchar **in, int *c1) {
@@ -273,7 +274,7 @@ uchar try_get_next_two_chars(uchar **in, int *c1, int *c2) {
 		if (str_ni_equals((const char*)*in, "UCHR{", 5)) {
 			char *ptr;
 			unsigned int unicode = strtol((const char*)(*in)+5, &ptr, 16);
-			GLECoreFont* cfont = get_core_font_ensure_loaded(p_fnt);
+			GLECoreFont* cfont = set_tex_font(p_fnt);
 			int font_code = cfont->unicode_map(unicode);
 			if (font_code != -1) {
 				code = 1;
@@ -290,7 +291,7 @@ uchar try_get_next_two_chars(uchar **in, int *c1, int *c2) {
 			if (str_ni_equals((const char*)(*in)+1, "UCHR{", 5)) {
 				char *ptr;
 				unsigned int unicode = strtol((const char*)(*in)+6, &ptr, 16);
-				GLECoreFont* cfont = get_core_font_ensure_loaded(p_fnt);
+				GLECoreFont* cfont = set_tex_font(p_fnt);
 				int font_code = cfont->unicode_map(unicode);
 				if (font_code != -1) {
 					*c2 = font_code;
@@ -317,8 +318,7 @@ void text_topcode(uchar *in, int *out, int *lout) {
 		/* if next char is normal, then check for ligature and kern */
 norm_again:
 		w = 0;
-	    p_fnt = g_font_fallback(p_fnt);
-		cfont = get_core_font_ensure_loaded(p_fnt);
+		cfont = set_tex_font(p_fnt);
 		if (c2 != 0) {
 			if (!g_CmdLine.hasOption(GLE_OPT_NO_LIGATURES)) {
 				if (cfont->char_lig(&c1, c2)) {
@@ -331,16 +331,15 @@ norm_again:
 		}
 		outlong(1);
 		outlong(c1 | p_fnt*1024);
-		dbg gprint("==char width %d %f %f \n", c1, cfont->cdata[c1]->wx, w);
-		outfloat((cfont->cdata[c1]->wx + w)*p_hei);
+		dbg gprint("==char width %d %f %f \n", c1, cfont->getCharDataThrow(c1)->wx, w);
+		outfloat((cfont->getCharDataThrow(c1)->wx + w)*p_hei);
 		skip_space = false;
 		break;
 	    case 2: /* Single space */
 		if (skip_space) break;
 		skip_space = true;
 		outlong(2);
-		p_fnt = g_font_fallback(p_fnt);
-		cfont = get_core_font_ensure_loaded(p_fnt);
+		cfont = set_tex_font(p_fnt);
 		outfloat(cfont->info.space*p_hei);
 		outfloat(cfont->info.space_stretch*p_hei*10*stretch_factor);
 		outfloat(cfont->info.space_shrink*p_hei*10);
@@ -392,19 +391,19 @@ norm_again:
 	}
 }
 
-uchar* cmdParam(uchar **inp, char** pm, int* pmlen, int npm) {
+uchar* cmdParam(uchar **inp, uchar **pm, int* pmlen, int npm) {
 	int gcnt=0,i;
-	uchar *save_inp = *inp; /* need this to look ahead but not move ahaead */
-	char *in = (char*)*inp;
+	uchar* save_inp = *inp; /* need this to look ahead but not move ahaead */
+	uchar* in = *inp;
 	gcnt = 0;
 	for (i=0;i<npm;i++) {
 		pm[i] = in;
 		pmlen[i] = 0;
-		if (chr_code[(unsigned int)*in]==7) { /* begin group */
+		if (chr_code[*in]==7) { /* begin group */
 			pm[i] = ++in;
 			for (;*in!=0;in++) {
-				if (chr_code[(unsigned int)*in]==7) gcnt++;
-				if (chr_code[(unsigned int)*in]==8) {
+				if (chr_code[*in]==7) gcnt++;
+				if (chr_code[*in]==8) {
 					if (gcnt==0) break;
 					gcnt--;
 				}
@@ -412,7 +411,7 @@ uchar* cmdParam(uchar **inp, char** pm, int* pmlen, int npm) {
 			pmlen[i] = in - pm[i];
 			in++;
 		} else {
-			if (chr_code[(unsigned int)*in]==6) { /* backslash look for non-alpha */
+			if (chr_code[*in]==6) { /* backslash look for non-alpha */
 				pm[i] = ++in;
 				if (isalpha(*pm[i])) {
 					for (;*in!=0;in++) {
@@ -529,15 +528,16 @@ void mathchar_bbox(int m, double* x1, double* y1, double* x2, double* y2, double
 	if (mtyp == 7) mtyp = 0;
 	int font = fontfam[mfam][tofont[curstyle]];
 	char_bbox(font, mchar, x1, y1, x2, y2);
-	*cw = fnt[font]->cdata[mchar]->wx;
+	*cw = fnt[font]->getCharDataThrow(mchar)->wx;
 }
 
 void pp_fntchar(int ff, int ch, int *out,int *lout) {
+	ff = g_font_fallback(ff);
 	outlong(1);
 	if (ch == 0) ch = 254;
 	outlong(ch | ff*1024);
 	GLECoreFont* cfont = get_core_font_ensure_loaded(ff);
-	outfloat(cfont->cdata[ch]->wx * p_hei);
+	outfloat(cfont->getCharDataThrow(ch)->wx * p_hei);
 }
 
 void pp_pcode(int *pbuff, int plen, int *out,int *lout) {
@@ -560,7 +560,7 @@ void p_unichar(const string& str, int *out, int *lout) {
 		myfree(workbuff);
 	} else {
 		int i = 0;
-		int myfnt = 31;
+		int myfnt = g_font_fallback(31);
 		double savehei = p_hei;
 		p_sethei(0.4*savehei);
 		pp_move(0.0, 0.4*savehei, out, lout);
@@ -568,7 +568,7 @@ void p_unichar(const string& str, int *out, int *lout) {
 		double crxpos = 0.0;
 		while (str[i] != 0) {
 			int ch = str[i];
-			double wid = cfont->cdata[ch]->wx * p_hei;
+			double wid = cfont->getCharDataThrow(ch)->wx * p_hei;
 			switch (i) {
 				case 0:
 				case 1:
@@ -587,54 +587,54 @@ void p_unichar(const string& str, int *out, int *lout) {
 }
 
 void TexArgStrs::cmdParam1(uchar **in) {
-	char *s[2];
+	uchar *s[2];
 	int pmlen[2];
 	cmdParam(in, s, pmlen, 1);
-	str1.assign(s[0], pmlen[0]);
+	str1.assign((char*)s[0], pmlen[0]);
 }
 
 void TexArgStrs::cmdParam12(uchar **in) {
-	char *s[2];
+	uchar *s[2];
 	int pmlen[2];
 	cmdParam(in, s, pmlen, 1);
-	str2.assign(s[0],pmlen[0]);
+	str2.assign((char*)s[0],pmlen[0]);
 }
 
 void TexArgStrs::cmdParam2(uchar **in) {
-	char *s[3];
+	uchar *s[3];
 	int pmlen[3];
 	cmdParam(in, s, pmlen, 2);
-	str1.assign(s[0],pmlen[0]);
-	str2.assign(s[1],pmlen[1]);
+	str1.assign((char*)s[0],pmlen[0]);
+	str2.assign((char*)s[1],pmlen[1]);
 }
 
 void TexArgStrs::cmdParam3(uchar **in) {
-	char *s[4];
+	uchar *s[4];
 	int pmlen[4];
 	cmdParam(in, s, pmlen, 3);
-	str1.assign(s[0],pmlen[0]);
-	str2.assign(s[1],pmlen[1]);
-	str3.assign(s[2],pmlen[2]);
+	str1.assign((char*)s[0],pmlen[0]);
+	str2.assign((char*)s[1],pmlen[1]);
+	str3.assign((char*)s[2],pmlen[2]);
 }
 
 void TexArgStrs::cmdParam4(uchar **in) {
-	char *s[5];
+	uchar *s[5];
 	int pmlen[5];
 	cmdParam(in, s, pmlen, 4);
-	str1.assign(s[0],pmlen[0]);
-	str2.assign(s[1],pmlen[1]);
-	str3.assign(s[2],pmlen[2]);
-	str4.assign(s[3],pmlen[3]);
+	str1.assign((char*)s[0],pmlen[0]);
+	str2.assign((char*)s[1],pmlen[1]);
+	str3.assign((char*)s[2],pmlen[2]);
+	str4.assign((char*)s[3],pmlen[3]);
 }
 
 void TexArgStrs::cmdParam4_swap34(uchar **in) {
-	char *s[5];
+	uchar *s[5];
 	int pmlen[5];
 	cmdParam(in, s, pmlen, 4);
-	str1.assign(s[0],pmlen[0]);
-	str2.assign(s[1],pmlen[1]);
-	str4.assign(s[2],pmlen[2]);
-	str3.assign(s[3],pmlen[3]);
+	str1.assign((char*)s[0],pmlen[0]);
+	str2.assign((char*)s[1],pmlen[1]);
+	str4.assign((char*)s[2],pmlen[2]);
+	str3.assign((char*)s[3],pmlen[3]);
 }
 
 void tex_get_char_code(uchar** in, int* code) {
@@ -676,10 +676,10 @@ void tex_draw_accent(uchar **in, TexArgStrs* params, int *out, int *lout) {
 	// cout << "'" << params->str1 << "' '" << params->str2 << "' '" << params->str3 << "'" << endl;
 	// cout << *in << endl;
 	char_bbox(newfnt, ix, &lef, &dep, &wid, &hei);
-	cwid = p_hei * fnt[newfnt]->cdata[ix]->wx;
+	cwid = p_hei * fnt[newfnt]->getCharDataThrow(ix)->wx;
 	if (m == NULL) {
 		char_bbox(p_fnt, ix2, &lef2, &dep2, &wid2, &hei2);
-		cwid2 = p_hei * fnt[p_fnt]->cdata[ix2]->wx;
+		cwid2 = p_hei * fnt[p_fnt]->getCharDataThrow(ix2)->wx;
 	} else {
 		mathchar_bbox(*m, &lef2, &dep2, &wid2, &hei2, &cwid2);
 		cwid2 *= p_hei;
@@ -699,7 +699,6 @@ void tex_draw_accent(uchar **in, TexArgStrs* params, int *out, int *lout) {
 	p_fntchar(newfnt,ix);        /* cwid2/2 + cwid/2 */
 	p_move(-cwid+cwid2-lef2-wid2/2+wid/2-accent_x,-h-accent_y);
 	set_tex_font(savefnt);
-	font_load_metric(p_fnt);
 }
 
 void tex_draw_accent_cmb(uchar **in, TexArgStrs* params, int *out, int *lout) {
@@ -708,11 +707,11 @@ void tex_draw_accent_cmb(uchar **in, TexArgStrs* params, int *out, int *lout) {
 			int accent;
 			int ch = params->str3[0];
 			texint(params->str4, &accent);
-			GLECoreFont* cfont = get_core_font_ensure_loaded(p_fnt);
+			GLECoreFont* cfont = set_tex_font(p_fnt);
 			FontCompositeInfo* info = cfont->get_composite_char(ch, accent);
 			if (info != NULL) {
-				double w1 = cfont->cdata[info->c1]->wx * p_hei;
-				double w2 = cfont->cdata[info->c2]->wx * p_hei;
+				double w1 = cfont->getCharDataThrow(info->c1)->wx * p_hei;
+				double w2 = cfont->getCharDataThrow(info->c2)->wx * p_hei;
 				double dx1 = info->dx1 * p_hei;
 				double dx2 = info->dx2 * p_hei;
 				double dy1 = info->dy1 * p_hei;
@@ -745,21 +744,21 @@ int select_font_encoding(int font, int encoding, const char* defaultFont) {
 void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 	int ci;
 	int ix;
-	char cmdstr[20];
+	uchar cmdstr[20];
 	double lef,wid,hei,dep,savehei;
 	int *pbuff=0;
 	int plen;
-	char *pmu[10];
+	uchar *pmu[10];
 	int pmlen[10];
 	int *m,i,k,n,npm;
 
 	k = 0;
 
 	cmd_token(in,cmdstr);   /* finds command name and parameters */
-	ci = find_primcmd(cmdstr);
+	ci = find_primcmd((char*)cmdstr);
 
 	if (ci==0) { /* then maybe its a mathchar */
-		m = tex_findmathdef(cmdstr);
+		m = tex_findmathdef((char*)cmdstr);
 		if (m!=0) {
 			p_mathchar(*m);
 		} else {
@@ -774,7 +773,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		cmdParam(in,pmu,pmlen,1);
 		savehei = p_hei;
 		p_hei = p_hei * .7;
-		topcode(pmu[0],pmlen[0],0.0,&pbuff,&plen,&lef,&wid,&hei,&dep);
+		topcode((char*)pmu[0],pmlen[0],0.0,&pbuff,&plen,&lef,&wid,&hei,&dep);
 		/*g_measure(pm.s[0],&lef,&wid,&hei,&dep);
 		fftext_block(UC pm.s[0],0.0,0);
 		g_get_bounds(&lef,&wid,&hei,&dep);*/
@@ -783,13 +782,13 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		p_pcode(pbuff,plen);
 		/*printf("this P=[%s]\n",pm.s[0]);*/
 		p_move(0,-0.8*p_hei);
-		find_primcmd(cmdstr);
+		find_primcmd((char*)cmdstr);
 
 		/* do a look ahead to see if there is a superscript */
 		/* but keep in pointing to current spot */
 		*in = cmdParam(in,pmu,pmlen,1);
 		/* check to see if subscript is next */
-		if(strncmp(pmu[0],"sub ",4)==0) {
+		if(strncmp((char*)pmu[0],"sub ",4)==0) {
 			/* move back to starting point */
 			/*p_move(-wid,0);
 			printf("subscript next P=[%s]\n",pm.s[0]);
@@ -803,7 +802,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		cmdParam(in,pmu,pmlen,1);
 		savehei = p_hei;
 		p_hei = p_hei * .7;
-		topcode(pmu[0],pmlen[0],0.0,&pbuff,&plen,&lef,&wid,&hei,&dep);
+		topcode((char*)pmu[0],pmlen[0],0.0,&pbuff,&plen,&lef,&wid,&hei,&dep);
 		/*printf("lef=%g wid=%g hei=%g dep=%g\n",lef,wid,hei,dep);*/
 		p_move(0.0,-0.3*p_hei);
 		p_pcode(pbuff,plen);
@@ -814,7 +813,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 		/* but keep in pointing to current spot */
 		*in = cmdParam(in,pmu,pmlen,1);
 		/* check to see if superscript is next */
-		if(strncmp(pmu[0],"sup ",4)==0) {
+		if(strncmp((char*)pmu[0],"sup ",4)==0) {
 			/* move back to starting point */
 			/*p_move(-wid,0);*/
 			/*printf("supescript next B=[%s]\n",pm.s[0]);*/
@@ -852,7 +851,6 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 	  case tp_fontenc:
 		params->cmdParam2(in);
 		set_tex_font(select_font_encoding(p_fnt, atoi(params->getCStr1()), params->getCStr2()));
-		font_load_metric(p_fnt);
 		break;
 	  case tp_acccmb:
   		params->cmdParam4_swap34(in);
@@ -911,17 +909,16 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 	  case tp_mathcode:
 		params->cmdParam2(in);
 		texint(params->str2,&ix);
-		chr_mathcode[(unsigned int)params->str1[0]] = ix;
+		chr_mathcode[(unsigned char)params->str1[0]] = ix;
 		break;
 	  case tp_delcode:
 		params->cmdParam2(in);
 		texint(params->str2,&ix);
-		chr_mathcode[(unsigned int)params->str1[0]] = ix;
+		chr_mathcode[(unsigned char)params->str1[0]] = ix;
 		break;
 	  case tp_setfont:
 		params->cmdParam1(in);
 		set_tex_font(pass_font(params->getCStr1()));
-		font_load_metric(p_fnt);
 		break;
 	  case tp_presave:
 		gprint("Saving definitions\n");
@@ -973,7 +970,7 @@ void do_prim(uchar **in, int *out, int *lout, TexArgStrs* params) {
 
 double emtof(char *s) {       /* same as ATOF but if it sees EM then it multiplies by FONT HEI */
 	if (strstr(s,"sp")!=NULL) {
-		GLECoreFont* cfont = get_core_font_ensure_loaded(p_fnt);
+		GLECoreFont* cfont = set_tex_font(p_fnt);
 		return atof(s) * cfont->info.space*p_hei;
 	}
 	if (strstr(s,"em")!=NULL) {
@@ -984,7 +981,7 @@ double emtof(char *s) {       /* same as ATOF but if it sees EM then it multipli
 
 double emtof(const string& s) {
 	if (str_i_str(s, "sp") != -1) {
-		GLECoreFont* cfont = get_core_font_ensure_loaded(p_fnt);
+		GLECoreFont* cfont = set_tex_font(p_fnt);
 		return atof(s.c_str()) * cfont->info.space*p_hei;
 	}
 	if (str_i_str(s, "em") != -1) {
@@ -1071,10 +1068,13 @@ void text_wrapcode(int *in,int ilen,double width) {
 		crFont = g_font_fallback((*(in+ ++i)) / 1024);
 		cfont = get_core_font_ensure_loaded(crFont);
 		c = *(in+i) & 0x3ff;
-		if (cdep > cy + p_hei * cfont->cdata[c]->y1)
-			cdep = cy + p_hei * cfont->cdata[c]->y1;
-		if (chei < cy + p_hei * cfont->cdata[c]->y2)
-			chei = cy + p_hei * cfont->cdata[c]->y2;
+		{
+			GLEFontCharData* charData = cfont->getCharDataThrow(c);
+			if (cdep > cy + p_hei * charData->y1)
+				cdep = cy + p_hei * charData->y1;
+			if (chei < cy + p_hei * charData->y2)
+				chei = cy + p_hei * charData->y2;
+		}
 		/* gprint("chei=%f, cdep=%f \n",chei,cdep); */
 		cx += tofloat(*(in+ ++i));
 		ax = cx;
@@ -1202,7 +1202,6 @@ void text_wrapcode(int *in,int ilen,double width) {
 	    case 9: /* font     p_fnt   */
 	    crFont = g_font_fallback(*(in+ ++i));
 		font_load_metric(crFont);
-
 		break;
 	    case 20: /*  nop  */
 		break;
@@ -1318,8 +1317,11 @@ void text_draw(int *in,int ilen) {
 	    crFont = g_font_fallback((*(in+ ++i)) / 1024);
 		cfont = get_core_font_ensure_loaded(crFont);
 		c = *(in+i) & 0x3ff;
-		g_update_bounds(cx+p_hei*(cfont->cdata[c]->x1), cy+p_hei*(cfont->cdata[c]->y1));
-		g_update_bounds(cx+p_hei*(cfont->cdata[c]->x2), cy+p_hei*(cfont->cdata[c]->y2));
+		{
+			GLEFontCharData* cdata = cfont->getCharDataThrow(c);
+			g_update_bounds(cx+p_hei*(cdata->x1), cy+p_hei*(cdata->y1));
+			g_update_bounds(cx+p_hei*(cdata->x2), cy+p_hei*(cdata->y2));
+		}
 		dp {
 			g_move(cx,cy);
 			g_char(crFont,c);
@@ -1488,42 +1490,6 @@ void font_get_parskip(double *ls,double *gap) {
 #define get_exps(ss) polish(ss,(char *) pcode,plen,&etype)
 #define tok(n)  tk[n]
 
-void get_font(TOKENS tk,int *ntok,int *curtok,int *pcode,int *plen) {
-	int etype=1;
-	char vv[80];
-	char *p;
-	if (fnt.size() == 0) font_load();
-	if ( (*tok(*curtok) == '"' ) ||  (strchr(tok(*curtok),'$') != NULL) ) {
-		strcpy(vv,"cvtfont(");
-		strcat(vv,tok(*curtok)); strcat(vv,")");
-		get_exps(vv);
-		(*curtok)++;
-		return;
-	}
-	p = tk[*curtok];
-	(*curtok)++;
-	*(pcode+(*plen)++) = 8;
-	for (unsigned int i = 1; i < fnt.size(); i++) {
-		if (fnt[i]->name != NULL && str_i_equals(fnt[i]->name,p)) {
-			*(pcode+(*plen)++) = i; /* font number */
-			return;
-		}
-	}
-	ostringstream err;
-	err << "invalid font name: '" << p << "', expecting one of:";
-	for (unsigned int i = 1; i < fnt.size(); i++) {
-		if ((i-1) % 5 == 0) {
-			err << endl << "       ";
-		} else {
-			err << " ";
-		}
-		err << fnt[i]->name;
-	}
-	gprint(err.str().c_str());
-	/* default font number */
-	*(pcode+(*plen)++) = 1;
-}
-
 const char* get_font_name(int idx) {
 	GLECoreFont* cfont = get_core_font(idx);
 	return cfont->name;
@@ -1599,8 +1565,9 @@ void tex_chardef(int c, const char *defn) {
 	cdeftable[c] = sdup(defn);
 }
 
-char *tex_findchardef(int c) {
-	return cdeftable[c];
+char* tex_findchardef(int c) {
+	if (c < 0 || c > 255) return "";
+	else return cdeftable[c];
 }
 
 /*----------------------------------------------------------------------*/
@@ -1662,9 +1629,8 @@ typedef struct mdef_table_struct mdeftable;
 
 static mdeftable  *mdef_hashtab[HASHSIZE];
 
-int *tex_findmathdef(const char *s) {
-	mdeftable  *np;
-
+int* tex_findmathdef(const char *s) {
+	mdeftable* np;
 	for (np = mdef_hashtab[hash_str(s)]; np != NULL; np = np->next)
 		if (strcmp(s, np->name)==0) {
 			return &np->defn;
@@ -1817,7 +1783,7 @@ void fftext_block(const string& s,double width,int justify) {
 	g_set_hei(p_hei);
 }
 
-void cmd_token(uchar **in,char *cmdstr) {
+void cmd_token(uchar **in, uchar *cmdstr) {
 	int i = 0;
 	if ( (!isalpha(**in)) && (**in != 0)) {
 		if ((*in)[0] == '\'' && (*in)[1] == '\'') {
@@ -1834,7 +1800,7 @@ void cmd_token(uchar **in,char *cmdstr) {
 	}
 	*cmdstr = 0;
 	cmdstr -= 1;
-	if (chr_code[(unsigned int)*cmdstr]==1) {
+	if (chr_code[*cmdstr]==1) {
 		// if last character is alpha, then eat all subsequent space
 		// (chr_code alpha = 1, space = 2)
 		for (;(**in != 0) && (chr_code[**in]==2);) (*in)++;
@@ -1843,12 +1809,12 @@ void cmd_token(uchar **in,char *cmdstr) {
 
 void  text_tomacro(const string& in, uchar *out) {
 	/* find /cmdname  or defined characters */
-	char macroname[30];
+	uchar macroname[30];
 	uchar *s,*dfn,*r,*saves;
 	int dlen;
 	int nrep;
 	deftable  *np;
-	char *pmu[10];
+	uchar* pmu[10];
 	int pmlen[10];
 	nrep = 0;
 	strcpy(SC out,SC in.c_str());
@@ -1861,7 +1827,7 @@ void  text_tomacro(const string& in, uchar *out) {
 			s++;
 			cmd_token(&s,macroname);
 			// cout << "macro: " << macroname << endl;
-			np = tex_finddef(macroname);
+			np = tex_finddef((char*)macroname);
 			if (np != NULL) {
 				// cout << "found!" << endl;
 				nrep++;
@@ -1869,7 +1835,7 @@ void  text_tomacro(const string& in, uchar *out) {
 				dbg printf("Found macro {%s} = {%s} \n",macroname,dfn);
 				cmdParam(&s,pmu,pmlen,np->npm);
 				dlen = s-saves;
-				r = UC tex_replace(SC dfn,pmu,pmlen,np->npm);
+				r = UC tex_replace(SC dfn,(char**)pmu,pmlen,np->npm);
 				s = saves;
 				memmove(SC s+strlen(SC r),SC s+dlen,strlen(SC s)+1);
 				strncpy(SC s,SC r,strlen(SC r));
@@ -1877,17 +1843,17 @@ void  text_tomacro(const string& in, uchar *out) {
 				s--;
 			}
 			s = saves;
-			if (strcmp(macroname, "tex")==0) {
+			if (strcmp((char*)macroname, "tex")==0) {
 				// do not expand macros inside \tex{} expression
 				s = UC str_skip_brackets((char*)s, '{', '}');
 			}
-			if (strcmp(macroname, "unicode")==0) {
+			if (strcmp((char*)macroname, "unicode")==0) {
 				// do not expand macros inside \unicode{}{} expression
 				s = UC str_skip_brackets((char*)s, '{', '}');
 				if (s[0] == '}') s++;
 				s = UC str_skip_brackets((char*)s, '{', '}');
 			}
-			if (strcmp(macroname, "def")==0) {
+			if (strcmp((char*)macroname, "def")==0) {
 				// make sure not to expand macro's in macro name
 				// otherwise, during a second pass, the name will be replaced by the value
 				// e.g., \def\a{b} will trun into: \defb{b{}
@@ -1909,75 +1875,60 @@ void  text_tomacro(const string& in, uchar *out) {
 void tex_presave() {
 	int i;
 	deftable *dt;
-	FILE *fout;
+	GLEFileIO fout;
 	mdeftable *mdt;
 	/* Save all defined features possible */
 	string fname = gledir("inittex.ini");
-	fout = fopen(fname.c_str(), "wb");
-	if (fout==NULL) gprint("Could not create inittex.ini file \n");
-	fwrite(fontfam,sizeof(int),16*4,fout);
-	fwrite(fontfamsz,sizeof(double),16*4,fout);
-	fwrite(chr_mathcode,sizeof(char),256,fout);
+
+	fout.open(fname.c_str(), "wb");
+	if (!fout.isOpen()) gprint("Could not create inittex.ini file \n");
+	fout.fwrite(fontfam, sizeof(int), 16*4);
+	fout.fwrite(fontfamsz, sizeof(double), 16*4);
+	fout.fwrite(chr_mathcode, sizeof(char), 256);
 	for (i=0;i<HASHSIZE;i++) {
 	  for (dt = def_hashtab[i]; dt != NULL; dt = dt->next) {
-		fwrite(&i,sizeof(i),1,fout);
-		fwrite(&dt->npm,sizeof(i),1,fout);
-		fsendstr(dt->name,fout);
-		fsendstr(dt->defn,fout);
+		fout.fwrite(&i, sizeof(i), 1);
+		fout.fwrite(&dt->npm, sizeof(i), 1);
+		fout.fsendstr(dt->name);
+		fout.fsendstr(dt->defn);
 	  }
 	}
-	i = 0x0fff; fwrite(&i,sizeof(i),1,fout);
+	i = 0x0fff; fout.fwrite(&i, sizeof(i), 1);
 	for (i=0;i<HASHSIZE;i++) {
 	  for (mdt = mdef_hashtab[i]; mdt != NULL; mdt = mdt->next) {
-		fwrite(&i,sizeof(i),1,fout);
-		fwrite(&mdt->defn,sizeof(i),1,fout);
-		fsendstr(mdt->name,fout);
+		fout.fwrite(&i, sizeof(i), 1);
+		fout.fwrite(&mdt->defn, sizeof(i), 1);
+		fout.fsendstr(mdt->name);
 	  }
 	}
-	i = 0x0fff; fwrite(&i,sizeof(i),1,fout);
-	for (i=0;i<256;i++) fsendstr(cdeftable[i],fout);
+	i = 0x0fff; fout.fwrite(&i, sizeof(i), 1);
+	for (i=0;i<256;i++) fout.fsendstr(cdeftable[i]);
 	/* write unicode definitions */
 	for (IntStringHash::const_iterator it = m_Unicode.begin(); it != m_Unicode.end(); it++ ) {
 		int key = it->first;
 		const string& data = it->second;
 		int len = data.size();
-		fwrite(&key,sizeof(int),1,fout);
-		fwrite(&len,sizeof(int),1,fout);
-		fwrite(data.c_str(),sizeof(char),len,fout);
+		fout.fwrite(&key, sizeof(int), 1);
+		fout.fwrite(&len, sizeof(int), 1);
+		fout.fwrite(data.c_str(), sizeof(char),len);
 	}
 	i = 0;
-	fwrite(&i,sizeof(int),1,fout);
-	fclose(fout);
+	fout.fwrite(&i, sizeof(int), 1);
+	fout.close();
 }
 
-void fgetvstr(char **s,FILE *fmt) {
-	int i = fgetc(fmt);
+void fgetvstr(char** s, GLEFileIO* fmt) {
+	int i = fmt->fgetc();
 	if (i == 0) return;
 	if (*s != NULL) myfree(*s);
 	*s = (char*) myalloc(i+1);
-	fread(*s, 1, i, fmt);
+	fmt->fread(*s, 1, i);
 	*(*s+i) = 0;
 }
 
-void fgetcstr(char s[],FILE *fmt) {
-	int i = fgetc(fmt);
-	if (i == 0) return;
-	fread(s, 1, i, fmt);
-	s[i] = 0;
-}
-
-void fsendstr(char *s, FILE *fout) {
-	if (s == NULL) {
-		fputc(0, fout);
-		return;
-	}
-	fputc(strlen(s), fout);
-	fwrite(s, 1, strlen(s), fout);
-}
-
 void tex_preload() {
 	int i,j;
-	FILE *fout;
+	GLEFileIO fout;
 	char str1[80],str2[80];
 	/* reload all defined features */
 #ifdef DO_TEX_DECODE
@@ -1985,33 +1936,33 @@ void tex_preload() {
 	fhv = fopen("decode.tex","w");
 #endif
 	string fname = gledir("inittex.ini");
-	fout = fopen(fname.c_str(), "rb");
-	if (fout == NULL) {
+	fout.open(fname.c_str(), "rb");
+	if (!fout.isOpen()) {
 		if (!IS_INSTALL) gprint("Could not open inittex.ini file \n");
 		return;
 	}
-	fread(fontfam,sizeof(int),16*4,fout);
-	fread(fontfamsz,sizeof(double),16*4,fout);
-	fread(chr_mathcode,sizeof(char),256,fout);
+	fout.fread(fontfam, sizeof(int), 16*4);
+	fout.fread(fontfamsz, sizeof(double), 16*4);
+	fout.fread(chr_mathcode, sizeof(char), 256);
 #ifdef DO_TEX_DECODE
 	for(k=0;k<=16;k++){
 		fprintf(fhv,"*fontfam[%d]=%s  fontfamsz[%d]=%s\n",k,fontfam[k],k,fontfamsz[k]);
 	}
 	fprintf(fhv,"chr_mathcode=%s\n",chr_mathcode);
 #endif
-	for (;  (fread(&i,sizeof(i),1,fout)), i != 0x0fff;) {
-		fread(&j,sizeof(j),1,fout);
-		fgetcstr(str1,fout);
-		fgetcstr(str2,fout);
+	for (; fout.fread(&i, sizeof(i), 1), i != 0x0fff;) {
+		fout.fread(&j, sizeof(j), 1);
+		fout.fgetcstr(str1);
+		fout.fgetcstr(str2);
 		tex_def(str1,str2,j);
 #ifdef DO_TEX_DECODE
 		fprintf(fhv,"str1=%s  str2=%s j=%d\n",str1,str2,j);
 #endif
 	}
 	i=0;
-	for (;  (fread(&i,sizeof(i),1,fout)), i != 0x0fff;) {
-		fread(&j,sizeof(j),1,fout);
-		fgetcstr(str1,fout);
+	for (; fout.fread(&i, sizeof(i), 1), i != 0x0fff;) {
+		fout.fread(&j, sizeof(j), 1);
+		fout.fgetcstr(str1);
 		tex_mathdef(str1,j);
 #ifdef DO_TEX_DECODE
 		fprintf(fhv,"str1=%s  i=%d\n",str1,i);
@@ -2019,7 +1970,7 @@ void tex_preload() {
 	}
 	i=0;
 	for (i=0;i<256;i++){
-		fgetvstr(&cdeftable[i],fout);
+		fgetvstr(&cdeftable[i], &fout);
 #ifdef DO_TEX_DECODE
 		fprintf(fhv,"cdeftable[%d]=%s\n",i,cdeftable[i]);
 #endif
@@ -2027,23 +1978,23 @@ void tex_preload() {
 	/* load unicode definitions */
 	m_Unicode.clear();
 	int key;
-	fread(&key,sizeof(int),1,fout);
+	fout.fread(&key, sizeof(int), 1);
 	char* read_ptr = NULL;
 	int read_ptr_len = 0;
 	while (key != 0) {
 		int len;
-		fread(&len,sizeof(int),1,fout);
+		fout.fread(&len, sizeof(int), 1);
 		if (len >= read_ptr_len) {
 			read_ptr_len = 2*read_ptr_len + len + 1;
 			read_ptr = (char*)realloc(read_ptr, read_ptr_len);
 		}
-		fread(read_ptr,sizeof(char),len,fout);
+		fout.fread(read_ptr, sizeof(char), len);
 		read_ptr[len] = 0;
 		m_Unicode.add_item(key, read_ptr);
-		fread(&key,sizeof(int),1,fout);
+		fout.fread(&key, sizeof(int), 1);
 	}
 	if (read_ptr != NULL) free(read_ptr);
-	fclose(fout);
+	fout.close();
 #ifdef DO_TEX_DECODE
 	fclose(fhv);
 #endif
