@@ -109,9 +109,9 @@ TeXObject* TeXInterface::draw(const char* str) throw(ParserError) {
 	return draw(str, info, 1);
 }
 
-TeXObject* TeXInterface::draw(const char* str, GLERectangle* box) throw(ParserError) {
+TeXObject* TeXInterface::draw(const std::string& str, GLERectangle* box) throw(ParserError) {
 	TeXObjectInfo info;
-	return draw(str, info, 1, box);
+	return draw(str.c_str(), info, 1, box);
 }
 TeXObject* TeXInterface::drawUTF8(const char* str, GLERectangle* box) throw(ParserError) {
 	TeXObjectInfo info;
@@ -439,7 +439,7 @@ void TeXInterface::writeInc(ostream& out, const char* prefix) {
 		TeXObject* obj = m_TeXObjects[i];
 		obj->output(out);
 	}
-	out << "\\end{picture}" << endl;
+	out << "\\end{picture}%" << endl;
 }
 
 void TeXInterface::createInc(const string& prefix) {
@@ -482,6 +482,9 @@ void TeXInterface::createTeX(bool usegeom) {
 		}
 		string tex_name = m_MainOutputName.getFullPath();
 		tex_name += ".tex";
+		if (GLEFileExists(tex_name)) {
+			g_throw_parser_error("GLE needs to create a temporary file '", tex_name.c_str(), "', but this file already exists");
+		}
 		ofstream tex_file(tex_name.c_str());
 		createPreamble(tex_file);
 		tex_file << "\\usepackage{color}" << endl;
@@ -1519,20 +1522,18 @@ void begin_tex_preamble(int *pln, int *pcode, int *cp) {
 
 void begin_tex(GLERun* run, int *pln, int *pcode, int *cp) {
 	// Get optional params
-	string name;
-	double add = 0.0, x;
+	GLERC<GLEString> name;
+	double add = 0.0;
 	int ptr = *(pcode + (*cp)); /* add */
 	if (ptr) {
-		int zzcp = 0, otyp;
-		eval(pcode + (*cp) + ptr, &zzcp, &add, NULL, &otyp);
+		int zzcp = 0;
+		add = evalDouble(run->getStack(), run->getPcodeList(), pcode + (*cp) + ptr, &zzcp);
 	}
 	(*cp) = (*cp) + 1;
 	ptr = *(pcode + (*cp)); /* name */
 	if (ptr) {
-		int zzcp = 0, otyp;
-		const char* ostr = NULL;
-		eval(pcode + (*cp) + ptr, &zzcp, &x, &ostr, &otyp);
-		name = ostr;
+		int zzcp = 0;
+		name = evalString(run->getStack(), run->getPcodeList(), pcode + (*cp) + ptr, &zzcp, true);
 	}
 	// Start with pcode from the next line
 	(*pln)++;
@@ -1559,10 +1560,10 @@ void begin_tex(GLERun* run, int *pln, int *pcode, int *cp) {
 	decode_utf8_basic(text_block);
 	TeXInterface::getInstance()->draw(text_block.c_str(), nblines, &box);
 	// Name object
-	if (name.length() != 0) {
+	if (!name.isNull() && name->length() != 0) {
 		double x1, x2, y1, y2;
 		box.getDimensions(&x1,&y1,&x2,&y2);
 		x1 -= add; x2 += add; y1 -= add; y2 += add;
-		run->name_set(name.c_str(),x1,y1,x2,y2);
+		run->name_set(name.get(),x1,y1,x2,y2);
 	}
 }
