@@ -22,10 +22,15 @@
 #include <QtGui>
 #include <QtOpenGL>
 #include "3dviewer.h"
+#include "../config.h"
 #include "../gle/cutils.h"
 #include "../gle/gle-block.h"
 #include "../gle/surface/gsurface.h"
 
+#ifdef HAVE_LIBGLU_H
+	#include <GL/glu.h>
+#endif
+
 #include <math.h>
 
 QGLE3DWidget::QGLE3DWidget(QWidget *parent, GLEInterface* iface)
