@@ -115,20 +115,24 @@ ExportDialogue::ExportDialogue(GLEMainWindow *parent) {
 	p1->addStretch(1);
 	layout->addLayout(p1);
 	QHBoxLayout *p2 = new QHBoxLayout();
-	resolutionLabel = new QLabel(tr("Resolution:"));
+	resolutionLabel = new QLabel(tr("Resolution (dpi):"));
 	p2->addWidget(resolutionLabel);
 	resolution = new QSpinBox();
 	resolution->setMinimum(1);
 	resolution->setMaximum(2000);
-	resolution->setValue(96);
 	connect(resolution, SIGNAL(valueChanged(int)), this, SLOT(resolutionChanged(int)));
 	p2->addWidget(resolution);
 	sizeLabel = new QLabel();
 	p2->addWidget(sizeLabel);
 	p2->addStretch(1);
 	layout->addLayout(p2);
+	layout->addWidget(new QLabel(tr("(For vector formats, 'resolution' specifies the bitmap fallback resolution.)")));
 	transp = new QCheckBox(tr("Transparent"));
+	transp->setChecked(mainWin->settings->isExportTransparent());
 	layout->addWidget(transp);
+	grayScale = new QCheckBox(tr("Grayscale"));
+	grayScale->setChecked(mainWin->settings->isExportGrayScale());
+	layout->addWidget(grayScale);
 	openResult = new QCheckBox(tr("View resulting file"));
 	openResult->setChecked(mainWin->settings->isOpenExportedFigure());
 	layout->addWidget(openResult);
@@ -151,7 +155,7 @@ ExportDialogue::ExportDialogue(GLEMainWindow *parent) {
 	layout->addLayout(p3);
 	setLayout(layout);
 	resize(mainWin->width()/2, mainWin->height()/2);
-	exportFormatChanged(mainWin->settings->getExportFormat());
+	exportFormatChangedImpl(mainWin->settings->getExportFormat(), true);
 }
 
 void ExportDialogue::addFormat(const QString& name, int gleFormatID) {
@@ -166,8 +170,15 @@ int ExportDialogue::entryIDtoDevice(int id) {
 	return GLE_DEVICE_EPS;
 }
 
+bool ExportDialogue::isBitmap()
+{
+	return device == GLE_DEVICE_PNG || device == GLE_DEVICE_JPEG;
+}
+
 void ExportDialogue::resolutionChanged(int) {
-	updateResolution();
+	if (isBitmap()) {
+		updateResolution();
+	}
 }
 
 void ExportDialogue::exportAsClicked() {
@@ -190,25 +201,35 @@ void ExportDialogue::performExport(const QString& file) {
 	mainWin->clearConsoleWindow();
 	iface->clearAllCmdLine();
 	iface->setMakeDrawObjects(false);
-	bool isBitmap = device == GLE_DEVICE_PNG || device == GLE_DEVICE_JPEG;
-	if (isBitmap) {
-		QString dpi = QString("%1").arg(resolution->value());
-		iface->setCmdLineOptionString("resolution", dpi.toLatin1().constData());
-	}
+	QString dpi = QString("%1").arg(resolution->value());
+	iface->setCmdLineOptionString("resolution", dpi.toLatin1().constData());
 	if (device == GLE_DEVICE_PNG && transp->isChecked()) {
 		iface->setCmdLineOption("transparent");
 	}
+	if (grayScale->isChecked()) {
+		iface->setCmdLineOption("nocolor");
+	}
 	if (format->currentIndex() != 0) {
 		iface->setCmdLineOption("fullpage");
 	}
 	if (format->currentIndex() == 2) {
 		iface->setCmdLineOption("landscape");
 	}
+	if (mainWin->settings->isRenderUsingCairo()) {
+		iface->setCmdLineOption("cairo");
+	}
 	iface->setCompatibilityMode(mainWin->getCompatibility().toLatin1().constData());
 	mainWin->settings->setExportPageSize(format->currentIndex());
 	iface->renderGLE(mainWin->getGLEScript(), file.toLatin1().constData(), device);
 	iface->clearAllCmdLine();
 	mainWin->settings->setOpenExportedFigure(openResult->isChecked());
+	if (isBitmap()) {
+		mainWin->settings->setExportBitmapResolution(resolution->value());
+	} else {
+		mainWin->settings->setExportVectorResolution(resolution->value());
+	}
+	mainWin->settings->setExportGrayScale(grayScale->isChecked());
+	mainWin->settings->setExportTransparent(transp->isChecked());
 	if (openResult->isChecked()) {
 		mainWin->showLocalFile(file);
 	}
@@ -218,30 +239,39 @@ void ExportDialogue::performExport(const QString& file) {
 void ExportDialogue::updateResolution() {
 	double dpi = (double)resolution->value();
 	const GLEPoint& bb = mainWin->getGLEScript()->getBoundingBox();
-	int img_wd = (int)floor((double)dpi/72.0*bb.getX()+1);
-	int img_hi = (int)floor((double)dpi/72.0*bb.getY()+1);
+	int img_wd = (int)floor((double)dpi/PS_POINTS_PER_INCH*bb.getX()+1);
+	int img_hi = (int)floor((double)dpi/PS_POINTS_PER_INCH*bb.getY()+1);
 	sizeLabel->setText(QString("(%1 x %2 pixels)").arg(img_wd).arg(img_hi));
 }
 
 void ExportDialogue::exportFormatChanged(int idx) {
+	exportFormatChangedImpl(idx, false);
+}
+
+void ExportDialogue::exportFormatChangedImpl(int idx, bool updateAll) {
 	device = entryIDtoDevice(idx);
 	transp->setEnabled(device == GLE_DEVICE_PNG);
-	bool isBitmap = device == GLE_DEVICE_PNG || device == GLE_DEVICE_JPEG;
-	resolution->setEnabled(isBitmap);
-	resolutionLabel->setEnabled(isBitmap);
 	if (device == GLE_DEVICE_PS && format->currentIndex() == 0) {
 		// "Figure" output not compatible with PS
 		format->setCurrentIndex(1);
 	}
 	const char* ext = mainWin->getGLEInterface()->getDeviceFilenameExtension(device);
 	fname->setText(QGLE::addFileNameExtension(fname->text(), ext));
-	if (isBitmap) {
+	if (isBitmap()) {
 		updateResolution();
 	} else {
 		const GLEPoint& size = mainWin->getGLEScript()->getSize();
 		sizeLabel->setText(QString("(%1 x %2 cm)").arg(size.getX()).arg(size.getY()));
 	}
 	mainWin->settings->setExportFormat(idx);
+	if (previousWasBitmap != isBitmap() || updateAll) {
+		if (isBitmap()) {
+			resolution->setValue(mainWin->settings->getExportBitmapResolution());
+		} else {
+			resolution->setValue(mainWin->settings->getExportVectorResolution());
+		}
+	}
+	previousWasBitmap = isBitmap();
 }
 
 SoftwareLocateThread::SoftwareLocateThread(SoftwareLocateDialogue* parent, const QString& root) {
@@ -300,7 +330,7 @@ SoftwareLocateDialogue::SoftwareLocateDialogue(GLEMainWindow *parent, GLEInterfa
 	connect(searchButton, SIGNAL(clicked()), this, SLOT(searchAuto()));
 	buttonLayout->addWidget(searchButton);
 	if ((buttons & QGLE_SOFT_DIALOG_SEARCH_MANUAL) != 0) {
-		locButton = new QPushButton(tr("Manually Locate"));
+		locButton = new QPushButton(tr("Manually Locate GhostScript"));
 		connect(locButton, SIGNAL(clicked()), this, SLOT(locateManual()));
 		buttonLayout->addWidget(locButton);
 	}
