@@ -184,7 +184,6 @@ void GLEDrawingArea::clearDirty()
 	dirty = false;
 	// The following sets the '*' on the title bar
 	mainWin->setWindowModified(false);
-	qDebug() << "Dirty flag cleared";
 	emit dirtyFlagChanged(dirty);
 }
 
@@ -201,7 +200,6 @@ void GLEDrawingArea::setDirty()
 	if (dirty) return;
 	dirty = true;
 	mainWin->setWindowModified(true);
-	qDebug() << "Dirty flag set";
 	emit dirtyFlagChanged(dirty);
 }
 
@@ -256,6 +254,7 @@ void GLEDrawingArea::createNewArcFromScript(GLEArcDO* arc) {
 	objectList[currentIndex]->setPoint(GLEArc::MidPoint, gleToQt(arc->getPointMid(dummy)));
 	objectList[currentIndex]->setGLEObject(arc);
 	objectList[currentIndex]->setLineProperties();
+	objectList[currentIndex]->setPropertyNoUpdate(GLELine::ArrowHeadPresence, arc->getArrow());
 }
 
 // Create new text object based on GLE object
@@ -287,8 +286,10 @@ void GLEDrawingArea::createNewObjectFromScript(GLEObjectDO* obj) {
 	GLEPropertyStore* props = obj->getProperties();
 	double fontsize = props->getRealProperty(GLEDOPropertyFontSize);
 	newObj->setPropertyNoUpdate(GLEObjectBlock::FontSize, fontsize);
-	int number = props->getFontProperty(GLEDOPropertyFont)->getNumber();
-	newObj->setPropertyNoUpdate(GLEObjectBlock::FontName, number);
+	GLEFont* font = props->getFontProperty(GLEDOPropertyFont);
+	GLEFont* base = font->getBaseFont();
+	newObj->setPropertyNoUpdate(GLEObjectBlock::FontName, base->getNumber());
+	newObj->setPropertyNoUpdate(GLEObjectBlock::FontStyle, base->checkStyle(font));
 	if (props->getModel()->isSupportScale()) {
 		GLEArrayImpl* arr = props->getArray();
 		newObj->setLength(GLEObjectBlock::ObjectWidth, arr->getDouble(0), false);
@@ -495,8 +496,7 @@ void GLEDrawingArea::clearObjects()
 		delete obj;
 	}
 	objectList.clear();
-	qDebug() << "Object list cleared";
-   updateSelection();
+    updateSelection();
 }
 
 // Update the resolution and notify children
@@ -608,10 +608,8 @@ void GLEDrawingArea::createPolarSnaps(QPointF qtPoint)
 	SnapLine *snap;
 	SnapLine *sn;
 	bool duplicate;
-	qDebug() << "Creating snaps from point " << qtPoint;
 	for(double i = polarStartAngle; i < 360.0 ; i += polarAngleIncrement)
 	{
-		qDebug() << "Creating snap line at angle " << i;
 		snap = new SnapLine(dpi, pixmap.size(), this);
 		snap->setPoint(SnapLine::StartPoint, qtPoint);
 		snap->setPoint(SnapLine::Angle, QPointF(QGLE::degreesToRadians(i),0.0));
@@ -1384,13 +1382,11 @@ void GLEDrawingArea::changeLastPoint(QPointF p)
 void GLEDrawingArea::clearNewObjectsFlag()
 {
 	newObjects = false;
-	qDebug() << "NEW Objects flag cleared";
 }
 
 void GLEDrawingArea::setNewObjectsFlag()
 {
 	newObjects = true;
-	qDebug() << "NEW Objects flag set";
 }
 
 // Identify whether there are new objects
@@ -1454,7 +1450,6 @@ void GLEDrawingArea::setEditMode(bool state)
 // Handle key presses
 void GLEDrawingArea::hitKey(int key, int modifiers)
 {
-	qDebug() << "Key event: " << key;
 	if (modifiers != Qt::NoModifier)
 		return;
 
@@ -2074,7 +2069,6 @@ void GLEDrawingArea::createNewTextObject(QPointF referencePoint)
 	thisTextObject->setGLEObject(gleScript->newGLEObject(GDOText));
 	objectList.append(thisTextObject);
 	currentIndex = objectList.size() - 1;
-	qDebug() << "Added new text object at index " << currentIndex;
 	// Make sure the line is updated when the image size changes
 	connect(this, SIGNAL(sizeChanged(QSize,double)), objectList[currentIndex], SLOT(setPixmapSize(QSize,double)));
 	objectList[currentIndex]->setTextProperties();
@@ -2094,8 +2088,6 @@ void GLEDrawingArea::createNewLine(QPointF startPoint)
 	// Get the index
 	currentIndex = objectList.size() - 1;
 
-	qDebug() << "Added new line at index " << currentIndex;
-
 	// Make sure the line is updated when the image size changes
 	connect(this, SIGNAL(sizeChanged(QSize,double)),
 			objectList[currentIndex], SLOT(setPixmapSize(QSize,double)));
@@ -2148,8 +2140,6 @@ void GLEDrawingArea::createNewTanLine(QPointF startPoint)
 			// Remember the object to which we are drawing
 			baseObject = shortest_index;
 
-			qDebug() << "Added new tan line at index " << currentIndex;
-
 			// Make sure the line is updated when the image size changes
 			connect(this, SIGNAL(sizeChanged(QSize,double)),
 					objectList[currentIndex], SLOT(setPixmapSize(QSize,double)));
@@ -2212,8 +2202,6 @@ void GLEDrawingArea::createNewPerpLine(QPointF startPoint)
 			// Remember the object to which we are drawing
 			baseObject = shortest_index;
 
-			qDebug() << "Added new perpendicular line at index " << currentIndex;
-
 			// Make sure the line is updated when the image size changes
 			connect(this, SIGNAL(sizeChanged(QSize,double)),
 					objectList[currentIndex], SLOT(setPixmapSize(QSize,double)));
@@ -2249,7 +2237,6 @@ void GLEDrawingArea::createNewArc(QPointF startPoint)
 	objectList.append(currentArc);
 	// Get the index
 	currentIndex = objectList.size() - 1;
-	qDebug() << "Added new arc at index " << currentIndex;
 
 	// Make sure the line is updated when the image size changes
 	connect(this, SIGNAL(sizeChanged(QSize,double)),
@@ -2282,7 +2269,6 @@ void GLEDrawingArea::setAmove(QPointF pt)
 {
 	if (hasAMove)
 	{
-		qDebug() << "Removing old amove at index " << amoveIndex;
 		// Delete the object
 		delete(objectList[amoveIndex]);
 
@@ -2294,8 +2280,6 @@ void GLEDrawingArea::setAmove(QPointF pt)
 	objectList.append(amove);
 	amoveIndex = objectList.size() - 1;
 
-	qDebug() << "Added new amove at index " << amoveIndex;
-
 	connect(this, SIGNAL(sizeChanged(QSize,double)),
 			objectList[amoveIndex], SLOT(setPixmapSize(QSize,double)));
 
@@ -2319,7 +2303,6 @@ void GLEDrawingArea::createNewCircle(QPointF centrePoint)
 	objectList.append(currentCircle);
 	// Get the index
 	currentIndex = objectList.size() - 1;
-	qDebug() << "Added new circle at index " << currentIndex;
 
 	// Make sure the line is updated when the image size changes
 	connect(this, SIGNAL(sizeChanged(QSize,double)),
@@ -2351,7 +2334,6 @@ void GLEDrawingArea::createNewEllipse(QPointF cornerPoint)
 	objectList.append(currentEllipse);
 	// Get the index
 	currentIndex = objectList.size() - 1;
-	qDebug() << "Added new ellipse at index " << currentIndex;
 
 	// Make sure the line is updated when the image size changes
 	connect(this, SIGNAL(sizeChanged(QSize,double)),
@@ -2541,7 +2523,6 @@ void GLEDrawingArea::selectObjects(QPointF pt, bool multiSelect)
 				// TODO: Set basePoint to centre of selection
 				// NOTE: Currently disabled because generating GLE script code
 				//       from rotated objects is not yet implemented
-				qDebug() << "Beginning rotate";
 				isRotating = true;
 				basePoint.setX((handleList[TopLeft].x() + handleList[BottomRight].x())/2);
 				basePoint.setY((handleList[TopLeft].y() + handleList[BottomRight].y())/2);
