@@ -697,9 +697,9 @@ void GLEMainWindow::updateWindowTitle()
 	QString title(APP_NAME);
 	if (has) {
 		title += tr(" - ") + currFile->primaryFile();
-		// Always include "[*]", it is treated special by Qt and only filled in when actually modified
-		title += tr(" [*]");
 	}
+	// Always include "[*]", it is treated special by Qt and only filled in when actually modified
+	title += tr(" [*]");
 	setWindowTitle(title);
 }
 
@@ -1583,9 +1583,23 @@ void GLEMainWindow::renderComplete(QImage image)
 
 	updateWindowTitle();
 
+	addFilesToMonitor();
 	executeToDos();
 }
 
+void GLEMainWindow::addFilesToMonitor()
+{
+	GLEFileInfo* currFile = getCurrentFile();
+	currFile->clearFilesToMonitor();
+	GLEInterface* iface = getGLEInterface();
+	vector<GLEFileLocation> infos = iface->getFileInfos();
+	for (unsigned int i = 0; i < infos.size(); i++) {
+		GLEFileLocation& info = infos[i];
+		QString fullpath = QString::fromUtf8(info.getFullPath().c_str());
+		currFile->addFileToMonitor(fullpath);
+	}
+}
+
 //! Execute this after opening a file
 void GLEMainWindow::executeToDoAfterFileOpen()
 {
@@ -2299,8 +2313,8 @@ void GLEMainWindow::newFile()
 	{
 		gleScript = gleInterface->newGLEFile(code.toLatin1().constData(), tempFiles[NewGLE].toLatin1().constData());
 		// Since we've added a "size" line, the file has (arguably) changed
-		drawingArea->setDirty();
 		updateWindowTitle();
+		drawingArea->setDirty();
 	}
 	else
 	{
